;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :clasp)

;;; ***************************************************************************

(defun boot (data boot-function &rest boot-function-arguments &key
		  (sampling-function #'sample-with-replacement-from-sequence)
		  sampling-function-arguments
		  (number-of-iterations 500))
  (remf boot-function-arguments :sampling-function)
  (remf boot-function-arguments :sampling-function-arguments)
  (remf boot-function-arguments :number-of-iterations)
  (loop for i from 1 to number-of-iterations
	collect (apply boot-function
		       (apply sampling-function data
			      sampling-function-arguments)
		       boot-function-arguments)))

(defun sample-with-replacement-from-sequence
  (sequence &rest args &key number-of-samples &allow-other-keys)
  (declare (ignore args))
  (let ((len (length sequence)))
    (when (null number-of-samples)
	  (setq number-of-samples len))
    (loop for i from 1 to number-of-samples
	  collect (elt sequence (random len)))))

(defun sample-without-replacement-from-sequence
  (sequence &rest args &key number-of-samples &allow-other-keys)
  (if (listp sequence)
      (apply #'sample-without-replacement-from-list sequence args)
    (let ((indices (loop for i from 1 to (length sequence)
			 collect (- i 1))))
      (setq indices
	    (apply #'sample-without-replacement-from-list indices args))
      (loop for i in indices
	    collect (elt sequence i)))))

(defun sample-without-replacement-from-list
  (list &rest args &key number-of-samples &allow-other-keys)
  (declare (ignore args))
  (when (null number-of-samples)
	(setq number-of-samples (length list)))
  (sample-without-replacement-from-list-1 (copy-list list)
					  number-of-samples))

;; some of the "sequence" references below used to say "list"
;; I changed them, because it didn't work. ms 10-31-94
(defun sample-without-replacement-from-list-1 (sequence length)
  (when (and sequence
	     (> length 0))
	(let* ((pos (random (length sequence)))
	       (temp (elt sequence pos)))
	  (setf (elt sequence pos) (elt sequence 0))
	  (cons temp (sample-without-replacement-from-list-1 (cdr sequence)
							     (- length 1))))))

