;;;
;;; dataset-view.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dataset-view class and methods.


================================================================
Status =========================================================
================================================================
In-progress.

FIX: Define :help-spec that tells about contents, starting drags,
selecting then doing menu items, etc.


================================================================
Change history =================================================
================================================================
22-May-94 mc	Created.
7-Jul-94  ms    Changed double-click to option-click
                fixed scroll-bar-click-open problem

|#


(in-package "CLASP-INTERFACE")

(export '(*font-spec-variables*
          *font-spec-datasets*))

(require "OBJECT-DROPPER")


;;;
;;; Define variables.
;;;

;;;
;;; Define dataset-view .
;;;

(defclass dataset-view (sequence-dialog-item)
  ()
  (:documentation "Shows all defined datasets. Lists each CLASP
dataset with its variables listed and indented under it. Selection is
:disjoint . Abbreviated 'dv'.")
  (:DEFAULT-INITARGS
    :view-nick-name 'dataset-view
    ;:view-position #@(3 2)              ; set by parent window
    ;:view-size #@(294 146)              ; set by parent window
    :CELL-SIZE #@(167 11)               ;was #@(294 16) ;FIX
    :SELECTION-TYPE :DISJOINT
    :TABLE-HSCROLLP NIL
    :TABLE-VSCROLLP T
    :TABLE-SEQUENCE ()                  ; set by update-dataset-view
    :view-font *font-spec-variables*
    :table-print-function
    #'(lambda (dataset-or-var stream)
        "Indents dataset-or-var if it's a variable."
        (declare (ignore stream))
        (let ((font-spec (if (typep dataset-or-var 'clasp::dataset)
                           *font-spec-datasets*
                           *font-spec-variables*))
              (string (format nil "~A~A"
                              ;; Create indent for variables.
                              (if (typep dataset-or-var 'clasp::dataset) "" "   ")
                              ;; Create name. FIX: Use a prefix for datasets and variables?
                              (if (typep dataset-or-var 'clasp::dataset)
                                (clasp::name dataset-or-var)
                                (clasp::name dataset-or-var)))))
          (multiple-value-bind (ff ms) (font-codes font-spec)
            (with-font-codes ff ms
              (with-pstrs ((p-str string))
                (#_DrawString p-str))))))))


(defmethod initialize-instance :after ((dataset-view dataset-view) &key)
  "Calls update-dataset-view ."
  ;;
  (set-cell-size dataset-view (point-h (cell-size dataset-view)) (font-height *font-spec-variables*))
  (update-dataset-view dataset-view))


(defmethod update-dataset-view ((dataset-view dataset-view))
  "Initializes dataset-view's table-sequence."
  ;;
  (set-table-sequence dataset-view
   ;; Following creates an ordered list of datasets, each followed by
   ;;  its variables.
   (loop with l-dataset-and-vars = ()
         for dataset in clasp::*clasp-datasets*
         do
         (push dataset l-dataset-and-vars)
         (loop for var in (clasp:dataset-variables dataset)
               do (push var l-dataset-and-vars))
         finally return (reverse l-dataset-and-vars))))


(defmethod l-selected-items-dv ((dataset-view dataset-view))
  "Returns a list of items selected in dataset-view. FIX: Need
terminology for 'items', such as dataset, variable, result, graph, etc."
  ;;
  (let* ((l-items (table-sequence dataset-view))
         (l-cell-selected (selected-cells dataset-view))
         (l-selected-items (map 'list #'(lambda (cell)
                                          (elt l-items (point-v cell)))
                                l-cell-selected)))
    l-selected-items))


(defmethod view-click-event-handler ((view dataset-view)
                                     (pt-where fixnum))
  "Makes single-clicks on selected items do a drag of
l-selected-items-dv . Otherwise calls the usual."
  ;;
  (let* ((pt-cell-clicked (point-to-cell view pt-where))
         (f-clicked-on-selection (member pt-cell-clicked (selected-cells view))))
    (if (and f-clicked-on-selection
             (not (option-key-p))        ;ignore if selection editing keys down
             (not (shift-key-p))
             (not (command-key-p))
             (not (control-key-p)))
      (track-mouse-for-dropping view
                                (l-selected-items-dv view)
                                (local-to-global (view-container view) pt-where))
      (call-next-method))))


(defmethod view-click-event-handler :after ((view dataset-view)
                                            (pt-where fixnum))
  "Handles the special modified click options: send to dock, cache, or open."
  ;;
  (let* ((l-selected-items (l-selected-items-dv view))
         (l-openable-items (remove-if-not
                            #'(lambda (item)
                                (f-can-open-clasp-object item))
                            l-selected-items))
         (clicked-item (ignore-errors (elt (table-sequence view) 
                                           (cell-to-index view 0 (point-v (point-to-cell view pt-where)))))))
    (if clicked-item (ribbon-display (name-and-dataset clicked-item)))
    (if (and l-selected-items
             (< (point-h pt-where) (- (point-h (view-size view)) 15)))
      (progn
        (if (funcall *x-key-p*) 
          (if (find-window "CLASP Dock")
            (dock-item 'x-var-dock l-selected-items :auto-select t)
            (let* ((destwin (ignore-errors (find-if #'(lambda (item) (typep item 'clasp-interface::statistic-dialog-mixin)) (windows))))
                   (var-pum (ignore-errors (view-named 'x-var-item destwin))))
              (when var-pum
                (when (and (not (funcall *cache-key-p*)) (typep var-pum 'variable-pop-up-menu))
                  (loop for x in (menu-items var-pum) do
                        (remove-menu-items var-pum x)))
                (receive-drop var-pum l-selected-items 1)))))
        (if (funcall *y-key-p*) 
          (if (find-window "CLASP Dock")
            (dock-item 'y-var-dock l-selected-items :auto-select t)
            (let* ((destwin (ignore-errors (find-if #'(lambda (item) (typep item 'clasp-interface::statistic-dialog-mixin)) (windows))))
                   (var-pum (ignore-errors (view-named 'y-var-item destwin))))
              (when var-pum
                (when (and (not (funcall *cache-key-p*)) (typep var-pum 'variable-pop-up-menu))
                  (loop for x in (menu-items var-pum) do
                        (remove-menu-items var-pum x)))
                (receive-drop var-pum l-selected-items 1)))))
        (if (and (funcall *cache-key-p*) (find-window "CLASP Dock")) (dock-item 'gp-dock l-selected-items))
        (if (funcall *open-click-p*)
          (if l-openable-items
            (eval-enqueue `(map nil #'open-clasp-object ',l-openable-items))))))))
