;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/exports.lisp *-*
;;;; *-* Last-Edit: Tuesday, November 15, 1994  19:01:46; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       Data Manipulation Functions                      *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-25-93 File Created.  (Carlson)
;;;  11-15-94 Added `mode-for-continuous-data'.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp)

;;; --*--
;;; ***************************************************************************

(export
 '(
   ;; classes and definitions - defs.lisp
   data variable row-number-variable *dataset-class* *variable-class* *row-number-variable-class*
   name id description source dataset-variables variable-value
   
   ;; dataset and variable fns - dataset-fns.lisp
   make-dataset make-dataset-from-rows make-dataset-from-columns
   rename-dataset delete-dataset activate-dataset deactivate-dataset
   get-dataset
   make-variable add-variable-to-dataset add-variables-to-dataset
   rename-variable delete-variable get-variable
   add-row-to-dataset add-rows-to-dataset
   
   ;; statistical functions - statistical-fns.lisp
   data-length mean sum-of-squares variance standard-deviation minimum maximum
   range quantile median trimmed-mean mode
   mode-for-continuous-data
   multiple-modes interquartile-range
   skewness tukey-summary
   statistical-summary
   t-test-one-sample t-test d-test t-test-matched
   chi-square-2x2 chi-square-2x2-counts
   chi-square-rxc chi-square-rxc-counts
   covariance correlation correlation-from-summaries
   cross-correlation autocorrelation lagged-correlation
   confidence-interval-z confidence-interval-t confidence-interval-proportion
   confidence-interval-z-summaries confidence-interval-t-summaries
   anova-one-way-variables anova-one-way-groups print-anova-table
   anova-two-way-variables anova-two-way-groups
   anova-two-way-variables-unequal-cell-sizes
   scheffe-test print-scheffe-table
   linear-regression-minimal linear-regression-minimal-summaries
   linear-regression-brief linear-regression-brief-summaries
   linear-regression-verbose linear-regression-verbose-summaries
   smooth-median-2 smooth-median-3 smooth-median-4
   smooth-median-5 smooth-hanning smooth-4253H
   smooth-mean-2 smooth-mean-3 smooth-mean-4 smooth-mean-5
   
   ;; statistical functions - density-fns.lisp
   factorial factorial-ln binomial-coefficient binomial-probability
   error-function gaussian-cdf error-function-complement gaussian-significance
   poisson-cdf chi-square-significance students-t-significance f-significance
   binomial-cdf
   
   ;; statistical functions - find-critical.lisp
   find-critical-value
   
   ;; statistical functions - sampling-fns.lisp
   distribution binomial-distribution normal-distribution gamma-distribution
   poisson-distribution uniform-distribution
   sample-to-list sample-to-dataset
   sample-uniform-to-list sample-normal-to-list sample-binomial-to-list
   sample-poisson-to-list sample-gamma-to-list
   sample-uniform-distribution-to-dataset  sample-normal-distribution-to-dataset
   sample-binomial-distribution-to-dataset  sample-poisson-distribution-to-dataset
   sample-gamma-distribution-to-dataset
   
   ;; statistical-function - bootstrap.lisp
   boot sample-with-replacement-from-sequence
   sample-without-replacement-from-sequence

   ;; io functions - io-fns.lisp
   load-dataset save-dataset import-dataset export-dataset save
   
   ;; data manipulation functions - manipulation-fns.lisp
   create-new-column-from-function recode recode-list
   partition-dataset partition-on merge-datasets .on. row-number
   
   ;; data manipulation functions - translation-fns.lisp
   dataset-to-rows dataset-to-columns transpose
   )
 )

;;; ***************************************************************************
;;; EOF
