;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/io-fns.lisp *-*
;;;; *-* Last-Edit: Wednesday, January 25, 1995  15:53:19; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                         Input/Output Functions                         *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  06-30-92 File Created.  (Carlson)
;;;  10-30-97 Improved the speed of import datasetby having it push values on
;;;           the front.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp)

;;; --*--
;;; ***************************************************************************

(defmacro with-clasp-standard-io-syntax (&body body)
  "See `with-standard-io-syntax' in CLTL2 for details.
Also binds *package* to *clasp-package*."
  `(let ((*package* *clasp-package*))
     #+Lispworks
     (with-standard-io-syntax
      . ,body)
     #-Lispworks
     (progn
       . ,body)))

(defun load-dataset (filename &key
			      (defaults *data-file-defaults*)
                              report-stream)
  "Reads a dataset from a file in standard CLASP format.  `filename' is a
string or pathname.  If `filename' does not exist, an error is signaled. CLASP
format files have two parts, a header and a data section, the first line of the
header is the name of the dataset, represented as a string (e.g. it must appear
in double quotes.)  The following n lines each have one string with the name of
one of the variables in the dataset.  The rest of the file is the data section.
Each line in the data section is a lisp list containing one row of data..  The
order of the data in the rows is the same as the order of the names of the
variables in the header.  If a file is loaded more than once, each successive
load will return a new dataset."
  #+lucid (declare (ignore defaults))
  (let ((header nil)
	(variable-names nil)
	(data nil))
    (when defaults
      (setf filename (merge-pathnames filename defaults)))
    (unless (probe-file filename)
      (error 'file-does-not-exist :filename filename))
    (with-open-file (instream filename
		              :direction :input :if-does-not-exist nil)
      
      (multiple-value-setq (header variable-names data) (load-dataset-reader instream report-stream))
      (load-dataset-processer header data variable-names report-stream)
      )))

(defun load-dataset-reader (instream report-stream &aux variable-names line header data)
  (with-clasp-standard-io-syntax
   (setf header (read instream))
   (setf header (stupid-clip-header-parsing-hack (format nil "~a" header)))
   (do ((header-line (read instream) (read instream)))
       ((not (stringp header-line)) (setf line header-line))
     (push header-line variable-names))
   (when report-stream
     (terpri report-stream))
   (with-progress-thermometer (report-stream 100 10000 :initial-string "Reading" :print-total t)
     (loop for data-line = line then (read instream nil 'eof)
	   until (eq data-line 'eof) do
	   (push data-line data)
	   (note-progress)))
   (values header (nreverse variable-names) data)))
  
(defun load-dataset-processer (header data variable-names report-stream)
  ;; This code is a duplicate of that in `make-dataset-from-rows'.
  ;; `data' comes in in reverse order and we push each value onto a row
  ;; so we get things in "row number" order
  (check-type header (or symbol string))
  (check-type data sequence)
  (check-type variable-names sequence)
  
  (if (not (= (length variable-names) (length (car data))))
      (warn 'row-length-mismatch))
  (let* ((new-dataset (make-dataset :name header))
         ;; Create the variables and push the first data point.
         (variables (loop for n from 0
			  repeat (length variable-names)
			  collect  (add-variable-to-dataset new-dataset (list (nth n (car data)))
							    (nth n variable-names))))
	 (row-cnt 1) ; start at one since we added one point above
	 )

    ;; This code is similar to `add-rows-to-dataset'.
    (with-progress-thermometer (report-stream 100 10000 :start 1 :initial-string "Processing")
      (loop for data-row in (cdr data) do
            (add-row-to-dataset new-dataset data-row variables t t)
          (incf row-cnt)
	  (note-progress)))
      (update-row-number-for-changed-variable-value new-dataset
                                                    (setf (dataset-rows new-dataset) row-cnt))
    (values new-dataset)))

#+OLD
(defun load-dataset-old (filename &key
			          (defaults *data-file-defaults*)
                                  report-stream)
  "Reads a dataset from a file in standard CLASP format.  `filename' is a
string or pathname.  If `filename' does not exist, an error is signaled. CLASP
format files have two parts, a header and a data section, the first line of the
header is the name of the dataset, represented as a string (e.g. it must appear
in double quotes.)  The following n lines each have one string with the name of
one of the variables in the dataset.  The rest of the file is the data section.
Each line in the data section is a lisp list containing one row of data..  The
order of the data in the rows is the same as the order of the names of the
variables in the header.  If a file is loaded more than once, each successive
load will return a new dataset."
  #+lucid (declare (ignore defaults))
  (let ((header nil)
	(variable-names nil)
	(data nil)
	(line nil))
    (when defaults
      (setf filename (merge-pathnames filename defaults)))
    (unless (probe-file filename)
      (error 'file-does-not-exist :filename filename))
    (with-open-file (ifile filename
		     :direction :input :if-does-not-exist nil)
      (setf header (read ifile))
      (setf header (stupid-clip-header-parsing-hack (format nil "~a" header)))
      (do ((header-line (read ifile) (read ifile)))
	  ((not (stringp header-line)) (setf line header-line))
	(push header-line variable-names))
      
      (with-progress-thermometer (report-stream 100 10000 :initial-string "Reading")
        (do ((data-line line (read ifile nil 'eof)))
	    ((equal data-line 'eof))
	  (push data-line data)
          (note-progress))
	)
      (make-dataset-from-rows header (reverse data) (reverse variable-names)) nil report-stream)
    ))



(defun stupid-clip-header-parsing-hack (string
					&optional (keyword "Experiment:"))
  (let ((pos (search keyword string)))
    (when pos (setf pos (+ pos (length keyword))))
    (if pos
	(format nil "~a" (read-from-string string nil nil :start pos))
      string)))

(defun import-dataset (filename &key
                                (separator #\,) include-labels-p
			        (defaults *data-file-defaults*)
                                report-stream)
  "Reads a dataset in from a file, where `filename' is a string or pathname.  If
`filename' doesn't exist, an error is signaled.  The data on each line in the
file must be separated by `separator' characters or whitespace.  No error
checking is done to insure that each line of the file contains the same number
of data.  If `include-labels-p' is non-nil, the first line will be used a
variable names, otherwise they will be named variable, variable-1, variable-2,
etc.  The filename will be used for the dataset name."
  (when defaults
    (setf filename (merge-pathnames filename defaults)))
  (unless (probe-file filename)
    (error 'file-does-not-exist :filename filename))
  (with-open-file (instream filename
                            :direction :input :if-does-not-exist nil)
    
    (multiple-value-bind (variable-names data) (import-dataset-reader instream filename separator include-labels-p report-stream)
      (import-dataset-processor filename data variable-names report-stream))))


(defun import-dataset-reader (instream filename separator include-labels-p report-stream)
  (declare (ignore report-stream))
  (with-clasp-standard-io-syntax
     (let (data
	   data-line
	   datum-start-position
	   datum
	   variable-names)
       (when include-labels-p
	 (setf data-line (read-line instream nil 'eof))
	 (when (eq data-line 'eof)
	   (error 'read-from-empty-file :filename filename))
	 (setf data-line (substitute #\space separator data-line))
	 (setf datum-start-position 0)
	 (do ()
	     ((>= datum-start-position (length data-line)))
	   (multiple-value-setq (datum datum-start-position)
	       (read-from-string data-line nil nil
				 :start datum-start-position))
	   (push-end (string datum) variable-names)))
       (do ((input-line (read-line instream nil 'eof) (read-line instream nil 'eof)))
	   ((eq input-line 'eof))
	 (setf data-line nil)
	 (setf datum-start-position 0)
	 (setf input-line (substitute #\space separator input-line))
	 (do ()
	     ((>= datum-start-position (length input-line)))
	   (multiple-value-setq
	       (datum datum-start-position)
	       (read-from-string input-line nil nil
			         :start datum-start-position))
	   (push datum data-line))
	 (push (reverse data-line) data))
       ;(setf data (reverse data))
       (unless include-labels-p
	 (setf variable-names (make-list (length (car data))
					 :initial-element "Variable")))
       (values variable-names data))))

(defun import-dataset-processor (filename data variable-names report-stream)
  (make-dataset-from-rows
   (pathname-name (pathname filename))
   data variable-names
   :push-rows t
   :report-stream report-stream))
  

#+OLD
(defun import-dataset (filename &key (separator #\,) include-labels-p)
  "Reads a dataset in from a file, where `filename' is a string or pathname.  If
`filename' doesn't exist, an error is signaled.  The data on each line in the
file must be separated by `separator' characters or whitespace.  No error
checking is done to insure that each line of the file contains the same number
of data.  If `include-labels-p' is non-nil, the first line will be used a
variable names, otherwise they will be named variable, variable-1, variable-2,
etc.  The filename will be used for the dataset name."
  (unless (probe-file filename)
    (error 'file-does-not-exist :filename filename))
  (with-open-file (ifile filename
		   :direction :input :if-does-not-exist nil)
    (with-clasp-standard-io-syntax
     (let (data
	   data-line
	   datum-start-position
	   datum
	   variable-names)
       (when include-labels-p
	 (setf data-line (read-line ifile nil 'eof))
	 (when (eq data-line 'eof)
	   (error 'read-from-empty-file :filename filename))
	 (setf data-line (substitute #\space separator data-line))
	 (setf datum-start-position 0)
	 (do ()
	     ((>= datum-start-position (length data-line)))
	   (multiple-value-setq (datum datum-start-position)
	       (read-from-string data-line nil nil
				 :start datum-start-position))
	   (push-end (string datum) variable-names)))
       (do ((input-line (read-line ifile nil 'eof) (read-line ifile nil 'eof)))
	   ((eq input-line 'eof))
	 (setf data-line nil)
	 (setf datum-start-position 0)
	 (setf input-line (substitute #\space separator input-line))
	 (do ()
	     ((>= datum-start-position (length input-line)))
	   (multiple-value-setq
	       (datum datum-start-position)
	       (read-from-string input-line nil nil
			         :start datum-start-position))
	   (push datum data-line))
	 (push (reverse data-line) data))
       (setf data (reverse data))
       (unless include-labels-p
	 (setf variable-names (make-list (length (car data))
					 :initial-element "Variable")))
       (make-dataset-from-rows
        (pathname-name (pathname filename)) data variable-names)))))

(defun save-dataset (data filename &key (if-exists :error))
  "Writes a dataset out to a file in CLASP format.  `data' must be a dataset,
and filename is a string or pathname of a file.  If `filename' already exists,
an error will be signaled."
  (when (and (eq if-exists :error)
             (probe-file filename))
    (error 'file-already-exists :filename filename))
  (with-open-file (ofile filename :direction :output
                   :if-exists if-exists
		   :if-does-not-exist :create)
    (with-clasp-standard-io-syntax
     (format ofile "~s~%" (name data))
     (let (row-names
	   rows)
       (dolist (variable (dataset-variables data))
	 (unless (row-number-p variable)
	   (format ofile "~s~%" (name variable))
	   (push (name variable) row-names)))
       (setf row-names (reverse row-names))
       (setf rows (dataset-to-rows data :columns row-names))
       (let ((*print-length* nil))
	 (dolist (i rows)
	   (format ofile "~s~%" i))))))
  #+MCL ; should use the *clasp-application-creator* but it is in the interface package
  (ccl:set-mac-file-creator filename :CLSP)
  (values))

(defun export-dataset (dataset filename &key (separator #\,) include-labels-p)
  "Writes out the values in `dataset' in row-major order to the file `filename.'
Each row in `filename' will be one row of data in `dataset.' The data in a row
will be separated by the character `separator'.  If include-labels-p is non-nil,
the variable-names will be written out on the first line.  `export-dataset' will
signal an error if a file of name `filename' already exists."
  (when (probe-file filename)
    (error 'file-already-exists :filename filename))
  (with-open-file (ofile filename :direction :output
		   :if-does-not-exist :create)
    (with-clasp-standard-io-syntax
     (let (row-names
	   rows
	   (format-string (concatenate 'string "~{~s~^"
				       (string separator) "~}~%")))
       (dolist (variable (dataset-variables dataset))
	 (unless (row-number-p variable)
	   (push (name variable) row-names)))
       (setf row-names (reverse row-names))
       (if include-labels-p
	   (format ofile format-string row-names))
       (setf rows (dataset-to-rows dataset :columns row-names))
       (dolist (i rows)
	 (format ofile format-string i)))))
   #+MCL 
  (ccl:set-mac-file-creator filename :ttxt)
  (values))

;; A first pass at a generic routine for writing multiple output formats.
(defun write-dataset (dataset stream &optional (separator ",") row-start row-end include-labels-p)
  (with-clasp-standard-io-syntax
   (let* ((format-string (format nil "~@[~a~]~~{~~a~~^~a~~}~@[~a~]~~%"
                                 row-start separator row-end))
          (column-names
           (loop for variable in (dataset-variables dataset)
                 unless (row-number-p variable)
                 collect (name variable) into column-names
                 finally
                 (when include-labels-p
	           (format stream format-string column-names))
                 (return column-names))))
     
     (let ((rows (dataset-to-rows dataset :columns column-names)))
       (dolist (row rows)
	 (format stream format-string row))))))

;;; ***************************************************************************

(defmethod save ((self t) pathname)
  (declare (ignore pathname))
  (error 'not-implemented))

(defmethod save ((self dataset) pathname)
  (save-dataset self pathname))

;;; ***************************************************************************
;;; EOF
