;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/dev/shortcuts.lisp *-*
;;;; *-* Last-Edit: Monday, October 16, 1995  12:19:45; Edited-By: Westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                           Shortcut Functions                           *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-17-93 File Created.  (carlson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp)

;;; --*--
;;; ***************************************************************************

;;; These functions are either shortcuts for commonly used
;;; transformations, hacks for the clim interface or first drafts of
;;; functions which should eventually go into another file like
;;; statistical-fns.lisp

(defun string-append (&rest strings)
  (apply #'concatenate 'simple-string strings))

(defun add-a-constant (variable number)
  (create-new-column-from-function
    (variable-dataset variable)
    (string-append (name variable) "+" (princ-to-string number))
    #'(lambda (var)
        (+ var number))
    variable))

(defun natural-log (variable)
  (if (< (minimum (variable-value variable)) 0)
    (warn 'complex-results))
  (create-new-column-from-function
    (variable-dataset variable)
    (string-append "LOG[" (name variable) "]" )
    #'(lambda (var)
        (log var))
    variable))

(defun log10 (variable)
  (if (< (minimum (variable-value variable)) 0)
    (warn 'complex-results))
  (create-new-column-from-function
    (variable-dataset variable)
    (string-append "LOG10[" (name variable) "]" )
    #'(lambda (var)
        (log var 10))
    variable))

(defun smooth-variable-4253H (variable)
  (add-variable-to-dataset
   (variable-dataset variable)
   (smooth-4253H (variable-value variable))
   (string-append "SMOOTHED-" (name variable))))

(defun recode-categorical (variable old-values new-values)
  (create-new-column-from-function
    (variable-dataset variable)
    ;; Put the new name on the front since the back is more likely to be out of the window.
    (string-append "RECODED-" (name variable))
    #'(lambda (var)
        (recode var old-values new-values))
    variable))

(defun rebin (variable limits new-values)
  (create-new-column-from-function
   (variable-dataset variable)
   ;; Put the new name on the front since the back is more likely to be out of the window.
   (string-append "REBINNED-" (name variable))
   #'(lambda (var)
       (bin var limits new-values))
   variable))

(defun bin (value max-values bin-names)
  (let ((bins (mapcar #'list (append max-values (list most-positive-fixnum))
		      bin-names)))
    #+OLD
    (setf bins (sort bins #'(lambda (x &rest y)
			      (apply #'< (car x)
				     (mapcar #'car y)))))
    (setf bins (sort bins #'< :key #'first))
    
    (dolist (bin bins)
      (when (<= value (car bin))
	(return-from bin (cadr bin))))))

(defun formula (dataset expression)
  (create-new-column-from-expression dataset expression))

(defun difference-a-column (variable)
  (add-variable-to-dataset
   (variable-dataset variable)
   (pairwise-difference (variable-value variable))
   (string-append "DIFFERENCED-" (name variable))))

#+OLD
(defun pairwise-difference (data)  
  (cons 0 (mapcar #'(lambda (x1 x2) (- x1 x2))
		  data
		  (cdr data))))

#+MORE-EFFICIENT
(defun pairwise-difference (data)  
  (mapcar #'(lambda (x1 x2) (- x1 x2))
	  (cons (first data) data)
	  (cons (second data) (cdr data))))

;; In order to get a list with the same number of values as the data list the
;; original version just stuck a zero on the front. This version duplicates the
;; last value and sticks it on the end. This is not very efficient (we cdr
;; down the list 3 times).

(defun pairwise-difference (data)
  (let ((result (mapcar #'(lambda (x1 x2) (- x1 x2))
	                data
	                (cdr data))))
    (values (append result (last result)))))

(defun sort-variable (variable index-variable)
  (when (not (eq (variable-dataset variable)
		 (variable-dataset index-variable)))
    (error 'variables-from-different-datasets))
  (add-variable-to-dataset
   (variable-dataset variable)
   (variable-value variable
		   :order-by index-variable
                   #+OLD-VARS
                   (car (variable-attribute index-variable)))
   (if (eq variable index-variable)
       (string-append "SORTED-" (name variable))
       (string-append "SORTED-" (name variable) "-BY-" (name index-variable)))
   (if (eq variable index-variable)
       (string-append "sorted " (description variable))
       (string-append (description variable)
	              " sorted by "
                      (description index-variable)))))

;;; ***************************************************************************
;;; EOF
