;;
;; soon to be moved (schmill)
;;

(in-package :clasp)

;;
;; Below are some tidbits for code beautification. We represent a contrast with an alist,
;;  of the form (group-name . weight). Instead of cdr and car, we use these inline functions.
;;

(proclaim '(inline group weight))

;; 2-2-96 GROUP IS DEFINED IN UTILS - PROBABLY SHOULD RENAME THIS - Westy
(defun group (contrast-term)
  (car contrast-term))
(defun weight (contrast-term)
  (cdr contrast-term))

;;; ---------------------------------------------------------------------------
;;; the test for orthogonality

(defmethod orthogonal-contrasts-p (&rest contrasts)
  (if (<= (length contrasts) 1)
      (values NIL "Only one contrast.")
      (let* ((ilc 0) (olc 0) 
             (dcs (copy-tree contrasts))
             (reason (catch 'not
                       (loop while (not (null dcs)) do
                             (incf olc) (setq ilc olc)
                             (loop for com in (cdr dcs) do
                                   (incf ilc)
                                   (let ((cws (princ (append (car dcs) com))))
                                     (if (not (zerop (princ (loop for g in (princ (remove-duplicates (mapcar #'group cws)))
			                                   sum (* (or (weight (assoc g com)) 0.0)
                                                                  (or (weight (assoc g (car dcs))) 0.0))))))
                                       (throw 'not (list olc ilc)))))
                             (setq dcs (cdr dcs))))))
        (values (not reason) (when reason (format nil "Contrasts ~D and ~D are not orthogonal." (first reason) (second reason)))))))

;;; ---------------------------------------------------------------------------
;;; the significance (anova-style) test for contrasts (one-way)

(defun contrast-significance-unequal-cells-args (gv dv &rest contrasts)
  (let ((ght (make-hash-table))
        (op (apply #'orthogonal-contrasts-p contrasts))
        hm                       ;; harmonic mean
        (numgroups 0)
        cvals                    ;; contrast values
        ssl                      ;; sum squares
        fs                       ;; f statistics
        fss                      ;; f significances
        (SSW 0.0) MSW DFW)       ;; ss, ms df within
    ;; generate a group hash-table
    (loop for g in gv
          for d in dv do
          (unless (gethash g ght) (incf numgroups))
          (push d (gethash g ght)))
    ;; now do a mapping over the hash table to get the SSw and harmonic mean
    (let ((den 0) (num 0))
      (maphash #'(lambda (k g) 
                   (declare (ignore k))
                   (incf num) (incf den (/ 1 (length g)))
                   (let ((group-mean (clasp::mean g)))
                     (incf SSW (loop for x in g sum
                                     (expt (- x group-mean) 2)))))
               ght)
      (setq hm (/ num den)))
    ;; the mean square and df within
    (setq DFW (- (length gv) numgroups))
    (setq MSW (/ SSW DFW))
    ;; compute the contrast values
    (setq cvals (loop for c in contrasts collect
                      (#-RATIOK float #+RATIOK identity
                                (loop for g in c sum
                                      (* (weight g) (clasp::mean (gethash (group g) ght)))))))
    ;; compute the sum-of-squares
    (setq ssl (loop for c in contrasts
                    for cv in cvals
                    collect (#-RATIOK float #+RATIOK identity
                                      (* hm (/ (* cv cv)
                                               (loop for g in c sum
                                                     (* (weight g) (weight g))))))))
    ;; f statistic stuff
    (setq fs (mapcar #'(lambda (ss) (/ ss MSW)) ssl))
    (setq fss (mapcar #'(lambda (fs) (f-significance fs 1 DFW anova-f-one-tailed?)) fs))
    ;; returns: contrast-calues sum-squares-contrasts df-contrasts f-contrasts f-significances
    ;;          sum-squares-within mean-squares-within df-within
    ;;          (if orthogonal) sum-squares-between df-between mean-squares-between
    ;;          harmonic mean
    (values cvals ssl (make-list (length cvals) :initial-element 1) ssl fs fss
            SSW DFW MSW
            (if op `(,(reduce #'+ ssl) :df :ms) NIL)
            hm)))
;;; ---------------------------------------------------------------------------
;;;

(defparameter anova-f-one-tailed? t)

;;; ---------------------------------------------------------------------------
;;; one way repeated measures

(defun anova-one-way-repeated-measures (fix ran v &optional (scheffe-tests-p t))
  (let* ((fixs (extract-unique-values fix))
         (rans (extract-unique-values ran))
         (fixl (length fixs))
         (ranl (length rans))
         (dt (make-array (list fixl ranl) :initial-element NIL))
         (GM (mean v))
         SSfix (DFfix (1- fixl)) MSfix Ffix
         SSran (DFran (1- ranl)) MSran
         SSin (DFin (* (1- ranl) (1- fixl))) MSin
         SStot (DFtot (1- (length v)))
         scheffe
         )
    ;; the data table
    (loop for fixv in fix for ranv in ran for vv in v do
          (let ((fixi (position fixv fixs))
                (rani (position ranv rans)))
            (if (null (aref dt fixi rani))
                (setf (aref dt fixi rani) vv)
                (error "not a repeated measures design -- more than one datum in cell (~S ~S)." fixv ranv))))
    ;; sum squares
    (setq SSfix (* ranl (loop for fixi from 0 to (1- fixl) sum
                              (expt (- (mean (loop for rani from 0 to (1- ranl) collect (aref dt fixi rani))) GM) 2))))
    (setq SSran (* fixl (loop for rani from 0 to (1- ranl) sum
                              (expt (- (mean (loop for fixi from 0 to (1- fixl) collect (aref dt fixi rani))) GM) 2))))
    (setq SStot (loop for fixi from 0 to (1- fixl) sum
                      (loop for rani from 0 to (1- ranl) sum
                            (expt (- (aref dt fixi rani) GM) 2))))
    (setq SSin (- SStot SSfix SSran))
    (setq MSfix (/ SSfix DFfix) MSran (/ SSran DFran) MSin (/ SSin DFin))
    (setq Ffix (/ MSfix MSin))
    (when scheffe-tests-p
      ;; group-means, group-sizes, MSerror, DFerror
      (setq scheffe (scheffe-tests (loop for fixi from 0 to (1- fixl) collect
					 (mean (loop for rani from 0 to (1- ranl) collect (aref dt fixi rani))))
				   (make-list fixl :initial-element ranl)
                                   MSin DFin)))
    (values (list `(,SSfix ,DFfix ,MSfix ,Ffix ,(f-significance (float Ffix) DFfix DFin anova-f-one-tailed?))
                  `(,SSran ,DFran ,MSran)
                  `(,SSin ,DFin ,MSin)
                  `(,SStot ,DFtot))
            fixs
            rans
            scheffe)))

;;; ---------------------------------------------------------------------------
;;; two way repeated measures stuff

;;; from Keppel, p428

(defun anova-two-way-repeated-measures (av bv sv vv)
  (let* ((ags (extract-unique-values av))  ;; groups
         (bgs (extract-unique-values bv))
         (sgs (extract-unique-values sv))
         (ABS (make-array (list (length ags) (length bgs) (length sgs)))) ;; contingency-table
         (AB  (make-array (list (length ags) (length bgs)) :initial-element 0)) ;; contingency-table
         (AS  (make-array (list (length ags) (length sgs)) :initial-element 0)) ;; contingency-table
         (BS  (make-array (list (length bgs) (length sgs)) :initial-element 0)) ;; contingency-table
         T! A! B! S! AB! AS! BS! ABS!
         SSa SSb SSs MSa MSb MSs
         (DFa (1- (length ags))) (DFb (1- (length bgs))) (DFs (1- (length sgs)))
         SSab SSas SSbs MSab MSas MSbs
         (DFab (* DFa DFb)) (DFbs (* DFb DFs)) (DFas (* DFa DFs))
         SSabs MSabs (DFabs (* DFa DFb DFs))
         Fa Fb Fab Pa Pb Pab
         SSt DFt
         )
    ;; generate contingency tables
    (loop for a in av for b in bv for s in sv for v in vv do
          (let ((ai (position a ags))
                (bi (position b bgs))
                (si (position s sgs)))
            (if (null (aref ABS ai bi si))
                (setf (aref ABS ai bi si) v)
                (error "repeated measures design cannot handle multiple values per cell."))
            ;; increment the marginal tables
            (incf (aref AB ai bi) v)
            (incf (aref AS ai si) v)
            (incf (aref BS bi si) v)
            ))
    ;; preliminary values
    (setq T! (/ (expt (loop for ai from 0 to (1- (array-dimension AB 0)) sum
                            (loop for bi from 0 to (1- (array-dimension AB 1)) sum
                                  (aref AB ai bi))) 2)
                (* (length ags) (length bgs) (length sgs))))
    ;; compute sum of squares
    ;; single factor = sumsq(factor)/other_gsizes - T
    (setq SSa (- (setq A! (/ (loop for ai from 0 to (1- (array-dimension AB 0))
                                   sum (expt (loop for bi from 0 to (1- (array-dimension AB 1))
                                                   sum (aref AB ai bi)) 2))
                             (* (length bgs) (length sgs))))
                 T!))
    (setq SSb (- (setq B! (/ (loop for bi from 0 to (1- (array-dimension AB 1))
                                   sum (expt (loop for ai from 0 to (1- (array-dimension AB 0))
                                                   sum (aref AB ai bi)) 2))
                             (* (length ags) (length sgs))))
                 T!))
    (setq SSs (- (setq S! (/ (loop for si from 0 to (1- (array-dimension BS 1))
                                   sum (expt (loop for bi from 0 to (1- (array-dimension BS 0))
                                                   sum (aref BS bi si)) 2))
                             (* (length ags) (length bgs))))
                 T!))
    ;; interaction sum squares
    (setq SSab (+ (- (setq AB! (/ (loop for ai from 0 to (1- (array-dimension AB 0))
                                        sum (loop for bi from 0 to (1- (array-dimension AB 1))
                                                  sum (expt (aref AB ai bi) 2)))
                                  (length sgs)))
                     A! B!) T!))
    (setq SSas (+ (- (setq AS! (/ (loop for ai from 0 to (1- (array-dimension AS 0))
                                        sum (loop for si from 0 to (1- (array-dimension AS 1))
                                                  sum (expt (aref AS ai si) 2)))
                                  (length bgs)))
                     A! S!) T!))
    (setq SSbs (+ (- (setq BS! (/ (loop for bi from 0 to (1- (array-dimension BS 0))
                                        sum (loop for si from 0 to (1- (array-dimension BS 1))
                                                  sum (expt (aref BS bi si) 2)))
                                  (length ags)))
                     S! B!) T!))
    (setq SSabs (- (+ (setq ABS! (loop for ai from 0 to (1- (array-dimension AB 0))
				      sum (loop for bi from 0 to (1- (array-dimension AB 1))
						sum (loop for si from 0 to (1- (array-dimension ABS 2))
							  sum (expt (aref ABS ai bi si) 2)))))
                      A! B! S!) T! AB! BS! AS!))
    ;; mean squares
    (setq MSa (/ SSa DFa) MSb (/ SSb DFb) MSs (/ SSs DFs)
          MSab (/ SSab DFab) MSas (/ SSas DFas) MSbs (/ SSbs DFbs)
          MSabs (/ SSabs DFabs))
    ;; F ratios
    (setq Fa (/ MSa MSas) Fb (/ MSb MSbs) Fab (/ MSab MSabs))
    ;; P values
    (setq Pa (f-significance (float Fa) DFa DFas anova-f-one-tailed?)
          Pb (f-significance (float Fb) DFb DFbs anova-f-one-tailed?) 
          Pab (f-significance (float Fab) DFab DFabs anova-f-one-tailed?))
    ;; totals
    (setq SSt (- ABS! T!) DFt (1- (* (length ags) (length bgs) (length sgs))))
    (values `((,SSa ,DFa ,MSa ,Fa ,Pa) (,SSs ,DFs ,MSs) (,SSas ,DFas ,MSas) (,SSb ,DFb ,MSb ,Fb ,Pb)
              (,SSbs ,DFbs ,MSbs) (,SSab ,DFab ,MSab ,Fab ,Pab) (,SSabs ,DFabs ,MSabs) (,SSt ,DFt)))
    ))

;;; ---------------------------------------------------------------------------
;;; 3 way, unequal cell sizes.

;; the following are to make things readable. for unequal cell sizes, we need to use means for
;; part of the analysis, and the actual cell variates for the within component.
;; these DynamicCellSize functions are for decomposing data in the partition tables. A cell
;; looks like '((cell data) . mean)
(proclaim '(inline dcs-cell-mean dcs-cell))
(defun dcs-cell-mean (cell) (cdr cell))
(defun dcs-cell      (cell) (car cell))

(defun anova-three-way (av bv cv vv)
  (let* ((ags (extract-unique-values av))  ;; groups
	 (bgs (extract-unique-values bv))
	 (cgs (extract-unique-values cv))
         (am (length ags)) (bm (length bgs)) (cm (length cgs))
	 (ABC (make-array (list am bm cm)))                 ;; contingency-table
	 (AB  (make-array (list am bm) :initial-element 0)) ;; contingency-table
	 (AC  (make-array (list am cm) :initial-element 0)) ;; contingency-table
	 (BC  (make-array (list bm cm) :initial-element 0)) ;; contingency-table
	 (T! 0) A! B! C! AB! AC! BC! ABC!
	 SSa SSb SSc MSa MSb MSc
	 (DFa (1- am)) (DFb (1- bm)) (DFc (1- cm))
	 SSab SSac SSbc MSab MSac MSbc
         SSw (DFw (- (length vv) (* am bm cm))) MSw
         (DFab (* DFa DFb)) (DFbc (* DFb DFc)) (DFac (* DFa DFc))
	 SSabc MSabc (DFabc (* DFa DFb DFc))
	 Fa Fb Fc Fab Fac Fbc Fabc
         Pa Pb Pc Pab Pac Pbc Pabc
         SSt DFt
         HM
         )
    ;; Compile Contingency Table
    (loop for a in av for b in bv for c in cv for v in vv do
          (let ((ai (position a ags))
                (bi (position b bgs))
                (ci (position c cgs)))
            (if (null (aref ABC ai bi ci))
                (setf (aref ABC ai bi ci) (list v))
                (push v (aref ABC ai bi ci)))))
    ;; Calculate The Harmonic Mean
    (setq HM (/ (* am bm cm)
	        (loop for ai from 0 to (1- am) sum
                      (loop for bi from 0 to (1- bm) sum
                            (loop for ci from 0 to (1- cm) sum
                                  (/ 1 (length (aref ABC ai bi ci))))))))
    ;; calculate the SSwithin before we convert to cell means
    (let ((tmp 0))
      (setq SSw (loop for ai from 0 to (1- am) sum
		      (loop for bi from 0 to (1- bm) sum
			    (loop for ci from 0 to (1- cm) sum
                                  (progn
                                    (incf tmp (/ (expt (reduce #'+ (aref ABC ai bi ci)) 2)
                                                 (length (aref ABC ai bi ci))))
                                    (loop for ob in (aref ABC ai bi ci) sum
                                          (* ob ob)))))))
      (setq SSw (- SSw tmp)))
    ;; Calculate The Cell Means
    (loop for ai from 0 to (1- am) do
	  (loop for bi from 0 to (1- bm) do
		(loop for ci from 0 to (1- cm) do
                      (setf (aref ABC ai bi ci) (cons (aref ABC ai bi ci) (mean (aref ABC ai bi ci)))))))
    ;; Calculate T!
    (setq T! (/ (expt (loop for ai from 0 to (1- am) sum
                            (loop for bi from 0 to (1- bm) sum
                                  (loop for ci from 0 to (1- cm) sum
                                        (dcs-cell-mean (aref ABC ai bi ci)))))
                      2) (* am bm cm)))
    ;; Calculate Marginal 2 Way Frequency Tables
    (Loop for ci from 0 to (1- cm) do
	  (loop for ai from 0 to (1- am) do
	        (setf (aref AC ai ci) (loop for bi from 0 to (1- bm) sum
					    (dcs-cell-mean (aref ABC ai bi ci)))))
	  (loop for bi from 0 to (1- (array-dimension ABC 1)) do
	        (setf (aref BC bi ci) (loop for ai from 0 to (1- am) sum
					    (dcs-cell-mean (aref ABC ai bi ci))))))
    (loop for ai from 0 to (1- am) do
	  (loop for bi from 0 to (1- bm) do
	        (setf (aref AB ai bi) (loop for ci from 0 to (1- cm) sum
					    (dcs-cell-mean (aref ABC ai bi ci))))))
    ;; Calculate Sum Square Terms. from OLSON, p659.
    ;; we condense the whole process into one mess here, without using SSa'
    (setq SSa (* HM (- (setq A! (/ (loop for ai from 0 to (1- am)
                                         sum (expt (loop for bi from 0 to (1- bm)
                                                         sum (aref AB ai bi)) 2))
                                   (* bm cm)))
                       T!)))
    (setq SSb (* HM (- (setq B! (/ (loop for bi from 0 to (1- (array-dimension AB 1))
                                         sum (expt (loop for ai from 0 to (1- (array-dimension AB 0))
                                                         sum (aref AB ai bi)) 2))
                                   (* am cm)))
                       T!)))
    (setq SSc (* HM (- (setq C! (/ (loop for ci from 0 to (1- cm)
                                         sum (expt (loop for bi from 0 to (1- bm)
                                                         sum (aref BC bi ci)) 2))
                                   (* am bm)))
                       T!)))
    ;; Interaction Sum Squares
    ;; my current hypothesis is that all of these need the HM term
    (setq SSab (* HM (+ (- (setq AB! (/ (loop for ai from 0 to (1- am)
                                              sum (loop for bi from 0 to (1- bm)
                                                        sum (expt (aref AB ai bi) 2)))
                                        cm))
                           A! B!) T!)))
    (setq SSac (* HM (+ (- (setq AC! (/ (loop for ai from 0 to (1- am)
                                              sum (loop for ci from 0 to (1- cm)
                                                        sum (expt (aref AC ai ci) 2)))
                                        bm))
                           A! C!) T!)))
    (setq SSbc (* HM (+ (- (setq BC! (/ (loop for bi from 0 to (1- bm)
                                        sum (loop for ci from 0 to (1- cm)
                                                  sum (expt (aref BC bi ci) 2)))
                                        am))
                           C! B!) T!)))
    (setq SSabC (* HM (- (+ (setq ABC! (loop for ai from 0 to (1- am)
				             sum (loop for bi from 0 to (1- bm)
						       sum (loop for ci from 0 to (1- cm)
							         sum (expt (dcs-cell-mean (aref ABC ai bi ci)) 2)))))
                            A! B! C!) T! AB! BC! AC!)))
    (setq MSa (/ SSa DFa) MSb (/ SSb DFb) MSc (/ SSc DFc)
          MSab (/ SSab DFab) MSac (/ SSac DFac) MSbc (/ SSbc DFbc)
          MSabc (/ SSabc DFabc) MSw (/ SSw DFw))
    ;; figger out the F's and F-significances
    (if (/= MSw 0.0)
        (progn
          (setq Fa (/ MSa MSw) Fb (/ MSb MSw) Fc (/ MSc MSw)
                Fab (/ MSab MSw) Fac (/ MSac MSw) Fbc (/ MSbc MSw)
                Fabc (/ MSabc MSw))
          (setq Pa (f-significance Fa DFa DFw anova-f-one-tailed?)
                Pb (f-significance Fb DFb DFw anova-f-one-tailed?)
                Pc (f-significance Fc DFc DFw anova-f-one-tailed?)
                Pab (f-significance Fab DFab DFw anova-f-one-tailed?)
                Pac (f-significance Fac DFac DFw anova-f-one-tailed?)
                Pbc (f-significance Fbc DFbc DFw anova-f-one-tailed?)
                Pabc (f-significance Fabc DFabc DFw anova-f-one-tailed?)))
        (warn "Within cell variance is zero. F ratios are through the roof."))
    (values (list `(,SSa ,DFa ,MSa ,Fa ,Pa) `(,SSb ,DFb ,MSb ,Fb ,Pb) `(,SSc ,DFc ,MSc ,Fc ,Pc)
                  `(,SSab ,DFab ,MSab ,Fab ,Pab) `(,SSac ,DFac ,MSac ,Fac ,Pac) `(,SSbc ,DFbc ,MSbc ,Fbc ,Pbc)
                  `(,SSabc ,DFabc ,MSabc ,Fabc ,Pabc)
                  `(,SSw ,DFw ,MSw))
	    )))

;;; ---------------------------------------------------------------------------
;;; two way mixed design

(defun anova-two-way-mixed (fv rv vv)
  (let* ((fgs (extract-unique-values fv))  ;; groups
	 (rgs (extract-unique-values rv))
         (dt  (make-array (list (length fgs) (length rgs)) :initial-element nil))
         HM (T! 0) (N 0) TSON
         SSf (DFf (1- (length fgs))) MSf Ff Pf
         SSr (DFr (1- (length rgs))) MSr Fr Pr
         SSi (DFi (* DFf DFr)) MSi Fi Pin
         SSw DFw MSw)
    ;; build up the partition table
    (format t "~D  " N)
    (loop for f in fv
          for r in rv
          for v in vv do
          (incf N)
          (push v (aref dt (position f fgs) (position r rgs))))
    (setq DFw (- N (* (1+ DFf) (1+ DFr))))
    ;; compute SSw before converting the cells to DCS cells
    (let ((tmp 0))
      (setq SSw (loop for fi from 0 to DFf sum
		      (loop for ri from 0 to DFr sum
			    (progn
			      (incf tmp (/ (expt (reduce #'+ (aref dt fi ri)) 2)
					   (length (aref dt fi ri))))
			      (loop for ob in (aref dt fi ri) sum
				    (* ob ob))))))
      (setq SSw (- SSw tmp)))
    ;; compute the harmonic mean, compute T, and convert to DCS cells
    (setq HM (/ (* (1+ DFf) (1+ DFr))
                (loop for fi from 0 to DFf sum
	              (loop for ri from 0 to DFr sum
                            (prog1
                                ;; this is propogated to the HM computation
                                (/ 1 (length (aref dt fi ri)))
                              ;; this is the dcs cell mean computation
                              (setf (aref dt fi ri) (cons (aref dt fi ri) (mean (aref dt fi ri))))
                              ;; this is the T calculation
                              (incf T! (dcs-cell-mean (aref dt fi ri))))))))
    (setq TSON (/ (* T! T!) (* (1+ DFf) (1+ DFr))))
    ;; now compute the effects
    (setq SSf (- (/ (loop for fi from 0 to DFf sum
			  (expt (loop for ri from 0 to DFr sum (dcs-cell-mean (aref dt fi ri))) 2))
		    (1+ DFr))
		 TSON))
    (setq SSr (- (/ (loop for ri from 0 to DFr sum
			  (expt (loop for fi from 0 to DFf sum (dcs-cell-mean (aref dt fi ri))) 2))
		    (1+ DFf))
		 TSON))
    (setq SSi (* HM (- (loop for fi from 0 to DFf sum
                             (loop for ri from 0 to DFr sum
                                   (expt (dcs-cell-mean (aref dt fi ri)) 2)))
                       SSf SSr TSON)))
    (setq SSf (* HM SSf) SSr (* HM SSr))
    (setq MSf (/ SSf DFf) MSr (/ SSr DFr) MSi (/ SSi DFi) MSw (/ SSw DFw))
    (setq Ff  (/ MSf MSi) Fr  (/ MSr MSw) Fi  (/ MSi MSw))
    (setq Pf (f-significance (float Ff) DFf DFi anova-f-one-tailed?))
    (setq Pr (f-significance (float Fr) DFr DFw anova-f-one-tailed?)
          Pin (f-significance (float Fi) DFi DFw anova-f-one-tailed?))
    (values `((,SSf ,DFf ,MSf ,Ff ,Pf)
              (,SSr ,DFr ,MSr ,Fr ,Pr)
              (,SSi ,DFi ,MSi ,Fi ,Pin)
              (,SSw ,DFw ,MSw)))
    ))
;;; ---------------------------------------------------------------------------
;;; rewrite of the one way anova

(defun anova-one-way (iv dv &optional (scheffe-tests-p t) (confidence-intervals NIL))
   (let ((gt (make-hash-table))
         (j 0) (T! 0) (N 0) #+HARMONIC-MEAN HM
         (SSi 0) DFi MSi Fi Pin
         (SSw 0) DFw MSw
         SSt DFt
         (group-means NIL) (group-sizes NIL) (groups NIL))
     (loop for ivv in iv
           for dvv in dv do
           (incf T! dvv)
           (incf N)
           (when (null (gethash ivv gt))
             (incf j))
           (push dvv (gethash ivv gt)))
     (maphash #'(lambda (key val)
                  ;; record group incidence order
                  (push key groups)
                  ;; accumulate sum squares for factor A and store the mean
		  (let ((m (mean val)))
		    (push m group-means)
		    (push (length val) group-sizes)
		    (incf SSi (/ (square (reduce #'+ val)) (length val))))
		  ;; accumulate sum squares within
		  (incf SSw (reduce #'+ val :key #'square))
		  ) gt)
     (setq groups (nreverse groups) group-means (nreverse group-means) group-sizes (nreverse group-sizes))
     (setq SSw (- SSw SSi))
     (setq SSi (- SSi (/ (square T!) N)))
     (setq DFi (1- j) DFw (- N j))
     (setq MSi (/ SSi DFi) MSw (/ SSw DFw))
     (setq Fi  (/ MSi MSw))
     (setq Pin (f-significance (float Fi) DFi DFw anova-f-one-tailed?))
     (setq SSt (+ SSi SSw) DFt (+ DFi DFw))
     (values `((,SSi ,DFi ,MSi ,Fi ,Pin)
               (,SSw ,DFw ,MSw)
               (,SSt ,DFt))
             group-means
             (when scheffe-tests-p
               (scheffe-tests group-means group-sizes MSw DFw))
             (let ((gm (/ T! N)))
               (reduce #'+ dv :key #'(lambda (num) (square (- num gm)))))
	     (when (and (numberp confidence-intervals)
			(< 0.0 confidence-intervals 1.0))
	       (let ((gss NIL))
		 (maphash #'(lambda (key group)
			      (push (reduce #'+ group :key #'square) gss)) gt)
                 (setf gss (nreverse gss))
		 (mapcar #'(lambda (s2 mean n2)
                             (let ((num (- s2 (* n2 mean mean))))
                               ;; the following is a hack to trap zero variance
			       ;; round off error can get us into the negative
                               (if (<= num 0)
                                   (list mean mean mean)
                                   (multiple-value-list
			            (confidence-interval-t-summaries
			             mean
			             (- n2 1)
			             (sqrt (/ num (* n2 (- n2 1))))
			             confidence-intervals)))))
			 gss group-means group-sizes)))
             groups)))

;;; ---------------------------------------------------------------------------
;;; dataset garbache

#+KEPPEL-20-4
(progn
  (defparameter A1 NIL)
  (defparameter B1 NIL)
  (defparameter S1 NIL)
  (loop for ai in '(fa2 fa1) do
        (loop for bi in '(fb4 fb3 fb2 fb1) do
              (loop for si in '(fs5 fs4 fs3 fs2 fs1) do
                    (push ai A1)
                    (push bi B1)
                    (push si S1))))
  (defparameter V1 '(3 7 9 4 1  5 11 13 8 3  9 12 14 11 5  6 11 12 7 4
                       5 10 10 6 3  6 12 15 9 5  11 18 15 13 9  7 15 14 9 7)))

#+OLSON-19.4
(loop for ai in '(f2h f3h) do
      (loop for bi in '(f1t f2t f4t) do
            (loop for si in '(r1 r2 r3 r4) do
		  (push c1 ai)
                  (push c2 bi)
                  (push r1 si))))

#+OLSON-18.14
(progn
  (defparameter VAR NIL)
  (defparameter FER NIL)
  (defparameter WEA NIL)
  (loop for ai in '(bonanza klondike fairfield) do
        (loop for bi in '(normal dry) do
              (loop for si in '(fert nofert) do
                    (dotimes (i 2)
		      (push ai VAR)
                      (push bi WEA)
                      (push si FER)))))
  (defparameter VAL '(62 58 42 40 32 37 26 24
                         66 69 49 39 38 31 29 20
                         70 76 54 47 36 43 27 33)))

#+HAND-c.2
(progn
  (defparameter clin NIL)
  (defparameter stat NIL)
  (defparameter cdtn NIL)
  (defparameter valu NIL)
  (loop for row in '((EXP A ST-EXP 92.7 5)  (CON A ST-EXP 93.4 11)
                     (EXP B ST-EXP 98.2 5)  (CON B ST-EXP 96.2 4)
                     (EXP A ST-NOV 99.1 12) (CON A ST-NOV 99.2 6)
                     (EXP B ST-NOV 99.7 9)  (CON B ST-NOV 97.2 13)) do
        (dotimes (i (fifth row))
          (push (first row) cdtn)
	  (push (second row) clin)
          (push (third row) stat)
          (push (fourth row) valu))))

#+TRICKY
(progn
  (defparameter clin NIL)
  (defparameter stat NIL)
  (defparameter cdtn NIL)
  (defparameter valu NIL)
  (loop for row in '((EXP A ST-EXP 92.7 5)  (CON A ST-EXP 93.4 11)
                     (EXP B ST-EXP 98.2 5)  (CON B ST-EXP 96.2 4)
                     (EXP A ST-NOV 99.1 12) (CON A ST-NOV 99.2 6)
                     (EXP B ST-NOV 99.7 9)  (CON B ST-NOV 97.2 13)) do
        (let ((foo '()) tmp)
          (dotimes (i (fifth row))
            (setq tmp (random (/ (fourth row) 10)))
            (push tmp foo)
            (push (- tmp) foo))
          (dotimes (i (fifth row))
            (push (first row) cdtn)
	    (push (second row) clin)
            (push (third row) stat)
            (push (+ (fourth row) (or (car foo) 0)) valu)
            (setq foo (cdr foo))))))


#+KEPPEL-19-3
(progn
  (defparameter sub NIL)
  (defparameter fac NIL)
  (loop for c in '(a1 a2 a3 a4 a5 a6) do
        (loop for r in '(s1 s2 s3 s4 s5 s6 s7 s8) do
              (push c fac)
	      (push r sub)))
  (defparameter pos '(7 4 7 8 7 6 4 6
                        3 8 6 6 2 3 2 7
                        2 3 3 1 3 3 0 5
                        2 8 1 0 0 1 0 1
                        1 1 5 2 1 1 0 3
                        1 2 4 0 3 1 0 2)))

#+OLSON-15-8
(progn
  (defparameter prod NIL)
  (defparameter stor NIL)
  (loop for c in '(s1 s2 s3) do
        (loop for r in '(lettuce mushrooms tomatoes eggs cottage-cheese prok shrimp coffee) do
              (push c stor)
              (push r prod)))
  (defparameter price '(59 71 99 112 195 205 389 439
                           59 79 129 110 179 195 349 385
			   47 94 129 105 193 199 375 434)))

#+DEBUG
(progn
  ;; three way
  (format t "HM  = ~,3f  T! = ~,3F~%" HM T!)
  (format t "A   : ~,3f  ~,3f  ~,3f  ~%" SSa MSa DFa)
  (format t "B   : ~,3f  ~,3f  ~,3f  ~%" SSb MSb DFb)
  (format t "C   : ~,3f  ~,3f  ~,3f  ~%" SSc MSc DFc)
  (format t "AB  : ~,3f  ~,3f  ~,3f  ~%" SSab MSab DFab)
  (format t "AC  : ~,3f  ~,3f  ~,3f  ~%" SSac MSac DFac)
  (format t "BC  : ~,3f  ~,3f  ~,3f  ~%" SSbc MSbc DFbc)
  (format t "ABC : ~,3f  ~,3f  ~,3f  ~%" SSabc MSabc DFabc)
  ;; two way repeated
  (format t "A   : ~4,3F~4T~,3F~4T~,3F~4T~%" SSa MSa DFa)
  (format t "B   : ~4,3F~4T~,3F~4T~,3F~4T~%" SSb MSb DFb)
  (format t "S   : ~4,3F~4T~,3F~4T~,3F~4T~%" SSs MSs DFs)
  (format t "AB  : ~4,3F~4T~,3F~4T~,3F~4T~%" SSab MSab DFab)
  (format t "AS  : ~4,3F~4T~,3F~4T~,3F~4T~%" SSas MSas DFas)
  (format t "BS  : ~4,3F~4T~,3F~4T~,3F~4T~%" SSbs MSbs DFbs)
  (format t "ABS : ~4,3F~4T~,3F~4T~,3F~4T~%" SSabs MSabs DFabs)
  )

#+OLSON-UWM
(setq row '(m m m m m f f f f f f)
      col '(1 1 2 2 2 1 1 1 2 2 2)
      val '(4 6 2 3 4 8 9 10 10 11 12))

;; Documentation of probable errors :
;; 
;; 1) With unequal cell sizes, and the procedure in OLSON, the interaction effects involves subtraction
;;    of the SS main effects. the subtraction should NOT use the harmonic-adjusted SS, but the pre-adjusted
;;    one.

