;;;; -*- Mode:Common-Lisp; Package:CLASP; Base:10; Fonts:(MEDFNT) -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/dev/utilities.lisp *-*
;;;; *-* Last-edit: Tuesday, December 6, 1994  18:06:16; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                                Utilities                               *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  01-19-93 File Created.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :CLASP)

;;; --*--
;;; ***************************************************************************

(proclaim '(inline square))
(defun square (x)
  "(* x x)"
  (* x x))

#+Lispworks
(defun write-nicely (object)
  (typecase object
    (ratio (format nil "R~f" object))
    (cons (mapcar 'write-nicely object))
    (t object)))

#+IN-EKSL-UTILITIES
(defmacro spy (&rest forms &aux (stream #-Lispworks '*trace-output* #+Lispworks '*standard-output*))
  "A debugging tool: wrapping this around a form causes both the form and its
values to be printed on *trace-output*.  Like `progn,' this returns the value\(s\)
of the last form, so that it can be safely wrapped about functional code as
well.  In other words, \(spy \(foo\)\) returns the same values as \(foo\).
The first form can optionally be a list of options, eg., keyword value pairs\).
Options supported:
:STREAM <stream>      - directs output to `stream'
:NO-NEWLINE <boolean> - if non-nil suppresses insertion of newlines between forms
:NO-EXPAND <boolean>  - if true suppresses the form with values replacing subforms
"
  (let ((no-newline nil)
	(no-expand  nil))
    (when (and (consp (first forms)) (keywordp (first (first forms))))
      (let ((options (pop forms)))
	(setf stream (getf options :stream stream)
	      no-newline (getf options :no-newline)
	      no-expand  (getf options :no-expand))))
    (labels ((spy-2 (form)
	       ;; produces a form equivalent to the input, except that all non-car atoms
	       ;; are evaluated, so that when it prints, the user can see the "expanded"
	       ;; version of the form.  Certain special forms are handled specially.
	       (if (atom form)
                   #+Lispworks
		   `(write-nicely ,form)
                   #-Lispworks
                   form
		   (let ((car (car form)))
		     (case car
		       ((setq setf)
			`(list ',car ,@(let ((place? t))
					 (mapcar #'(lambda (subform)
						     (prog1 (if place?
								`',subform
								(spy-2 subform))
							    (setf place? (not place?))))
						 (cdr form)))))
                       ((and or prog1 progn)
                        `(list ',car ,@(mapcar #'spy-2 (cdr form))))
		       (t (if (or (macro-function car)
				  (special-form-p car))
			      `',form
			      `(list ',car ,@(mapcar #'spy-2 (cdr form))))))))))
      (let ((formo (gensym "ORIGINAL-FORM"))
	    (formx (gensym "EXPANDED-FORM"))
	    (values (gensym "VALUES")))
	(flet ((spy-1 (form)
		 `(let ((,formo   ',form)
			(,formx  ,(spy-2 form))
			(,values  (multiple-value-list ,form)))
		    ,(when no-expand `(declare (ignore ,formx)))
		    ,(unless no-newline
		       `(fresh-line ,stream))
		    ,(if no-expand
			 `(format ,stream "~s =>~{ ~s~}"
                                  ,formo
                                  #-Lispworks ,values
                                  #+Lispworks (mapcar #'write-nicely ,values))
			 `(format ,stream "~s =>~%~s =>~{ ~s~}" ,formo ,formx
                                  #-Lispworks ,values
                                  #+Lispworks (mapcar #'write-nicely ,values)))
		    ,(if no-newline
			 `(write-char #\space ,stream)
			 `(terpri ,stream))
		    (values-list ,values))))
	  `(prog1
	     (let ((*print-rationals-as-floats?* t))
	       ,@(mapcar #'spy-1 forms))
	     ,@(when no-newline
		 `((terpri ,stream)))))))))

#+test
(defun test-spy ()
  (let ((c 1/5)) (spy c))
  (spy (/ 1 2))
  (spy (:no-expand t) (floor 4 3))
  (let ((a 1) (b 2)) (spy (/ a b)))
  (let ((a 1) (b 2) c) (spy (/ (setf c (+ a b)) (+ a b))))
  (let ((a 1) (b 2) c) (spy (and a b (or (prog1 a b) (progn c)))))
  (let ((a 1) (b 2)) (spy (multiple-value-bind (c d) (floor a b) (values c d))))
  )

;;; ----------------------------------------------------------------------------

#+OLD
(defmacro spy-math (&rest forms)
  #+obsolete
  `(progn ,@(mapcar #'spy-math-1 forms))
  `(spy ,@forms))

#+obsolete
(defun spy-math-1 (form)
  `(let ((print-form ',form)
	 (inline-form ,(unless (atom form) (spy-math-2 form)))
	 (values (multiple-value-list ,form)))
     (format t "~s =>~@[~%~s =>~] ~{ ~s~}~%" print-form inline-form (mapcar #'float values))
     (values-list values)))

#+obsolete
(defun spy-math-2 (form &optional no-func)
  `(,@(unless no-func '(list))
      ,@(append (unless no-func `((quote ,(pop form))))
		(cond ((atom (first form))
		       `((float ,(first form))))
		      (t (list (spy-math-2 (first form)))))
		(when (cdr form)
		  (spy-math-2 (cdr form) t)))))
      

;;; ----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defmacro push-end (item item-list)
  "This is similar to PUSH except that it puts the new element at the end
of the existing list.  This preserves the order of the elements as they are added to
the list."
  `(setf ,item-list (nconc ,item-list (cons ,item nil))))

;;; ----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defmacro undefclass (name supers slots &rest options)
  (declare (ignore supers slots options))
  `(setf (find-class ',name) nil))

;;;----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defmacro map-over-classes (function class-list &key key predicate partial-order &aux (class (gensym)))
  `(dolist (,class ,@(if partial-order `((sort-using-list-order (copy-list ,class-list)
                                                                ,partial-order))
                         `(,class-list)))
     ,@(if key `((setf ,class (funcall ,key ,class))))
     (when (funcall ,predicate ,class)
       (funcall ,function ,class))))

;;; ----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defun sort-using-list-order (x list &key (test #'eq) key (list-key #'identity))
  (flet ((item-lessp-function (list  test item-key list-key)
           (if item-key
             #'(lambda (item-1 item-2)
                 (setf item-2 (funcall item-key item-2)
                       item-1 (funcall item-key item-1))
                 (let ((item-2-present (member item-2 list :test test :key list-key))
                       (item-1-present (member item-1 list :test test :key list-key)))
                   (if (and item-1-present item-2-present)
                     (not (member item-1 item-2-present :test test :key list-key))
                     ;; otherwise
                     (if item-1-present t nil))))
             #'(lambda (item-1 item-2)
                 (let ((item-2-present (member item-2 list :test test :key list-key))
                       (item-1-present (member item-1 list :test test :key list-key)))
                   (if (and item-1-present item-2-present)
                     (not (member item-1 item-2-present :test test :key list-key))
                     ;; otherwise
                     (if item-1-present t nil)))))))
    (let ((lessp-function (item-lessp-function list test key list-key)))
      (stable-sort x lessp-function))))

;;; ----------------------------------------------------------------------------

(defun alphalessp (x y)
  "T if printed representation of X is less than that of Y.
Numbers come before characters, before symbols/strings, before random objects, before lists.
Numbers are compared using <, characters are compared using CHAR<; symbols/strings with STRING-LESSP;
random objecs by printing them(!); lists are compared recursively."
  (cond
    ((numberp x) (or (not (numberp y)) (< x y)))
    ((numberp y) nil)
    ((characterp x) (or (not (characterp y)) (char< x y)))
    ((characterp y) nil)
    ((or (symbolp x) (stringp x)) (or (not (or (symbolp y) (stringp y))) (string-lessp x y)))
    ((or (symbolp y) (stringp y)) nil)
    ((atom x) (or (consp y) (string-lessp (prin1-to-string x) (prin1-to-string y))))
    ((atom y) nil)
    (t
     (do ((x1 x (cdr x1))
	  (y1 y (cdr y1)))
	 ((null y1))
       (or x1 (return t))
       (and (alphalessp (car x1) (car y1)) (return t))
       (and (alphalessp (car y1) (car x1)) (return ())))))) 

;;;----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defun class-built-on-class (class potential-superclass)
  (member potential-superclass
          #-MCL
	  (if #+Explorer (ticlos::class-composed-p class)
              #-Explorer (class-finalized-p class)
	      (class-precedence-list class)
	      (class-direct-superclasses class))
          #+MCL
          (class-precedence-list class)
	  :test #'eq))

;;; ----------------------------------------------------------------------------

(defun format-if (stream format-string &rest format-args)
  (when stream (apply #'format stream format-string format-args)))

(defun report-if (flag stream format-string &rest format-args)
  (when (and flag stream)
    (apply #'format stream format-string format-args)))

;;;----------------------------------------------------------------------------

(defun fill-all (array elt &key (start 0) (end (array-total-size array)))
  "Fills an array ignoring the fill pointer."
  (let ((fp (fill-pointer array)))
    (setf (fill-pointer array) end)
    (fill array elt :start start :end end)
    (setf (fill-pointer array) fp)))

;;; ----------------------------------------------------------------------------

(defvar *temporary-vector* nil
  "A temporary vector for use by statistical functions such as `quantile,' which
uses it for sorting data.  This avoids consing or rearranging the user's data.")

(defmacro with-temp-vector ((temp min-size) &body forms)
  "Binds `temp' to a vector of length at least `min-size.' It's a vector of
pointers and has a fill-pointer, initialized to `min-size.'"
  ;; The following will NOT prevent sychronization problems in a parallel lisp,
  ;; but it will make them less likely, because the only timing problem will be
  ;; if a process swap allows another process to grab the same
  ;; *temporary-vector*, that is, before the variable gets set to nil.  The real
  ;; solution is critical sections, which is not in Common Lisp.
  `(let ((,temp *temporary-vector*))
     ;; no process swap between the previous LET and the next SETF, we hope!
     (setf *temporary-vector* nil) 
     (when (or (null ,temp)
	       (< (array-total-size ,temp) ,min-size))
       (setf ,temp (make-array ,min-size :fill-pointer ,min-size)))
     ;; nulling out the array makes any bugs in the code more likely to
     ;; show up, but is not really necessary
     #+TEST
     (fill-all ,temp nil)
     (setf (fill-pointer ,temp) ,min-size)
     (multiple-value-prog1 (progn . ,forms)
			   (setf *temporary-vector* ,temp))))

(defvar *temporary-table* nil
  "A temporary table. This avoids consing.")

(defmacro with-temp-table ((temp) &body forms)
  "Binds `temp' to a hash table."
  ;; The following will NOT prevent sychronization problems in a parallel lisp,
  ;; but it will make them less likely, because the only timing problem will be
  ;; if a process swap allows another process to grab the same
  ;; *temporary-table*, that is, before the variable gets set to nil.  The real
  ;; solution is critical sections, which is not in Common Lisp.
  `(let ((,temp *temporary-table*))
     ;; no process swap between the previous LET and the next SETF, we hope!
     (setf *temporary-table* nil) 
     (when (null ,temp)
       (setf ,temp (make-hash-table)))
     ;; nulling out the array makes any bugs in the code more likely to
     ;; show up, but is not really necessary
     (clrhash ,temp)
     (multiple-value-prog1 (progn . ,forms)
			   (setf *temporary-table* ,temp))))

;; A faster version of `remove-duplicates'. Note you cannot specify a :TEST (it is always #'eq).
(defun extract-unique-values (sequence)
  (with-temp-table (table)
    (loop for element in sequence do
	  (unless (gethash element table)
            (setf (gethash element table) element))
          finally (return (let ((val nil)) 
                            (maphash #'(lambda (key value)
                                         (declare (ignore value))
                                         (push key val))
                                     table)
                            (values val))))))

#+test
(defun test-extract-unique-values ()
  (loop for size in '(100 500 1000 #-MCL 5000 #-MCL 10000 #-MCL 50000 #-MCL 100000)
        for list = (loop repeat size collect (random size))
        for theirs = (time (remove-duplicates list))
        for ours =  (time (extract-unique-values list))
        do
        (format t "~&Lengths: initial = ~d; after: theirs = ~d, ours = ~d~%" (length list) (length theirs) (length ours))))

;;; ----------------------------------------------------------------------------

#+IN-EKSL-UTILITIES
(defmacro doplist ((key value plist) &body body)
  "Iterate over each key/value pair in a plist.  Key and Value are
bound to successive pairs in Plist."
  (let ((atvar (gensym "PLIST")))
    `(do* ((,atvar ,plist (cddr ,atvar))
	   (,key (car ,atvar) (car ,atvar))
	   (,value (second ,atvar) (second ,atvar)))
	  ((null ,atvar) nil)
       ,@body))) 

;;; ----------------------------------------------------------------------------

;; A version of this is in EKSL UTILITIES.
(defun DATE-AND-TIME (&OPTIONAL (time (get-universal-time)))
  
  "DATE-AND-TIME [universal-time]

Returns formatted date/time string."
  
  (multiple-value-bind (second minute hour date month year day)
                       (decode-universal-time time)
    (format nil 
            "~a, ~a ~a, ~a  ~2,'0d:~2,'0d:~2,'0d"
            (nth day
                 '("Monday" "Tuesday" "Wednesday" "Thursday"
                   "Friday" "Saturday" "Sunday"))
            (nth (1- month)
                 '("January" "February" "March" "April" "May"
                   "June" "July" "August" "September"
                   "October" "November" "December"))
            date
            year
            hour
            minute
            second)))

;;;----------------------------------------------------------------------------

#-lcl4.0
(DEFUN xor (&rest args)
  "Takes any number of arguments and returns T if an odd number of its arguments are non-NIL, 
otherwise returns NIL."
  (LET ((flag nil))
    (DOLIST (form args flag)
      (WHEN form
	    (SETF flag (not flag))))))

;;;; --------------------------------------------------------------------------
;;;;   Creating symbols
;;;; --------------------------------------------------------------------------
#+IN-EKSL-UTILITIES
(defmacro form-symbol-in-package (pkg &rest names)
  "FORM-SYMBOL-IN-PACKAGE pkg &rest names
   Return a symbol interned in PKG whose print name is the concatenation
   of NAMES.  Each name must be acceptable to the string function."
  `(intern (concatenate
	     'simple-string
	     ,@(mapcar #'(lambda (name)
			   (if (stringp name) name `(string ,name)))
		       names))
	   ,pkg))
#+IN-EKSL-UTILITIES
(defmacro form-symbol (&rest names)
  "FORM-SYMBOL &rest names
   Return a symbol interned in the current package whose print name is the
   concatenation of NAMES.  Each name must be acceptable to the string function."
  `(form-symbol-in-package *package* ,@names))
#+IN-EKSL-UTILITIES
(defmacro form-keyword (&rest names)
  "FORM-KEYWORD &rest names
   Return a symbol interned in the keyword package whose print name is the
   concatenation of NAMES.  Each name must be acceptable to the string function."
  `(form-symbol-in-package "KEYWORD" ,@names))
#+IN-EKSL-UTILITIES
(defmacro form-uninterned-symbol (&rest names)
  "FORM-UNINTERNED-SYMBOL &rest names
   Return an uninterned symbol whose print name is the concatenation of NAMES.
   Each name must be acceptable to the string function."
  `(make-symbol (concatenate
		  'simple-string
		  ,@(mapcar #'(lambda (name)
				(if (stringp name) name `(string ,name)))
			    names))))

;;;; --------------------------------------------------------------------------
;;;;   Functions used in the following macros
;;;; --------------------------------------------------------------------------

;;; Sometimes it's nice to have your gensyms mean something when
;;; you're reading the macroexpansion of some form.  The problem
;;; is that if you give a prefix to GENSYM it remains the prefix
;;; until you change it.  
#+IN-EKSL-UTILITIES
(eval-when (compile load eval)

(defvar *newsym-counter* 0
  "Counter used by NEWSYM for generating print names.")

(defun newsym (&optional (prefix "X"))
  "Create a new uninterned symbol whose print name begins with PREFIX.
   This differs from GENSYM in that the prefix is not sticky."
  (unless (stringp prefix)
    (setf prefix (string prefix)))
  (make-symbol (format nil "~a~4,'0d" prefix (incf *newsym-counter*))))

) ;; End of Eval-When

;;;; --------------------------------------------------------------------------
;;;;   Keyword Parsing
;;;; --------------------------------------------------------------------------

;;; WITH-KEYWORDS-BOUND:
;;;
;;; (with-keywords-bound (((key1 default1)
;;;                        (key2 default2)
;;;                        ...)
;;;                       arglist)
;;;   <body>)
#+IN-EKSL-UTILITIES
(defmacro with-keywords-bound ((key-specs arg-list &rest args)
			       &body body)

  "WITH-KEYWORDS-BOUND (key-specs arg-list [error-msg | keyvar] &rest otherwise)
                 &body body)

   Look through ARG-LIST for keywords and bind them, similar to the
   processing of lambda-list keywords.  ARG-LIST is evaluated and should
   be a list of keywords and values.  KEY-SPECS describes the keywords
   to check for.  Each element should be a symbol that will be bound to
   the keyword value from ARG-LIST or a list of the symbol and a default
   value.

   If the third argument is a string it is used as the format string for
   the error message when an unknown keyword is found.

   If the third argument is a symbol then it is used as the variable
   which is used internally to hold the remaining part of the list during
   processing.  OTHERWISE is code that will be executed (in an implicit
   progn) if any keyword from ARG-LIST is not a recognized keyword.
   This code can refer to KEYVAR."

  (let* ((key-symbols (mapcar #'(lambda (key)
				  (if (symbolp key) key (car key)))
			      key-specs))
	 (error-msg (if (stringp (car args)) (car args) nil))
	 (keyvar (if (and (symbolp (car args))
			  (not (null (car args))))
		     (car args)
		     (newsym "KEY")))
	 (otherwise (if error-msg nil (cdr args))))
    (or otherwise
	error-msg
	(setf error-msg (format nil "~~s is not one of (~{:~a~^ ~})."
				key-symbols)))
    `(let ,key-specs
       (when (consp ,arg-list)
	 (do ((,keyvar ,arg-list (cdr ,keyvar)))
	     ((null ,keyvar))
	   (case (car ,keyvar)
	     ,@(mapcar #'(lambda (key)
			   `(,(form-keyword key)
			     (setf ,key (car (setf ,keyvar (cdr ,keyvar))))))
		       key-symbols)
	     (otherwise
	      ,@(or otherwise
		    `((error ,error-msg (car ,keyvar))))))))
       ,@body)))

#+TEST
(with-keywords-bound ((filter index pred)
			     '(:filter (t nil) :pred fred a b c)
			     keyvar (or stuff (setf stuff keyvar))) 
  (spy filter index pred stuff))

(make-obsolete make-variables with-variables)

#+IN-EKSL-UTILITIES                         
(defmacro make-variables (symbols &body body)
  "Using gensyms/gentemps is necessary to prevent variables produced by
macro expansions from interfering with user variables, and naming them
mnemonically helps make macro expansions and compiled code easier to
read, but it's a pain to create them properly.  This macro creates them
for you, which makes writing nice macros easier.  For example, if you
are writing a macro to iterate over an array, you used to have to write:

 (defmacro do-2d-array ((elt array) &body body)
   (let ((row (gentemp \"ROW\"))
         (col (gentemp \"COL\")))
     `(dotimes (,row (array-dimension 0))
        (dotimes (,col ,(array-dimension 1))
           (let ((,elt (aref ,array ,row ,col)))
               . ,body)))))

Now you can just write the following, which eliminates the need to
laboriously create the mnemonic gensyms/gentemps.

 (defmacro do-2d-array ((elt array) &body body)
   (make-variables (row col)
      `(dotimes (,row ,(array-dimension 0))
          (dotimes (,col ,(array-dimension 1))
             (let ((,elt (aref ,array ,row ,col)))
                 . ,body))))
"
  `(let ,(mapcar #'(lambda (sym)
		     `(,sym (gentemp ,(symbol-name sym))))
		 symbols)
     . ,body))

;;; ---------------------------------------------------------------------------
#+IN-EKSL-UTILITIES
(defun mapappend (fun &rest args)
  (if (some #'null args) 
      ()
      (append (apply fun (mapcar #'car args))
	      (apply #'mapappend  fun (mapcar #'cdr args)))))

;;; ---------------------------------------------------------------------------

#+ANOTHER-SIMILAR-FUNCTION
(defun p/c (l1 l2)
  (if (null l1)
      '(())
      (mapcan #'(lambda (a)
                  (mapcar #'(lambda (b) (cons (list (car l1) a) b))
                          (p/c (cdr l1) (remove a l2))))
              l2)))

(defun power-set (lists)
  (setf lists (reverse lists))
  (power-set-helper (first lists) (rest lists) nil))

(defun power-set-helper (first rest accum)
  (if (null first)
      (values accum)
      (power-set-helper (first rest) (rest rest) (power-set-2-args first accum))))
  
(defun power-set-2-args (new old)
  (if (null old)
      new
      (mapappend #'(lambda (n) (mapcar #'(lambda (o) (if (consp o) (list* n o) (list n o))) old)) new)))

;;; ---------------------------------------------------------------------------

(defmacro with-progress-thermometer ((stream dots-num num-num &key (start 0) initial-string print-total) &body body)
  (let ((cnt (newsym 'cnt)))
    `(let ((,cnt ,start))
       (labels ((note-progress ()
               (incf ,cnt)
               (when ,stream
	         (cond ((= (nth-value 1 (truncate ,cnt ,num-num)) 0)
	                (print ,cnt ,stream)
                        (finish-output ,stream))
	               ((= (nth-value 1 (truncate ,cnt ,dots-num)) 0)
	                (write-char #\. ,stream)
			(finish-output ,stream))
	               (t nil))))
                (doit ()
                      (progn
                        (when (and ,stream ,initial-string)
                          (write-string ,initial-string ,stream))
                        . ,body)
                      
                      (when ,stream
                        (when ,print-total
                          (print ,cnt ,stream))
                        (terpri ,stream))))
         (let ((*trace-output* *standard-output*))
	   (#+TEST time #-TEST progn (doit)))))))

;;; ---------------------------------------------------------------------------

(defun maybe-compile-lambda (lambda)
  (or (ignore-errors
       (compile nil lambda
                #+lucid :optimize-message #+lucid nil))
      (coerce lambda 'function)))

;;; ***************************************************************************
;;; EOF


