;;;; -*- Mode:Common-Lisp; Package:cl-user; Fonts:(medfnt); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/loading/generic-load-utils.lisp *-*
;;;; *-* Last-edit: Friday, Feb 21, 1997; Edited-By: westy *-* 

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                     Generic Load Utilities                             *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************

;;; Copyright (c) 1990 - 1997 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and
;;; its documentation is hereby granted without fee for non-commercial
;;; uses only (not for resale), provided that the above copyright
;;; notice of EKSL, this paragraph and the one following appear in all
;;; copies and in supporting documentation.

;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possibility of
;;; such damages.

;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  03-09-93 File Created.  (Westy)
;;;  05-19-93 Converted to use logical pathnames in Lucid.  (Westy)
;;;  06-08-94 Split into Generic and Clasp-specific files.  (Anderson)
;;;  06-23-94 Added `compile-load-if'.  (Westy)
;;;  06-25-94 Added explorer types to `file-type-for-*' (Anderson)
;;;  06-25-94 Added more defaults to `file-write-date' in `compile-load-if
;;;           (anderson)
;;;  06-25-94 Prepend `eksl-' to `file-type-for-*' to avoid conflicts with
;;;           functions in load-dwim. (Anderson)
;;;  07-07-94 Added error checking to `load-pathname,' since bugs can be 
;;;           obscure.  (anderson)
;;;  07-09-94 Changed name of `relative-path-namestring' to
;;;           `relative-path-directory-list'.  (Westy)
;;;  07-12-94 Added `define-eksl-system'.  (Westy)
;;;  07-27-94 Added code to `eksl-load-system' and `eksl-compile-system' to 
;;;           remove the bin-identifiers information on the Explorer, since
;;;           `make-system' doesn't understand that.  (Anderson)
;;;  12-06-94 Improved the previous patch to also remove :base  (anderson)
;;;  03-18-95 Modified `compile-load-if' to translate logical pathnames in
;;;           Lispworks as well as MCL.  The function constructs bad pathnames
;;;           otherwise.  (anderson)
;;;  05-07-95 Anderson: Here's something for the wish list: It would be great if
;;;          `compile-load-if' were enhanced to have a restart that offered to
;;;           recompile and reload the file if there's an error during loading.
;;;           Lispworks automatically gives a "recompile" restart for
;;;           compilation errors, but I'm talking about errors that aren't
;;;           discovered until loading: reloading just isn't going to help.
;;;  05-08-95 Removed support for Explorer defsystem. (Westy)
;;;  05-08-95 Changed the syntax of `define-eksl-system' to allow multiple base
;;;           directories. Also removed the need to supply the :FILES keyword
;;;           (although it is still supported for compatibility). Finally,
;;;           added `map-eksl-system-files' to make everything work.  (Westy)
;;;  05-17-95 Changed `compile-eksl-system' to correctly handle the :compile 
;;;           and :recompile init keywords on the Explorer.  (anderson)
;;;  05-17-95 Modified `define-eksl-system' to macro-expand into an Explorer
;;;           defsystem on the Explorer.  Simplifies foo.system files.  (anderson)
;;;  05-18-95 Modified to avoid the Explorer `defsystem' and to use our own 
;;;           code instead.  This allows for the new structured syntax for 
;;;           systems.  (anderson)
;;;  05-18-95 Added a `restart-case' to recompile for errors during loading.  
;;;           (anderson)
;;;  05-25-95 Added host component to bin-directory.  (anderson)
;;;  05-18-95 I implemented my wish of 05-07-95. (anderson)
;;;  06-30-95 Implemented `file-newer-than-file-p' so I could use it separately.
;;;           (anderson)
;;;  07-01-95 Added a `restart-case' to recompile if binary not found. (anderson)
;;;  07-07-95 Fixed the logic for unknown file write dates.  It was defaulting to
;;;           zero, which meant it would never get reloaded if it had once been
;;;           loaded.  Change the default to most-positive-short-float.
;;;           (anderson)
;;;  08-04-95 Extended make-bin-directory to handle symbols other than the
;;;           special keywords  (anderson)
;;;  09-15-95 Added some code to `eksl-load-system' to allow partial loading
;;;           of systems.  (Westy)
;;;  03-01-96 Changed the name of `compile-load-if' to `compile-load-source-if'
;;;           to reflect that fact that is really only usefull for compiling 
;;;           and loading source sources defined to be part of a defsystem (it
;;;           does all of that base-pathname and bin-directory defaulting). 
;;;           To get a real `compile-load-if' I ripped the guts out of the old
;;;           one and made it only do simple binary extension merging if the
;;;           binary is not specified. `compile-load-source-if' calls 
;;;           `compile-load-if' after it has figured out the bin directory.
;;;           (Westy)
;;;  04-10-96 Added code (again) to canonicalize the names used for EKSL
;;;           systems to keywords (no more package problems).
;;;  06-14-96 It would be nice if we handled component systems in here -
;;;           you know all that compilation propagation stuff. Also, maybe 
;;;           provided a method for not reloading system def files (turned 
;;;           off for development). - Westy
;;;  08-20-96 Added a CCL-4 feature for MCL so we can differentiate binaries
;;;           (which are incompatible between 3.9 and 4.0).  (Westy)
;;;  10-10-96 Added a restart-case to 'eksl-load-system' to allow reloading 
;;;           or compiling from an error.  (Westy)
;;;  09-19-97 Changed 'load-pathname' in Allegro to return a translated
;;;           pathname. I'm considering changing the name of to
;;;           load-truename, because that is the functionality that it
;;;           turns out to be providing. (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package #-MCL :user #+MCL :COMMON-LISP-USER)

#+MCL
(ccl::%add-nicknames  '("USER") (find-package 'cl-user))

;;; --*--
;;; ***************************************************************************

;;; So that we can avoid reloading this file many times, this file should always
;;; be loaded conditional on the following feature.
(pushnew :eksl-generic-load-utils *features*)

;;; ===========================================================================
;;; Feature hacking for CLIM

#-clim
;Add CLIM feature.
(eval-when (compile load eval)
  (when (find-package 'clim)
    (pushnew :clim *features*)))

; 5-26-93 These will suffice for now.  (Westy)
#+(and clim (or allegro-v4.1 lcl4.0 lcl4.1) (not clim-2))
(pushnew :clim-1 *features*)
#+(and clim (or allegro-v4.1 lcl4.0 lcl4.1 lispworks) (not clim-2))
(pushnew :clim-1.1 *features*)
#+(and CLIM-2 Lucid4.1)
(pushnew :CLIM *features*)

(proclaim '(special *default-binary-path*))

;;; ===========================================================================

(defun load-pathname* ()
  "Primitive version of load-pathname, which is the function to use."
  #+allegro (translate-logical-pathname excl:*source-pathname*)
  #+MCL (if *load-pathname* 
          (translate-logical-pathname *load-pathname*)
          ;; This makes it work in a fred buffer...
          (translate-logical-pathname *loading-file-source-file*))
  #+Lispworks (translate-logical-pathname *load-pathname*)
  #-(or allegro Explorer MCL Lispworks) 
  (translate-logical-pathname *load-pathname*))


(defun load-pathname ()
  "Returns a pathname that is useful for merging with filenames to get a
complete pathname for a file in the same directory as the one currently being
loaded.  This function is a more portable version of the Common Lisp variable
*load-pathname*, since not all vendors implemented that correctly."
  (let ((pn (load-pathname*)))
    (assert (pathnamep pn) (pn)
            "load-pathname* did not return a pathname, it returned ~s;
this might be because it executed in a context where you're not loading a file
or because the file is in the current working directory.  In any event, this 
will break the code in generic-load-utils, because it works with the components
of pathnames."
	    pn)
    (values
     #-lispworks
     pn
     ;; Deal with the lispworks using an empty pathname for *default-pathname-defaults*
     ;; if you are loading from the current working directory.
     #+lispworks
     (if (null (pathname-directory pn))
         (merge-pathnames pn (current-pathname))
         pn))))

;;; ===========================================================================

;; Obsolete
(defun dns (dir)
  "Directory namestring: Returns the namestring of a path created from a
directory list, such as the result of relative-path-namestring."
  (check-type dir list)
  (namestring (make-pathname :directory dir)))

(defconstant *wild-inferiors*
  #-lucid :wild-inferiors
  #+lucid "**"
  "A platform-independent definition of wild inferiors, suitable for appending to
directory lists for pathnames.")

;;; ===========================================================================
;;; RELATIVE-PATH-DIRECTORY-LIST

(defun relative-namestring (&rest dir-names)
   "Starts with the directory of `load-pathname'; it assumes the directory is
represented as a list.  It then modifies that list by appending things, such
as subdirectories, or removing things, to go up the directory tree. Each
dir-name argument can be :back, which goes up a subdirectory level, a string,
which should be uppercase to specify common case, or :wild-inferiors.  Returns
a namestring."
  (namestring (make-pathname :directory (apply #'relative-path-directory-list dir-names))))

;; changed to call 'relative-path-directory-list-with-base-pathname'
(defun relative-path-directory-list (&rest dir-names)
  "See `relative-namestring'."
  (apply #'relative-path-directory-list-with-base-pathname nil dir-names))

#+OLD
(defun relative-path-directory-list (&rest dir-names)
  "See `relative-namestring'."
  (let ((pn (copy-list (pathname-directory (load-pathname)))))
    (dolist (dir dir-names)
       (cond ((eq dir :back)
	      (setf pn (nbutlast pn)))
	     ((eq dir :wild-inferiors)
	      (setf pn (nconc pn (list *wild-inferiors*))))
	     ;; So far, no platform correctly implements common case, so we have
	     ;; to convert strings to lowercase.
	     ((stringp dir)
	      (setf pn (nconc pn (list (string-downcase dir)))))
	     (T
	      (error "Not a known kind of dir-name: ~s" dir))))
    pn))

(defun relative-path-directory-list-with-base-pathname (base-pathname &rest dir-names)
  "See `relative-namestring'."
  (let ((pn (copy-list (pathname-directory (or base-pathname 
                                               (load-pathname))))))
    (dolist (dir dir-names)
       (cond ((eq dir :back)
	      (setf pn (nbutlast pn)))
	     ((eq dir :wild-inferiors)
	      (setf pn (nconc pn (list *wild-inferiors*))))
	     ;; So far, no platform correctly implements common case, so we have
	     ;; to convert strings to lowercase.
	     ((stringp dir)
	      (setf pn (nconc pn (list (string-downcase dir)))))
	     (T
	      (error "Not a known kind of dir-name: ~s" dir))))
    pn))

(defun relative-namestring-with-base-dir (base-directory &rest dir-names)
  (namestring (make-pathname
               :directory 
               (apply #'relative-path-directory-list-with-base-pathname
                      base-directory dir-names))))

(defconstant +eksl-define-logical-pathname-translations-documentation+
  "A more declarative version of 'eksl-logical-pathname-translations'.
Syntax:

(eksl-define-logical-pathname (<name> [<base-pathname>])
  (<translation>*))

translation :=  <directory-name> | (<logical-part>) | (<logical-part> <physical-part>)
logical-part := <directory-name>
directory-name := <symbol> | <string>
physical-part := <directory-name> | <dir-spec-list>
dir-spec-list := (<dir-spec>* | (:BINARY-DIRECTORY (<bin-identifiers>) <directory-name>)
dir-spec := <directory-name> | :BACK

The current directory is used as the default directory unless `base-pathname' is supplied. 
A translation consisting of just a directory-name describes a pathname translation to a 
subdirectory under the default directory whose logical and physical names are identical. 
A translation with just a logical part describes a pathname translation to the default directory.
A translation with a logical part and a physical part describes a pathname translation to a 
subdirectory under the default directory with the corresponding names for each.
See also `relative-namestring' for a description of the use of :BACK. See also 
`bin-directory-name' for a description of `bin-identifiers'.

Example:

(eksl-define-logical-pathname-translations (ACS)
   (acs)
   (source \"CODE\")
   (binary (:BINARY-DIRECTORY (:platform :vendor) \"CODE\"))
   speech
   images
   demos
   (interface #+CLIM \"INTERFACE\" #+MCL \"INTERFACE-MCL\")
   (interface-binary (:BINARY-DIRECTORY (:platform :vendor :gui) #+CLIM \"INTERFACE\" #+MCL \"INTERFACE-MCL\"))
   (eksl-utils (:back :back \"UTILS\" \"DEV\"))
   (frame      (:back :back \"FRAME\" \"DEV\"))
   (mess       (:back :back \"MESS\" \"DEV\"))
   (clip       (:back :back \"CLIP\" \"DEV\"))
   (plotter    (:back :back \"PLOTTER\" \"DEV\")))
")


(defmacro eksl-define-logical-pathname-translations ((name &optional base-pathname) &rest definitions)
  #.+eksl-define-logical-pathname-translations-documentation+
  (let ((logical-name (string name)))
    `(let ((translated-base-pathname 
            ,(when base-pathname 
               `(translate-logical-pathname ,base-pathname))))
       (setf (eksl-logical-pathname-translations ,logical-name)
             (list 
              ,@(mapcar 
                 #'(lambda (def)
                     (cond 
                      ; FRED -> ("FRED;**;*.*.*" <BASE>+"FRED")
                      ((atom def)
                       `(list ,(concatenate 'string (string def) ";**;*.*.*")
                              ,(build-eksl-physical-translation 
                                def)))
                      ; (FRED) -> ("FRED;**;*.*.*" <BASE>)
                      ((null (cdr def))
                       `(list ,(concatenate 'string (string (first def)) ";**;*.*.*")
                              (relative-namestring-with-base-dir 
                               translated-base-pathname
                               :wild-inferiors)))
                      ; (FRED "sam") -> ("FRED;**;*.*.*" <BASE>+"sam")
                      ((symbolp (first def))
                       `(list ,(concatenate 'string (string (first def)) ";**;*.*.*")
                              ,(build-eksl-physical-translation 
                                (second def))))
                      ; (<something> "sam") -> (<something> <BASE>+"sam")
                      (t
                       `(list ,(string (first def))
                              ,(build-eksl-physical-translation 
                                (second def))))))
                 definitions)
              ;; Add the default translation...
              (list "**;*.*.*" (relative-namestring-with-base-dir translated-base-pathname
                                                                 :wild-inferiors)))))))


(defun build-eksl-physical-translation (translation)
  (etypecase translation 
    ((or symbol string)
     `(relative-namestring-with-base-dir
       translated-base-pathname
       ,(string translation)
       :wild-inferiors) )
    (cons 
     (cond ((eq (first translation) :BINARY-DIRECTORY)
            `(relative-namestring-with-base-dir
              translated-base-pathname 
              ,(string (third translation))
              (bin-directory-name "BIN" ',(second translation))
              :wild-inferiors))
           (t
            `(relative-namestring-with-base-dir
              translated-base-pathname
              ,@translation
              :wild-inferiors))))))
    

;; Obsolete poorly named version
(defun relative-path-namestring (&rest dir-names)
  (format *standard-output* "~&; In ~a; RELATIVE-PATH-NAMESTRING is obsolete. See RELATIVE-NAMESTRING.~%" (load-pathname))
  (apply #'relative-path-directory-list dir-names))

;;; ===========================================================================
;;; Some code to figure out what bin directory to use.
;;; ---------------------------------------------------------------------------
;;; The basic idea here is that we usually want a distinct binary directory for each 
;;; combination of the platform, the vendor and the gui. While it is usually the 
;;; case that some vendors choose a different filename extension for each different
;;; platform some do not (Allegro). In addition there are currently some name collisions 
;;; between vendors (allegro and MCL both use fasl). 
;;; Having the GUI in this mix allows us to differentiate binaries compiled 
;;; for MCL with CLIM 1.1 vs. native toolkit or for unix CLIM 1 vs. CLIM 2.
;;; Another reason for separate binary
;;; directories is that vendors frequently produce new versions that are not 
;;; binary compatible with the old version. Finally, keeping the binaries separate
;;; makes it slightly easier to do some types of maintenence and distribution.
;;; An alternative 
;;; scheme used by other multi-platform Lisp developers is to ignore the standard 
;;; extensions used by the particular implementation and come up with their own set.
;;; We decided against this because 1) some of the tools of the implementation might
;;; depend on knowing the default extension (sad but true) 2) it can break down if 
;;; software from two sources use this scheme in incompatible ways 3) we like not having 
;;; 8 (at least) different types of binaries cluttering up our source directories

(defvar *default-bin-identifiers* '(:platform :vendor :gui))

#+MCL ; ; new incompatible binary format
(when (search "4." (lisp-implementation-version))
   (pushnew :CCL-4 *features*))

(defun bin-directory-name (&optional (prefix "BIN")
				     (identifiers *default-bin-identifiers*))
  "Returns as an uppercase string the name of a directory for binaries in this
implementation."
  ;; The number of different binaries you must have is
  ;; the cross product of the instruction set and the gui.
  ;; 9-15-93 Added platform for Allegro which uses .fasl on all platforms. (Westy)
  ;(assert (intersection identifiers  *default-bin-identifiers*))
  (let ((platform 
         #+ALPHA            "ALPHA"
	 #+MIPS             "MIPS"
	 #+(OR SPARC SUN4)  "SPARC"
	 #-(or MIPS SPARC SUN4 ALPHA) nil
	 )
	(vendor
         #+(and MCL (not CCL-3))       "MCL"
         #+(and MCL CCL-3 (not CCL-4)) "MCL3" ; new incompatible binary format
         #+(and MCL CCL-4)             "MCL4" ; another new incompatible binary format
	 #+(and LUCID DBCS)      "LUCID"
	 #+(and LUCID (not DBCS)) "LUCID-N"
         #+(and ALLEGRO-VERSION>= (not (version>= 4 3))) "ALLEGRO"
	 #+(and ALLEGRO-VERSION>= (version>= 4 3))  "ALLEGRO4.3"
	 #+LISPWORKS3.1        "LISPWORKS3.1"
         #+LISPWORKS3.2        "LISPWORKS"
	 #+GENERA           "GENERA"
         #+Explorer         "EXPLORER"
	 )
        (gui
         #+(and mcl (not clim)) "MAC"
         #+(and genera (not clim)) "DW"
         #+clim-0.9                "CLIM-0-9"
         #+(and clim-1.0 (not clim-1.1)) "CLIM-1-0" ; in case both features exist (Westy)
         #+clim-1.1                "CLIM-1-1"
         #+clim-2                  "CLIM-2"))
    (format nil "~A~@[-~A~]~@[-~A~]~@[-~A~]~{-~A~}"
	    prefix 
	    (and (member :platform identifiers) platform)
	    (and (member :vendor identifiers) vendor)
	    (and (member :gui identifiers) gui)
            ;; Any remaining symbols are used literally.  This second list should be ALL of
            ;; the specially interpreted symbols, above.  SDA 8/4/95
            (set-difference identifiers '(:platform :vendor :gui)))))

(defun bin-directory (&key (base-path (load-pathname))
		      (prefix "BIN")
		      (identifiers *default-bin-identifiers*))
  "Returns the namestring of a pathname relative to `base-path' for binaries
for this implementation."
  (let ((bin-dir (bin-directory-name prefix identifiers)))
    #-Explorer
    (setf bin-dir (string-downcase bin-dir))
    (namestring (make-pathname
		  :host (pathname-host base-path)
		  :directory
		  (append
		    (if base-path
			(pathname-directory base-path)
			'(:relative))
		    (list bin-dir))))))

;;; ===========================================================================
;;; Some load time feature checking

;; Load logical pathnames
#+(and lucid (not logical-pathnames-mk))
(load (make-pathname 
       :defaults (merge-pathnames "logical-pathnames" (load-pathname))
       :type nil))

;;; ---------------------------------------------------------------------------
;;; Paper over some differences between logical-pathname function packages.

;;; This is a macro so that it is setf-able
(defmacro eksl-logical-pathname-translations (host)
  #-lucid `(logical-pathname-translations ,host)
  #+lucid `(lp:logical-pathname-translations ,host))

(defun eksl-translate-logical-pathname (pathname)
  ;; `string-downcase' and related Lucid kludges added by SDA 6/19/95
  #+Lucid    (if (and (stringp pathname)
		      (not (find #\: pathname)))
		 ;; assume it's already been translated
		 pathname
	       (string-downcase (lp:translate-logical-pathname pathname)))
  ;; ditto for allegro by Westy 8/24/95
  #+Allegro
  (if (and (stringp pathname)
	   (not (find #\: pathname)))
      ;; assume it's already been translated
      pathname
    (pathname
     (string-downcase 
      (namestring
       (translate-logical-pathname pathname)))))
  #+Explorer (common-lisp:translate-logical-pathname pathname)
  #-(or Lucid Explorer allegro)
  (translate-logical-pathname pathname))

;;; ---------------------------------------------------------------------------
;;; A simple macro to eliminate some of the drudgery.

(defmacro define-eksl-system (name &rest options)
  "
DEFINE-EKSL-SYSTEM (NAME FILES &REST OPTIONS)

Define a system that can be loaded or compiled using `eksl-load-system' or `eksl-compile-system'.
The options are :BASE-DIR and :BIN-IDENTIFIERS, (see below).
`files' is a list of this form:

files   :== (<filename>*) | (((<filename*>) options))
options :== :BASE-DIR <pathname>, :BIN-IDENTIFIERS (<ident>*)
ident   :== :PLATFORM,  :VENDOR, :GUI
"
  (setf name (canonicalize-eksl-system-name name))
  ;; Allow old syntax using :FILES or new syntax where the files are the first
  ;; expression in the options list.
  (let ((files (if (member :FILES options)
                   (getf options :FILES)
                   (pop options)))
        (base-dir (getf options :BASE-DIR))
	(bin-identifiers (getf options :BIN-IDENTIFIERS)))
    `(create-eksl-system-definition 
      ',name 
      ',files
      ;; Note that `base-dir' is evaluated...
      ,base-dir
      ',bin-identifiers)
    #+OLD
    `(progn
       ;; Tells us that this is the name of a system.
       (setf (get ',name :eksl-system) t)
       (setf (get ',name :eksl-system-files) ',files)
       ;; This one gets evaluated
       (setf (get ',name :eksl-system-base-dir) (or ,base-dir (load-pathname)))
       (setf (get ',name :eksl-system-bin-identifiers) ',bin-identifiers)
       ',name)))

(defun create-eksl-system-definition (name files base-dir bin-identifiers)
  ;; Check for redefining a system...
  (let ((old-pathname (get name :eksl-system-definition-file)))
    (when (and old-pathname
	       (not (equal old-pathname (load-pathname))))
      (cerror "Load ~2@*~a and redefine ~@*~a."
	      "~a system previously defined in ~a, now being redefined in ~a." name old-pathname (load-pathname))))
  (setf (get name :eksl-system) t)
  (setf (get name :eksl-system-definition-file) (load-pathname))
  (setf (get name :eksl-system-files) files)
  ;; This one gets evaluated
  (setf (get name :eksl-system-base-dir) (or base-dir (load-pathname)))
  (setf (get name :eksl-system-bin-identifiers) bin-identifiers)
  (values name))

(defun eksl-system-definition-file (name)
  (get (canonicalize-eksl-system-name name) :eksl-system-definition-file))

(defun canonicalize-eksl-system-name (name)
  (etypecase name
    (keyword name)
    (string (intern (string-upcase name) (find-package "KEYWORD")))
    (symbol (intern (symbol-name name) (find-package "KEYWORD")))))

(defun convert-to-flat-list (elt)
  "Flattens the directory-structure syntax of `define-eksl-system' files."
  (declare (special *dir*))
  (etypecase elt
    (string (list (concatenate 'string *dir* (string-upcase elt))))
    (list   (let ((*dir* (getf (cdr elt) :base-dir)))
	      (declare (special *dir*))
	      (mapcan #'convert-to-flat-list (car elt))))))

(defun map-eksl-system-files (name function)
  "Function is called with the arguments name, base-dir, bin-identifiers."
  (setf name (canonicalize-eksl-system-name name))
  (let ((base-dir   (or (get name :eksl-system-base-dir) (load-pathname)))
        (bin-idents (or (get name :eksl-system-bin-identifiers) *default-bin-identifiers*)))
    (dolist (item (get name :eksl-system-files))
      (if (consp item)
	  (dolist (file (first item))
	    (funcall function file
                     (getf (rest item) :BASE-DIR base-dir)
                     (getf (rest item) :BIN-IDENTIFIERS bin-idents)))
	  (funcall function item base-dir bin-idents)))))

#+NOT-USED?
(defun eksl-system-files (name)
  (let ((files nil))
    (map-eksl-system-files
     name
     #'(lambda (file base bin-idents)
         (declare (ignore bin-idents))
         (push (make-pathname :defaults base
		              :name file)
               files)))
    (values files)))
  
;;; ============================================================================
;;; Getting rid of unwanted keyword options.  This is necessary on the Explorer,
;;; which doesn't always understand :allow-other-keys.

(defun remove-options (keywords option-list)
  (labels ((remove-1 (keyword options)
	     ;; The following removes `keyword' from the options, because certain of our
	     ;; options aren't recognized by the Explorer make-system and neither
	     ;; :allow-other-keys nor remf works.  SDA 7/27/94
	     (let ((sublist (member keyword options)))
	       (if sublist
		   ;; In case it occurs more than once.
		   (remove-1 keyword
			     (subst (cddr sublist) sublist options))
		   options))))
    (labels ((remove-many (keywords option-list)
	       (if (null keywords)
		   option-list
		   (remove-many (cdr keywords)
				(remove-1 (car keywords) option-list)))))
      (remove-many keywords option-list))))

#+test
(defun test-remove-options ()
  (spy (remove-options '(:a :c) '(:a 1 :b 2 :c 3 :d 4 :a 5))))

;;; ---------------------------------------------------------------------------
;;; Generic load and compile system functions.

(defun eksl-load-system (name &rest options &key (files :ALL) compile &allow-other-keys)
  "Loads all files in system, but does not compile any of them."
  (setf name (canonicalize-eksl-system-name name))
  (unless (get name :eksl-system)
    (error "Not the name of a known system: ~s" name))
  (tagbody retry
    (common-lisp:restart-case
      (map-eksl-system-files
       name
       #'(lambda (file base-dir bin-idents)
           (when (or (eq files :ALL)
                     (member file files :test #'string-equal))
             (apply #'compile-load-source-if file :compile compile
	            ;; translation added by SDA 6/19/95
                    :base-path #-Lucid base-dir #+Lucid (eksl-translate-logical-pathname base-dir)
                    :bin-identifiers bin-idents
                    :allow-other-keys t options))))
      ;; If an error occurs give them the option to retry compiling or loading the entire system.
      (nil () :report (lambda (s)
                        (format s "Retry ~:[loading~;compiling~] ~a system." compile name))
           (go retry)))))

(defun eksl-compile-system (name &rest options)
  "Loads all files of system, compiling those that need it."
  (apply #'eksl-load-system name :compile t options))

(defun eksl-recompile-system (name &rest options)
  (apply #'eksl-compile-system name :recompile t options))

(defun compile-load-source-if (name &rest options
			  &key (compile t) (reload nil) (recompile nil) (base-path (load-pathname))
			       (bin-identifiers *default-bin-identifiers*)
			  &allow-other-keys)
  #+(or MCL Lispworks Explorer Allegro)
  (setf base-path (eksl-translate-logical-pathname base-path))
  #+Explorer
  (setf name (string-upcase name))
  (let* ((source-dir (make-pathname :defaults base-path
				    :name name))
	 (source (make-pathname :defaults source-dir
				:type (eksl-file-type-for-sources)))
	 (bin-dir (bin-directory :base-path base-path :identifiers bin-identifiers))
	 (binary (make-pathname :defaults bin-dir
				:name name
				:type (eksl-file-type-for-binaries))))
    
    (apply #'compile-load-if source :binary binary :compile compile :reload reload :recompile recompile :allow-other-keys t options)))

;;; ===========================================================================
;;; COMPILE-LOAD-IF
;;; ---------------------------------------------------------------------------
;;; As an alternative define some file loading utilities similar to what defsystems
;;; use to load files and insure that binaries are up to date. It is the
;;; bare bones of our defsystem.

(defvar *generic-pathname-load-date-table* (make-hash-table :test #'equal)
  "Records the last load time of a pathname.")

(defun generic-pathname (pathname)
  ;; Equal hashtables with pathnames as keys do not work correctly in
  ;; Lucid so we will use strings instead.
  (#-Lucid pathname #+Lucid namestring (truename pathname)))

(defun file-load-date (pathname)
  (values 
   (gethash (generic-pathname pathname) *generic-pathname-load-date-table* 0)))

(defun set-file-load-date (pathname value)
  (setf (gethash (generic-pathname pathname) *generic-pathname-load-date-table*)
        value))

(defsetf file-load-date set-file-load-date)

(defun note-file-load-date (pathname)
  (setf (file-load-date pathname) (get-universal-time)))

(defun eksl-file-type-for-sources ()
  #+MCL #.(pathname-type *.lisp-pathname*)
  #+Allegro "lisp"
  #+genera "LISP"
  #+unix "lisp"
  #+explorer "LISP"
  #-(or mcl genera unix explorer) (error "Not yet implemented."))

(defun eksl-file-type-for-binaries ()
  #+MCL                      #.(pathname-type *.fasl-pathname*)
  #+genera                   si:*default-binary-file-type*
  #+allegro                  #.(if (fboundp 'compile-file-pathname)
				   (pathname-type (compile-file-pathname "foo"))
				 "fasl")
  #+lucid                    (car lcl:*load-binary-pathname-types*)
  #+lispworks                compiler::*fasl-extension-string*
  #+explorer                 "XLD"
  #-(or genera allegro lucid mcl lispworks explorer)
  (error "Not yet implemented."))

(defun file-newer-than-file-p (file1 file2)
  "Compares the write dates of `file1' and `file' and returns T if `file' is newer than
`file2' or if it cannot be determined.  `file1' is usually the source file and `file2'
the object file."
  ;; File write dates default to 0 and 1 so that if they can't be
  ;; determined, the file is recompiled, just to be safe.
  (< (or (file-write-date file2) 0)
     (or (file-write-date file1) 1)))

(defvar *load-verbose-excessive* nil)
(defvar *load-verbose-excessive-to-the-max* nil)

;;; ---------------------------------------------------------------------------

(defun compile-load-if (source &rest options &key binary (compile t) (reload nil) (recompile nil) &allow-other-keys)
  (unless binary 
    (setf binary (make-pathname :defaults source :type (eksl-file-type-for-binaries))))
  (when (and compile
	     (or recompile
		 (not (probe-file binary))
		 (file-newer-than-file-p source binary)))
      (apply #'compile-file source :output-file binary :allow-other-keys t options)
      (setf reload t))
  (tagbody recompile
    (common-lisp:restart-case
	;; Tell them if the binary is missing, and let them recompile.
	(assert (probe-file binary) () "no binary ~a for ~a ~:[(which is also missing)~;; try compiling it!~]"
		binary source (probe-file source))               
      (nil () :report "Recompile source."
	(apply #'compile-file source :output-file binary :allow-other-keys t options)
	(setf reload t)
	(go recompile))))
  (cond ((or reload
	     ;; File write date defaults to most-positive-short-float, so that if it
	    ;; can't be determined, the file is reloaded, just to be safe.
	    ;; Most-positive-fixnum is not big enough.
	    (< (file-load-date binary)
	       (or (file-write-date binary) most-positive-short-float)))
	 (prog1 ; return the results of the load
	     (tagbody
	      reload
	       (common-lisp:restart-case
		   #-Explorer (apply #'load binary :allow-other-keys t options)
		   #+Explorer (apply #'load binary (remove-options
						    '(:compile :recompile :reload :base-path
						      :bin-identifiers :allow-other-keys)
						    options))
	       (nil () :report "Recompile source and then reload."
		 (apply #'compile-file source :output-file binary :allow-other-keys t options)
		 (go reload))))
	   (note-file-load-date binary)))
	(t
	 (when *load-verbose-excessive*
	   (if *load-verbose-excessive-to-the-max*
	       (flet ((dt (ut)
			(if ut
			    (multiple-value-bind (sec min hr day mon year)
				(decode-universal-time ut)
			      (format nil "~d/~d/~d ~2,'0d:~2,'0d:~2,'0d" mon day year hr min sec))
			  "unknown")))
		 (format t "~&; Not loading ~s because it was written at ~a and loaded at ~a~%"
			 binary
			 (dt (file-write-date binary))
			 (dt (file-load-date binary))))
	     (format t "~&; Not loading ~s; up to date~%" binary))))))

;;; `load-if' defaults to not compiling the file, and but will only load it if
;;; the binary has changed since the last load. It actually lets `compile-load-if'
;;; do all the work.
(defun load-if (pathname &rest options &key compile reload recompile &allow-other-keys)
  (if (string-equal (pathname-type pathname) (eksl-file-type-for-binaries))
      ;; If `pathname' is a binary file we get the source and pass that to `compile-load-if'.
      (apply #'compile-load-if (make-pathname :defaults pathname :type (eksl-file-type-for-sources))
	     :binary pathname
	     :compile compile :reload reload :recompile recompile options)
    ;; otherwise we just pass the pathname to `compile-load-if' with the correct options.
    (apply #'compile-load-if pathname :compile compile :reload reload :recompile recompile options)))

;; For debugging
(defun show-pathnames-loaded (&key (stream *standard-output*) by-date)
  (let ((pathnames (make-array (hash-table-count *generic-pathname-load-date-table*)
                               :fill-pointer 0))
	(sort-test (if by-date #'< #'(lambda (p1 p2) (string-lessp (namestring p1) (namestring p2)))))
		       
	(sort-key (if by-date #'second #'first)))
    (maphash #'(lambda (key val) (vector-push (list key val) pathnames))
             *generic-pathname-load-date-table*)
    (sort pathnames sort-test :key sort-key)
    (fresh-line stream)
    (loop for p across pathnames 
	do (multiple-value-bind (sec min hr day mon year)
	       (decode-universal-time (second p))
	     (format stream "~d/~d/~d ~2,'0d:~2,'0d:~2,'0d:: ~a~%" 
		     mon day year hr min sec
		     (first p))))))

;;; ===========================================================================
;;; Misc...

(defmacro bind-implementation-dependent-parameters-for-compile ((&key development (load-verbose (or *load-verbose* development))) 
                                                                &body body)
  `(flet ((do-body ()
            (progn ,@body)))
     (cond (,development
            (#+MCL let #+MCL ((*load-verbose* ,load-verbose)
                              (*save-local-symbols* t)
                              (*fasl-save-local-symbols* t)
                              (*record-source-file* t)
                              (*save-fred-window-positions* t)
                              (*save-definitions* t)
                              (*save-doc-strings* t))
             #-MCL progn
             (do-body)))
           (t
            (#+MCL let #+MCL ((*load-verbose* ,load-verbose)
                              (*save-local-symbols* nil)
                              (*fasl-save-local-symbols* nil)
                              (*record-source-file* nil)
                              (*save-fred-window-positions* nil)
                              (*save-definitions* nil)
                              (*save-doc-strings* nil))
             #-MCL progn
             (do-body))))))

;;; ===========================================================================
;;; Patches
;;; ---------------------------------------------------------------------------
;;; Each system has a file called patches.  If there are multiple patches, as
;;; with the Explorer patch system, they are appended to this single file, 
;;; which has the advantage that you can't forget to load a patch or load them in
;;; the wrong order and the disadvantage that you can't load patches selectively.
;;; The load file for every system that wants to allow post-hoc patch loading
;;; via `eksl-load-all-patches' will have in it a line like this:
;;;
;;; (push "FOO:BINARIES;PATCHES" *eksl-all-patch-files*)
;;;
;;; If a user, say with a saved image, wants to load patches without loading
;;; the system from scratch, they call the function (eksl-load-all-patches)

(defvar *eksl-all-patch-files* nil
  "A list of filenames loaded, in the correct order, by `eksl-load-all-patches'.")

(defun eksl-load-all-patches ()
  "Loads all files on *eksl-all-patch-files*.  Usually, the load-foo file for the FOO
system will push a logical binary pathname onto this list."
  ;; Since files are pushed on, they are in reverse order, so we load them thus:
  (flet ((load-lp (logical-pathname)
            ;; This could be improved by checking the source and possibly
            ;; compiling the file
            (load (eksl-translate-logical-pathname logical-pathname))))
    (mapc #'load-lp (reverse *eksl-all-patch-files*))))

;;; ===========================================================================

(defun eksl-copyright-notice (software-name &optional mailing-address)
  "Prints the copyright notice for `software-name' to *standard-output*.  If a
mailing address is supplied, it prints that as well, in a sentence like ``for
more info, write to ~s.''"
  (format t "~2%Copyright notice for ~a~%" software-name)
  (format t "
Copyright (c) 1996 University of Massachusetts
Department of Computer Science
Experimental Knowledge Systems Laboratory
Professor Paul Cohen, Director.
All rights reserved.

Permission to use, copy, modify and distribute this software and its
documentation is hereby granted without fee for non-commercial uses
only (not for resale), provided that the above copyright notice of EKSL,
this paragraph and the one following appear in all copies and in supporting
documentation.

EKSL makes no representation about the suitability of this software for any
purposes.  It is provided \"AS IS\", without express or implied warranties
including (but not limited to) all implied warranties of merchantability
and fitness for a particular purpose, and notwithstanding any other provision
contained herein.  In no event shall EKSL be liable for any special, indirect
or consequential damages whatsoever resulting from loss of use, data or profits,
whether in an action of contract, negligence or other tortuous action, arising
out of or in connection with the use or performance of this software, even if
EKSL is advised of the possibility of such damages.

")
  (when mailing-address
    (format t "For more information, write to ~a.~2%" mailing-address)))

;;; ---------------------------------------------------------------------------
;;; Implementation specific work-arounds for file problems


;; Fix a problem with :UNSPECIFIC devices and calling `pathname-device' with nil
#+(and Lispworks Defsystem)
(lispworks:defadvice (pathname-device fix :around) (pathname)
  (if (null pathname) 
      nil
      (let ((device (call-next-advice pathname)))
	(if (eq device :UNSPECIFIC) nil device))))

;;; ---------------------------------------------------------------------------

#+MCL
(let ((*warn-if-redefine* nil))
;; When using MCL with a CAP server timing problems (I think) often cause
;; the compiler to hang when attempting to delete a file that is locked.

(defvar *auto-unlock-before-delete-file* nil)

(defun maybe-unlock-before-delete (file)
  (when (and *auto-unlock-before-delete-file*
             (file-locked-p file))
    (unlock-file file)))

(ccl:advise delete-file 
  (progn 
    (maybe-unlock-before-delete (first ccl:arglist))
    (:DO-IT))
  :when :around
  :name maybe-unlock-before-delete)

(ccl:advise compile-file
   (let ((*auto-unlock-before-delete-file* t))
     (:DO-IT))
   :when :around
   :name set-unlock-before-delete)

)

;;; ***************************************************************************
;;; EOF
