#|  

Caddy class for MacCLASP. You can just fill these suckers up
with stuff and bring out the caddy to look at them again.

The caddy is basically a display class, you'll have to worry about
keeping things on hand yourself. See the graph caddy for details.

8-Aug-94 ms   Started.
18-Oct-94 westy  changed the label to "palette" because it's happier

|#

(in-package "CLASP-INTERFACE")

(defclass caddy-sequence-item  (sequence-dialog-item)
  ()
  (:DOCUMENTATION "A sequence dialog item that only works in the context
of a clasp caddy. See its click-handler for details."))

(defmethod view-click-event-handler :after ((csi caddy-sequence-item)
                                                (pt-where fixnum))
  "updates the name portion of the clasp-caddy. That's why this dialog-item
won't work out of context."
  (let ((clicked-item (ignore-errors (elt (table-sequence csi) 
                                          (cell-to-index csi 0 (point-v (point-to-cell csi pt-where)))))))
    (if clicked-item (ribbon-display (name-and-dataset clicked-item))))
  (let* ((selected-cells (selected-cells csi))
         (nselected (length selected-cells))
         (pt-cell-clicked (point-to-cell csi pt-where))
         (f-clicked-on-selection (member pt-cell-clicked 
                                         (selected-cells csi)))
         (item-clicked (if f-clicked-on-selection
                         (elt (table-sequence csi) (point-v pt-cell-clicked))))
         (window (view-container csi))
         (display (if window (view-named 'name-display window))))
    (if display
      (set-dialog-item-text display (format nil "~D/~D item~:P selected." nselected (length (table-sequence csi)))))
    (if (funcall *open-click-p*)
      (if (f-can-open-clasp-object item-clicked)
        (open-clasp-object item-clicked)))))

(defclass clasp-caddy (window)
  ((caddy-for
    :initform nil
    :initarg :caddy-for
    :type SYMBOL
    :accessor clasp-caddy-for
    :documentation "the symbol for which this is a caddy, like '*clasp-graphs*"))
  ;; in an instantiable class, there will be a list of "things" here
  (:DOCUMENTATION "This is a display class for getting at old graphs
which are stored in a caddy.")
  (:DEFAULT-INITARGS
    :window-title "Generic Caddy"
    :window-type :document-with-grow
    :view-size #@(200 100)
    :VIEW-POSITION (best-spot 200 100)
    :view-subviews
    (list (make-instance 'static-text-dialog-item
            :view-nick-name 'name-display
            :dialog-item-text "0 items selected.")
          (make-instance 'caddy-sequence-item
            :view-nick-name 'sequence-view
            :cell-size #@(190 11)
            :view-size #@(190 90)
            :selection-type :DISJOINT
            :table-hscrollp NIL
            :table-vscrollp T
            :table-sequence NIL
            :view-font *caddy-font*)
          (make-instance 'button-dialog-item
            :view-nick-name 'delete-button
            :dialog-item-text "Delete"
            :border-p NIL
            :dialog-item-action #'(lambda (item)
                                    (delete-items (view-container item))))
          (make-instance 'button-dialog-item
            :view-nick-name 'update-button
            :dialog-item-text "Update"
            :border-p NIL
            :dialog-item-action #'(lambda (item)
                                    (fill-caddy (view-container item))))
          )))

(defmethod initialize-instance :after ((clasp-caddy clasp-caddy) &key)
  "fill in the sequence with the supplied caddy-for."
  (set-cell-size (view-named 'sequence-view clasp-caddy)
                                (point-h (cell-size (view-named 'sequence-view clasp-caddy))) 
                                (font-height *caddy-font*))
  (fill-caddy clasp-caddy))

(defmethod view-activate-event-handler :after ((clasp-caddy clasp-caddy))
  "check to see if we need an update"
  (let ((was (table-sequence (view-named 'sequence-view clasp-caddy)))
        (is (eval (clasp-caddy-for clasp-caddy))))
    (if (not (equal is was))
      (fill-caddy clasp-caddy)
      )))

(defmethod delete-items ((clasp-caddy clasp-caddy))
  "delete the selected caddy items"
  (let* ((table (view-named 'sequence-view clasp-caddy))
         (l-items (table-sequence table))
         (l-cells-selected (selected-cells table))
         (selected-items (map 'list #'(lambda (cell) (elt l-items (point-v cell)))
                              l-cells-selected))
         (caddy-for (clasp-caddy-for clasp-caddy)))
    (loop for x in selected-items do
          (set caddy-for (remove x (eval caddy-for))))
    (deselect-cells table)
    (set-dialog-item-text (view-named 'name-display clasp-caddy) "0 items selected.")
    (fill-caddy clasp-caddy)))

(defmethod caddy-contents ((clasp-caddy clasp-caddy))
  (table-sequence (view-named 'sequence-view clasp-caddy)))

(defmethod l-caddy-selected-items ((clasp-caddy clasp-caddy))
  "return the selected items."
  (let* ((table (view-named 'sequence-view clasp-caddy))
         (l-items (table-sequence table))
         (l-cells-selected (selected-cells table))
         (selected-items (map 'list #'(lambda (cell) (elt l-items (point-v cell)))
                              l-cells-selected))
         )
    selected-items))
                              

(defmethod fill-caddy ((clasp-caddy clasp-caddy))
  (let* ((table (view-named 'sequence-view clasp-caddy))
         (caddy-symbol (clasp-caddy-for clasp-caddy))
         (the-stuff (eval `,caddy-symbol))
         )
    (setf (table-sequence table) the-stuff)))

(defmethod window-size-parts :after ((clasp-caddy clasp-caddy))
  (let* ((table (view-named 'sequence-view clasp-caddy))
         (grow-icon-dim nil)
         (int-grow-icon-top
          (multiple-value-bind (pt-top-left pt-bottom-right)
                               (ccl::grow-icon-corners clasp-caddy)
            (setf grow-icon-dim
                  (- (point-v pt-bottom-right) (point-v pt-top-left)))
            (point-v pt-top-left)))
         (name (view-named 'name-display clasp-caddy))
         (ub (view-named 'update-button clasp-caddy))
         (db (view-named 'delete-button clasp-caddy))
         )
    (when (and table name ub db) ;; need this in MCL3 - Westy
      (set-view-position db 1 (1+ int-grow-icon-top))
      (set-view-position ub (- (point-h (view-size clasp-caddy))
                               (point-h (view-size ub))
                               grow-icon-dim
                               2)
                         (1+ int-grow-icon-top))
      (set-view-position name 0 0)
      (set-view-size name (point-h (view-size clasp-caddy)) 15)
      (set-view-position table 0 (1+ (point-v (view-size name))))
      (set-view-size table (point-h (view-size clasp-caddy)) 
                     (- (point-v (view-size clasp-caddy))
                        (point-v (view-size name))
                        grow-icon-dim))
      )))
  
(defun open-graph-caddy ()
  (make-instance 'clasp-caddy :window-title "Graph Palette"
                 :caddy-for '*clasp-graphs*))
(defun open-table-caddy ()
  (make-instance 'clasp-caddy :window-title "Table Palette"
                 :caddy-for '*clasp-tables*))
(defun open-dataset-caddy ()
  (make-instance 'clasp-caddy :window-title "Dataset Palette"
                 :caddy-for 'clasp::*clasp-datasets*))