;;;-*- Mode: Lisp; Package: CLASP-INTERFACE -*-

(in-package :CLASP-INTERFACE)

;;; -----------------------------------------------------------------------------------------------
;;; STARTUP CODE

;;; sets up the environment for saving as an application

#| Out with the old...

;; changed these to use pushnew in case the file gets loaded twice
(pushnew 'load-user-preferences-file *lisp-startup-functions*)
#-DEMO
(pushnew 'do-license-agreement-and-about-box *lisp-startup-functions*)
#+DEMO
(pushnew 'display-about-box *lisp-startup-functions*)
(pushnew 'find-np-fonts *lisp-startup-functions*)
(pushnew 'open-ribbon *lisp-startup-functions*)
(pushnew 'make-user-package *lisp-startup-functions*)
(pushnew 'select-current-or-new-clasp-window *lisp-startup-functions*)
#+DEMO
(pushnew 'initialize-clasp-demo *lisp-startup-functions*)
#-DEMO
(pushnew 'load-clasp-patches *lisp-startup-functions*)

|#

;; In with the new...
(defun do-clasp-initializations ()
  (load-user-preferences-file)
  #-DEMO
  (do-license-agreement-and-about-box)
  #+DEMO
  (display-about-box)
  (find-np-fonts)
  (open-ribbon)
  (make-user-package)
  (select-current-or-new-clasp-window)
  #+DEMO
  (initialize-clasp-demo)
  #-DEMO
  (load-clasp-patches))

(pushnew 'do-clasp-initializations *lisp-startup-functions*)

#+TEST-NO-WINDOWS
(setf *lisp-startup-functions* 
      '(LOAD-CLASP-PATCHES
        MAKE-USER-PACKAGE
        LOAD-USER-PREFERENCES-FILE))

#+TEST-NO-ABOUT-BOX
(setf *lisp-startup-functions* 
      '(LOAD-CLASP-PATCHES
         select-current-or-new-clasp-window
        MAKE-USER-PACKAGE
        open-ribbon
        find-np-fonts
        LOAD-USER-PREFERENCES-FILE
        ))

#+TEST
(setf *lisp-startup-functions* 
      '(LOAD-CLASP-PATCHES
          select-current-or-new-clasp-window
         MAKE-USER-PACKAGE
         open-ribbon
         find-np-fonts
         do-license-agreement-and-about-box
         LOAD-USER-PREFERENCES-FILE
         ))

(defun make-user-package ()
  #+OLD
  (defpackage :clasp-user
    (:use clasp clasp-interface cl common-lisp-user ccl))
  (setf *package* (find-package :clasp-user)))

;;; --------------------------------------------------------------------------------------------------------------
;;; CLASP APPLICATION CLASSES

(defclass clasp-development-system (ccl::lisp-development-system)
  ())

(defmethod application-name ((app clasp-development-system)) "CLASP Development")

(defmethod application-file-creator ((app clasp-development-system))
  *clasp-application-creator*)

(defmethod application-sizes ((app clasp-development-system))
  '(3670016 10485760))

(defmethod application-resource-file ((app clasp-development-system))
  "CLASP:interface-mcl:clasp-resources")

(defmethod ccl::application-toplevel ((app clasp-development-system))
  'toplevel-function)

(defmethod ccl::application-custom-modules ((app clasp-development-system))
  nil)

(defmethod ccl::application-optional-modules ((app clasp-development-system))
  nil)

(defmethod ccl::application-error-mode ((app clasp-development-system))
  :listener)

(defmethod toplevel-function ((a clasp-development-system) init-file)
  (declare (ignore init-file))
  ;; Condensed from startup-ccl...
  (unwind-protect
    (let ((file-list (finder-parameters)))
      (with-simple-restart (abort "Abort startup.")
        (when cl-user::*clasp-development-version*
          (when *arglist-on-space*
            (when (ccl::open-doc-string-file nil nil)
              (ccl::maybe-load-help-map))))
        (ccl::make-mcl-listener-process)
        (cond ((eq (car file-list) :open)
               (dolist (f (cdr file-list))
                 (open-application-document *application* f t)))
              ((eq (car file-list) :print)
               (dolist (f (cdr file-list))
                 ;; For now only print text files...
                 (when (eq :text (mac-file-type f))
                   (print-application-document *application* f t)))))
        #+OLD
        (u::awhen (clasp-window-front)
          (update-dataset-view it))
        (format t "~&Welcome to CLASP ~A!~%" clasp::*clasp-version*)))
    (ccl::startup-finished)))

#+OLD
(defmethod toplevel-function ((a clasp-development-system) init-file)
  (call-next-method))

#+LATER
(defmethod toplevel-function ((app clasp-development-system) init-file)
  (declare (ignore init-file))
  (let* ((file-list (finder-parameters))
         (spec (cdr file-list)))
    (case (car file-list)
      (:open
       (dolist (f spec)
         (ccl:open-application-document app f t)))
      (:print
       (dolist (f spec)
         (ccl:print-application-document app f t))))
    (ccl::startup-finished)
    (ccl::make-mcl-listener-process)
    #+OLD
    (catch :toplevel
      (loop (event-dispatch)))))

#+DEFAULT
; specialize this for your application
(defmethod open-application ((self application) startup)
  (declare (ignore startup))
  nil)

#+DEFAULT
; specialize this for your application
(defmethod print-application-document ((a application) path &optional startup)
  (declare (ignore path startup)))

; specialize this for your application
(defmethod open-application-document ((a clasp-development-system) path &optional startup)
  (declare (ignore startup))
  (case (mac-file-type path)
    (:data
     (load-dataset path)
     (u::awhen (clasp-window-front)
       (update-dataset-view it)))
    (:text
     (cond ((file-is-lisp-code path)
            (load path)
            (ed path))
           (t
            (import-dataset path)
            (u::awhen (clasp-window-front)
              (update-dataset-view it)))))
    (:fasl
     (load path))))

(defun file-is-lisp-code (path)
  (with-open-file (stream path :direction :input)
    (when (search (read-line stream) "-*- Mode:Common-Lisp;")
      (return-from file-is-lisp-code t))
    (loop for line = (read-line stream nil :EOF)
          until (eq line :EOF) do
          (when (and (plusp (length line)) (char= (aref line 0) #\())
            (return-from file-is-lisp-code t)))))
          
     
     

#+LATER
(defmethod application-about-view ((app clasp-development-system))
  #+DEFAULT
  (make-dialog-item 'static-text-dialog-item
                    #@(15 20)
                    nil
                    (application-name app)))

#+LATER
(defmethod application-about-dialog ((app clasp-development-system))
  #+DEFAULT
  (let* ((about-view (application-about-view app))
         (view-size (view-size about-view))
         (next-y (if view-size
                   (+ (point-v (view-position about-view))
                      (point-v view-size)
                      15)
                   50))
         (dialog-width  (if view-size
                          (max (point-h view-size) 350)
                          350))
         (dialog-height (if view-size
                          (max (+ (point-v view-size) 100) 150)
                          150)))
    (make-instance 'dialog
      :auto-position :centermainscreen
      :view-size (make-point dialog-width dialog-height)
      :window-type :double-edge-box
      :window-show nil
      :view-subviews
      (list
       about-view
       (make-dialog-item 'static-text-dialog-item
                         (make-point 15 next-y)
                         #@(300 60)
                         "Written in Macintosh Common Lisp."
                         nil
                         :view-font '("Geneva" 9))
       (make-dialog-item 'default-button-dialog-item
                         (make-point (ash (- dialog-width 70) -1)
                                     (incf next-y 70))
                         #@(70 18)
                         "OK"
                         #'(lambda (item)
                             (declare (ignore item))
                             (return-from-modal-dialog t)))))))


#| Other methods you can optionally define:

(defmethod application-menubar ((app clasp-development-system))
  ...)

(defmethod application-suspend-event-handler ((app clasp-development-system))
  ...)

(defmethod application-resume-event-handler ((app clasp-development-system))
  ...)

(defmethod application-eval-enqueue ((app clasp-development-system) quote (form))
  ...)
|#
