#|
======================
Variable-Dock=========
======================

The Variable Docks for CLASP. The variable docks are
a set of views within a window which holds "selected"
variables for forcing into CLASP functions as arguments.

The window is the CLASP-DOCK.

There are three views : x-var view, y-var view and a
general argument view.

in progress

25-Jul-94 ms   Created.

|#

(in-package "CLASP-INTERFACE")

#+OLD
(require "OBJECT-DROPPER")

(defvar *variable-dock-font* '("Geneva" 9))
(defvar *variable-dock-font-ex* '(:bold "Geneva" 9))

;;;
;;; variable dock implementation
;;;


(defclass variable-dock (drag-aware-mixin sequence-dialog-item)
  ()
  (:documentation "a general purpose sequence dialog item
for holding onto variable arguments to be launched into
statistic-dialog boxes.")
  (:DEFAULT-INITARGS
    :CELL-SIZE #@(105 11)
    :SELECTION-TYPE :DISJOINT
    :TABLE-HSCROLLP NIL
    :TABLE-VSCROLLP T
    :TABLE-SEQUENCE NIL
    :view-font *variable-dock-font*
    :table-print-function
    #'(lambda (item stream)
        (declare (ignore stream))
        (let ((font-spec (if (typep item 'CLASP::variable)
                           *variable-dock-font*
                           *variable-dock-font-ex*))
              (string (if (typep item 'clasp::variable)
                        (format nil "~A [~A]" (clasp::name item) 
                                (clasp::name (clasp::variable-dataset item)))
                        (format nil "~A" (clasp::name item)))))
          (multiple-value-bind (ff ms) (font-codes font-spec)
            (with-font-codes ff ms
              (with-pstrs ((p-str string))
                (#_DrawString p-str))))))))

(defmethod docked-items ((dock variable-dock) &key (select-type :all))
  "Returns a list of the docked items"
  (let* ((l-items (table-sequence dock))
         (l-cell-selected (selected-cells dock))
         (docked-items (map 'list #'(lambda (cell) (elt l-items (point-v cell)))
                            l-cell-selected)))
    (cond
     ((equal select-type :all) l-items)
     ((equal select-type :selected-only) docked-items)
     ((equal select-type :select-or-all) (if docked-items 
                                           docked-items l-items))
     (t (warn "unsupported keyword to docked-items")))))

(defmethod f-can-receive-drop ((view variable-dock)
                                  (object t)
                                  (pt-global integer))
  (declare (ignore object))
  t)

(defmethod invert-drop-highlight ((view variable-dock)
                                      (object t)
                                      (pt-global integer))
  (invert-inside-gray-frame view))

(defmethod receive-drop ((view variable-dock)
                           (object t)
                           (pt-global integer))
  "Object is any item. the variable dock will weed out anything
that's not a clasp::varaible or a clasp::dataset."
  (if (or (typep object 'clasp::variable)
          (typep object 'clasp::dataset))
    (set-table-sequence view (append (table-sequence view) (list object))))
  (invalidate-view view t))

(defmethod receive-drop ((view variable-dock) 
                           (object list) 
                           (pt-global integer))
  "Object is a list of dropppable items."
  (loop for thing in (reverse object)
        do (put-in-dock view thing)))

(defmethod put-in-dock ((dock variable-dock) (item t))
  "puts an item in the dock."
  (if (or (typep item 'clasp::variable)
          (typep item 'clasp::dataset))
    (set-table-sequence dock (append (table-sequence dock) (list item)))))

#+OLD
(defmethod view-click-event-handler ((view variable-dock)
                                         (pt-where fixnum))
  "Initiates the dragging of items on a non-modified click."
  (let* ((pt-cell-clicked (point-to-cell view pt-where))
         (f-clicked-on-selection (member pt-cell-clicked (selected-cells view))))
    (if (and f-clicked-on-selection
             (not (option-key-p))
             (not (control-key-p))
             (not (command-key-p))
             (not (shift-key-p)))
      (track-mouse-for-dropping view
                                (docked-items view :select-type :selected-only)
                                (local-to-global (view-container view) pt-where))
      (call-next-method))))

(defmethod view-click-event-handler :after ((view variable-dock)
                                                (pt-where fixnum))
  "opens the item on *open-click-p*"
  (when (and (funcall *open-click-p*)
             (member (point-to-cell view pt-where) (selected-cells view)))
    (let* ((l-selected-items (docked-items view :select-type :selected-only))
           (l-openable-items (remove-if-not #'(lambda (item)
                                                (f-can-open-clasp-object item))
                                            l-selected-items)))
      (cond (l-openable-items
             (eval-enqueue `(map nil #'open-clasp-object ',l-openable-items)))
            (t NIL)))) ;; do nothing
  "displays the long form of the item clicked."
  (let ((clicked-item (ignore-errors (elt (table-sequence view) 
                                          (cell-to-index view 0 (point-v (point-to-cell view pt-where)))))))
    (if clicked-item (ribbon-display (name-and-dataset clicked-item))))
  "Takes care of dock launches to the currently active function dialog."
  (let* ((selected-window (front-window))
         (var-pum (if (funcall *x-key-p*) (view-named 'x-var-item selected-window)
                      (if (funcall *y-key-p*) (view-named 'y-var-item selected-window)
                          nil))))
    (if var-pum
      (progn
        (if (and (not (funcall *cache-key-p*)) (typep var-pum 'variable-pop-up-menu))
          (loop for x in (menu-items var-pum) do
                (remove-menu-items var-pum X)))
        (receive-drop var-pum (docked-items view :select-type :selected-only) pt-where)
        ))))

;;; dock access functions
         
(defun dock-item (dock-tag item &key (auto-select NIL))
  "sends an item to the clasp dock named dock-tag."
  (let* ((clasp-dock (find-clasp-dock :force-access t))
         (the-dock (view-named dock-tag clasp-dock)))
    (if the-dock
      (if (consp item)
        (loop for x in item do
              (put-in-dock the-dock x))
        (put-in-dock the-dock item)))
    (if (and auto-select the-dock)
      (progn (deselect-cells the-dock)
             (dotimes (i (length item))
               (let ((cell (index-to-cell the-dock (- (length (table-sequence the-dock)) i 1))))
                 (cell-select the-dock (point-h cell) (point-v cell))))))
    ))

(defun clear-dock (view dock-tag)
  "clears out the dock indicated by the dock tag."
  (let* ((dock (view-named dock-tag view))
         (selected-items (if dock (docked-items dock :select-type :selected-only))))
    (if dock
      (if (null selected-items) 
        (set-table-sequence dock '())
        (let ((rt (docked-items dock :select-type :all)))
          (loop for x in selected-items do
                (setq rt (remove x rt)))
          (set-table-sequence dock rt)
          (deselect-cells dock))))))
                          
;;; this function call is for those who don't know where to find the
;;; right views & stuff
(defun items-in-dock (dock-tag &key (select-type :all))
  "returns a list of items found in the dock named dock-tag."
  (let* ((main-dock (find-clasp-dock))
         (the-dock  (if main-dock (view-named dock-tag main-dock))))
    (if the-dock (docked-items the-dock :select-type select-type))))

(defconstant +clasp-dock-name+ "CLASP Dock")

(defun find-clasp-dock (&key force-access)
  "Returns the clasp-dock window. If force-access is non-nil, it creates
a clasp dock and returns it."
  (let ((dock (find-window +clasp-dock-name+ 'clasp-dock)))
    (if dock
      dock
      (if force-access
        (make-instance 'clasp-dock)
        nil))))

(defun dock-cache (view dock-tag)
  "Dock all of the cache items in the dock named dock-tag."
  (let* ((dock (view-named dock-tag view))
         (cache (view-named 'gp-dock view))
         (items-to-dock (if cache (docked-items cache :select-type :select-or-all)))
         )
    (if dock
      (set-table-sequence dock (append (table-sequence dock)
                                       items-to-dock)))))

(defun cache-dock (view dock-tag)
  "Cache all the items in the dock named dock-tag."
  (let* ((dock (view-named dock-tag view))
         (cache (view-named 'gp-dock view))
         (items-to-cache (if dock (docked-items dock :select-type :select-or-all))))
    (if dock
      (set-table-sequence cache (append (table-sequence cache)
                                        items-to-cache)))))

;;;
;;; CLASP dock implementation
;;;

(defparameter *dock-button-font* '("Geneva" 9 :bold))

(defclass clasp-dock (WINDOID)
  ()
  (:documentation "The window which harbors the x and y variable
docks and the general-purpose-dock.")
  (:DEFAULT-INITARGS
    :WINDOW-TITLE +clasp-dock-name+
    :WINDOW-TYPE :DOCUMENT-WITH-GROW
    :VIEW-SIZE #@(320 200)
    :view-position (make-point (- *screen-width* 320 10) (- *screen-height* 210))
    :view-subviews
    (list (make-instance 'variable-dock :view-nick-name 'x-var-dock)
          (make-instance 'variable-dock :view-nick-name 'y-var-dock)
          (make-instance 'variable-dock :view-nick-name 'gp-dock)
          (make-instance 'clickable-text-item 
                          :view-nick-name 'x-cache
                         :dialog-item-text "X>>"
                         :item-action 'cache-dock
                         :item-action-arg 'x-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'clickable-text-item :view-nick-name 'y-cache
                         :dialog-item-text "Y>>"
                         :item-action 'cache-dock
                         :item-action-arg 'y-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'clickable-text-item :view-nick-name 'cache-x
                         :dialog-item-text "X<<"
                         :item-action 'dock-cache
                         :item-action-arg 'x-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'clickable-text-item :view-nick-name 'cache-y
                         :dialog-item-text "Y<<"
                         :item-action 'dock-cache
                         :item-action-arg 'y-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'static-text-dialog-item :view-nick-name 'x-tag
                         :dialog-item-text "X(s)")
          (make-instance 'static-text-dialog-item :view-nick-name 'y-tag
                         :dialog-item-text "Y(s)")
          (make-instance 'static-text-dialog-item :view-nick-name 'gp-tag
                         :dialog-item-text "Cache")
          (make-instance 'clickable-text-item :view-nick-name 'x-clear
                         :dialog-item-text "Clear"
                         :item-action 'clear-dock
                         :item-action-arg 'x-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'clickable-text-item :view-nick-name 'y-clear
                         :dialog-item-text "Clear"
                         :item-action 'clear-dock
                         :item-action-arg 'y-var-dock
                         :view-font *dock-button-font*)
          (make-instance 'clickable-text-item :view-nick-name 'gp-clear
                         :dialog-item-text "Clear"
                         :item-action 'clear-dock
                         :item-action-arg 'gp-dock
                         :view-font *dock-button-font*)
          )))

(defmethod window-size-parts :after ((clasp-dock clasp-dock))
  "Sets the positions and sizes of the variable docks in the
clasp dock according to the new size of the window."
  (let* ((x-dock (view-named 'x-var-dock clasp-dock))
         (y-dock (view-named 'y-var-dock clasp-dock))
         (gp-dock (view-named 'gp-dock clasp-dock))
         (x-tag (view-named 'x-tag clasp-dock))
         (y-tag (view-named 'y-tag clasp-dock))
         (gp-tag (view-named 'gp-tag clasp-dock))
         (x-clear (view-named 'x-clear clasp-dock))
         (y-clear (view-named 'y-clear clasp-dock))
         (gp-clear (view-named 'gp-clear clasp-dock))
         (x-cache (view-named 'x-cache clasp-dock))
         (y-cache (view-named 'y-cache clasp-dock))
         (cache-x (view-named 'cache-x clasp-dock))
         (cache-y (view-named 'cache-y clasp-dock))
         (width (point-h (view-size clasp-dock)))
         (h-view-size (round width 3))
         (grow-icon-height nil)  ;; set up in the next messy binding (hack)
         (int-grow-icon-top
          (multiple-value-bind (pt-top-left pt-bottom-right)
                               (ccl::grow-icon-corners clasp-dock)    ;FIX: undocumented
            (setf grow-icon-height 
             (- (point-v pt-bottom-right) (point-v pt-top-left)))
            (point-v pt-top-left)))
         (v-view-size (- int-grow-icon-top (+ 3 grow-icon-height)))
         )
    ;; For some reason in MCL 3.0 this method is getting called before 
    ;; the views have been added so we have to add this check
    (when (and x-dock y-dock gp-dock x-tag y-tag  gp-tag x-clear y-clear 
               gp-clear x-cache y-cache cache-x cache-y)
      (set-view-position x-tag 0 0)
      (set-view-position x-dock 0 (+ 3 grow-icon-height))
      (set-view-size x-dock (1- h-view-size) v-view-size)
      (set-view-position x-clear 0 (1+ int-grow-icon-top))
      (set-view-position x-cache (+ (point-h (view-size x-clear)) 3) (1+ int-grow-icon-top))
      
      (set-view-position y-tag (1+ h-view-size) 0)
      (set-view-position y-dock h-view-size (+ 3 grow-icon-height))
      (set-view-size y-dock (1- h-view-size) v-view-size)
      (set-view-position y-clear (1+ h-view-size) (1+ int-grow-icon-top))
      (set-view-position y-cache (+ h-view-size
                                    (point-h (view-size y-clear)) 
                                    3) 
                         (1+ int-grow-icon-top))
      
      (set-view-position gp-tag (1+ (* 2 h-view-size)) 0)
      (set-view-position gp-dock (* 2 h-view-size) (+ 3 grow-icon-height))
      (set-view-size gp-dock h-view-size v-view-size)
      (set-view-position gp-clear (* 2 h-view-size) (1+ int-grow-icon-top))
      (set-view-position cache-x (+ (* 2 h-view-size)
                                    (point-h (view-size gp-clear)) 
                                    3) 
                         (1+ int-grow-icon-top))
      (set-view-position cache-y (+ (* 2 h-view-size)
                                    (point-h (view-size gp-clear)) 
                                    6 
                                  (point-h (view-size cache-x)))
                         (1+ int-grow-icon-top))
      )))
  
;; 5-Aug-94 ms
(defmethod remove-renegades ((clasp-dock clasp-dock) (renegades list))
  (let* ((x-dock (view-named 'x-var-dock clasp-dock))
         (y-dock (view-named 'y-var-dock clasp-dock))
         (gp-dock (view-named 'gp-dock clasp-dock))
         (xs (table-sequence x-dock))
         (ys (table-sequence y-dock))
         (gps (table-sequence gp-dock)))
    (loop for r in renegades do
         (if (typep r 'clasp::variable)
           (progn
             (if (member r xs)
               (setq xs (remove r xs)))
             (if (member r ys)
               (setq ys (remove r ys)))
             (if (member r gps)
               (setq gps (remove r gps)))))
         (if (typep r 'clasp::dataset)
           (progn
             (loop for v in xs do
                  (if (null (clasp::variable-dataset v))
                    (setq xs (remove v xs))))
             (loop for v in ys do
                  (if (null (clasp::variable-dataset v))
                    (setq ys (remove v ys))))
             (loop for v in gps do
                  (if (null (clasp::variable-dataset v))
                    (setq gps (remove v gps)))))))
    (set-table-sequence x-dock xs)
    (set-table-sequence y-dock ys)
    (set-table-sequence gp-dock gps)))