;;;
;;; cut-n-paste.lisp
;;;

#|

Cut and Paste functionality for Mac CLASP.
Allows user to cut results from the current window (whatever that is)
and then switch applications and paste them in.

Uses Matt Cornell's copy-str-or-h-pict function in event-logging.lisp

7-Sep-94 ms  Created

|#

(in-package "CLASP-INTERFACE")

(defun press-to-clipboard (items)
  (let* ((result "")
         buffer bogus)
    (with-cursor *watch-cursor* 
      (if items
        (progn
          (loop for citem in items do
                (if (and (typep citem 'graph) (> (length items) 1))
                  (progn 
                    (gui-warn "Sorry. Can't cut multiple graphics or mix graphics and text.")
                    (setf bogus t)))
                (if (not bogus)
                  (progn
                    (setf buffer (make-log-item citem))
                    (if (typep citem 'graph)
                      (setf result buffer)
                      (setf result (concatenate 'string result (format nil "~%") buffer))))))
          (copy-str-or-h-pict result))))))

(defmethod cut-selection ((window t) &key)
  nil)

(defmethod cut-selection ((window clasp-window) &key)
  (let ((selection (reverse (l-history-selected-cw window))))
    (press-to-clipboard selection)))

(defmethod cut-selection ((window clasp-notebook) &key)
  (let ((selection (reverse (l-caddy-selected-items window))))
    (press-to-clipboard selection)))

;; bypasses the press-to-clipboard hoop
(defmethod cut-selection ((window plotter::plot-window) &key)
  (with-slots (plotter::picture) window
    (copy-str-or-h-pict plotter::picture)))

(defmethod cut-selection ((window table-view-window) &key)
  (press-to-clipboard (list (table-view-table (view-named 'table-array-item window)))))

#+NEW-CLASP-TABLES
(defmethod cut-selection ((window composite-table-window) &key)
  (press-to-clipboard (list (ctw-table window))))

(defmethod cut-selection ((window clasp-caddy) &key)
  (let ((selection (reverse (l-caddy-selected-items window))))
    (press-to-clipboard selection)))

(defmethod cut-selection ((window plotter::key-window) &key)
  (with-slots (plotter::picture) window
    (copy-str-or-h-pict plotter::picture)))

(defmethod cut-selection ((window listener) &key)
  (copy window))

(defmethod paste-into-clasp ((window t) &key)
  ())

(defmethod paste-into-clasp ((window window) &key)
  (paste window))

(defmethod cut-selection ((window ribbon-window) &key)
  (press-to-clipboard 
   (list (what-i-display window))))


#+SEE-YOU
(defun string-substitute (target substitute string)
  (let (loc)
    (loop while (setf loc (search target string)) do
          (setf string (concatenate 'string (subseq string 0 loc)
                                    substitute
                                    (subseq string (+ loc (length target)) (length string)))))
    string))

#+GOODBYE
(string-substitute "#.(DATASET" "(CLASP::GET-DATASET"
                   (string-substitute "#.(VARIABLE" "(CLASP::GET-VARIABLE" (what-i-display window)))

