#|
*********************************
* Dustribution generation funcs *
* for Macintosh CLASP           *
*********************************
|#

;;;
;;; uniform-distribution.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dialog box for creating a variable from a uniform
distribution.

================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
12-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#


(in-package "CLASP-INTERFACE")


;;;
;;; Define uniform-dist-dialog .
;;;

(defclass uniform-dist-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::uniform-distribution function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Uniform Distribution"
    :VIEW-SIZE #@(250 165)
    :VIEW-POSITION (best-spot 250 165)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*uniform-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Minimum"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'min-number-item
                         :view-position #@(125 15)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Maximum"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'max-number-item
                         :view-position #@(125 42)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Samples"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "1"
                         :view-nick-name 'number-samples-number-item
                         :allow-floats NIL
                         :view-position #@(125 70)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Sample Size"
                         :view-position #@(15 97))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'sample-size-number-item
                         :allow-floats NIL
                         :view-position #@(125 97)
                         :view-size #@(70 16))
          (make-instance 'compute-button-di :view-position #@(125 132)))))

(defmethod f-can-compute-statistics-dialog ((uniform-dist-dialog uniform-dist-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((uniform-dist-dialog uniform-dist-dialog))
  (let* ((min (number-item-value (view-named 'min-number-item uniform-dist-dialog)))
         (max (number-item-value (view-named 'max-number-item uniform-dist-dialog)))
         (ns (number-item-value (view-named 'number-samples-number-item uniform-dist-dialog)))
         (ss (number-item-value (view-named 'sample-size-number-item uniform-dist-dialog)))
         )
    (generate-uniform-distribution min max ss ns)))


;;;
;;;
;;;

(defmethod generate-uniform-distribution (min max ss ns)
  "Makes a Univform Distribution from the arguments, saving
results in a new dataset."
  ;;
  (let* ((clasp-command (make-clasp-command
                         `(clasp::sample-to-dataset
                           (make-instance 'clasp::uniform-distribution
                             :minimum ,min
                             :maximum ,max)
                           ,ss ,ns)
                         :description
                         (list (list 'command 'uniform-distribution)
                               (list 'min min)
                               (list 'max max)
                               (list 'samples ns)
                               (list 'sample-size ss)))))
    (run-clasp-command clasp-command
                       :str-message "Computing Uniform Distributions"
                       :f-update-dataset-view t)))


(defun uniform-dist-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
uniform-distribution class."
  ;;
  (make-instance 'uniform-dist-dialog))

;;;
;;; binomial-distribution.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dialog box for creating a variable from a binomial
distribution.

================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
13-Jul-94 ms	Created.
20-Jul-94 ms    Beta finish.

|#


;;;
;;; Define binomial-dist-dialog .
;;;

(defclass binomial-dist-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::binomial-distribution function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Binomial Distribution"
    :VIEW-SIZE #@(250 165)
    :VIEW-POSITION (best-spot 250 165)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*binomial-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "P"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "0.500"
                         :view-nick-name 'p-number-item
                         :allow-negatives NIL
                         :view-position #@(125 15)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "N"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "12"
                         :view-nick-name 'n-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 42)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Samples"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "1"
                         :view-nick-name 'number-samples-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 70)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Sample Size"
                         :view-position #@(15 100))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'sample-size-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 100)
                         :view-size #@(70 16))
          (make-instance 'compute-button-di :view-position #@(125 133)))))

(defmethod f-can-compute-statistics-dialog ((binomial-dist-dialog binomial-dist-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((binomial-dist-dialog binomial-dist-dialog))
  (let* ((p (number-item-value (view-named 'p-number-item binomial-dist-dialog)))
         (n (number-item-value (view-named 'n-number-item binomial-dist-dialog)))
         (ns (number-item-value (view-named 'number-samples-number-item binomial-dist-dialog)))
         (ss (number-item-value (view-named 'sample-size-number-item binomial-dist-dialog)))
         )
    (generate-binomial-distribution p n ss ns)))


;;;
;;;
;;;

(defmethod generate-binomial-distribution (p n ss ns)
  "Calls clasp::sample-to-dataset on the arguments, saving
results in the history."
  ;;
  (let* ((clasp-command (make-clasp-command
                         `(clasp::sample-to-dataset
                           (make-instance 'clasp::binomial-distribution
                             :p ,p :n ,n)
                           ,ss ,ns)
                         :description
                         (list (list 'command 'binomial-distribution)
                               (list 'p p)
                               (list 'n n)
                               (list 'samples ns)
                               (list 'sample-size ss)))))
    (run-clasp-command clasp-command
                       :str-message "Computing binomial Distributions"
                       :f-update-dataset-view t)))


(defun binomial-dist-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
binomial distribution creation facility."
  ;;
  (make-instance 'binomial-dist-dialog))

;;;
;;; normal-distribution.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dialog box for creating a variable from a normal
distribution.

================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
13-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish.

|#


;;;
;;; Define normal-dist-dialog .
;;;

(defclass normal-dist-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::normal-distribution function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Normal Distribution"
    :VIEW-SIZE #@(235 165)
    :VIEW-POSITION (best-spot 235 165)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*normal-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Mean"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "0.0"
                         :view-nick-name 'mean-number-item
                         :allow-negatives t
                         :view-position #@(125 15)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Std. Dev"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "1.0"
                         :view-nick-name 'sd-number-item
                         :view-position #@(125 42)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Samples"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "1"
                         :view-nick-name 'number-samples-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 70)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Sample Size"
                         :view-position #@(15 100))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'sample-size-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 100)
                         :view-size #@(70 16))
          (make-instance 'compute-button-di :view-position #@(125 133)))))

(defmethod f-can-compute-statistics-dialog ((normal-dist-dialog normal-dist-dialog))
  "We can compute if the parameters are the right kind of numbers."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((normal-dist-dialog normal-dist-dialog))
  (let* ((mean (number-item-value (view-named 'mean-number-item normal-dist-dialog)))
         (sd (number-item-value (view-named 'sd-number-item normal-dist-dialog)))
         (ns (number-item-value (view-named 'number-samples-number-item normal-dist-dialog)))
         (ss (number-item-value (view-named 'sample-size-number-item normal-dist-dialog)))
         )
    (generate-normal-distribution mean sd ss ns)))


;;;
;;;
;;;

(defmethod generate-normal-distribution (mean sd ss ns)
  "Calls clasp::sample-to-dataset on the arguments, saving
results in the history."
  ;;
  (let* ((clasp-command (make-clasp-command
                         `(clasp::sample-to-dataset
                           (make-instance 'clasp::normal-distribution
                             :mean ,mean :standard-deviation ,sd)
                           ,ss ,ns)
                         :description
                         (list (list 'command 'normal-distribution)
                               (list 'mean mean)
                               (list 'std-dev sd)
                               (list 'samples ns)
                               (list 'sample-size ss)))))
    (run-clasp-command clasp-command
                       :str-message "Computing normal Distributions"
                       :f-update-dataset-view t)))


(defun normal-dist-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
normal distribution creation facility."
  ;;
  (make-instance 'normal-dist-dialog))

;;;
;;; poisson-distribution.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dialog box for creating a variable from a poisson
distribution.

================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
12-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish.

|#


;;;
;;; Define poisson-dist-dialog .
;;;

(defclass poisson-dist-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::poisson-distribution function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Poisson Distribution"
    :VIEW-SIZE #@(235 135)
    :VIEW-POSITION (best-spot 235 165)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*poisson-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Mean"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "1.0"
                         :view-nick-name 'mean-number-item
                         :view-position #@(125 15)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Samples"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "1"
                         :view-nick-name 'number-samples-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 42)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Sample Size"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'sample-size-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 70)
                         :view-size #@(70 16))
          (make-instance 'compute-button-di :view-position #@(125 105)))))

(defmethod f-can-compute-statistics-dialog ((poisson-dist-dialog poisson-dist-dialog))
  "We can compute if the numbers are ok."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((poisson-dist-dialog poisson-dist-dialog))
  (let* ((mean (number-item-value (view-named 'mean-number-item poisson-dist-dialog)))
         (ns (number-item-value (view-named 'number-samples-number-item poisson-dist-dialog)))
         (ss (number-item-value (view-named 'sample-size-number-item poisson-dist-dialog)))
         )
    (generate-poisson-distribution mean ss ns)))

;;;

(defmethod generate-poisson-distribution (mean ss ns)
  "Calls clasp::sample-to-dataset on the arguments, saving
results in the history."
  ;;
  (let* ((clasp-command (make-clasp-command
                        `(clasp::sample-to-dataset
                          (make-instance 'clasp::poisson-distribution
                            :mean ,mean)
                          ,ss ,ns)
                        :description
                        (list (list 'command 'poission-distribution)
                              (list 'mean mean)
                              (list 'samples ns)
                              (list 'sample-size ss)))))
    (run-clasp-command clasp-command
                       :str-message "Computing Poisson Distributions"
                       :f-update-dataset-view t)))


(defun poisson-dist-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
poisson distribution creation facility."
  ;;
  (make-instance 'poisson-dist-dialog))

;;;
;;; gamma-distribution.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the dialog box for creating a variable from a gamma
distribution.

================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
13-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#


;;;
;;; Define gamma-dist-dialog .
;;;

(defclass gamma-dist-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::gamma-distribution function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Gamma Distribution"
    :VIEW-SIZE #@(235 135)
    :VIEW-POSITION (best-spot 235 135)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*gamma-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Integer Order"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "2"
                         :view-nick-name 'order-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 15)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Samples"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "1"
                         :view-nick-name 'number-samples-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 42)
                         :view-size #@(70 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Sample Size"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "100"
                         :view-nick-name 'sample-size-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 70)
                         :view-size #@(70 16))
          (make-instance 'compute-button-di :view-position #@(125 105)))))

(defmethod f-can-compute-statistics-dialog ((gamma-dist-dialog gamma-dist-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((gamma-dist-dialog gamma-dist-dialog))
  (let* ((order (number-item-value (view-named 'order-number-item gamma-dist-dialog)))
         (ns (number-item-value (view-named 'number-samples-number-item gamma-dist-dialog)))
         (ss (number-item-value (view-named 'sample-size-number-item gamma-dist-dialog)))
         )
    (generate-gamma-distribution order ss ns)))

;;;

(defmethod generate-gamma-distribution (order ss ns)
  "Calls clasp::sample-to-dataset on the arguments, saving
results in the history."
  ;;
  (let* ((clasp-command (make-clasp-command
                         `(clasp::sample-to-dataset
                           (make-instance 'clasp::gamma-distribution
                             :ia ,order)
                           ,ss ,ns)
                         :description
                         (list (list 'command 'gamma-distribution)
                               (list 'order order)
                               (list 'samples ns)
                               (list 'sample-size ss)))))
    (run-clasp-command clasp-command
                       :str-message "Computing Gamma Distributions"
                       :f-update-dataset-view t)))


(defun gamma-dist-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
gamma distribution creation facility."
  ;;
  (make-instance 'gamma-dist-dialog))

;;; ========================================================================
;;; bootstrapping stuff
;;; ========================================================================
;;;
;;; monte-carlo
;;;

(defclass monte-carlo-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "Monte carlo sampling dialog")
  (:default-initargs
    :window-title "Monte Carlo"
    :view-size #@(400 200)
    :view-position #@(100 100)
    :help '*monte-carlo-help*
    :view-subviews
    (list (make-instance 'bs-ub-radio-button :view-nick-name 'univariate
                         :deactivate-ids '(CSAMS-2 FUNCTION-2)
                         :view-font '(:bold "Helvetica" 12)
                         :activation-form `(setup-iim 
                                            (view-named 'stats (view-container 'self))
                                            *univariate-stat-names*)
                         :view-position #@(30 5) :dialog-item-text "Univariate (X)")
          (make-instance 'bs-ub-radio-button :view-nick-name 'bivariate
                         :view-font '(:bold "Helvetica" 12)
                         :activate-ids '(CSAMS-2 FUNCTION-2)
                         :activation-form `(setup-iim 
                                            (view-named 'stats (view-container 'self))
                                            *bivariate-stat-names*)
                         :view-position #@(230 5) :dialog-item-text "Bivariate (X,Y)")
          (make-instance 'item-injector-menu :view-nick-name 'csams-1
                         :injections *sample-calls* :inject-where 'function-1
                         :view-font '("Helvetica" 12)
                         :view-position #@(5 23) :view-size #@(190 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'function-1
                         :view-font '("Helvetica" 10)   :allow-returns t
                         :view-position #@(5 45) :view-size #@(190 24)
                         :dialog-item-text "<lisp function call>")
          (make-instance 'item-injector-menu :view-nick-name 'csams-2
                         :injections *sample-calls* :inject-where 'function-2
                         :enabledp nil
                         :view-font '("Helvetica" 12)
                         :view-position #@(205 23) :view-size #@(190 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'function-2
                         :view-font '("Helvetica" 10) :dialog-item-enabled-p nil :allow-returns t
                         :view-position #@(205 45) :view-size #@(190 24)
                         :dialog-item-text "<lisp function call>")
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(175 80) :dialog-item-text "Statistic")
          (make-instance 'item-injector-menu :view-nick-name 'stats
                         :injections *univariate-stat-names*
                         :inject-where 'boot-function
                         :view-font '("Helvetica" 12)
                         :view-position #@(5 97) :view-size #@(190 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'boot-function
                         :view-font '("Helvetica" 12) :view-position #@(205 100)
                         :view-size #@(185 12) :dialog-item-text "clasp::mean")
          (make-instance 'static-text-dialog-item :dialog-item-text "Number of resamples"
                         :view-font '("Helvetica" 12) :view-position #@(5 125))
          (make-instance 'number-entry-item :view-nick-name 'bootstraps
                         :allow-negatives nil :allow-floats nil :view-font '("Helvetica" 12)
                         :view-position #@(145 125) :view-size #@(40 12)
                         :dialog-item-text "100")
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(200 125) :dialog-item-text "Name")
          (make-instance 'editable-text-dialog-item :view-nick-name 'dataset-name
                         :view-position #@(250 125) :view-size #@(140 12)
                         :dialog-item-text "Monte-Carlo" :view-font '("Helvetica" 12))
          (make-instance 'compute-button-di :view-position #@(170 165))
          )))

(defmethod f-can-compute-statistics-dialog ((monte-carlo-dialog monte-carlo-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate monte-carlo-dialog)))
         (fun-1 (read-from-string (dialog-item-text (view-named 'function-1 monte-carlo-dialog))))
         (fun-2 (read-from-string (dialog-item-text (view-named 'function-2 monte-carlo-dialog))))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function monte-carlo-dialog))))
         (btsts (number-item-value (view-named 'bootstraps monte-carlo-dialog)))
         (can-p t) why)
    (cond
     ((not (ignore-errors (fboundp (car fun-1))))
      (setf can-p nil) (setf why "invalid X function specification"))
     ((and (not (ignore-errors (fboundp (car fun-2)))) bivariate)
      (setf can-p nil) (setf why "invalid Y function specification"))
     ((< btsts 2)
      (setf can-p nil) (setf why "too few resamples to bootstrap"))
     ((not (fboundp b-fun))
      (setf can-p nil) (setf why "invalid bootstrapping function"))
     ((and (assoc (format nil "~S" fun-1) *sample-calls* :test #'string-equal))
      (setf can-p nil) (setf why "the X sampling distribution function call arguments haven't been filled in."))
     ((and (assoc (format nil "~S" fun-2) *sample-calls* :test #'string-equal) bivariate)
      (setf can-p nil) (setf why "the Y sampling distribution function call arguments haven't been filled in."))
     (t "Let er rip"))
    (values can-p why)))

(defmethod compute-statistics-dialog ((monte-carlo-dialog monte-carlo-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate monte-carlo-dialog)))
         (fun-1 (read-from-string (dialog-item-text (view-named 'function-1 monte-carlo-dialog))))
         (fun-2 (read-from-string (dialog-item-text (view-named 'function-2 monte-carlo-dialog))))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function monte-carlo-dialog))))
         (btsts (number-item-value (view-named 'bootstraps monte-carlo-dialog)))
         (name (dialog-item-text (view-named 'dataset-name monte-carlo-dialog)))
         )
    (do-bootstrap name b-fun btsts
                  (if bivariate (list (car fun-1) (car fun-2))
                      (list (car fun-1)))
                  (if bivariate (list (cdr fun-1) (cdr fun-2))
                      (list (cdr fun-1)))
                  (if bivariate
                    (list (cdr fun-1) (cdr fun-2))
                    (list (cdr fun-1))))))

(defun monte-carlo-dialog-make ()
  (make-instance 'monte-carlo-dialog))

;;;
;;; randomization/bootstrapping
;;;

(defclass data-sampling-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting at randomization and bootstrapping stuff.")
  (:default-initargs
    :window-title "Bootstrapping/Randomization"
    :view-size #@(400 200)
    :view-position #@(100 100)
    :help '*boot-help*
    :view-subviews
    (list (make-instance 'bs-ub-radio-button :view-nick-name 'univariate
                         :deactivate-ids '(y-var-item swap-1 pool-nr-1 pool-wr-1)
                         :view-font '(:bold "Helvetica" 12)
                         :activation-form `(progn 
                                             (unless (radio-button-pushed-p 
                                                      (view-named 'noreplace-1 (view-container 'self)))
                                               (radio-button-push
                                                (view-named 'replacement-1 (view-container 'self))))
                                             (setup-iim 
                                              (view-named 'stats (view-container 'self))
                                              *univariate-stat-names*))
                         :view-position #@(5 5) :dialog-item-text "Univariate (X)")
          (make-instance 'bs-ub-radio-button :view-nick-name 'bivariate
                         :view-font '(:bold "Helvetica" 12)
                         :activate-ids '(y-var-item swap-1 pool-nr-1 pool-wr-1)
                         :activation-form `(setup-iim 
                                            (view-named 'stats (view-container 'self))
                                            *bivariate-stat-names*)
                         :view-position #@(205 5) :dialog-item-text "Bivariate (X,Y)")
          (make-instance 'bs-ub-radio-button :view-nick-name 'replacement-1
                         :view-font '("Helvetica" 12) 
                         :view-position #@(5 49) :dialog-item-text "Sample with replacement"
                         :radio-button-cluster 2)
          (make-instance 'bs-ub-radio-button :view-nick-name 'noreplace-1
                         :view-font '("Helvetica" 12) 
                         :view-position #@(5 64) :dialog-item-text "Sample without replacement"
                         :radio-button-cluster 2)
          (make-instance 'bs-ub-radio-button :view-nick-name 'swap-1
                         :view-font '("Helvetica" 12) :dialog-item-enabled-p NIL
                         :view-position #@(205 49) :dialog-item-text "Swap paired items"
                         :radio-button-cluster 2)
          (make-instance 'bs-ub-radio-button :view-nick-name 'pool-nr-1
                         :view-font '("Helvetica" 12) :dialog-item-enabled-p NIL
                         :view-position #@(205 79) :dialog-item-text "Without replacement from pool"
                         :radio-button-cluster 2)
          (make-instance 'bs-ub-radio-button :view-nick-name 'pool-wr-1
                         :view-font '("Helvetica" 12) :dialog-item-enabled-p NIL
                         :view-position #@(205 64) :dialog-item-text "With replacement from pool"
                         :radio-button-cluster 2)

          (make-instance 'variable-stdi :view-nick-name 'x-var-item :dialog-item-enabled-p t
                         :view-font '("Helvetica" 12) :view-position #@(25 30)
                         :view-size #@(160 14))
          (make-instance 'variable-stdi :view-nick-name 'y-var-item :dialog-item-enabled-p nil
                         :view-font '("Helvetica" 12) :view-position #@(225 30)
                         :view-size #@(160 14))
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(175 99) :dialog-item-text "Statistic")
          (make-instance 'item-injector-menu :view-nick-name 'stats
                         :injections *univariate-stat-names* :inject-where 'boot-function
                         :view-font '("Helvetica" 12)
                         :view-position #@(5 119) :view-size #@(185 19))
          (make-instance 'editable-text-dialog-item :view-nick-name 'boot-function
                         :view-font '("Helvetica" 12) :view-position #@(205 122)
                         :view-size #@(185 12) :dialog-item-text "clasp::mean")
          (make-instance 'static-text-dialog-item :dialog-item-text "Number of resamples"
                         :view-font '("Helvetica" 12) :view-position #@(5 145))
          (make-instance 'number-entry-item :view-nick-name 'bootstraps
                         :allow-negatives nil :allow-floats nil :view-font '("Helvetica" 12)
                         :view-position #@(145 145) :view-size #@(40 12)
                         :dialog-item-text "100")
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(200 145) :dialog-item-text "Name")
          (make-instance 'editable-text-dialog-item :view-nick-name 'dataset-name
                         :view-position #@(250 145) :view-size #@(140 12)
                         :dialog-item-text "sample-statistic" :view-font '("Helvetica" 12))
          (make-instance 'compute-button-di :view-position #@(170 175))
          )))

(defmethod f-can-compute-statistics-dialog ((data-sampling-dialog data-sampling-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate data-sampling-dialog)))
         (var-1 (variable-vset-stdi (view-named 'x-var-item data-sampling-dialog)))
         (var-2 (variable-vset-stdi (view-named 'y-var-item data-sampling-dialog)))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function data-sampling-dialog))))
         (btsts (number-item-value (view-named 'bootstraps data-sampling-dialog)))
         (can-p t) why)
    (cond
     ((not var-1)
      (setf can-p nil) (setf why "no X variable to sample from"))
     ((and (not var-2) bivariate)
      (setf can-p nil) (setf why "no Y variable to sample from"))
     ((< btsts 2)
      (setf can-p nil) (setf why "too few resamples to bootstrap"))
     ((not (fboundp b-fun))
      (setf can-p nil) (setf why "invalid bootstrapping function"))
     (t "Let er rip"))
    (values can-p why)))

(defmethod compute-statistics-dialog ((data-sampling-dialog data-sampling-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate data-sampling-dialog)))
         (var-1 (variable-vset-stdi (view-named 'x-var-item data-sampling-dialog)))
         (var-2 (variable-vset-stdi (view-named 'y-var-item data-sampling-dialog)))
         (mode (cond 
                ((radio-button-pushed-p (view-named 'replacement-1 data-sampling-dialog))
                 :disjoint-with-replace)
                ((radio-button-pushed-p (view-named 'noreplace-1 data-sampling-dialog))
                 :disjoint-without-replace)
                ((radio-button-pushed-p (view-named 'swap-1 data-sampling-dialog))
                 :swap-in-place)
                ((radio-button-pushed-p (view-named 'pool-wr-1 data-sampling-dialog))
                 :joint-with-replace)
                (t :joint-without-replace)))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function data-sampling-dialog))))
         (btsts (number-item-value (view-named 'bootstraps data-sampling-dialog)))
         (name (dialog-item-text (view-named 'dataset-name data-sampling-dialog)))
         )
    (if bivariate
      (do-bootstrap-special name b-fun btsts var-1 var-2 mode)
      (do-bootstrap name b-fun btsts
                    (list (if (radio-button-pushed-p (view-named 'noreplace-1 data-sampling-dialog))
                            'clasp::sample-without-replacement-from-sequence
                            (if (radio-button-pushed-p (view-named 'replacement-1 data-sampling-dialog))
                              'clasp::sample-with-replacement-from-sequence
                              'identity)))
                    (list (list (clasp::simplify var-1)))
                    (list var-1)))))
  
(defun data-sampling-dialog-make ()
  (make-instance 'data-sampling-dialog))


;;;
;;; big heavy one.
;;;

(defclass bootstrap-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting arguments to the compute-bootstrap function.")
  (:DEFAULT-INITARGS
    :window-title "Monte Carlo/Bootstrap/Randomization"
    :view-size #@(400 200)
    :view-position #@(100 100)
    :help '*mixed-help*
    :view-subviews
    (list (make-instance 'bs-ub-radio-button :view-nick-name 'univariate
                         :deactivate-ids '(CSAMS-2 EXPRESSION-2 FUNCTION-2 VARIABLE-2 REPLACEMENT-2 NOREPLACE-2)
                         :activation-form `(setup-iim 
                                            (view-named 'stats (view-container 'self))
                                            *univariate-stat-names*)
                         :view-font '(:bold "Helvetica" 12)
                         :view-position #@(30 5) :dialog-item-text "Univariate (X)")
          (make-instance 'bs-ub-radio-button :view-nick-name 'bivariate
                         :activate-ids '(CSAMS-2 EXPRESSION-2 REPLACEMENT-2 NOREPLACE-2)
                         :activation-form `(progn
                                             (if (radio-button-pushed-p (view-named 'expression-2
                                                                                    (view-container 'self)))
                                               (dialog-item-enable (view-named 'FUNCTION-2
                                                                               (view-container 'self)))
                                               (dialog-item-enable (view-named 'variable-2
                                                                               (view-container 'self))))
                                             (setup-iim (view-named 'stats (view-container 'self))
                                                        *bivariate-stat-names*))
                         :view-font '(:bold "Helvetica" 12)
                         :view-position #@(230 5) :dialog-item-text "Bivariate (X,Y)")
          (make-instance 'bs-ub-radio-button :view-nick-name 'expression-1
                         :view-font '("Helvetica" 12)
                         :activate-ids '(function-1) :deactivate-ids '(variable-1)
                         :view-position #@(5 25) :dialog-item-text "Function Call"
                         :radio-button-cluster 2)
          (make-instance 'item-injector-menu :view-nick-name 'csams-1
                         :injections *sample-calls* :inject-where 'function-1
                         :view-font '("Helvetica" 12)
                         :view-position #@(110 23) :view-size #@(87 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'function-1
                         :view-font '("Helvetica" 10)   :allow-returns t
                         :view-position #@(5 45) :view-size #@(190 24)
                         :dialog-item-text "<lisp function call>")
          (make-instance 'bs-ub-radio-button :view-nick-name 'replacement-1
                         :view-font '("Helvetica" 12) 
                         :deactivate-ids '(function-1) :activate-ids '(variable-1)
                         :view-position #@(5 74) :dialog-item-text "Sample with replacement"
                         :radio-button-cluster 2)
          (make-instance 'bs-ub-radio-button :view-nick-name 'noreplace-1
                         :view-font '("Helvetica" 12) 
                         :deactivate-ids '(function-1) :activate-ids '(variable-1)
                         :view-position #@(5 89) :dialog-item-text "Sample without replacement"
                         :radio-button-cluster 2)
          (make-instance 'variable-stdi :view-nick-name 'variable-1 :dialog-item-enabled-p nil
                         :view-font '("Helvetica" 12) :view-position #@(25 104)
                         :view-size #@(160 14))
          (make-instance 'bs-ub-radio-button :view-nick-name 'expression-2
                         :view-font '("Helvetica" 10) :dialog-item-enabled-p nil
                         :activate-ids '(function-2) :deactivate-ids '(variable-2)
                         :view-position #@(205 25) :dialog-item-text "Function Call"
                         :radio-button-cluster 3)
          (make-instance 'item-injector-menu :view-nick-name 'csams-2
                         :injections *sample-calls* :inject-where 'function-2
                         :view-font '("Helvetica" 12)
                         :view-position #@(310 23) :view-size #@(87 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'function-2
                         :view-font '("Helvetica" 10) :dialog-item-enabled-p nil :allow-returns t
                         :view-position #@(205 45) :view-size #@(190 24)
                         :dialog-item-text "<lisp function call>")
          (make-instance 'bs-ub-radio-button :view-nick-name 'replacement-2
                         :view-font '("Helvetica" 12) :dialog-item-enabled-p nil
                         :deactivate-ids '(function-2) :activate-ids '(variable-2)
                         :view-position #@(205 74) :dialog-item-text "Sample with replacement"
                         :radio-button-cluster 3)
          (make-instance 'bs-ub-radio-button :view-nick-name 'noreplace-2
                         :view-font '("Helvetica" 12) :dialog-item-enabled-p nil
                         :deactivate-ids '(function-2) :activate-ids '(variable-2)
                         :view-position #@(205 89) :dialog-item-text "Sample without replacement"
                         :radio-button-cluster 3)
          (make-instance 'variable-stdi :view-nick-name 'variable-2 :dialog-item-enabled-p nil
                         :view-font '("Helvetica" 12) :view-position #@(225 104)
                         :view-size #@(160 14))
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(5 122) :dialog-item-text "Bootstrap Function")
          (make-instance 'item-injector-menu :view-nick-name 'stats
                         :injections *univariate-stat-names* :inject-where 'boot-function
                         :view-font '("Helvetica" 12)
                         :view-position #@(115 119) :view-size #@(75 18))
          (make-instance 'editable-text-dialog-item :view-nick-name 'boot-function
                         :view-font '("Helvetica" 12) :view-position #@(205 122)
                         :view-size #@(185 12) :dialog-item-text "clasp::mean")
          (make-instance 'static-text-dialog-item :dialog-item-text "Number of resamples"
                         :view-font '("Helvetica" 12) :view-position #@(5 145))
          (make-instance 'number-entry-item :view-nick-name 'bootstraps
                         :allow-negatives nil :allow-floats nil :view-font '("Helvetica" 12)
                         :view-position #@(145 145) :view-size #@(40 12)
                         :dialog-item-text "100")
          (make-instance 'static-text-dialog-item :view-font '("Helvetica" 12)
                         :view-position #@(200 145) :dialog-item-text "Name")
          (make-instance 'editable-text-dialog-item :view-nick-name 'dataset-name
                         :view-position #@(250 145) :view-size #@(140 12)
                         :dialog-item-text "Bootstrap" :view-font '("Helvetica" 12))
          (make-instance 'compute-button-di :view-position #@(170 175))
          )))

(defmethod f-can-compute-statistics-dialog ((bootstrap-dialog bootstrap-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate bootstrap-dialog)))
         (fun-1 (read-from-string (dialog-item-text (view-named 'function-1 bootstrap-dialog))))
         (fun-2 (read-from-string (dialog-item-text (view-named 'function-2 bootstrap-dialog))))
         (ufu-1 (radio-button-pushed-p (view-named 'expression-1 bootstrap-dialog)))
         (ufu-2 (radio-button-pushed-p (view-named 'expression-2 bootstrap-dialog)))
         (var-1 (variable-vset-stdi (view-named 'variable-1 bootstrap-dialog)))
         (var-2 (variable-vset-stdi (view-named 'variable-2 bootstrap-dialog)))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function bootstrap-dialog))))
         (btsts (number-item-value (view-named 'bootstraps bootstrap-dialog)))
         (can-p t) why)
    (cond
     ((and (not (ignore-errors (fboundp (car fun-1)))) ufu-1)
      (setf can-p nil) (setf why "invalid X function specification"))
     ((and (not (ignore-errors (fboundp (car fun-2)))) ufu-2 bivariate)
      (setf can-p nil) (setf why "invalid Y function specification"))
     ((and (not var-1) (not ufu-1))
      (setf can-p nil) (setf why "no X variable to sample from"))
     ((and (not var-2) (not ufu-2) bivariate)
      (setf can-p nil) (setf why "no Y variable to sample from"))
     ((< btsts 2)
      (setf can-p nil) (setf why "too few resamples to bootstrap"))
     ((not (fboundp b-fun))
      (setf can-p nil) (setf why "invalid bootstrapping function"))
     ((and (assoc (format nil "~S" fun-1) *sample-calls* :test #'string-equal) ufu-1)
      (setf can-p nil) (setf why "the X sampling distribution function call arguments haven't been filled in."))
     ((and (assoc (format nil "~S" fun-2) *sample-calls* :test #'string-equal) bivariate ufu-2)
      (setf can-p nil) (setf why "the Y sampling distribution function call arguments haven't been filled in."))
     (t "Let er rip"))
    (values can-p why)))

(defmethod compute-statistics-dialog ((bootstrap-dialog bootstrap-dialog))
  (let* ((bivariate (radio-button-pushed-p (view-named 'bivariate bootstrap-dialog)))
         (fun-1 (read-from-string (dialog-item-text (view-named 'function-1 bootstrap-dialog))))
         (fun-2 (read-from-string (dialog-item-text (view-named 'function-2 bootstrap-dialog))))
         (ufu-1 (radio-button-pushed-p (view-named 'expression-1 bootstrap-dialog)))
         (ufu-2 (radio-button-pushed-p (view-named 'expression-2 bootstrap-dialog)))
         (var-1 (variable-vset-stdi (view-named 'variable-1 bootstrap-dialog)))
         (var-2 (variable-vset-stdi (view-named 'variable-2 bootstrap-dialog)))
         (rep-1 (radio-button-pushed-p (view-named 'replacement-1 bootstrap-dialog)))
         (rep-2 (radio-button-pushed-p (view-named 'replacement-2 bootstrap-dialog)))
         (b-fun (read-from-string (dialog-item-text (view-named 'boot-function bootstrap-dialog))))
         (btsts (number-item-value (view-named 'bootstraps bootstrap-dialog)))
         (name (dialog-item-text (view-named 'dataset-name bootstrap-dialog)))
         )
    (do-bootstrap name b-fun btsts
                  (if bivariate (list (if ufu-1 (car fun-1)
                                          (if rep-1 'clasp::sample-with-replacement-from-sequence
                                              'clasp::sample-without-replacement-from-sequence))
                                      (if ufu-2 (car fun-2)
                                          (if rep-2 'clasp::sample-with-replacement-from-sequence
                                              'clasp::sample-without-replacement-from-sequence)))
                      (list (if ufu-1 (car fun-1)
                                (if rep-1 'clasp::sample-with-replacement-from-sequence
                                    'clasp::sample-without-replacement-from-sequence))))
                  (if bivariate (list (if ufu-1 (cdr fun-1)
                                          (list (clasp::simplify var-1)))
                                      (if ufu-2 (cdr fun-2)
                                          (list (clasp::simplify var-2))))
                      (list (if ufu-1 (cdr fun-1)
                                (list (clasp::simplify var-1)))))
                  (if bivariate
                    (list (if ufu-1 (cdr fun-1) var-1) (if ufu-2 (cdr fun-2) var-2))
                    (list (if ufu-1 (cdr fun-1) var-1))))))

(defun bootstrap-dialog-make ()
  (make-instance 'bootstrap-dialog))

;;;
;;; what the hypothesis testing boils down to
;;;

(defun do-bootstrap (dataset-name boot-function resamples sampling-functions sampling-arguments arg-syms)
  (let ((clasp-command
         (make-clasp-command 
          `(clasp::make-dataset-from-columns
            ,dataset-name
            (list
             (clasp::compute-bootstrap
              ',boot-function
              :sampling-functions ',sampling-functions
              :sampling-function-arguments ',sampling-arguments
              :number-of-iterations ,resamples))
            ',(list (symbol-name boot-function)))
          :description
          (list (list 'command 'clasp::compute-bootstrap)
                (list 'boot-statistic boot-function)
                (list 'samples arg-syms)
                (list 'resamples resamples)))))
    (run-clasp-command clasp-command
                       :str-message "Computing bootstrap data...."
                       :f-update-dataset-view t)))

(defun do-bootstrap-special (dataset-name boot-function btsts var-1 var-2 mode)
  (let ((clasp-command
         (make-clasp-command
          `(clasp::make-dataset-from-columns
            ,dataset-name
            (list
             (clasp::compute-bootstrap-special
              ',boot-function
              (clasp::simplify ,var-1)
              (clasp::simplify ,var-2)
              ,btsts
              :mode ,mode))
            ',(list (symbol-name boot-function)))
          :description
          (list (list 'command 'clasp::compute-bootstrap-special)
                (list 'boot-statistic boot-function)
                (list 'variables var-1 var-2)
                (list 'resamples btsts)))))
    (run-clasp-command clasp-command
                       :str-message "Computing bootstrap distribution...."
                       :f-update-dataset-view t)))