;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE -*-

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                  Package Definition for the MCL Interface              *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1996 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and
;;; its documentation is hereby granted without fee for non-commercial
;;; uses only (not for resale), provided that the above copyright
;;; notice of EKSL, this paragraph and the one following appear in all
;;; copies and in supporting documentation.

;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possibility of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  08-16-96 File Created.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp-interface)

;;; --*--
;;; ***************************************************************************

;; The flavor for items we pass around to other drag-aware views.
(defconstant $CLASP-Flavor :|CLSP|)

;;; DRAGABLE-ITEMS-SEQUENCE-VIEW - a table you can drag stuff out of

(defclass dragable-items-sequence-view (sequence-dialog-item
                                        drag-view-mixin)
  ())

(defmethod drag-selection-p ((view dragable-items-sequence-view) (pt-where fixnum))
  (let* ((pt-cell-clicked (point-to-cell view pt-where))
         (f-clicked-on-selection (member pt-cell-clicked (selected-cells view))))
    (and f-clicked-on-selection
         (not (option-key-p))        ;ignore if selection editing keys down
         (not (shift-key-p))
         (not (command-key-p))
         (not (control-key-p)))))

(defmethod selected-items ((view dragable-items-sequence-view))
  (let* ((l-items (table-sequence view))
         (l-cell-selected (selected-cells view))
         (l-selected-items (map 'list #'(lambda (cell)
                                          (elt l-items (point-v cell)))
                                l-cell-selected)))
    l-selected-items))

; Add a flavor, which is an MCL object and the drag region.
(defmethod drag-add-drag-contents ((view dragable-items-sequence-view))
  (drag-add-mcl-object-flavor 1 $CLASP-Flavor (selected-items view))
  (multiple-value-bind (upper-left lower-right) (selected-cells-bounds view)
    (let ((region (new-region)))
      (unwind-protect
        (progn
          (set-rect-region region upper-left lower-right)
          (drag-create-item-bounds view 1 region))
        (dispose-region region))))
  (values t))

(defmethod selected-cells-bounds ((view dragable-items-sequence-view))
  (let* ((dimensions (table-dimensions view))
         (vis-dimensions (visible-dimensions view))
         (top-cell (scroll-position view))
         (bottom-cell (add-points vis-dimensions top-cell))
         (superview (view-container view)))
    (values
     (loop for cell-counter from 0 to (point-v dimensions) do
           (when (and (cell-selected-p view 0 cell-counter)
                      (>= cell-counter (point-v top-cell))
                      (<= cell-counter (point-v bottom-cell)))
             (return (convert-coordinates (cell-position view 0 cell-counter)
                                          superview
                                          view))))
     
     (loop for cell-counter from (point-v dimensions) downto 0 do
           (when (and (cell-selected-p view 0 cell-counter)
                      (>= cell-counter (point-v top-cell))
                      (<= cell-counter (point-v bottom-cell)))
             (return (add-points (convert-coordinates (cell-position view 0 cell-counter)
                                                      superview
                                                      view)
                                 (cell-size view))))))))

;;; --------------------------------------------------------------------------------------
;;; DRAG-AWARE-MIXIN - add this to classes that you want to drop stuff on

(defclass drag-aware-mixin (drag-view-mixin)
  ()
  (:default-initargs
    :drag-accepted-flavor-list $CLASP-Flavor
    ))

(defmethod drag-allow-drop-p ((view drag-aware-mixin))
  (with-drag-flavor (1 $CLASP-Flavor data-ptr data-size)
    (declare (ignore data-size))
    ;; `with-drag-flavor' does not return the value of its last form
    (return-from drag-allow-drop-p (f-can-receive-drop view data-ptr 0))))

(defmethod drag-receive-dropped-flavor ((view drag-aware-mixin)
                                        (flavor (eql $CLASP-Flavor))
                                        (data-ptr t) (data-size integer)
                                        (item-reference integer))
  (declare (ignore data-size item-reference))
  (receive-drop view data-ptr 0) 
  t)
  

;;; *******************************************************************************************
;;; EOF