#|
*************************
* Macintosh dialog item *
* subclasses for CLASP  *
*************************

24-Jul-94 ms   Created.

|#

(in-package "CLASP-INTERFACE")

;;
;; static text item with attachable function & args
;; 24-Jul-94 ms

(defclass clickable-text-item (static-text-dialog-item)
  ((item-action :initarg :item-action)
   (item-action-arg :initarg :item-action-arg)
   )
  (:documentation "something cool."))

(defmethod view-click-event-handler ((view clickable-text-item)
                                     (pt-where integer))
  (with-slots (item-action item-action-arg) view
    (funcall item-action (view-container view) item-action-arg)))

;;;
;;; number-entry-item
;;;

#|
================================================================
Purpose ========================================================
================================================================
defines the dialog item (view) class and methods for the
number entry gadget


================================================================
Status =========================================================
================================================================
In-progress.

================================================================
Change history =================================================
================================================================
1-Jul-94 ms	Created.
8-Jul-94 ms     Changed from float- to number- with :allow-floats initarg
24-Jul-94 ms    Beta finish with :allow-negatives

|#


(defclass number-entry-item (editable-text-dialog-item)
  ((allow-floats :initarg :allow-floats)
   (allow-negatives :initarg :allow-negatives)
   (allow-zero :initarg :allow-zero))
  (:documentation "An editable-text-dialog-item that only accepts number values.")
  (:default-initargs
    :allow-floats t
    :allow-negatives t
    :allow-zero t))

;;;
;;; custom key handler to constrain input
;;;

(defmethod view-key-event-handler :around ((number-entry-item number-entry-item) char)
  "Custom key handler for the number-entry-item."

  (with-slots (allow-floats allow-negatives) number-entry-item
    (let* ((dialog-text (dialog-item-text number-entry-item)))
      (cond ((and (or (char< char #\0)
                      (char> char #\9))
                  (or (char/= char #\.)
                      (not allow-floats))
                  (or (char/= char #\-)
                      (not allow-negatives))
                  (char/= #\BackArrow char)
                  (char/= #\ForwardArrow char)
                  (char/= #\Delete char)
                  (char/= #\rubout char))
             (return-from view-key-event-handler))
            ((and (char= #\. char)
                  (> (count #\. dialog-text) 0))
             (return-from view-key-event-handler))
            ((and (char= #\- char)
                  (> (count #\- dialog-text) 0))
             (return-from view-key-event-handler))
            (t t))
      (call-next-method))))

(defmethod view-key-event-handler :after ((nei number-entry-item) char)
  (when (and (char= char #\-)
             (> (position char (string-trim " " (dialog-item-text nei))) 0))
    (set-dialog-item-text nei (remove char (dialog-item-text nei)))))

;;;
;;; number-value method
;;;

(defmethod number-item-value ((number-entry-item number-entry-item) &key (report-error nil))
  "Parses the text of the number-entry-item into a floating point number."
  (let ((value (parse-float (dialog-item-text number-entry-item) :report-errors report-error)))
    (cond ((or (slot-value number-entry-item 'allow-zero)
               (not (zerop value)))
           (values value))
          (t
           (set-dialog-item-text number-entry-item "1")
           (values 1)))))
        


;;;
;;; Define variable-stdi .
;;;

(defclass variable-stdi (drag-aware-mixin static-text-dialog-item)
  ((variable-vset-stdi
    :initform nil
    :initarg :variable-vset-stdi
    :type CLASP::VARIABLE
    :accessor variable-vset-stdi
    :documentation "The most recently stored variable in this item."))
  (:documentation "A static-text-dialog-item subclass that supports
drag-and-drop of lists of clasp variables. Abbreviated 'dset-stdi'.")
  (:default-initargs
    :dialog-item-text "<NONE>"))

(defmethod (setf variable-vset-stdi) :after ((variable-new clasp::variable)
                                            (variable-stdi variable-stdi))
  "Updates variable-stdi's dialog-item-text to be variable-new's name or
'<NONE>' ."
  ;;
  (set-dialog-item-text variable-stdi (if variable-new
                                       (clasp::name variable-new)
                                       "<NONE>")))

(defmethod view-draw-contents :after ((variable-stdi variable-stdi))
  "Draws a frame around variable-stdi ."
  ;;
  ;; FIX: Frame dataset-stdi in its container.
  ;;
  )

(defmethod f-can-receive-drop ((view variable-stdi)
                               (object t)
                               (pt-global fixnum))
  "Return non-nil if object is a clasp variable or a sequence of clasp
variables."
  ;;
  (or (typep object 'clasp::variable)
      (and (typep object 'sequence)
           (every #'(lambda (item)
                      (typep item 'clasp::variable))
                  object))))

(defmethod invert-drop-highlight ((view variable-stdi)
                                 (object t)
                                 (pt-global integer))
  (invert-inside-gray-frame view))


(defmethod receive-drop ((view variable-stdi)
                         (object clasp::variable)
                         (pt-global integer))
  "Object is a clasp::variable . Sets view's dataset-dset-stdi to
object ."
  ;;
  (setf (variable-vset-stdi view) object))

(defmethod receive-drop ((view variable-stdi)
                         (object list)
                         (pt-global integer))
  "Object is a list of clasp variables Recurses on each element in object."
  ;;
  (loop for variable in (reverse object)
        do (when (f-can-receive-drop view variable pt-global)
             (receive-drop view variable pt-global))))

;;; dataset-pum.lisp

#|
================================================================
Purpose ========================================================
================================================================
Defines classes, functions, etc. for dataset drag-and-drop 
pop-up-menus

================================================================
Status =========================================================
================================================================
In-progress.



================================================================
Change history =================================================
================================================================
13-Jun-94 ms	Created.
21-Jul-94 ms    Beta finish.

|#

(defclass dataset-menu-item (menu-item)
  ((dataset
    :initform nil
    :initarg :dataset
    :type CLASP::dataset
    :accessor dataset-dsmi
    :documentation "The clasp dataset represented by this menu-item."))
  (:documentation "A menu-item subclass that represents a clasp
dataset. The title is set automatically based on the dataset. Used
by dataset-pop-up-menu . Abbreviated 'dsmi' ."))

(defmethod initialize-instance :after ((dataset-menu-item dataset-menu-item)
                                     &key dataset)
  "Errors if dataset is not a dataset Sets dataet-menu-item's title."
  ;;
  (check-type dataset clasp::dataset)
  ;; FIX: Is there ever not a name?:
  (set-menu-item-title dataset-menu-item
                       (clasp::name (dataset-dsmi dataset-menu-item))))

;;;
(defclass dataset-pop-up-menu (drag-aware-mixin pop-up-menu)
  ()
  (:documentation "A pop-up-menu subclass whose items must be
dataset-menu-item, and that supports drag-and-drop of lists of
datasets. Abbreviated 'dataset-pum'."))


(defmethod delete-selected-dataset-pum ((dataset-pop-up-menu dataset-pop-up-menu))
  "Removes the selected menu-item from dataset-pop-up-menu ."
  ;;
  (let ((menu-item-selected (selected-item dataset-pop-up-menu)))
    (cond (menu-item-selected
           (remove-menu-items dataset-pop-up-menu menu-item-selected)
           (invalidate-view dataset-pop-up-menu t)     ;show the removed item
           )
          (t
           ;; Beep to show there's no selection. FIX: Should we show a dialog?
           (ed-beep)))))

(defmethod l-datasets-dataset-pum ((dataset-pop-up-menu dataset-pop-up-menu))
  "Returns a list of datasets corresponding to dataset-menu-item in
dataset-pop-up-menu, ordered from top to bottom."
  ;;
  (map 'list #'(lambda (dataset-menu-item)
                 (dataset-dsmi dataset-menu-item))
       (menu-items dataset-pop-up-menu)))

(defmethod f-can-receive-drop ((view dataset-pop-up-menu)
                               (object t)
                               (pt-global fixnum))
  "Return non-nil if object is a clasp dataset or a sequence of clasp
datasets."
  ;;
  (or (typep object 'clasp::dataset)
      (and (typep object 'sequence)
           (every #'(lambda (item)
                      (typep item 'clasp::dataset))
                  object))))


(defmethod invert-drop-highlight ((view dataset-pop-up-menu)
                                 (object t)
                                 (pt-global integer))
  ;; Invert a gray frame just inside view. (Headache if it's thick!)
  (invert-inside-gray-frame view))


(defmethod receive-drop ((view dataset-pop-up-menu)
                         (object clasp::dataset)
                         (pt-global integer))
  "Object is a clasp::dataset. Adds to view a 'dataset-menu-item for
object."
  ;;
  ;; FIX: Make the last item (the one just dropped) selected/current.
  ;;
  (add-menu-items view (make-instance 'dataset-menu-item
                         :dataset object))
  (invalidate-view view t)              ;show the new item
  )


(defmethod receive-drop ((view dataset-pop-up-menu)
                         (object list)
                         (pt-global integer))
  "Object is a list of clasp datasets. Recurses on each element in object."
  ;;
  (loop for dataset in (reverse (remove-if-not #'(lambda (item) (typep item 'clasp::dataset))
                                               object))
        do (receive-drop view dataset pt-global)))

(defclass dataset-delete-button-di (button-dialog-item)
  ((dataset-pum
    :initform nil
    :initarg :dataset-pum
    :type dataset-POP-UP-MENU
    :accessor dataset-pum-dsdbdi
    :documentation "The dataset-pop-up-menu instance that this button
deletes selected items from."))
  (:documentation "A button-dialog-item subclass that's associated
with a dataset-pop-up-menu and allows deleting that pop-up's
currently selected dataset. The default dialog-item-text is 'Delete'.
Abbreviated 'vdbdi'.")
  (:default-initargs
    :dialog-item-text "Delete"
    :dialog-item-action
    #'(lambda (item)
        (delete-selected-dataset-pum (dataset-pum-dsdbdi item)))))


(defmethod initialize-instance :after ((dataset-delete-button-di dataset-delete-button-di)
                                     &key dataset-pum)
  "Errors if :dataset-pum is not a dataset-pop-up-menu ."
  ;;
  (check-type dataset-pum dataset-pop-up-menu))

;;;
;;; comment machine
;;;

; the comment class
(defclass comment (clasp::named-object-mixin)
  ((comment
    :initform nil
    :initarg :comment
    :type STRING
    :accessor comment-contents
    :documentation "The actual comment."))
  (:documentation "A class of comments for the history view."))

(defmethod initialize-instance :after ((comment comment) &key)
  (setf (clasp::name comment) (comment-contents comment)))

; the gadget that drives it
(defvar *comment-entry-text* '("Geneva" 9))

(defclass comment-entry-item (editable-text-dialog-item)
  ((destination
    :initform nil
    :initarg :destination
    :type symbol
    :accessor comment-destination
    :documentation "The nickname of the view in which the comment will be inserted."))
  (:default-initargs 
    :view-size #@(100 15)
    :dialog-item-text "enter comment here."
    :view-font *comment-entry-text*))

(defmethod activate-comment ((cei comment-entry-item))
  (let ((parent (view-named (comment-destination cei) (view-container cei))))
    (if parent
      (receive-comment parent
                       (make-instance 'comment
                         :comment (dialog-item-text cei))))
    (select-all cei)))

(defclass comment-activator (default-button-dialog-item)
  ((comment-item-tag
    :initform nil
    :initarg :activate-who-tag
    :type symbol
    :accessor activate-who-tag
    :documentation "The nickname of the editable-comment item to activate."))
  (:default-initargs
    :dialog-item-text "!"
    :view-font *comment-entry-text*
    :border-p nil
    :dialog-item-action #'(lambda (item) 
                            (activate-comment (view-named (activate-who-tag item)
                                                          (view-container item)))))
  (:documentation "Activates the comment item pointed to by comment-item-tag"))

;;
;; Result type
;;   for simple results from clasp like trimmed-mean
;;

(defclass clasp-result ()
  ((result-line
    :initform nil
    :initarg :result-text
    :type 'string
    :accessor clasp-result-text
    :documentation "The text line that we like to look at.")
   (result
    :initform nil
    :initarg :primary-result
    :type t
    :accessor clasp-primary-result
    :documentation "The actual result that we can spew."))
  (:documentation "This is a class of simple results that are not big enough to bother
with a table for. It has a verbose output for clasp windows and a primary-value for
drag & drop if that ever comes about."))

;;
;; HaCK text item.
;; sends a #'handle-arrow with the key in question to its view container
;; make sure you set up a handle-arrow method !

(defclass hack-text-item (editable-text-dialog-item)
  ()
  (:documentation "A wicked hack. Wickedly awesome. If a hack-text-item gets
an #\UpArrow or #\DownArrow it spits a handle-arrow to its view container.
Make sure you have a method set up for any of this item's possible view
container, and good luck."))

(defmethod view-key-event-handler :after ((hack-text-item hack-text-item) char)
  "Calls handle-arrow on its view-container with the character in question."
  (if (or (equal char #\UpArrow)
          (equal char #\DownArrow)
          (equal char #\Tab))
    (progn (handle-arrow (view-container hack-text-item) char) 
           (select-all hack-text-item))))

;;
;; font-entry-items : font, style, point-size pop ups
;; 9-13-94 ms

(defvar *np-fonts* (remove-if-not #'(lambda (item) (eq (string-width "W" (list item 12))
                                                         (string-width "i" (list item 12))))
                                    *font-list*))

(defun find-np-fonts ()
  (setf *np-fonts* (remove-if-not #'(lambda (item) (eq (string-width "W" (list item 12))
                                                         (string-width "i" (list item 12))))
                                    *font-list*)))

(defclass font-pum (pop-up-menu)
  ()
  (:documentation "A pop up menu for selecting fonts."))

(defmethod initialize-instance :after ((font-pum font-pum) &key (proportional-p t) (default nil))
  (if default
    (progn (add-menu-items font-pum (make-instance 'menu-item :menu-item-title default))
           (add-menu-items font-pum (make-instance 'menu-item :menu-item-title "-" :disabled t))))
  (mapcar #'(lambda (item) (add-menu-items font-pum
                                           (make-instance 'menu-item
                                             :menu-item-title item)))
          (if proportional-p
            *font-list*
            *np-fonts*)))

(defvar *point-sizes* '(8 9 10 12 14 16 18 24 30))

(defclass point-size-pum (pop-up-menu)
  ()
  (:documentation "A pop up menu for selecting point size."))

(defmethod initialize-instance :after ((point-size-pum point-size-pum) &key (default nil))
  (if default
    (progn (add-menu-items point-size-pum (make-instance 'menu-item :menu-item-title (format nil "~D" default)))
           (add-menu-items point-size-pum (make-instance 'menu-item :menu-item-title "-" :disabled t))))
  (mapcar #'(lambda (item) (add-menu-items point-size-pum
                                           (make-instance 'menu-item
                                             :menu-item-title (format nil "~D" item))))
          *point-sizes*))

(defvar *font-styles* '(:plain :bold))

(defclass font-style-pum (pop-up-menu)
  ()
  (:documentation "A pop up menu for selecting font style."))

(defmethod initialize-instance :after ((font-style-pum font-style-pum) &key (default nil))
  (if default
    (progn (add-menu-items font-style-pum (make-instance 'menu-item :menu-item-title (format nil "~S" default)))
           (add-menu-items font-style-pum (make-instance 'menu-item :menu-item-title "-" :disabled t))))
  (mapcar #'(lambda (item) (add-menu-items font-style-pum
                                           (make-instance 'menu-item
                                             :menu-item-title (format nil "~S" item))))
          *font-styles*))

;;;
;;; a checkbox class for disabling other items in the same view-container
;;;

(defclass disabler-checkbox-item (check-box-dialog-item)
  ((wathces-over
    :initform nil
    :initarg :watches-over
    :type LIST
    :accessor watches-over-who
    :documentation "a list of nicknames that the checkbox disables/enables on clicking."))
  (:documentation "A class of checkbox items which controls enabledness of other items
in the same dialog box."))

(defmethod initialize-instance :after ((dci disabler-checkbox-item) &key)
  "calls the click handler to get everything set up correctly.")

(defmethod view-click-event-handler :after ((dci disabler-checkbox-item) (pt-where fixnum))
  (declare (ignore pt-where))
  (let ((parent (view-container dci))
        (state (check-box-checked-p dci))
        item)
    (loop for item-name in (watches-over-who dci) do 
          (setf item (view-named item-name parent))
          (if item (if state 
                     (dialog-item-enable item)
                     (dialog-item-disable item))))))

;;;
;;; funky sampling items
;;;

;;; controller radio buttons
(defclass bs-ub-radio-button (radio-button-dialog-item)
  ((activate-ids ;; what to activate when it gets clicked
    :initform nil :initarg :activate-ids
    :accessor activate-ids)
   (deactivate-ids ;; what to deactivate
    :initform nil :initarg :deactivate-ids
    :accessor deactivate-ids)
   (activation-form ;; a form to evaluate
    :initform nil :initarg :activation-form
    :accessor activation-form))
  (:documentation "Radio buttons that de/activate other items in the bs dialog."))

(defmethod view-click-event-handler :after ((bs-ub-radio-button bs-ub-radio-button) (pt-where fixnum))
  (let* ((container (view-container bs-ub-radio-button)))
    (mapcar #'(lambda (item) (dialog-item-enable (view-named item container)))
            (activate-ids bs-ub-radio-button))
    (mapcar #'(lambda (item) (dialog-item-disable (view-named item container)))
            (deactivate-ids bs-ub-radio-button))
    (eval (subst bs-ub-radio-button 'self (activation-form bs-ub-radio-button)))))

;;; item pop-up-menu injector
(defvar *bivariate-stat-names* 
  '("clasp::correlation" "clasp::covariance" "clasp::t-test" "clasp::t-test-matched-pairs"))
(defvar *univariate-stat-names* '("clasp::mean" "clasp::median" "clasp::interquartile-range"
                                  "clasp::variance" "clasp::standard-deviation" "clasp::skew"))
(defvar *sample-calls* '(("(clasp::sample-normal-to-list
   mean sd sample-size)" . "Normal")
                         ("(clasp::sample-poisson-to-list
   mean sample-size)" . "Poisson")
                         ("(clasp::sample-binomial-to-list 
   p n sample-size)" . "Binomial")
                         ("(clasp::sample-uniform-to-list
   min max sample-size)" . "Uniform")
                         ("(clasp::sample-gamma-to-list
   order sample-size)" . "Gamma")))

(defclass item-injector-menu (pop-up-menu)
  ((inject-where :initform nil :initarg :inject-where :accessor inject-where)
   (inijections :initform nil :initarg :injections :accessor injections))
  (:documentation "When you select an item, it injects it into a dialog item."))

(defmethod initialize-instance :after ((iim item-injector-menu) &key)
  (setup-iim iim (injections iim)))

(defmethod setup-iim ((iim item-injector-menu) items)
  (setf (injections iim) items)
  (apply #'remove-menu-items iim (menu-items iim))
  (mapcar #'(lambda (item) 
              (add-menu-items iim (make-instance 'menu-item :menu-item-title 
                                                 (if (listp item)
                                                   (cdr item)
                                                   item))))
          items)
  (invalidate-view iim t))

(defmethod view-click-event-handler :after ((iim item-injector-menu) (pt-where fixnum))
  (let ((mitem (menu-item-title (selected-item iim)))
        (titem (view-named (inject-where iim) (view-container iim))))
    (if (listp (car (injections iim)))
      (setf mitem (car (rassoc mitem (injections iim)))))
    (set-dialog-item-text titem mitem)))

;;;
;;; menu items with values.
;;;

(defclass valued-menu-item (menu-item)
  ((value :initform NIL :initarg :value :accessor value))
  (:DOCUMENTATION "A menu item with a value."))

;;;

