#|
*****************************
* generic X variable dialog *
* boxes for Macintosh CLASP *
*****************************

22-Jul-94 ms    gathered into complete volume

|#

#|
================================================================
__One var generic===============================================
================================================================
dialog box for one variable generic functions

6-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#

(in-package "CLASP-INTERFACE")

;;;
;;; Define one-variable-dialog .
;;;

(defclass one-variable-dialog (statistic-dialog-mixin dialog)
  ((statistical-function :initarg :statistical-function)
   (statistical-function-name :initarg :statistical-function-name)
   (format-function :initarg :format-function))
  (:documentation "A dialog for getting data for the
generic one variable functions.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Function"
    :VIEW-SIZE #@(350 85)
    :VIEW-POSITION (best-spot 350 85)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(140 50)))))

(defmethod initialize-instance :after ((ovd one-variable-dialog) &key (categorical-ok t)
                                       &allow-other-keys)
  (setf (allow-categorical-p (view-named 'x-var-item ovd)) categorical-ok))

(defmethod f-can-compute-statistics-dialog ((one-variable-dialog one-variable-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   one-variable-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))

(defmethod compute-statistics-dialog ((one-variable-dialog one-variable-dialog))
  (with-slots (statistical-function statistical-function-name format-function)  one-variable-dialog
    (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                     one-variable-dialog))))
      (loop for x in l-var-x
            do (compute-statistics x statistical-function statistical-function-name format-function)))))


(defmethod compute-statistics ((iv clasp::variable) function name ffunc)
  "Calls function on the arguments, saving
results in the history."
  (let ((clasp-command (make-clasp-command
                        `(,function
                          (clasp::simplify ,iv))
                        :format-function
                        ffunc
                        :description
                        (list (list 'command function)
                              (list 'x-args iv))
                        )))
    (run-clasp-command clasp-command
                       :str-message (format nil "Computing ~a" name)
                       :f-update-dataset-view t)))

(defun one-variable-dialog-make (function &optional (name (string-capitalize (string function))) (ffunc nil)
                                          &key (help "Sorry, no help available.") (categorical-ok t))
  "Returns a new dialog instance for editing the function."
  ;;
  (let ((selected-variables (items-in-dock 'x-var-dock :select-type :select-or-all)))
    (if (and selected-variables *bypass-dialogs*)
      (loop for x in selected-variables do
            (compute-statistics x function name ffunc))
      (make-instance 'one-variable-dialog 
        :window-title name
        :statistical-function function 
        :statistical-function-name name
        :help help
        :format-function ffunc
        :categorical-ok categorical-ok))))

#|
================================================================
__two variable generic functions================================
================================================================
dialog box for two variable generic functions

6-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish.

|#
;;;
;;; Define two-variable-dialog .
;;;

(defclass two-variable-dialog (statistic-dialog-mixin dialog)
  ((statistical-function :initarg :statistical-function)
   (statistical-function-name :initarg :statistical-function-name)
   (format-function :initarg :format-function))
  (:documentation "A dialog for getting data for the
Median function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "goober"
    :VIEW-SIZE #@(350 100)
    :VIEW-POSITION (best-spot 350 100)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'y-var-item
                         :view-position #@(9 42)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(140 70)))))

(defmethod initialize-instance :after ((tvd two-variable-dialog) &key (categorical-x t) (categorical-y t)
                                       &allow-other-keys)
  (setf (allow-categorical-p (view-named 'x-var-item tvd)) categorical-x)
  (setf (allow-categorical-p (view-named 'y-var-item tvd)) categorical-y))

(defmethod f-can-compute-statistics-dialog ((two-variable-dialog two-variable-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   two-variable-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   two-variable-dialog)))
         (f-can-compute (and l-var-x l-var-y))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one x item")
                                     ((null l-var-y)
                                      "Need at least one y item")
                                     (t "Must compute on equal amounts of x & y variables."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((two-variable-dialog two-variable-dialog))
  (with-slots (statistical-function statistical-function-name format-function)  two-variable-dialog
    (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                     two-variable-dialog)))
           (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                     two-variable-dialog))))
      (if (> (length l-var-y) 1)
        (loop for x in l-var-x
              for y in l-var-y
              do (compute-tv-statistic x y 
                                     statistical-function 
                                     statistical-function-name 
                                     format-function))
        (loop for x in l-var-x
              do (compute-tv-statistic x (car l-var-y) statistical-function 
                                     statistical-function-name 
                                     format-function))))))

(defmethod compute-tv-statistic ((dv clasp::variable) (iv clasp::variable) function name format-function)
  "Calls function on the arguments, saving
results in the history."
  (let ((clasp-command (make-clasp-command
                        `(,function
                          (clasp::simplify ,iv)     
                          (clasp::simplify ,dv))
                        :format-function
                        format-function
                        :description
                        (list (list 'command function)
                              (list 'x-args dv)
                              (list 'y-args iv))
                        )))
    (run-clasp-command clasp-command
                       :str-message (format nil "Computing ~a" name)
                       :f-update-dataset-view t)))


(defun two-variable-dialog-make (function &optional (name (string-capitalize (string function))) 
                                          (format-function nil) &key (help "Sorry, no help available.")
                                          (categorical-x t) (categorical-y t))
  "Returns a new dialog instance for editing the function."
  ;;
  (let ((xvars (items-in-dock 'x-var-dock :select-type :select-or-all))
        (yvars (items-in-dock 'y-var-dock :select-type :select-or-all)))
    (if (and *bypass-dialogs* xvars yvars)
      (if (> (length yvars) 1)
        (loop for x in xvars
              for y in yvars do
              (compute-tv-statistic x y function name))
        (loop for x in xvars do
              (compute-tv-statistic x (car yvars) function name format-function)))
      (make-instance 'two-variable-dialog 
        :statistical-function function 
        :statistical-function-name name
        :window-title name
        :help help
        :format-function format-function
        :categorical-x categorical-x
        :categorical-y categorical-y
        ))))

#|
================================================================
__One variable transformations==================================
================================================================
dialog box for One variable transformations

6-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish.

|#
;;;
;;; Define one-var-trans-dialog .
;;;

(defclass one-var-trans-dialog (statistic-dialog-mixin dialog)
  ((transform-function :initarg :transform-function)
   (transform-function-name :initarg :transform-function-name))
  (:documentation "A dialog for getting data for the
one variable transformation functions.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "ov trans"
    :VIEW-SIZE #@(350 85)
    :VIEW-POSITION (best-spot 350 85)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Variables"
                         :view-nick-name 'x-var-item
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(140 50)))))

(defmethod initialize-instance :after ((ovtd one-var-trans-dialog) &key (categorical-ok t)
                                       &allow-other-keys)
  (setf (allow-categorical-p (view-named 'x-var-item ovtd)) categorical-ok))

(defmethod f-can-compute-statistics-dialog ((one-var-trans-dialog one-var-trans-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   one-var-trans-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((one-var-trans-dialog one-var-trans-dialog))
  (with-slots (transform-function transform-function-name)  one-var-trans-dialog
    (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                     one-var-trans-dialog))))
      (loop for x in l-var-x
            do (compute-transformation x transform-function transform-function-name)))))


(defmethod compute-transformation ((iv clasp::variable) function name)
  "Calls function on the arguments, saving
results in the history."
  (let ((clasp-command (make-clasp-command
                        `(,function
                          ,iv)
                        :description
                        (list (list 'command function)
                              (list 'x-args iv)))))
    (run-clasp-command clasp-command
                       :str-message (format nil "Computing ~a" name)
                       :f-update-dataset-view t)))


(defun one-var-trans-dialog-make (function &optional (name (string-capitalize (string function)))
                                           &key (help "Sorry, no help available.")
                                           (categorical-ok t))
  "Returns a new dialog instance for editing the function."
  ;;
  (let ((vars (items-in-dock 'x-var-dock :select-type :select-or-all)))
    (if (and *bypass-dialogs* vars)
      (loop for x in vars do
            (compute-transformation x function name))
      (make-instance 'one-var-trans-dialog 
        :window-title name
        :help help
        :transform-function function 
        :transform-function-name name
        :categorical-ok categorical-ok))))
