;;;
;;; documentation/help package for CLASP
;;;

(in-package :clasp-interface)

(defclass doc-viewer (dialog)
  ((doc-string :initarg :doc-string :accessor doc-string))
  (:default-initargs 
    :view-size #@(350 150)
    :window-type :document-with-grow
    :view-subviews
    (list (make-instance 
            #-CCL-3 'ccl::scrolling-fred-dialog-item
            #+CCL-3 'ccl::scrolling-fred-view
            :view-nick-name 'fred
            :view-size #@(335 135)))))

(defmethod initialize-instance :after ((dv doc-viewer) &key)
  (let ((ds (doc-string dv)))
    (ed-insert-with-style (view-named 'fred dv)
                          (if (stringp ds)
                            ds
                            (if (and ds (symbolp ds))
                              (eval ds)
                              "No documentation on this item."))
                          NIL)))

(defmethod window-close :before ((dv doc-viewer))
  (if (symbolp (doc-string dv))
    (set (doc-string dv) (dialog-item-text (view-named 'fred dv)))))

(defmethod window-size-parts ((dv doc-viewer))
  (when (view-named 'fred dv)
    (set-view-position (view-named 'fred dv)
                       0 0)
    (set-view-size (view-named 'fred dv)
                   (- (point-h (view-size dv)) 15)
                   (- (point-v (view-size dv)) 15))))

(defmethod show-help ((title string) help)
  (make-instance 'doc-viewer
    :window-title (concatenate 'string title " Help")
    :doc-string help))

;;; **********************************************
;;;  HELP STRINGS FOR MENU ITEM DIALOGS
(defmethod display-help ((window statistic-dialog-mixin))
  (show-help (window-title window) (help window)))

;;; ***********************
(defvar *histogram-help* "Creates a histogram of X.")
;;; ***********************
(defvar *scatterplot-help* "Creates a scatterplot of X on Y.

X and Y must be numeric variables.
Color by (optional) should be a variable to
  partition the graph by.")
(defvar *cluster-plot-help* "Creates a cluster plot of X on Y.

X and Y must be numeric variables.")
;;; ***********************
(defvar *row-line-help* "Creates a line plot of Y on row number.

Y must be a numeric variable.
X shift indicates a constant to shift
  the x axis (row number) by.")
;;; ***********************
(defvar *line-plot-help* "Creates a line plot of X on Y.

X and Y must be numeric variables.")
;;; ***********************
(defvar *regression-plot-help* "Creates a scatterplot and regression line
  for X on Y.

X and Y must be numeric variables.
Color by (optional) should be a variable to
  partition the graph by.")
;;; ***********************
(defvar *overlay-graph-help* "Overlays the specified graphs.

The graphs to overlay may be selected from either
  the graph caddy or the history view.")
;;; ***********************
(defvar *data-length-help* "Computes the data length of X.")
;;; ***********************
(defvar *mean-help* "Computes the arithmetic mean of X.
X must be a numeric variable.")
;;; ***********************
(defvar *variance-help* "Computes the variance of X.
X must be a numeric variable.")
;;; ***********************
(defvar *std-dev-help* "Computes the standard deviation of X.
X must contain numeric data.")
;;; ***********************
(defvar *minimum-help* "Determines the minimum value of X.
X must contain numeric data.")
;;; ***********************
(defvar *maximum-help* "Determines the maximum value of X.
X must contain numeric data.")
;;; ***********************
(defvar *range-help* "Computes the range of X, which is
  maximum - minimum.

X must contain numeric data.")
;;; ***********************
(defvar *quantile-help* "Computes the qth quantile k of X such that
q * n elements of the sorted distribution are <= k
and (1-q) * n elements are >= k.

Q is between 0 and 1; Q=0.5 computes the median.
X must contain numeric data.")
;;; ***********************
(defvar *median-help* "Computes the median of X, which is the
  value that divides the sorted distribution in
  half.

X must contain numeric data.")
;;; ***********************
(defvar *trimmed-mean-help* "Computes the mean of X after trimming 
  a percentage of the highest and lowest elements
  of the sorted data. The amount trimmed from each
  tail is equal to Percentage * n, where n is the
  data length.

X must contain numeric data.
Percentage is the amount to trim the data, 
  and is between 0 and 1.")
;;; ***********************
(defvar *mode-help*  "Computes the mode of X.

If X is discrete, computes the highest frequency item.
If X is continuous, computes the center of the highest 
  density cluster (estimated mode).
Best Guess tells CLASP to make its best guess as to
  whether X is continuous or discrete.")
;;; ***********************
(defvar *iqr-help* "Computes the interquartile range of X, which
  is the difference between the .75 and .25
  quantiles of the data.

X must contain numeric data.")
;;; ***********************
(defvar *skew-help* "Computes the skew of X.

X must contain numeric data.
X must have standard deviation <> 0.")
;;; ***********************
(defvar *statistical-summary-help* "Produces a table of statistics for X.
X must contain numeric data.
Computes data length, minimum, maximum, range, median, 
  mode, mean, variance, standard deviation, interquartile 
  range, and skewness.")
;;; ***********************
(defvar *covariance-help* "Computes the covariance of X and Y.

X and Y must be numeric variables.")
;;; ***********************
(defvar *correlation-help* "Computes the Pearson's correlation coefficient
  for X and Y.

X and Y must be numeric variables.")
;;; ***********************
(defvar *contingency-help* "Creates a contingency table for X and Y.")

(defvar *cross-correlation-help* "Computes the correlation coefficient for
  pairs of X and Y where the pairs are taken
  <X(n),Y(n+lag)>.

One coefficient is produced for each lag in the
  range minlag..maxlag.
X and Y must contain numerical data.")

(defvar *autocorrelation-help* "Computes the correlation coefficient for
  pairs within X where the pairs are taken to be
  <X(n),X(n+lag)>.

One coefficient is produced for each lag in the
  range minlag..maxlag.
X must contain numerical data.")

(defvar *partial-correlation-help* "Computes the correlation coefficient for X and Y
  holding a set of variables constant.

X and Y must contain numerical data.")

(defvar *correlation-table* "Computes a table of the correlation coefficients
  for all pairings of the supplied X variables.

All X variables must contain numerical data.")

#+FBD
(defvar *fbd-help* "Produces a causal model for the IV(s) on DV.

Both the IV(s) and DV must be numerical variables.
The result is an adjacency matrix. A <-' indicates that
  the column appears to have an effect on the row where the
  symbol is located.")

(defvar *cdf-help* "Computes p values for common distributions including
  F, Z, and t.

DF is degrees of freedom and must be a positive integer.")
;;; **********************************************
;;; MANIPULATE DIALOG HELP

(defvar *merge-help* "Combines the rows of two or more datasets.

For best results, datasets to be merged should
  share variable names.")
;;; ***********************
(defvar *partition-help* "Creates a new dataset by extracting rows
  which satisfy a partition clause.

Include vars are the columns to include in
  the resulting dataset. Supplying no args
  uses the default, which is to include all
  columns from the original dataset.

The partition clause should be a LISP form
  which references variables by name and
  evaluates to true or nil.

example: (and (> var-1 var-2) (eql var-3 'good))")
;;; ***********************
(defvar *partition-on-help* "Partitions a dataset by each possible
  value for the partition variable.

The partition variable should be discrete
  and must be from the dataset being partitioned.

When multiple partition variables are specified,
  the dataset is partitioned by each possible value
  in the cross product of the specified variables.")
;;; **********************************************
;;; TRANSFORMATION DIALOG HELP

(defvar *add-constant-help* "Creates a new variable by adding a constant
  to each row in X.

X must contain numerical data.")
;;; ***********************
(defvar *mul-constant-help* "Creates a new variable by multiplying each
  element in X by a constant.

X must contain numerical data.")
;;; ***********************
(defvar *sort-help* "Creates a new variable by sorting X in
  ascending order.")
;;; ***********************
(defvar *sort-index-help* "Creates a new variable by sorting a variable and 
  using that as an index to reorder X.")
;;; ***********************
(defvar *rebin-help* "Creates a new variable by discretizing X
  into specified bins.

X must contain numerical data.
The bins may be specified by defining a start and
  end bin and then specifying the bin size or
  number of bins.")
;;; ***********************
(defvar *recode-help* "Creates a new variable by replacing distinct
  elements of X with a specified value.

Elements are recoded by selecting the original
  values, entering a new value in the text box,
  and clicking SET. When all proper assignments
  have been made, click RECODE.")
;;; ***********************
(defvar *user-defined-help* "Creates a new variable by computing an
  expression over a given dataset.

Variables are refered to by name in the LISP expression.
example :
  (/ variable-a (log variable-b))")
;;; ***********************
(defvar *natural-log-help* "Creates a new variable by computing the
  natural log on the data of X.

If X contains negative values, this transformation
  produces complex results.")
;;; ***********************
(defvar *log10-help* "Creates a new variable by computing the
  natural log on the data of X.

If X contains negative values, this transformation
  produces complex results.")
;;; ***********************
(defvar *derivative-help* "Creates a new variable by computing the
  discrete derivative of X.

X must be a numeric variable.")
;;; ***********************
(defvar *smooth-4253h-help* "Creates a new variable by smoothing X.
  Performs median-4, median-2, median-5, median-3,
  then hanning smoothing on X.

X must be a numeric variable.")
;;; ***********************
(defvar *smooth-help* "Creates a new variable by smoothing X with a
  mean or median smooth with window size in
  the range 2..5")
;;; ***********************
(defvar *standardize-help* "Creates a new variable by standardizing
  X to the range -1..1

X must be a numeric variable.")

;;; **********************************************
;;; TEST DIALOG HELP

(defvar *confi-t-help* "Computes confidence intervals on X using
  the t statistic.

X must be a numeric variable.
Confidence is in the range 0..1")
;;; ***********************
(defvar *confi-z-help* "Computes confidence intervals on X using
  the z statistic.

X must be a numeric variable.
Confidence is in the range 0..1")
;;; ***********************
(defvar *confi-prop-help* "Computes confidence intervals of a proportion
  of trials and successes.

Confidence is in the range 0..1")
;;; ***********************
(defvar *d-test-help* "Performs a D Test on X and Y, which is
a bootstrap version of the two sample t-test.

X and Y must be numerical variables.")
;;; ***********************
(defvar *t-test1-help* "Calculates the T statistic for the mean of
  X under the null hypothesis that it is
  a given mean.

X must contain numerical data.")
;;; ***********************
(defvar *t-test2-help* "Calculates the T statistic for the difference
  in means for X and Y under the null
  hypothesis that it should equal zero.

X and Y must contain numerical data.")
;;; ***********************
(defvar *t-testm-help* "Returns the T statistic for two matched
  samples X and Y.

X and Y must be equal length and contain
  numerical data.")
;;; ***********************
(defvar *chi-2x2-help* "Performs chi square test on X and Y.

X and Y must be binary (two-valued) variables.
Chi square will fail if the contingency table of X on Y has
  any zero row or column margins.")
;;; ***********************
(defvar *chi-RxC-help* "Performs chi square and g tests on X and Y.

Chi square will fail if the contingency table of X on Y has
  any zero row or column margins.
The g test will fail if the contingency table of X on Y has
  any zero cells.")
;;; ***********************
(defvar *anova-one-help* "Performs an analysis of variance on
  X and Y.

X is the independant variable, and should be discrete.
Y is the dependant, and must contain numerical data.
Enabling scheffe tests will produce a scheffe table.
Enabling confidence intervals will compute comfidence
  intervals for the group means.")
;;; ***********************
(defvar *anova-two-help* "Performs a two-way analysis of variance
  on X and Y.

X1 and X2 are the independant variables, and should 
  be discrete.
Y is the dependant, and must contain numerical data.")
;;; ***********************
(defparameter *anova-toolbox-help*
"A toolbox filled with different ANOVA designs.

See the CLASP manual section on ANOVA to decide which
  design is best for your data.

X, fixed, and random factors must be discrete.
Y is the dependant variable and must contain
  numerical data.")
;;; ***********************
(defparameter *contrast-toolbox-help*
"Allows hypothesis testing on specific groups within
  variables.

First, specify the grouping variable, GV, and the data
  variable, DV.

To create contrasts, load the the two contrast boxes with
  groups by selecting each group in the large list and
  clicking 'Add' under the appropriate contrast box.
  When you have fully specified a contrast, click on
  'Insert Contrast'. The contrast will appear in the
  pull down menu above.

Once you have specified a set of contrasts, click 
  'Significance' to perform the test. Remember that if
  all of the contrasts are not orthogonal, your results
  will not be independant.")
;;; ***********************
(defvar *chi-counts-help* "Performs chi square analysis on a given
  2 X 2 contingency table.

No row or column should have a total of zero.")
;;; ***********************
(defvar *3d-ct-help* "Tests several hypotheses about three-way relationships
  based on contingency table analysis.

Each test precludes the next. 
 (i.e. choosing not to test for singular independence
  precludes testing for conditional independence, etc.)

HA Iterations must be an integer, and specifies the 
  number of iterations that the homogenous association
  test will take when creating the table of expected
  values. Higher = more accuracy, but slower")
;;; ***********************
(defvar *brief-lr-help* "Produces a brief summary of the regression
  of X on Y.

Computes the slope, intercept, r square, standard
  error of the slope, and p.

X and Y must be numeric variables.")
;;; ***********************
(defvar *verbose-lr-help* "Produces a full summary of the regression
  of X on Y.

Produces the brief summary with an anova
  table of X on Y.")
;;; ***********************
(defvar *multi-lr-help* "Performs a linear regression with multiple
  independant variables (Xs).

All Xs and Y must contain numerical data.")
;;; ***********************
(defparameter *contrast-tb-help* "Performs significance tests on orthogonal contrasts.

A grouping variable, GV, and a dependant variable, DV,
should be specified in the appropriate drop slots. GV
must be categorical. When specified, the values of GV
are displayed in left-hand side of the dialog. The data
in DV can be grouped under the headings of GV, and are
used to make the contrasts A and B.")
;;; **********************************************
;;; DISTRIBUTION DIALOG HELP

(defvar *uniform-help* "Creates a dataset consisting of variables drawn
  from a uniform distribution.

The distribution is distributed uniformly from min 
  to max.")
;;; ***********************
(defvar *binomial-help* "Creates a dataset with variables drawn from a 
  binomial distribution.

P is probability of success between 0 and 1
N is number of retrials/datum")
;;; ***********************
(defvar *normal-help* "Creates a dataset with variables drawn from a 
  normal distribution.

Each variable (sample) has the specified mean and 
  standard distribution.")
;;; ***********************
(defvar *poisson-help* "Creates a dataset with variables drawn from a
  poisson distribution.")
;;; ***********************
(defvar *gamma-help* "Creates a dataset with variables drawn from a 
  gamma distribution.")
;;; ***********************
(defvar *monte-carlo-help* "Creates a dataset by calculating a statistic
  on a reproducable distribution over a series
  of resamples.

Function call should be a LISP expression which 
  generates a sample. 
If using one of the canned function calls, be sure to 
  replace the arguments with actual numerical data.
The statistic must be supplied by name and must accept
  a sequence (univariate) or two sequences (bivariate)
  as arguments.")
;;; ***********************
(defvar *boot-help* "Creates a dataset by calculating a statistic
  on samples from a variable or variables.

The samples may be drawn in a variety of ways :
 - Sample with replacement
   generates samples by drawing with replacement from 
   only the corresponding variable (i.e. X from X)
 - Sample without replacement 
   generates samples by drawing without replacement from 
   only the corresponding variable (i.e. X from X)
 - Swap paired items (bivariate only)
   generates samples by swapping pairwise elements
   between the X and Y groups with probability = 0.5
 - With replacement from pool (bivariate only)
   generates samples by drawing both variates from X and
   Y's joint distribution with replacement
 - Without replacement from pool (bivariate only)
   generates samples by drawing both variates from X and
   Y's joint distribution without replacement

The statistic must be supplied by name and must accept
  a sequence (univariate) or two sequences (bivariate)
  as arguments.")
;;; ***********************
(defvar *mixed-help* "Creates a dataset by calculating a statistic
  on monte carlo, randomized, or bootstrapped 
  distributions.

Function call should be a LISP expression which 
  generates a sample. (if using monte carlo)
If using one of the canned function calls, be sure to 
  replace the arguments with actual numerical data.
  (if using monte carlo)

The statistic must be supplied by name and must accept
  a sequence (univariate) or two sequences (bivariate)
  as arguments.")

;;; **********************************************
;;; **********************************************
;;; methods & variables for other kinds of windows

(defvar *clasp-window-help* "The main CLASP window displays all
  active datasets and a log of results.

Control clicking sends variable items
  to the X Dock and option click sends them
  to the Y Dock.

Option clicking sends history items to the
  Notebook and double clicking graph or table
  items opens them for viewing.")

(defmethod display-help ((clasp-window clasp-window))
  (show-help "CLASP Window" '*clasp-window-help*))

;;; ********************

(defvar *clasp-notebook-help* "The CLASP Notebook is for storing special
  results.

Results from a session may be brought to the CLASP
  Notebook by option clicking them in the CLASP
  window's history view.

Items in the Notebook may be automatically
  pasted into another apple events aware 
  application by clicking the log button.")

(defmethod display-help ((clasp-notebook clasp-notebook))
  (show-help "CLASP Notebook" '*clasp-notebook-help*))

;;; ********************
(defvar *palette-help* "CLASP Palettes store results of a certain
  type for easy browsing. There are graph and
  table palettes, which respond to clicks in the
  same way the CLASP window's history view does.")

(defmethod display-help ((clasp-caddy clasp-caddy))
  (show-help (window-title clasp-caddy)
             '*palette-help*))
;;; ********************
(defvar *listener-help* "The listener is a window in which LISP expressions
  may be evaluated. This window responds to standard
  cut and paste operations as well as emacs-style
  editing keystrokes such as control-k.")

(defmethod display-help ((listener listener))
  (show-help (window-title listener)
             '*listener-help*))
;;; ********************
(defvar *graph-help* "Graph windows represent their data graphically.

Graphs may be resized, cut to the clipboard,
  and edited by control-double clicking the
  graph window or choosing Edit from the graph
  menu.")

(defmethod display-help ((pw graph-window))
  (show-help "Graph Window" '*graph-help*))
;;; ********************
(defvar *table-help* "Statistical tables are structured representations
  of numerical results.

Tables may be cut to the clipboard.")
(defvar *group-means-help* "Group Means tables represent the difference
  between group means produced out by an
  analysis of variance.

Group means may be plotted and cut to the
  clipboard.")

(defvar *scheffe-help* "Scheffe tables present the scheffe statistic
  for pairwise matches of group means in a
  one way analysis of variance.

Control click displays the scheffe statistic.
Option click displays the significance of the 
  scheffe statistic.")

(defvar *lag-table-help* "Lat tables represent statistics which
  are performed over a series of different
  lag values, such as autocorrelation.

Lag tables may be plotted and cut to the
  clipboard.")
(defvar *contingency-table-help* "Contingency tables represent observed
  frequencies between the possible values
  of two variables.

Control-click displays row proportions.
Option-click displays column proportions.
Double-click displays observed frequencies.

Contingency tables may be cut to the clipboard.")
(defvar *censored-data-help* "Censored data tables are created when one or more
  of the X variables in a multiple linear regression
  are found to be linearly dependant.

The table lists variables which were determined to be 
  dependant on others in the regression; it is 
  suggested that the regression be recomputed without
  these variables.")

#| NEW-TABLES 11/14 
(defmethod display-help ((statistical-table statistical-table))
  (show-help "Statistical Table" '*table-help*))
(defmethod display-help ((group-means-table group-means))
  (show-help "Group Means Table" '*group-means-help*))
(defmethod display-help ((scheffe-table scheffe-table))
  (show-help "Scheffe Table" '*scheffe-help*))
(defmethod display-help ((lag-table lag-table))
  (show-help "Lag Table" '*lag-table-help*))
(defmethod display-help ((contingency-table contingency-table))
  (show-help "Contingency Table" '*contingency-table-help*))
(defmethod display-help ((censored-data-table censored-data-table))
  (show-help "Censored Columns Table" '*censored-data-help*))
(defmethod display-help ((table-view-window table-view-window))
  (display-help (table-view-table (view-named 'table-array-item table-view-window))))
|#

;;; ******************
(defvar *data-view-help* "Data view windows display the contents
  of a variable or dataset.

Double clicking an item provides a dialog
  to change that item.")
(defmethod display-help ((data-view-window data-view-window))
  (show-help "Variable/Dataset View" '*data-view-help*))

;;; ******************
#| NEW-TABLES 11/14 
(defvar *composite-table-help* "Composite table windows group together the
  individual tables produced by verbose analyses.

Composite tables may be cut to the clipboard.")

(defmethod display-help ((composite-table-window composite-table-window))
  (show-help "Composite Table Window" '*composite-table-help*))
|#
;;; ******************
(defvar *graph-editor-help* "The graph editor allows you to tune each
  graph to the desired parameters.

- Custom X and Y bounds define the portion of the graph 
    to be displayed in the graph window.

The following parameters refer to individual plot
  groups within the graph. First select a plot group
  in the list at the center of the graph preference
  window, then set the preferences, and then choose
  CHANGE to apply the preferences.

- Color colors all the graph elements of the plot group
    to the specified color.
- X shift adds a constant to the X portion of the data
    points in the graph. This preference only applies to
    line and row-line plots.
- Number of bins defines the number of bins into which
    continuous data should be divided, and only applies
    to histograms.

Click the PLOT button to view or redraw graphs which
  have changed.")

(defmethod display-help ((graph-editor graph-editor))
  (show-help "Graph Editor" '*graph-editor-help*))

;;; ******************
;;; WARNING !! THE FOLLOWING IS TRULY EVIL
;;; BUT THEY LEFT ME NO CHOICE. HAVE A NICE DAY.

(fmakunbound 'ccl::ed-help)
(defmethod ccl::ed-help (&rest lame)
  (declare (ignore lame))
  (display-help (front-window)))
