;;;
;;;  interface-support-fns.lisp
;;;

#|

support functions for the proper operation of CLASP through a mac interface

13-Jul-94  ms  File created

|#

(in-package "CLASP-INTERFACE")

(defun number-or-val (input val)
  (if (typep input 'number)
    input
    val))

(defun parse-int-or-zero (invar)
  "if the input string can be parsed as an integer, returns that integer, else returns 0"
  (let ((val (parse-integer invar :junk-allowed t)))
    (if val
      val
      0)))

;;;
;;; WARNING WARNING :: THE FOLLOWING FUNCTION PASSES CONTROL TO AN UNDOCUMENTED
;;;    MCL FUNCTION CCL::PARSE-FLOAT. I DO NOT KNOW WHY THEY WERE HIDING IT,
;;;    BUT I EXPECT THEY COULD YANK IT OR SOMETHING AND LEAVE US HANGING.
;;;

(defun parse-float (number-string &key (report-errors NIL))
  (let* ((useable-string (string-trim " " number-string))
         (int-val (ignore-errors (parse-integer useable-string)))
         (val (ignore-errors (ccl::parse-float useable-string
                                               (length useable-string) 0))))
    (if int-val
      int-val
      (if (and (null val) report-errors)
        :unparsable-number
        val))))

;;;

(defmethod deselect-cells ((view sequence-dialog-item))
  "deselects all the selected cells in a sequence-dialog-item"
  (let* ((s-cells (selected-cells view))
         )
    (loop for x in s-cells do
          (cell-deselect view (point-h x) (point-v x)))))

;;;
;;; functions for placing windows
(defun intersect-level (dx1 dy1 sx1 sy1 dx2 dy2 sx2 sy2)
  "returns an intersection level of the windows in question."
  (let ((px1 (max dx1 dx2))
        (py1 (max dy1 dy2))
        (px2 (min (+ dx1 sx1) (+ dx2 sx2)))
        (py2 (min (+ dy1 sy1) (+ dy2 sy2)))
        level)
    (setf level (/ (* (- px2 px1) (- py2 py1)) (* sx1 sy1)))
    (if (or (< px2 px1) (< py2 py1)) 0 
        (if (> level 1) 1 (coerce level 'float)))))

(defun best-spot (sx sy &optional (dxi 0) (dyi 0) (level 0))
  "Finds a good location for a window and returns that location"
  (let* ((dock (find-clasp-dock))
         (dock-pos (if dock (view-position dock)))
         (dock-size (if dock (view-size dock)))
         (cw (clasp-window-front))
         (cw-pos (if cw (view-position cw)))
         (cw-size (if cw (view-size cw)))
         (dx dxi)
         (dy dyi)
         (recur nil)
         )

    (if (> 40 dy) (setf dy 40))
    ;; make sure we don't come up under the dock (if possible)
    (if dock
      (if (> (intersect-level dx dy sx sy
                              (point-h dock-pos)
                              (point-v dock-pos)
                              (point-h dock-size)
                              (point-v dock-size)) 0)
        ;; better move it
        (if (< (+ (point-h dock-pos) (point-h dock-size) sx) *screen-width*)
          (setq dx (+ (point-h dock-pos) (point-h dock-size) 3))
          (if (< (+ (point-v dock-pos) (point-v dock-size) sy) *screen-height*)
            (setq dy (+ (point-v dock-pos) (point-v dock-size) 20))
            ))))
    (if cw
      (if (> (intersect-level dx dy sx sy
                              (point-h cw-pos)
                              (point-v cw-pos)
                              (point-h cw-size)
                              (point-v cw-size)) 0)
        ;; better move it more
        (progn
          (setf recur t)
          (if (< (+ (point-v cw-pos) (point-v cw-size) sy) *screen-height*)
            (setq dy (+ (point-v cw-pos) (point-v cw-size) 20))
            (if (< (+ (point-h cw-pos) (point-h cw-size) sx) *screen-width*)
              (setq dx (+ (point-h cw-pos) (point-h cw-size) 3))
              (setf recur nil)
              )))))
    (if (and recur (<= level 1))         ;; limited recursion
      (best-spot sx sy dx dy (1+ level)) ;; make sure we didn't put it someplace dumb
      (make-point dx dy))                ;; DONE !!
    ))

(defmethod place-window-brilliantly ((window window))
  "actually moves a window out from under the clasp dock & away from the clasp window."
  (let ((point (best-spot (point-h (view-size window))
                          (point-v (view-size window))
                          (point-h (view-position window))
                          (point-v (view-position window)))))
    (set-view-position window (point-h point) (point-v point))))

;;;
(defun view-font-height (view)
  "determines the total height of the view's font"
  (multiple-value-bind (as de mx ld) (font-info (view-font view))
    (declare (ignore mx ld))
    (+ as de)))

(defun font-height (fs)
  "determines the height of a font."
  (multiple-value-bind (as de mx ld) (font-info fs)
    (declare (ignore mx ld))
    (+ as de)))

(defun explain-missing-function (function-name)
  (gui-warn (format nil "This function must be performed in the listener : ~A" function-name)))

;;
;; trim-to-maxlen
;; trims an item so that it doesn't exceed maxlen
;;

(defmethod trim-to-maxlen ((item symbol) (maxlen integer) &key)
  (trim-to-maxlen (symbol-name item) maxlen))

(defmethod trim-to-maxlen ((item string) (maxlen integer) &key (too-long-cond :truncate-add-elipses))
  (if (< (length item) maxlen)
    item
    (ecase too-long-cond
      (:truncate
       (subseq item 0 maxlen))
      (:truncate-add-elipses
       (if (< maxlen 3)
         (error "~s cannot be truncated to length ~D." item maxlen)
         (concatenate 'string (subseq item 0 (- maxlen 3)) "...")))
      (:error
       (error "~s cannot be truncated to length ~D." item maxlen)))))
       
(defmethod trim-to-maxlen ((item ratio) (maxlen integer) &key)
  (trim-to-maxlen (coerce item 'float) maxlen))

(defmethod trim-to-maxlen ((item t) (maxlen integer) &key)
  item)

;; the float method is solid, but you can't get too restrictive, or else, it won't
;;  be entirely predictable
(defmethod trim-to-maxlen ((item float) (maxlen integer) &key (too-long-cond :error))
  (let* ((flstr (format nil "~F" item))
         (dploc (search "." flstr)))
    (if (<= (length flstr) maxlen)
      item
      (if (or (not dploc) (ignore-errors (> dploc maxlen)))
        ;; problem
        (ecase too-long-cond
          (:ignore item)
          (:error (error "~F cannot be truncated to length ~D." item maxlen))
          (:convert (format nil (format nil "~~~D,~DE" maxlen (abs (- maxlen 5)))
                            item)))
        (if (or (= dploc maxlen) (= (1+ dploc) maxlen))
          (prog1 (round item))
          (coerce (/ (round (* item (expt 10 
                                                 (- (- (length flstr) dploc 1) 
                                                    (- (length flstr) maxlen)))))
                     (expt 10 (- (- (length flstr) dploc 1) 
                                 (- (length flstr) maxlen)))) 
                  'float))))))

(defmethod name-and-dataset ((item t))
  (format nil "~:(~A~)" item))

(defmethod name-and-dataset ((item clasp::named-object-mixin))
  (let ((dataset (ignore-errors (clasp::variable-dataset item))))
    (if dataset 
      (format nil "~:(~A~)[~:(~A~)] " (clasp::name item) (clasp::name dataset))
      (format nil "~:(~A~)" (clasp::name item)))))

;;;
;;; font & window rebuild stuff for preference editing
;;;

(defun resize-cells-cw ()
  "resize the cells of the clasp windows."
  (let* ((cw (clasp-window-front))
         (hv (if cw (view-named 'history-view cw)))
         (dv (if cw (view-named 'dataset-view cw))))
    (if cw
      (progn 
        (set-cell-size dv (point-h (cell-size dv)) (font-height *font-spec-variables*))
        (invalidate-view dv t)
        (set-cell-size hv (point-h (cell-size hv)) (font-height *font-spec-variables*))
        (invalidate-view hv t)))
    (loop for x in (remove-if-not #'(lambda (item) (typep item 'clasp-caddy)) (windows)) do
          (progn (set-view-font (view-named 'sequence-view x) *caddy-font*)
                 (set-cell-size (view-named 'sequence-view x) 
                                (point-h (cell-size (view-named 'sequence-view x))) 
                                (font-height *caddy-font*))
                 (invalidate-view (view-named 'sequence-view x) t)))))

(defun resize-cells-tb ()
  "resize the cells of the clasp tables."
  (loop for x in (remove-if-not #'(lambda (item) 
                                    (typep item 'table-view-window))
                                (windows))
        do (progn (set-cell-size (if (typep x 'table-view-window) (view-named 'table-array-item x) (view-named 'table-item x))
                                 (string-width (make-string (table-largest-datum (table-view-table (view-named 'table-array-item x)))
                                                            :initial-element #\9) *table-header-font*)
                                 (table-char-height))
                  (invalidate-view x t))))

(defun build-view-fonts ()
  (setq *caddy-font* *font-spec-variables*)
  (setq *font-spec-datasets* (append '(:bold) *font-spec-variables*))
  (setq *font-spec-history* *font-spec-variables*)
  (setq *font-spec-history-result* *font-spec-variables*)
  (setq *font-spec-history-label* (append '(:bold) *font-spec-variables*))
  (setq *font-spec-history-command* (append '(:bold) *font-spec-variables*))
  (setq *font-spec-history-comment* (list "Geneva" (car (last *font-spec-variables*)))))

;;;
;;; status updating on window select
;;;

(defmethod view-activate-event-handler :after ((window window))
  (window-help-status window))

(defmethod window-help-status ((window window))
  (update-status "Ready."))

(defmethod window-help-status ((window dialog))
  (update-status "Command-X: grab selection as X arg; Command-Y: grab selection as Y."))

(defmethod window-help-status ((window clasp-window))
  (update-status "Control click: (data) to X dock; Option click: (data) to X dock (history) to Notebook."))

(defmethod window-help-status ((window table-view-window))
  (ignore-errors (window-help-status (table-view-table (view-named 'table-array-item window)))))

(defmethod window-help-status ((table generic-table))
  (update-status "Ready."))

(defmethod window-help-status ((table contingency-table))
  (update-status "Control click: row proportions; Option click: col. proportions; Double click: frequencies."))

(defmethod window-help-status ((window data-view-window))
  (update-status "Double click: edit data item."))

(defmethod window-help-status ((table scheffe-table))
  (update-status "Control click: scheffe statistic; Option click: significance"))

(defmethod window-help-status ((window graph-window))
  (update-status "Drag: zoom in; Option click: zoom out, Option-Control click: zoom out full."))

;;
;; table sorting crap
;;

#| OLD TABLE STUFF THAT MIGHT COME IN HANDY

(defun sort-rows-to-match (index-list rows to-match)
  (mapcar #'(lambda (item) (elt rows (position item index-list)))
          to-match))

(defun sort-cols-to-match (index-list rows to-match)
  (clasp::transpose
   (mapcar #'(lambda (match-item)
               (let ((pos (position match-item index-list)))  
                 (fresh-line)
                 (mapcar #'(lambda (rows-row)
                             (elt rows-row pos))
                         rows)))
           to-match)))

|#

;;;
;;; help for generic windows
;;;

(defmethod display-help ((window t))
  (show-help "No Help" (format nil "No help available for this window with
CLASP version ~A" clasp::*clasp-version*)))


;;;
;;; re-executing clasp code via the #. reader macro
;;;

(export '(variable dataset table graph))
          

(defun variable (var-name &optional dataset)
  (if dataset
    (clasp::get-variable var-name (clasp::get-dataset dataset) nil t)
    (clasp::get-variable var-name nil nil t)))

(defmethod print-object :around ((object variable) (stream stream))
  (format stream "#.(VARIABLE '~A '~A)" (clasp::name object) (clasp::name (clasp::variable-dataset object))))

(defun dataset (dataset-name)
  (clasp::get-dataset dataset-name nil t))

;;
;; This is for objects no one is supposed to touch.
;;

(defun strings-are-dashes (string)
  (substitute #\- #\Space string))

#+OLD-TABLES
(defmethod print-object :around ((object statistical-table) (stream stream))
  #+IGNORE(call-next-method)
  (format stream "#.(TABLE '~A '~A)" (type-of object) (strings-are-dashes (clasp::name object))))

(defmethod print-object :around ((object graph) (stream stream))
  (format stream "#.(GRAPH '~A '~A)" (type-of object) (strings-are-dashes (clasp::name object))))

#+OLD-TABLES
(defmethod print-object :around ((object composite-statistical-table) (stream stream))
  (format stream "#.(TABLE 'COMPOSITE-TABLE ~A)" (strings-are-dashes (clasp::name object))))

(define-condition cant-manipulate-that-object (error)
  ((type :initform NIL :initarg :type))
  (:report (lambda (condition stream)
             (with-slots (type) condition
               (if type 
                 (format stream "Objects of type ~A may not be manipulated from the listener in this way." type)
                 (princ "An attempt to access a protected object was made." stream)))))
  (:documentation "When someone tries to execute an interface object, we signal this type of error, because they shouldn't touch some things."))

(defun table (&rest args)
  (declare (ignore args))
  (error 'cant-manipulate-that-object :type 'table))

(defun graph (&rest args)
  (declare (ingore args))
  (error 'cant-manipulate-that-object :type 'graph))

;;;
;;; demo-session-stuff
;;;

#+DEMO
(defconstant *weird-time-interval* 900)

#+DEMO
(defclass cdvl-record ()
  ((aenfastt :initform (+ (get-universal-time) *weird-time-interval*))))

#+DEMO
(defvar cds-record '())

#+DEMO
(defun initialize-clasp-demo ()
  (setq cds-record (make-instance 'cdvl-record)))

#+DEMO
(defvar *demo-time-expired* NIL)

#+DEMO
(defmethod view-key-event-handler :around ((w listener) char)
  (if (> (slot-value cds-record 'aenfastt) (get-universal-time))
    (call-next-method)
    (if (< *weird-time-interval* (- (slot-value cds-record 'aenfastt) (get-universal-time)))
      (princ "Demo version is mangled. Please use a fresh copy.")
      (when (not *demo-time-expired*)
        (setf *demo-time-expired* t)
        (format t
"~%You have reached the ~D minute time limit for using the CLASP demo. If you
are interested in purchasing a copy, please send email to clasp-support@cs.umass.edu
or send US mail to :

David Hart, Lab Manager
Computer Science Dept., LGRC
University of Massachusetts
Box 34610
Amherst, MA 01003-4610
" (round *weird-time-interval* 60))))))

#+DEMO
(let* ((pn-dataset (choose-file-dialog :button-string "Load")))
  (clasp::load-dataset pn-dataset))

#+DEMO
(defmethod sorry-demo-version ()
  (message-dialog "Sorry, this feature is not available in the demo version. For ordering information, send email to clasp-support@cs.umass.edu or US mail to :

Peggy Weston, EKSL Lab Manager
Computer Science Dept., LGRC
University of Massachusetts
Box 34610
Amherst, MA 01003-4610
" :ok-text "OK !" :size #@(400 200)))

#+DEMO
(defmethod demo-times-up ()
  (message-dialog (format nil "The CLASP demo only runs for ~D minutes. For ordering information, send email to clasp-support@cs.umass.edu or US mail to :

Peggy Weston, EKSL Lab Manager
Computer Science Dept., LGRC
University of Massachusetts
Box 34610
Amherst, MA 01003-4610
" (round *weird-time-interval* 60)) :ok-text "OK !" :size #@(400 200)))

;;
;; window size thingy
;;

(defclass intense-number-item (number-entry-item)
  ((recompute-views :initarg :recompute-views :accessor views-to-recompute)
   (recompute-fn    :initarg :recompute-fn    :accessor recompute-fn)))

(defmethod view-key-event-handler :after ((ini intense-number-item) key)
  (apply (recompute-fn ini) (view-container ini) (views-to-recompute ini)))

(defclass window-dimension-editor (dialog)
  ((window          :initarg :window          :accessor window))
  (:default-initargs
    :window-type :document
    :window-title "Resize Window"
    :view-size #@(200 100)
    :view-position (best-spot 200 100)
    :view-subviews
    (list (make-instance 'static-text-dialog-item :dialog-item-text "Width"
                         :view-position #@(50 2))
          (make-instance 'static-text-dialog-item :dialog-item-text "Height"
                         :view-position #@(100 2))
          (make-instance 'intense-number-item :view-position #@(50 22) :view-size #@(45 16)
                         :view-nick-name 'pixwid :recompute-views '(pixwid pixht reso inwid inht)
                         :allow-negatives NIL :allow-floats NIL
                         :recompute-fn 'pix->inches)
          (make-instance 'intense-number-item :view-position #@(100 22) :view-size #@(45 16)
                         :view-nick-name 'pixht :recompute-views '(pixwid pixht reso inwid inht)
                         :allow-negatives NIL :allow-floats NIL
                         :recompute-fn 'pix->inches)
          (make-instance 'static-text-dialog-item :dialog-item-text "Pixels"
                         :view-position #@(150 22))
          (make-instance 'intense-number-item :view-position #@(50 47) :view-size #@(45 16)
                         :view-nick-name 'inwid :recompute-views '(inwid inht reso pixwid pixht)
                         :allow-negatives NIL
                         :recompute-fn 'inches->pix)
          (make-instance 'intense-number-item :view-position #@(100 47) :view-size #@(45 16)
                         :view-nick-name 'inht :recompute-views '(inwid inht reso pixwid pixht)
                         :allow-negatives NIL
                         :recompute-fn 'inches->pix)
          (make-instance 'static-text-dialog-item :dialog-item-text "Inches"
                         :view-position #@(150 47))
          (make-instance 'button-dialog-item :view-position #@(20 72) :view-size #@(30 16)
                         :dialog-item-text "Set" :default-button t
                         :dialog-item-action 'resize-from-ini)
          (make-instance 'intense-number-item :view-position #@(75 72) :view-size #@(45 16)
                         :view-nick-name 'reso :recompute-views '(inwid inht reso pixwid pixht)
                         :allow-negatives NIL :allow-floats NIL
                         :recompute-fn 'inches->pix)
          (make-instance 'static-text-dialog-item :dialog-item-text "DPI"
                         :view-position #@(150 72)))))

(defun resize-from-ini (child)
  (let ((ini (view-container child)))
    (set-view-size (window ini) 
                   (number-item-value (view-named 'pixwid ini))
                   (number-item-value (view-named 'pixht ini)))))

(defmethod initialize-instance :after ((wde window-dimension-editor) &key (window NIL))
  (set-dialog-item-text (view-named 'pixwid wde) (princ-to-string (point-h (view-size window))))
  (set-dialog-item-text (view-named 'pixht wde)  (princ-to-string (point-v (view-size window))))
  (set-dialog-item-text (view-named 'reso wde)  (princ-to-string 72))
  (pix->inches wde 'pixwid 'pixht 'reso 'inwid 'inht)
  (set-window-title wde (format nil "Resize ~A" (window-title window))))

(defun pix->inches (window widgad htgad resgad widdes htdes)
  (set-dialog-item-text (view-named widdes window) 
                        (format nil "~,2F" 
                         (/ (number-item-value (view-named widgad window))
                            (number-item-value (view-named resgad window)))))
  (set-dialog-item-text (view-named htdes window)
                        (format nil "~,2F" 
                                (/ (number-item-value (view-named htgad window))
                                   (number-item-value (view-named resgad window))))))

(defun inches->pix (window widgad htgad resgad widdes htdes)
  (set-dialog-item-text (view-named widdes window) 
                        (format nil "~D" 
                                (round (* (number-item-value (view-named widgad window))
                                          (number-item-value (view-named resgad window))))))
  (set-dialog-item-text (view-named htdes window) 
                        (format nil "~D"
                                (round (* (number-item-value (view-named htgad window)) 
                                          (number-item-value (view-named resgad window)))))))

(defun edit-window-dimensions (window)
  (make-instance 'window-dimension-editor :window window))

(defmethod view-click-event-handler :around ((win window) pt-where)
  (declare (ignore pt-where))
  (when (and (eql (window-type win) :document-with-zoom)
             (double-click-p) (command-key-p))
    (edit-window-dimensions win))
  (call-next-method))

;;;
;;; patch loading stuff
;;;

(defparameter *patch-pathname* ":CLASP Patches:")

(defparameter *compiled-file-extension* #-POWERPC "fasl" #+POWERPC "pfsl")

(defun load-clasp-patches ()
  (if (probe-file *patch-pathname*)
    (let ((patches (directory (merge-pathnames *patch-pathname* (format nil "*.~A" *COMPILED-FILE-EXTENSION*)))))
      (format t "Loading patches...~%")
      (loop for patch in patches do
            (format t "Loading ~A...~%" patch)
            (load patch)))))
