#|

**************************
* Transform Function     *
* interfaces for Mac     *
* CLASP                  *
**************************
|#

(in-package "CLASP-INTERFACE")

;;;
;;; delete-results.lisp
;;;

#|

no dialog box. just deletes the items selected in the data view.

13-Jul-94 ms created
24-Jul-94 ms Beta finish

|#

(defun delete-these-string (list)
  (format nil "Really delete selected items ?~%~:(~A~)~A" (clasp::name (car list))
          (if (> (length list) 1) ", etc.." "")))


(defun delete-selected-items ()
  "deletes the selected items from the dataset view."
  (let ((items (l-data-selected-cw (clasp-window-front))))
    (if (null items)
      (message-dialog "Select the items to delete on the dataset view and then chose Delete Results."
                      :ok-text "OK")
      (if (or (not *confirm-deletes*)
              (y-or-n-dialog (delete-these-string items)
                             :cancel-text NIL))
        (progn 
          (loop for x in items do
                (delete-item x))
          ;; unselect contents of dataset view
          (deselect-cells (view-named 'dataset-view (clasp-window-front)))
          (when (find-clasp-dock)
            (remove-renegades (find-clasp-dock) items))
          (when (and *most-recent-vresult* (find *most-recent-vresult* items :test #'equal))
            (setf *most-recent-vresult* NIL))
          )
        ))))

(defmethod delete-item ((self clasp::dataset))
  (let ((clasp-command (make-clasp-command
                        `(clasp::delete-dataset ,self)
                        :description
                        (list (list 'command 'clasp::delete-dataset)
                              (list 'dataset (clasp::name self))))))
    (run-clasp-command clasp-command
                       :str-message "Deleting dataset..."
                       :f-update-dataset-view t)))

(defmethod delete-item ((self clasp::variable))
  (let ((clasp-command (make-clasp-command
                        `(clasp::delete-variable ,self)
                        :description
                        (list (list 'command 'clasp::delete-variable)
                              (list 'variable (clasp::name self))))))
    (run-clasp-command clasp-command
                       :str-message "Deleting variable"
                       :f-update-dataset-view t)))

;;;
;;; partition-dataset-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::partition-dataset .


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
20-Jun-94 mc	Created.

|#


;;;
;;; Define partition-dataset-dialog .
;;;

(defclass partition-dataset-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::partition-dataset function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Partition Dataset"
    :VIEW-SIZE #@(331 141)
    :VIEW-POSITION (best-spot 331 141)
    :help '*partition-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Include Vars"
                         :view-nick-name 'y-var-item
                         :view-position #@(4 93)
                         :view-size #@(229 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(251 119))
          (MAKE-DIALOG-ITEM 'EDITABLE-TEXT-DIALOG-ITEM #@(97 27) #@(228 59) "<Partition Clause here>" 'NIL
                            :VIEW-NICK-NAME 'partition-clause-etdi
                            :VIEW-FONT '("Monaco" 9 :SRCOR :PLAIN)
                            :ALLOW-RETURNS NIL          ;FIX: Allow?
                            )
          (MAKE-DIALOG-ITEM 'STATIC-TEXT-DIALOG-ITEM #@(7 26) #@(62 31) "Partition Clause" 'NIL)
          (MAKE-DIALOG-ITEM 'STATIC-TEXT-DIALOG-ITEM #@(7 7) #@(56 16) "Dataset" 'NIL)
          (make-instance 'dataset-stdi :view-position #@(93 7)
                                       :view-size #@(154 16)
                                       :VIEW-NICK-NAME 'x-var-item))))


(defmethod f-can-compute-statistics-dialog ((partition-dataset-dialog partition-dataset-dialog))
  "We can compute when there's a dataset in 'dataset-stdi and a
non-empty partition clause in 'partition-clause-etdi ."
  ;;
  (let* ((dataset (dataset-dset-stdi (view-named 'x-var-item
                                                 partition-dataset-dialog)))
         (str-clause (dialog-item-text (view-named 'partition-clause-etdi
                                                   partition-dataset-dialog)))
         ;; FIX: Might compute f-clause-readable .
         (f-can-compute (and dataset
                             str-clause
                             (not (string-equal "" str-clause))))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null dataset)
                                      "Need one dataset")
                                     ((string-equal "" str-clause)
                                      "Need a non-empty partition clause"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((partition-dataset-dialog partition-dataset-dialog))
  (let* ((dataset (dataset-dset-stdi (view-named 'x-var-item
                                                 partition-dataset-dialog)))
         (str-clause (read-from-string  (dialog-item-text (view-named 'partition-clause-etdi
                                                                      partition-dataset-dialog))))
         (l-var-include (l-variables-var-pum (view-named 'y-var-item
                                                         partition-dataset-dialog))))
    (if (null l-var-include) (setf l-var-include (clasp::dataset-variables dataset)))
    (compute-partition-dataset dataset str-clause l-var-include)))


;;; FIX: This results in no data in rows!?
(defmethod compute-partition-dataset ((dataset clasp::dataset)
                                     (str-clause cons)
                                     (l-var-include list))
  "Calls clasp::partition-dataset on the arguments, saving results in
the history."
  ;;
  (let* (;; FIX: Catch read errors:
  ;;       (sexp-clause (read-from-string str-clause))
         (clasp-command (make-clasp-command
                         `(clasp::partition-dataset ,dataset
                                                    #+OLD-VARS
                                                    ,sexp-clause
                                                    #-OLD-VARS
                                                    ,(build-where-lambda (dataset-variables dataset) str-clause)
                                                    #-OLD-VARS
                                                    ',(dataset-variables dataset)
                                                    ',l-var-include
                                                    #-OLD-VARS
                                                    ',str-clause)
                         :description
                         (list (list 'command 'clasp::partition-dataset)
                               (list 'dataset dataset)
                               (list 'clause str-clause)
                               (list 'include l-var-include))
                                     )))
    (run-clasp-command clasp-command
                       :str-message "Partitioning dataset"
                       :f-update-dataset-view t)))

(defun build-where-lambda (variables expression)
  (clasp::maybe-compile-lambda 
   `(lambda ,(mapcar #'(lambda (v) (clasp::form-symbol (name v))) variables)
      ,@(mapcar #'(lambda (v) (clasp::form-symbol (name v))) variables)
      ,expression)))

(defun partition-dataset-dialog-make ()
  "Returns a new dialog instance for editing CLASP's partition-dataset
function."
  ;;
  (make-instance 'partition-dataset-dialog))


#|
(partition-dataset-dialog-make)

;;; For PA-SCALED :
(.and. (.<<. sd 120)
       (.==. success 1))
|#

#|
================================================================
merge-datasets==================================================
================================================================
dialog box for merge-datasets

6-Jul-94  ms	Created.
21-Jul-94 ms    Beta finish.

|#
;;;
;;; Define merge-datasets-dialog .
;;;

(defclass merge-datasets-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
merge-datasets function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Merge Datasets"
    :VIEW-SIZE #@(350 85)
    :help '*merge-help*
    :VIEW-POSITION (best-spot 350 85)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'dataset-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Datasets"
                         :view-nick-name 'x-var-item
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(220 50)))))

(defmethod f-can-compute-statistics-dialog ((merge-datasets-dialog merge-datasets-dialog))
  "We can compute if there is at least one sample in the ds-ds-pop-up-menu."
  ;;
  (let* ((datasets (l-datasets-dataset-pum (view-named 'x-var-item
                                                      merge-datasets-dialog)))
         (f-can-compute (>= (length datasets) 2))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((< (length datasets) 2)
                                      "Need at least two datasets")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((merge-datasets-dialog merge-datasets-dialog))
  (let* ((datasets (l-datasets-dataset-pum (view-named 'x-var-item
                                                   merge-datasets-dialog))))
    (compute-merge-datasets datasets)))

(defmethod compute-merge-datasets (datasets)
  "Calls merge-datasets on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::merge-datasets
                          ',datasets)
                        :description
                        (list (list 'command 'clasp::merge-datasets)
                              (cons 'datasets datasets)))))
    (run-clasp-command clasp-command
                       :str-message "Merging datasets"
                       :f-update-dataset-view t)))


(defun merge-datasets-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
merge-datasets-one-sample function."
  ;;
    (make-instance 'merge-datasets-dialog))

;;;
;;; partition-on-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::partition-dataset using the .on. construct.


================================================================
Status =========================================================
================================================================
In-progress.
FIX :: multiple-partition-on variables breaks CLASP

================================================================
Change history =================================================
================================================================
05-Aug-94 ms	Created.
25-Aug-94 ms    Beta finish/patched - requires partition-patch.lisp to work right
|#

(defclass partition-on-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::partition-dataset function using the .on. construct.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Partition On"
    :VIEW-SIZE #@(331 120)
    :help '*partition-on-help*
    :VIEW-POSITION (best-spot 331 120)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Include Vars"
                         :view-nick-name 'include-var-pop-up-menu
                         :view-position #@(4 70)
                         :view-size #@(229 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(251 95))
          (make-instance 'variable-pop-up-menu :view-position #@(4 40)
                         :view-size #@(229 20)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Partition On"
                         :view-nick-name 'x-var-item)
          (MAKE-DIALOG-ITEM 'STATIC-TEXT-DIALOG-ITEM #@(7 7) #@(56 16) "Dataset" 'NIL)
          (make-instance 'dataset-stdi :view-position #@(93 7)
                                       :view-size #@(154 16)
                                       :VIEW-NICK-NAME 'dataset-stdi))))


(defmethod f-can-compute-statistics-dialog ((partition-on-dialog partition-on-dialog))
  "We can compute when there's a dataset in 'dataset-stdi and a
non-empty partition clause in 'partition-clause-etdi ."
  ;;
  (let* ((dataset (dataset-dset-stdi (view-named 'dataset-stdi
                                                 partition-on-dialog)))
         (on-variables (l-variables-var-pum (view-named 'x-var-item
                                                    partition-on-dialog)))
         
         ;; FIX: Might compute f-clause-readable .
         (f-can-compute (and dataset
                             on-variables
                             ))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null dataset)
                                      "Need one dataset")
                                     (t "Need a variable to partition on."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((partition-on-dialog partition-on-dialog))
  (let* ((dataset (dataset-dset-stdi (view-named 'dataset-stdi
                                                 partition-on-dialog)))
         (variables (l-variables-var-pum (view-named 'x-var-item partition-on-dialog)))
         (l-var-include (l-variables-var-pum (view-named 'include-var-pop-up-menu
                                                         partition-on-dialog))))
    (if (null l-var-include)
      (setf l-var-include (clasp::dataset-variables dataset)))
    (compute-partition-on dataset variables l-var-include)))

;; 9-1-94 Started to update this for CLASP 1.3.4 and decided to bag it for now. (Westy)
;; 9-7-94 Commented out the CLASP 1.3.4 stuff because it made it break.
(defmethod compute-partition-on ((dataset clasp::dataset)
                                 (variables list)
                                 (l-var-include list))
  "Calls clasp::partition-dataset on the arguments, saving results in
the history."
  #-OLD-VARS
  ;; should only be one variable for now...
  (let* ((clasp-command (make-clasp-command
                         `(clasp::partition-on ,dataset ',variables
                                               ',l-var-include))))
    ;(princ (format nil "dset ~S ~%clause ~S ~%vars ~S~%" dataset clause l-var-include))
    (run-clasp-command clasp-command
                       :str-message "Partitioning dataset"
                       :f-update-dataset-view t))
  #+OLD-VARS
  (let* ((clause (push '.on. variables))
         (clasp-command (make-clasp-command
                         `(clasp::partition-dataset ,dataset ',clause
                                                    ',l-var-include))))
    (run-clasp-command clasp-command
                       :str-message "Partitioning dataset"
                       :f-update-dataset-view t)))

(defun partition-on-dialog-make ()
  "Returns a new dialog instance for editing CLASP's partition-dataset
function."
  ;;
  (make-instance 'partition-on-dialog))

;;;
;;; change type of variable
;;;

(defun change-selected-variable-types ()
  (let ((vars (remove-if-not #'(lambda (item) (typep item 'clasp::variable)) (l-data-selected-cw (clasp-window-front)))))
    (if (y-or-n-dialog "What type should the selected variables be ?"
                       :yes-text "Interval" :no-text "Categorical")
      (mapcar #'(lambda (variable)
                  (ignore-errors (setf (slot-value variable 'clasp::type) :numerical)))
              vars)
      (mapcar #'(lambda (variable)
                  (ignore-errors (setf (slot-value variable 'clasp::type) :categorical)))
              vars))))

