#|
***************************************
*Miscellaneous dialog boxes for the   *
*Describe menu of Macintosh CLASP     *
***************************************

22-Jul-94 ms    Gathered into this volume.

|#

(in-package "CLASP-INTERFACE")

#|
================================================================
__Trimmed-Mean==================================================
================================================================
dialog box for Trimmed-Mean

6-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#
;;;
;;; Define Trimmed-Mean-dialog .
;;;

(defclass Trimmed-Mean-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
Trimmed-Mean function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Trimmed Mean"
    :HELP '*trimmed-mean-help*
    :VIEW-SIZE #@(350 85)
    :VIEW-POSITION (best-spot 350 85)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'static-text-dialog-item :dialog-item-text "Percentage"
                         :view-position #@(20 50))
          (make-instance 'number-entry-item :dialog-item-text "0.25"
                         :view-nick-name 'percent-number-item
                         :view-position #@(105 50)
                         :view-size #@(110 16))
          (make-instance 'compute-button-di :view-position #@(240 50)))))

(defmethod f-can-compute-statistics-dialog ((Trimmed-Mean-dialog Trimmed-Mean-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   Trimmed-Mean-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((Trimmed-Mean-dialog Trimmed-Mean-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   Trimmed-Mean-dialog)))
         (q (number-item-value (view-named 'percent-number-item Trimmed-Mean-dialog))))
    (loop for x in l-var-x
          do (compute-Trimmed-Mean x q))))


(defmethod compute-Trimmed-Mean ((iv clasp::variable) q)
  "Calls Trimmed-Mean on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::Trimmed-Mean
                          (clasp::simplify-and-sort ,iv)
                          ,q)
                        :format-function
                        (make-line-mapper "~5F trimmed mean of ~A is ~5F"
                                          '(percent x-args 0))
               ;;         'trimmed-mean-format
                        :description
                        (list (list 'command 'clasp::trimmed-mean)
                              (list 'x-args iv)
                              (list 'percent q))
                        )))
    (run-clasp-command clasp-command
                       :str-message "Computing Trimmed-Mean"
                       :f-update-dataset-view t)))


(defun Trimmed-Mean-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
Trimmed-Mean function."
  ;;
    (make-instance 'Trimmed-Mean-dialog))

#|
================================================================
__Quantile======================================================
================================================================
dialog box for Quantile
6-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish.

|#
;;;
;;; Define Quantile-dialog .
;;;

(defclass Quantile-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
Quantile function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Quantile"
    :HELP '*quantile-help*
    :VIEW-SIZE #@(350 85)
    :VIEW-POSITION (best-spot 350 85)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'static-text-dialog-item :dialog-item-text "Q"
                         :view-position #@(42 50))
          (make-instance 'number-entry-item :dialog-item-text "0.50"
                         :view-nick-name 'q-number-item
                         :view-position #@(55 50)
                         :view-size #@(120 16))
          (make-instance 'compute-button-di :view-position #@(220 50)))))

(defmethod f-can-compute-statistics-dialog ((Quantile-dialog Quantile-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   Quantile-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((Quantile-dialog Quantile-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   Quantile-dialog)))
         (q (number-item-value (view-named 'q-number-item Quantile-dialog))))
    (loop for x in l-var-x
          do (compute-Quantile x q))))

(defmethod compute-Quantile ((iv clasp::variable) q)
  "Calls Quantile on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::Quantile
                          (clasp::simplify-and-sort ,iv)
                          ,q)
                        :format-function
                        (make-line-mapper "~,4F is ~5F quantile of ~A with actual values of ~5f and ~5F"
                                          '(0 q x-args 1 2))
                        :description
                        (list (list 'command 'clasp::quantile)
                              (list 'x-args iv)
                              (list 'q q))
                        )))
    (run-clasp-command clasp-command
                       :str-message "Computing Quantile"
                       :f-update-dataset-view t)))


(defun Quantile-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
Quantile-one-sample function."
  ;;
    (make-instance 'Quantile-dialog))

#|
================================================================
__cross-correlation=============================================
================================================================
dialog box for cross-correlation

8-Jul-94 ms	created.
21-Jul-94 ms    Beta finish.

|#
;;;
;;; Define cross-correlation-dialog .
;;;

(defclass cross-correlation-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
cross-correlation function.")
  (:default-initargs
    :WINDOW-TYPE :DOcUMENT
    :WINDOW-TITLE "cross correlation"
    :VIEW-SIZE #@(340 145)
    :VIEW-POSITION (best-spot 340 145)
    :VIEW-FONT '("chicago" 12 :SRcOR :PLAIN)
    :HELP '*cross-correlation-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'y-var-item :allow-categorical-p nil
                         :view-position #@(12 42)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(12 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'static-text-dialog-item :dialog-item-text "Min Lag"
                         :view-position #@(25 85))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'min-lag-number-item
                         :allow-floats NIL
                         :view-position #@(100 85)
                         :view-size #@(50 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Max Lag"
                         :view-position #@(182 85))
          (make-instance 'number-entry-item :dialog-item-text "2"
                         :view-nick-name 'max-lag-number-item
                         :allow-floats NIL
                         :view-position #@(250 85)
                         :view-size #@(50 16))
          (make-instance 'compute-button-di :view-position #@(140 115)))))


(defmethod f-can-compute-statistics-dialog ((cross-correlation-dialog cross-correlation-dialog))
  "We can compute if there is at least one sample in the x-var-ietm."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   cross-correlation-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   cross-correlation-dialog)))
         (f-can-compute (and l-var-x l-var-y))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one X variable.")
                                     (t "Need at least one Y variable."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((cross-correlation-dialog cross-correlation-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   cross-correlation-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   cross-correlation-dialog)))
         (min-lag (number-item-value (view-named 'min-lag-number-item cross-correlation-dialog)))
         (max-lag (number-item-value (view-named 'max-lag-number-item cross-correlation-dialog))))
    (if (= (length l-var-y) 1)
      (loop for x in l-var-x
            do (compute-cross-correlation (car l-var-y) x max-lag min-lag))
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-cross-correlation y x max-lag min-lag)))))


(defmethod compute-cross-correlation ((dv clasp::variable)
                                      (iv clasp::variable) 
                                      maxlag
                                      minlag)
  "calls cross-correlation on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::cross-correlation
                          (clasp::simplify ,dv)
                          (clasp::simplify ,iv)
                          ,maxlag
                          ,minlag)
                        :format-function
                        'correlation-lag-formatter
                        :description
                        (list (list 'command 'clasp::cross-correlation)
                              (list 'x-args dv)
                              (list 'y-args iv)
                              (list 'min-lag minlag)
                              (list 'max-lag maxlag)))))
    (run-clasp-command clasp-command
                       :str-message "computing cross-correlation"
                       :f-update-dataset-view t)))


(defun cross-correlation-dialog-make ()
  "Returns a new dialog instance for editing cLASP's
cross-correlation-one-sample function."
  ;;
    (make-instance 'cross-correlation-dialog))

#|
================================================================
__Autocorrelation===============================================
================================================================
dialog box for autocorrelation

6-Jul-94 ms	Created.
20-Jul-94 ms    Beta finish.

|#
;;;
;;; Define Autocorrelation-dialog .
;;;

(defclass autocorrelation-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
Autocorrelation function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Autocorrelation"
    :VIEW-SIZE #@(340 115)
    :VIEW-POSITION (best-spot 340 115)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :help '*autocorrelation-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(12 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'static-text-dialog-item :dialog-item-text "Min Lag"
                         :view-position #@(25 50))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'min-lag-number-item
                         :allow-floats NIL
                         :view-position #@(100 50)
                         :view-size #@(50 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Max Lag"
                         :view-position #@(182 50))
          (make-instance 'number-entry-item :dialog-item-text "2"
                         :view-nick-name 'max-lag-number-item
                         :allow-floats NIL
                         :view-position #@(250 50)
                         :view-size #@(50 16))
          (make-instance 'compute-button-di :view-position #@(140 85)))))

(defmethod f-can-compute-statistics-dialog ((Autocorrelation-dialog Autocorrelation-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   Autocorrelation-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "inexplicable reason not to work."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((Autocorrelation-dialog Autocorrelation-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   autocorrelation-dialog)))
         (min-lag (number-item-value (view-named 'min-lag-number-item autocorrelation-dialog)))
         (max-lag (number-item-value (view-named 'max-lag-number-item autocorrelation-dialog))))
    (loop for x in l-var-x
          do (compute-auto-correlation x max-lag min-lag))))


(defmethod compute-Auto-Correlation ((iv clasp::variable) 
                                     maxlag
                                     minlag)
  "Calls autocorrelation on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::autocorrelation
                          (clasp::simplify ,iv)
                          ,maxlag
                          ,minlag)
                        :format-function
                        'correlation-lag-formatter
                        :description
                        (list (list 'command 'clasp::autocorrelation)
                              (list 'x-args iv)
                              (list 'min-lag minlag)
                              (list 'max-lag maxlag)))))
    (run-clasp-command clasp-command
                       :str-message "Computing autocorrelation"
                       :f-update-dataset-view t)))


(defun autocorrelation-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
autocorrelation function."
  ;;
    (make-instance 'Autocorrelation-dialog))


;;;
;;; correlation table maker
;;;

(defclass corr-table-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
Correlation-table function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Correlation Table"
    :VIEW-SIZE #@(350 70)
    :VIEW-POSITION (best-spot 350 70)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :help '*correlation-table-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(245 45)))))

(defmethod f-can-compute-statistics-dialog ((corr-table-dialog corr-table-dialog))
  "We can compute if there is at least one sample in the x-var-item."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   corr-table-dialog)))
         (f-can-compute (and
                         (> (length l-var-x) 1)
                         (every #'(lambda (ds) (eql (clasp::variable-dataset ds)
                                                    (clasp::variable-dataset (car l-var-x))))
                                l-var-x)))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((< (length l-var-x) 2) "Correlation table requires 2 or more X variables.")
                                     (t "Variables must be from same dataset."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((corr-table-dialog corr-table-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   corr-table-dialog))))
    (corr-table l-var-x)))


(defmethod corr-table ((iv LIST))
  "Applies make-correlation-table on the arguments, saving
results in the history."
  ;;
  (let* ((iv-names (mapcar #'clasp::name iv)) ;; we're doing this for cutting & pasting purposes
         (clasp-command (make-clasp-command
                         `(apply #'clasp::make-correlation-table
                                 (mapcar #'clasp::simplify ',iv))
                         :format-function
                         'correlation-table-format
                         :description
                         (list (list 'command 'clasp::make-correlation-table)
                               (list 'x-args iv)))))
    (run-clasp-command clasp-command
                       :str-message "Computing Correlation Table"
                       :f-update-dataset-view t)))


(defun corr-table-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
Trimmed-Mean function."
  ;;
    (make-instance 'corr-table-dialog))

;;;
;;; partial correlations
;;;

;;;
;;; Define partial-corr-dialog .
;;;

(defclass partial-corr-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::partial-correltation function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Partial Correlation"
    :VIEW-SIZE #@(300 120)
    :view-position (best-spot 300 120)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*partial-correlation-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(S)"
                         :view-nick-name 'x-var-item :allow-categorical-p nil
                         :view-position #@(8 15)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(S)"
                         :view-nick-name 'y-var-item :allow-categorical-p nil
                         :view-position #@(8 40)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Fixed"
                         :view-nick-name 'z-var-item :allow-categorical-p nil
                         :view-position #@(8 65)
                         :view-size #@(200 20))
          (make-instance 'compute-button-di :view-position #@(120 95)))))


(defmethod f-can-compute-statistics-dialog ((partial-corr-dialog partial-corr-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s)."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   partial-corr-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   partial-corr-dialog)))
         (l-var-cv (l-variables-var-pum (view-named 'z-var-item
                                                   partial-corr-dialog)))
         (f-can-compute (and l-var-x
                             l-var-y
                             l-var-cv))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one x variable")
                                     ((null l-var-y)
                                      "Need at least one y variable")
                                     (t "Variables must be of the same length"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((partial-corr-dialog partial-corr-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   partial-corr-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   partial-corr-dialog)))
         (l-var-cv (l-variables-var-pum (view-named 'z-var-item
                                                   partial-corr-dialog)))
         )
    (if (= (length l-var-y) 1)
      (loop for x in l-var-x do
            (compute-partial-corr x (car l-var-y) l-var-cv))
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-partial-corr x y l-var-cv)))))

;;;
;;;

(defmethod compute-partial-corr ((x clasp::variable)
                                 (y clasp::variable)
                                 (cv list))
  "Calls clasp::partials-from-parents on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::partials-from-parents
                          (clasp::simplify ,x)
                          (clasp::simplify ,y)
                          (mapcar #'clasp::simplify ',cv))
                        :format-function
                        (make-line-mapper "~,5F is partial correlation of ~:(~A~) on ~:(~A~) holding ~A constant."
                                          '(0 X-ARGS Y-ARGS CONSTANTS))
                        :description
                        (list (list 'command 'clasp::partials-from-parents)
                              (list 'x-ARGS x)
                              (list 'y-ARGS y)
                              (cons 'constants cv)))))
    (run-clasp-command clasp-command
                       :str-message "Computing partial correlation"
                       :f-update-dataset-view t)))


(defun partial-corr-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
partials-from-parents function."
  ;;
  (make-instance 'partial-corr-dialog))

;;;
;;; a little helper for the mode function, which breaks on categorical data, but doesn't need to
;;;

(defun mode-1 (sequence)
  (if (not (numberp (car sequence)))
    (clasp::mode sequence :key #'sxhash)
    (clasp::mode sequence)))

(defclass mode-dialog (statistic-dialog-mixin dialog)
  ()
  (:default-initargs
    :view-size #@(320 85)
    :view-position (best-spot 320 85)
    :help '*mode-help*
    :window-title "Mode"
    :view-subviews
    (list (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(S)"
                         :view-nick-name 'x-var-item
                         :view-position #@(8 6)
                         :view-size #@(200 20))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Discrete"
                         :view-nick-name 'discrete
                         :view-position #@(8 30) :view-size #@(90 15))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Continuous"
                         :view-nick-name 'continuous
                         :view-position #@(110 30) :view-size #@(90 15))
          (make-instance 'radio-button-dialog-item :dialog-item-text "Best Guess"
                         :view-nick-name 'best-guess
                         :view-position #@(218 30) :view-size #@(90 15))
          (make-instance 'compute-button-di :view-position #@(135 55)))))

(defmethod f-can-compute-statistics-dialog ((mode-dialog mode-dialog))
  (values (l-variables-var-pum (view-named 'x-var-item mode-dialog))
          "Need at least one variable to compute the mode for."))

(defmethod compute-statistics-dialog ((mode-dialog mode-dialog))
  (let ((correct-mode-fun (cond
                           ((radio-button-pushed-p (view-named 'discrete mode-dialog))
                            #'clasp::mode)
                           ((radio-button-pushed-p (view-named 'continuous mode-dialog))
                            #'clasp::mode-for-continuous-data)
                           (t #'clasp::smart-mode)))
        (xs (l-variables-var-pum (view-named 'x-var-item mode-dialog))))
    (loop for x in xs do
          (compute-mean x correct-mode-fun))))

(defun compute-mean (seq mode-fun)
  (let* ((key (if (not (numberp (car (clasp::simplify seq)))) #'sxhash))
         (clasp-command 
          (make-clasp-command 
           `(apply ,mode-fun
                   (append (list (clasp::simplify ,seq)) (if ,key (list :key ,key))))
           :description (list (list 'command 'clasp::mode)
                              (list 'X-args seq))
           :format-function
           (cond
            ((eql mode-fun #'clasp::mode)
             (make-line-mapper "~S is mode of ~:(~A~) and occurs ~D times"
                               '(0 x-args 1)))
            ((eql mode-fun #'clasp::mode-for-continuous-data)
             (make-line-mapper "~,4F is estimated mode of ~:(~A~)"
                               '(0 x-args)))
            (t #'smart-mode-result)))))
    (run-clasp-command clasp-command
                       :str-message "Computing mode"
                       :f-update-dataset-view t)))

(defun mode-dialog-make ()
  (make-instance 'mode-dialog))

;;;
;;; FBD Interface created 12-29-94 ms
;;;
#+FBD
(export '(compute-fbd compute-fbd-verbose))

#+FBD
(defclass fbd-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
path-analysis::fbd function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "FBD Causal Model"
    :VIEW-SIZE #@(300 140)
    :view-position (best-spot 300 140)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP *partial-correlation-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "DV"
                         :view-position #@(8 5))
          (make-instance 'variable-stdi :dialog-item-text "<NONE>"
                         :view-nick-name 'x-var-item
                         :view-position #@(50 5)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "IV(S)"
                         :view-nick-name 'y-var-item :allow-categorical-p nil
                         :view-position #@(8 27)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'check-box-dialog-item :dialog-item-text "Verbose"
                         :view-nick-name 'verbose-p
                         :view-position #@(223 3)
                         :view-size #@(100 20))
          (make-instance 'disabler-checkbox-item :view-nick-name 'bootstrapped :view-position #@(10 53)
                         :dialog-item-text "Bootstrap ?" :watches-over '(resampling res-lb resamples shuffle))
          (make-instance 'number-entry-item :view-nick-name 'resamples :view-position #@(105 73)
                         :allow-floats NIL :allow-negatives NIL :dialog-item-text "100" :dialog-item-enabled-p NIL)
          (make-instance 'static-text-dialog-item :view-nick-name 'res-lb :view-position #@(15 73)
                         :dialog-item-text "Resamples" :dialog-item-enabled-p NIL)
          #+NOPE(make-instance 'number-entry-item :view-nick-name 'length :view-position #@(105 98)
                         :allow-floats NIL :allow-negatives NIL :dialog-item-text "100" :dialog-item-enabled-p NIL)
          #+NOPE(make-instance 'static-text-dialog-item :view-nick-name 'len-lb :view-position #@(15 98)
                         :dialog-item-text "Sample Size" :dialog-item-enabled-p NIL)
          (make-instance 'check-box-dialog-item :view-nick-name 'shuffle :view-position #@(15 98)
                         :dialog-item-text "Shuffle  ?" :dialog-item-enabled-p NIL)
          (make-instance 'check-box-dialog-item :view-nick-name 'resampling :view-position #@(15 118)
                         :dialog-item-text "Resample ?" :dialog-item-enabled-p NIL)
          (make-instance 'compute-button-di :view-position #@(223 55)))))

#+FBD
(defmethod f-can-compute-statistics-dialog ((fbdd fbd-dialog))
  (values (and (variable-vset-stdi (view-named 'x-var-item fbdd))
               (l-variables-var-pum (view-named 'y-var-item fbdd)))
          "You must specify one dependent variable and at least one independent variable"))

#+FBD
(defmethod compute-statistics-dialog ((fbdd fbd-dialog))
  (let* ((verbose (check-box-checked-p (view-named 'verbose-p fbdd)))
         (x (variable-vset-stdi (view-named 'x-var-item fbdd)))
         (ys (l-variables-var-pum (view-named 'y-var-item fbdd)))
         (fun (if verbose #'compute-fbd-verbose #'compute-fbd))
         (bs (check-box-checked-p (view-named 'bootstrapped fbdd)))
         (us (check-box-checked-p (view-named 'shuffle fbdd)))
         (ur (check-box-checked-p (view-named 'resampling fbdd)))
         (rs (number-item-value (view-named 'resamples fbdd)))
         (clasp-command
          (if bs
            (make-clasp-command 
             `(compute-bs-fbd ,x ',ys ,us ,ur ,rs ,verbose))
            (make-clasp-command
             `(apply ,fun ,x ',ys)
             :format-function 'fbd-formatter
             :description (list (list 'command 'fbd-model)
                                (list 'dv x)
                                (list 'ivs ys)
                                (list 'verbose-p verbose))))))
    (run-clasp-command clasp-command :f-update-dataset-view t :str-message "Building model....")))

#+FBD
(defun fbd-dialog-make ()
  (make-instance 'fbd-dialog))

#+FBD
(defun compute-bs-fbd (dv ivs sh rs nrs vb)
  (let ((mx (path-analysis::bootstrap-model (mapcar #'clasp::simplify (append ivs (list dv)))
                                            (mapcar #'clasp::name  (append ivs (list dv)))
                                            :num-times nrs :verbosep vb
                                            :num-samples (and rs (clasp::data-length (clasp::simplify dv)))
                                            :shuffle sh)))
    (path-analysis::show-links mx (mapcar #'clasp::name  (append ivs (list dv))) :num-times nrs)
    mx))

#+FBD
(defun compute-fbd (dep &rest indep)
  (path-analysis::fbd
   (path-analysis::make-model-data
    (mapcar #'clasp::simplify
            (append indep (list dep)))
    :names-alist
    (path-analysis::variable-names-alist
     :names (mapcar #'clasp::name (append indep (list dep)))))))

#+FBD
(defun compute-fbd-verbose (dep &rest indep )
  (path-analysis::fbd
   (path-analysis::make-model-data
    (mapcar #'clasp::simplify
            (append indep (list dep)))
    :names-alist
    (path-analysis::variable-names-alist
     :names (mapcar #'clasp::name (append indep (list dep)))))
   :verbosep t))

;;;
;;; CDF functions
;;;

(defparameter *cdf-distributions* '(("F" . f-significance)
                                    ("T" . students-t-significance) 
                                    ("Z" . z-significance)
                                    ("Binomial" . binomial-cdf)
                                    ("Chi Square" . chi-square-significance)))

(defclass cdf-dialog (statistic-dialog-mixin dialog)
  ((ov :initform NIL :accessor old-value)
   (disposable-items :initform NIL :accessor disposable-items))
  (:documentation "A dialog for doing CDF p values or something.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "CDF Statistics"
    :VIEW-SIZE #@(300 110)
    :view-position (best-spot 300 110)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*cdf-help*
    :VIEW-SUBVIEWS
    (list (make-instance 'pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Distribution"
                         :view-nick-name 'dist-menu :view-position #@(5 5)
                         :view-size #@(180 18) 
                         :menu-items (mapcar #'(lambda (def) (make-instance 'valued-menu-item :value (cdr def)
                                                                            :menu-item-title (car def)))
                                             *cdf-distributions*)
                         #-CCL-3 :dialog-item-action #-CCL-3 #'(lambda (item) (reconsider-subviews (view-container item))))
          (make-instance 'compute-button-di :view-position #@(205 6)))))

(defmethod initialize-instance :after ((cdfd cdf-dialog) &key)
  (reconsider-subviews cdfd))

(defmethod view-click-event-handler :after ((cdfd cdf-dialog) (pt-where fixnum))
  (reconsider-subviews cdfd))

(defmethod reconsider-subviews ((cdfd cdf-dialog))
  "When you click a menu item, this thing makes sure all the other dialog-items are a-ok."
  (let ((new-statistic (value (selected-item (view-named 'dist-menu cdfd)))))
    (when (not (eql (old-value cdfd) new-statistic))
      (when (disposable-items cdfd)
        (apply #'remove-subviews cdfd (disposable-items cdfd)))
      (setf (old-value cdfd) new-statistic)
      (setf (disposable-items cdfd) (compute-cdf-subviews new-statistic))
      (apply #'add-subviews cdfd (disposable-items cdfd)))))

#+UNGA-BUNGA
(cdf-dialog-make)

(defmethod f-can-compute-statistics-dialog ((cdfd cdf-dialog))
  t)

(defmethod compute-statistics-dialog ((cdfd cdf-dialog))
  (let ((statistic (value (selected-item (view-named 'dist-menu cdfd)))))
    (compute-cdf-statistic cdfd statistic)))


(defun cdf-dialog-make ()
  (make-instance 'cdf-dialog))

;;;;;; BELOW ARE THE SPECIALIZERS FOR COMPUTING SUBVIEWS AND COMPUTING THE STATISTIC
;;; f-significance
(defmethod compute-cdf-subviews ((stat (eql 'f-significance)))
  (list (make-instance 'static-text-dialog-item :dialog-item-text "F" :view-position #@(75 28))
        (make-instance 'number-entry-item :dialog-item-text "0" :allow-negatives NIL :allow-floats t
                       :view-nick-name 'f-item :view-position #@(5 28) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "DF numerator" :view-position #@(75 53))
        (make-instance 'number-entry-item :dialog-item-text "2" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'dfn-item :view-position #@(5 53) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "DF denominator" :view-position #@(75 78))
        (make-instance 'number-entry-item :dialog-item-text "4" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'dfd-item :view-position #@(5 78) :view-size #@(60 16))
        (make-instance 'check-box-dialog-item :view-position #@(195 28) :dialog-item-text "One tailed ?"
                       :check-box-checked-p t :view-nick-name 'one-tailed)))

(defmethod compute-cdf-statistic ((cdfd cdf-dialog) (stat (eql 'f-significance)))
  (let* ((dof-num (number-item-value (view-named 'dfn-item cdfd)))
        (dof-den (number-item-value (view-named 'dfd-item cdfd)))
        (f-stat  (float (number-item-value (view-named 'f-item cdfd))))
        (onetail (check-box-checked-p (view-named 'one-tailed cdfd)))
        (clasp-command (make-clasp-command `(f-significance ,f-stat ,dof-num ,dof-den ,onetail)
                                            :format-function (make-line-mapper "P = ~,4F for F = ~A with ~A/~A DF." '(0 f df-num df-den))
                                            :description (list '(command F-significance)
                                                               `(f ,f-stat) `(df-num ,dof-num) `(df-den ,dof-den)))))
    (run-clasp-command clasp-command)))

;;; t-significance
(defmethod compute-cdf-subviews ((stat (eql 'students-t-significance)))
  (list (make-instance 'static-text-dialog-item :dialog-item-text "t" :view-position #@(75 28))
        (make-instance 'number-entry-item :dialog-item-text "0" :allow-negatives t :allow-floats t
                       :view-nick-name 't-item :view-position #@(5 28) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "DF" :view-position #@(75 53))
        (make-instance 'number-entry-item :dialog-item-text "2" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'df-item :view-position #@(5 53) :view-size #@(60 16))
        (make-instance 'tails-pop-up-menu :view-position #@(165 28) #+CCL-3 :menu-title #-CCL-3 :dialog-item-text "Tails"
                       :view-nick-name 'tails)))

(defmethod compute-cdf-statistic ((cdfd cdf-dialog) (stat (eql 'students-t-significance)))
  (let* ((dof (number-item-value (view-named 'df-item cdfd)))
         (t-stat  (float (number-item-value (view-named 't-item cdfd))))
         (tail (f-menu-value-tails-pum (view-named 'tails cdfd)))
         (clasp-command (make-clasp-command `(students-t-significance ,t-stat ,dof ,tail)
                                            :format-function (make-line-mapper "P = ~,4F for T = ~A with ~A DF and ~A tail(s)." '(0 t df tails))
                                            :description (list '(command t-significance)
                                                               `(t ,t-stat) `(df ,dof) `(tails ,tail)))))
    (run-clasp-command clasp-command)))

;;; chi-square-significance
(defmethod compute-cdf-subviews ((stat (eql 'chi-square-significance)))
  (list (make-instance 'static-text-dialog-item :dialog-item-text "X" :view-position #@(75 28))
        (make-instance 'number-entry-item :dialog-item-text "0" :allow-negatives t :allow-floats t
                       :view-nick-name 'x-item :view-position #@(5 28) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "DF" :view-position #@(75 53))
        (make-instance 'number-entry-item :dialog-item-text "2" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'df-item :view-position #@(5 53) :view-size #@(60 16))))

(defmethod compute-cdf-statistic ((cdfd cdf-dialog) (stat (eql 'chi-square-significance)))
  (let* ((dof (number-item-value (view-named 'df-item cdfd)))
         (x (float (number-item-value (view-named 'x-item cdfd))))
         (clasp-command (make-clasp-command `(chi-square-significance ,x ,dof)
                                            :format-function (make-line-mapper "P = ~,4F for X = ~A with ~A DF." '(0 x df))
                                            :description (list '(command chi-square-significance)
                                                               `(x ,x) `(df ,dof)))))
    (run-clasp-command clasp-command)))

;;; binomial
(defmethod compute-cdf-subviews ((stat (eql 'binomial-cdf)))
  (list (make-instance 'static-text-dialog-item :dialog-item-text "P" :view-position #@(75 28))
        (make-instance 'number-entry-item :dialog-item-text "0.5" :allow-negatives NIL :allow-floats t
                       :view-nick-name 'p-item :view-position #@(5 28) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "Trials" :view-position #@(75 53))
        (make-instance 'number-entry-item :dialog-item-text "20" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'n-item :view-position #@(5 53) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "Successes" :view-position #@(75 78))
        (make-instance 'number-entry-item :dialog-item-text "4" :allow-negatives NIL :allow-floats NIL
                       :view-nick-name 'k-item :view-position #@(5 78) :view-size #@(60 16))))

(defmethod compute-cdf-statistic ((cdfd cdf-dialog) (stat (eql 'binomial-cdf)))
  (let* ((p (float (number-item-value (view-named 'p-item cdfd))))
         (n (number-item-value (view-named 'n-item cdfd)))
         (k (number-item-value (view-named 'k-item cdfd)))
         (clasp-command (make-clasp-command `(binomial-cdf ,p ,n ,k)
                                            :format-function (make-line-mapper "P = ~,4F for Bin(~A,~A,~A)." '(0 p n k))
                                            :description (list '(command F-significance)
                                                               `(p ,p) `(n ,n) `(k ,k)))))
    (run-clasp-command clasp-command)))

;;; z significance GAUSSIAN
(defmethod compute-cdf-subviews ((stat (eql 'z-significance)))
  (list (make-instance 'static-text-dialog-item :dialog-item-text "X" :view-position #@(75 28))
        (make-instance 'number-entry-item :dialog-item-text "0.5" :allow-negatives t :allow-floats t
                       :view-nick-name 'x-item :view-position #@(5 28) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "Mean" :view-position #@(75 53))
        (make-instance 'number-entry-item :dialog-item-text "0" :allow-negatives t :allow-floats t
                       :view-nick-name 'm-item :view-position #@(5 53) :view-size #@(60 16))
        (make-instance 'static-text-dialog-item :dialog-item-text "Std. Deviation" :view-position #@(75 78))
        (make-instance 'number-entry-item :dialog-item-text "1" :allow-negatives t :allow-floats t
                       :view-nick-name 'sd-item :view-position #@(5 78) :view-size #@(60 16))
        (make-instance 'tails-pop-up-menu :view-position #@(165 28) #+CCL-3 :menu-title #-CCL-3 :dialog-item-text "Tails"
                       :view-nick-name 'tails)))

(defmethod compute-cdf-statistic ((cdfd cdf-dialog) (stat (eql 'z-significance)))
  (let* ((x (float (number-item-value (view-named 'x-item cdfd))))
         (m (number-item-value (view-named 'm-item cdfd)))
         (sd (number-item-value (view-named 'sd-item cdfd)))
         (tails (f-menu-value-tails-pum (view-named 'tails cdfd)))
         (clasp-command (make-clasp-command `(gaussian-significance ,x ,tails ,m ,sd)
                                            :format-function (make-line-mapper "P = ~,4F for ~A at Z(~A,~A) with ~A tails." '(0 x m sd tails))
                                            :description (list '(command Z-significance)
                                                               `(x ,x) `(m ,m) `(sd ,sd) `(tails ,tails)))))
    (run-clasp-command clasp-command)))

#+UNGA-BUNGA
(cdf-dialog-make)
