;;;
;;; menus.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines prototype CLASP menus.


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
16-May-94 mc	Created.

|#


(in-package "CLASP-INTERFACE")

(export '(install-clasp-menu            ;need these three?
          install-clasp-menubar
          install-default-menubar))

(defvar *rw-menu-item* nil) ; stores the ribbon window menu item so I can check it's box

;;;
;;; Define menu variables.
;;;

(defun clasp-menubar-make ()
  ;; Check this sometime to see if we could go back and put this in a global data structure 
  ;; instead of recreating it each time.
  "Returns a list of the clasp menus. Note: We could bind this result to a
global variable but this caused errors in install-clasp-menu because
*windows-menu* was already installed. We got a similar error for the
File menu. Reconstructing the menus solves the problem but makes
installation slower and use more memory."
  ;;
  (LIST (MAKE-INSTANCE 'MENU
          :MENU-TITLE "File"
          :help-spec "File Menu

Use this menu to save and load datasets, to import and export data, and to quit CLASP."
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Load Dataset"
                               :help-spec "Prompts you to select a dataset file then loads it."
                               #-MCG-TEST :command-key #-MCG-TEST #\O
                               #+MCG-TEST :command-key #+MCG-TEST #\0 ;; zero
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(choose-and-load-dataset))))
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Import Dataset"
                               :help-spec "Prompts you to select a data file then imports it. Use the Preferences commands to specify parameters such as file separator."
                               #-MCG-TEST :command-key #-MCG-TEST #\I
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(choose-and-import-dataset))))
                (MAKE-INSTANCE 'one-dataset-menu-item
                  :MENU-ITEM-TITLE "Save Dataset"
                  :help-spec "Prompts you to name a data file in which to save the selected dataset. Enabled when exactly one dataset is selected in the frontmost clasp window."
                  #-MCG-TEST :command-key #-MCG-TEST #\S
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(save-selected-dataset))))
                (MAKE-INSTANCE 'one-dataset-menu-item
                  :MENU-ITEM-TITLE "Export Dataset"
                  :help-spec "Prompts you to name a data file then exports the selected dataset to it. Use the Preferences commands to specify parameters such as file separator."
                  #-MCG-TEST :command-key #-MCG-TEST #\F
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(export-selected-dataset))))
                (MAKE-INSTANCE 'menu-item :MENU-ITEM-TITLE "Close"
                               :command-key #\W
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(window-close (front-window)))))
                (MAKE-INSTANCE 'menu-item :menu-item-title "Close All"
                               ;:command-key #\-
                               :menu-item-action
                               #'(lambda ()
                                   (mapc #'(lambda (w) (unless (or (typep w 'listener) (typep w 'clasp-dock)
                                                                   (typep w 'fred-window) (typep w 'clasp-window))
                                                         (window-close w))) (windows))))
                ;;
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)

                (MAKE-INSTANCE 'window-menu-item :MENU-ITEM-TITLE "Clear History"
                               :command-key #\Backspace
                               :help-spec "Clears the frontmost CLASP window's history view. Enabled when the front window is a CLASP window."
                               :menu-item-action #'(lambda () (clear-history-view (clasp-window-front))))
                #+IGNORE
                (MAKE-INSTANCE 'window-menu-item :MENU-ITEM-TITLE "Save History"
                               :help-spec "Saves the frontmost CLASP window's history view. Enabled when the front window is a CLASP window."
                               :menu-item-action 'save-history-dialog-cw)
                #+IGNORE
                (MAKE-INSTANCE 'window-menu-item :MENU-ITEM-TITLE "Restore History"
                               :help-spec "Restores the frontmost CLASP window's history view. Enabled when the front window is a CLASP window."
                               :menu-item-action 'restore-history-dialog-cw)
                (MAKE-INSTANCE 'window-menu-item :MENU-ITEM-TITLE "Update Dataset View"
                               :help-spec "Updates the dataset view in the frontmost CLASP window. Enabled when the front window is a CLASP window."
                               :menu-item-action #'(lambda () (update-dataset-view (clasp-window-front))))
                (MAKE-INSTANCE 'history-selection-menu-item :MENU-ITEM-TITLE "Log Selection"
                               #-MCG-TEST :command-key #-MCG-TEST #\T
                               :help-spec "Logs the frontmost CLASP window's selected history items. Enabled when the front window is a CLASP window."
                               :menu-item-action 'log-selected-items)
                (MAKE-INSTANCE 'menu-item :menu-item-title "Copy to Clipboard" 
                               :command-key #\C
                               :help-spec "Cuts the current window's selection to the clipboard."
                               :menu-item-action 
                               #+DEMO 'sorry-demo-version
                               #-DEMO #'(lambda () (eval-enqueue `(cut-selection (front-window)))))
                (MAKE-INSTANCE 'ribbon-open-menu-item :menu-item-title "Copy Ribbon to Clipboard" 
                               :command-key #\B
                               :help-spec "Cuts the ribbon window's contents to the clipboard."
                               :menu-item-action #'(lambda () (eval-enqueue `(cut-selection (find-window "gibbon-monkey" 'ribbon-window)))))
                (MAKE-INSTANCE 'menu-item :menu-item-title "Paste from Clipboard"
                               :command-key #\V
                               :menu-item-action #'(lambda () (eval-enqueue `(paste-into-clasp (front-window))))
                               :help-spec "Pastes the current clipboard to the current window.")
                ;;
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                ;;
                #+DOES-NOTHING (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Print"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(warn-feature-missing))))
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Preferences"
                               :command-key #\`
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(edit-preferences))))
                ;;
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                ;;
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Quit"
                               :COMMAND-KEY #\Q
                               :help-spec "Closes all windows and exits from CLASP. If the contents of any window have not been saved, you will loose that data."
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(quit-clasp))))))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Graph"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Histogram"
                               :command-key #\H
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(graph-ov-dialog-make 'histogram "Histogram" :help '*histogram-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Scatter Plot"
                               :command-key #\A 
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(graph-tv-dialog-make 'scatterplot "Scatterplot" t 
                                                   :help '*scatterplot-help* :categorical-x nil :categorical-y nil))))
		(MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Cluster Plot"
			       ;:command-key #\A
			       :menu-item-action
			       #'(lambda ()
				   (eval-enqueue '(graph-tv-dialog-make 'cluster-plot "Cluster Plot" t
						   :help '*cluster-plot-help* :categorical-x nil :categorical-y nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Line Plot"
                               :command-key #\L 
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(graph-tv-dialog-make 'line-plot "Line Plot" NIL 
                                                   :help '*line-plot-help* :categorical-x nil :categorical-y nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Row Line Plot"
                               :command-key #\N 
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(graph-ov-dialog-make 'row-line-plot "Row Line Plot" 
                                                   :help '*row-line-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Regression Plot"
                               :command-key #\G 
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(graph-tv-dialog-make 'regression-plot "Regression Plot" t 
                                                   :help '*regression-plot-help* :categorical-x nil :categorical-y nil))))
                (MAKE-INSTANCE 'table-window-menu-item :MENU-ITEM-TITLE "Table plot"
                               :command-key #\P :slot-to-check nil
                               :menu-item-action 
                               #'(lambda ()
                                   (eval-enqueue `(make-table-plot (front-window)))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Overlay Graphs"
                               :command-key #\=
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(make-instance 'overlaid-graph-dialog))))
                (MAKE-INSTANCE 'menu-item :MENU-ITEM-TITLE "-"
                               :DISABLED t)
                (MAKE-INSTANCE 'graph-window-menu-item :MENU-ITEM-TITLE "Edit"
                               :command-key #\E 
                               :menu-item-action #'(lambda () 
                                                    (eval-enqueue `(make-graph-editor (window-graph (front-window))))))

                (MAKE-INSTANCE 'graph-window-menu-item :MENU-ITEM-TITLE "Graph Key"
                               :command-key #\K
                               :menu-item-action #'(lambda () (eval-enqueue `(plotter::pop-key (front-window)))))))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Describe"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Data Length"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::data-length "Data Length"
                                                              (make-line-mapper "~D is data length of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*data-length-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Mean"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::mean "Mean"
                                                              (make-line-mapper "~,4F is mean of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*mean-help*
                                                              :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Variance"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::variance "Variance"
                                                              (make-line-mapper "~,4F is variance of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*variance-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Standard Deviation"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::standard-deviation "Standard Deviation"
                                                              (make-line-mapper "~,4F is standard deviation of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*std-dev-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Minimum"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::minimum "Minimum"
                                                              (make-line-mapper "~,4F is minimum of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*minimum-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Maximum"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::maximum "Maximum"
                                                              (make-line-mapper "~,4F is maximum of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*maximum-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Range"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::range "Range"
                                                              (make-line-mapper "~,4F is range of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*range-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Quantile"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(quantile-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Median"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::median "Median"
                                                              (make-line-mapper "~,4F is median of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*median-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Trimmed Mean"
                               :menu-item-action
                               #'(lambda ()
                                   ;; FIX: Following needs additional args:
                                   (eval-enqueue '(trimmed-mean-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Mode"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(mode-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Interquartile Range"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::interquartile-range "Interquartile Range"
                                                              (make-line-mapper "~,4F is interquartile Range of ~:(~A~)"
                                                                                '(0 x-args))
                                                              :help '*iqr-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Skew"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                    `(one-variable-dialog-make 'clasp::skewness "Skew"
                                                               (make-line-mapper "~,4F is skew of ~:(~A~)"
                                                                                 '(0 x-args))
                                                               :help '*skew-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Statistical Summary"
                               :command-key #\M
                               :menu-item-action
                               #'(lambda ()
                                   ;; FIX: Do what with following output?
                                   (eval-enqueue
                                   `(one-variable-dialog-make 'clasp::statistical-summary "Statistical Summary" 
                                                              (make-table-maker *statistical-summary-labels*
                                                                                :table-type :column-table
                                                                                :name-format "Statistical Summary of ~A"
                                                                                :name-args '(X-ARGS))
                                                              :help '*statistical-summary-help* :categorical-ok nil))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Covariance"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(two-variable-dialog-make 
                                                   'clasp::covariance "Covariance"
                                                   (make-line-mapper "~,4F is covariance of ~:(~A~) and ~:(~A~)"
                                                                     '(0 X-ARGS Y-ARGS))
                                                   :help '*covariance-help*
                                                   :categorical-x NIL :categorical-y NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Correlation"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(two-variable-dialog-make
                                                   'clasp::correlation "Correlation"
                                                   (make-line-mapper "~,4F is correlation of ~:(~A~) and ~:(~A~)"
                                                                     '(0 X-ARGS Y-ARGS))
                                                   :help '*correlation-help*
                                                   :categorical-x NIL :categorical-y NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Partial Correlation"
                               :menu-item-action 'partial-corr-dialog-make)
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Cross Correlation"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(cross-correlation-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Autocorrelation"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(autocorrelation-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Correlation Table"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(corr-table-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Contingency Table"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue
                                    `(two-variable-dialog-make 'clasp::make-contingency-table "Contingency Table"
                                                               'ctable-format
                                                               :help '*contingency-help*))))
                #+FBD
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "FBD Causal Model"
                               :menu-item-action
                               'fbd-dialog-make)
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "CDF Functions"
                               :menu-item-action 'cdf-dialog-make)))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Manipulate"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Rename"
                               :help-spec "Renames the selected dataset. Enabled when the front window is a CLASP window with one dataset selected."
                               :command-key #\R
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(rename-selected-things))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Delete Result"
                               :command-key #\D
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(delete-selected-items))))
#| NOT IMPLEMENTED 
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Make Dataset From Rows"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(explain-missing-function "clasp::make-dataset-from-rows"))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Make Dataset from Columns"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(explain-missing-function "clasp::make-dataset-from-columns"))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Add Variable To Dataset"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(explain-missing-function "clasp::add-variable-to-dataset"))))
|#
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Partition Dataset"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(partition-dataset-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Partition On"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(partition-on-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Merge Datasets"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(merge-datasets-dialog-make))))
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                (MAKE-INSTANCE 'MENU
                  :MENU-TITLE "Tables"
                  :MENU-ITEMS 
                  (LIST (MAKE-INSTANCE 'table-window-menu-item :MENU-ITEM-TITLE "do nothing"
                                       :help-spec "Control-click menu item for table."
                                       :slot-to-check 'control
                                       :menu-item-action #'(lambda () (control-click 
                                                                       (table-view-table
                                                                        (view-named
                                                                         'table-array-item
                                                                         (front-window))))
                                                            (revalidate-view (view-named
                                                                              'table-array-item
                                                                              (front-window)))))
                        (MAKE-INSTANCE 'table-window-menu-item :MENU-ITEM-TITLE "do nothing"
                                       :help-spec "Option-click menu item for table."
                                       :slot-to-check 'option
                                       :menu-item-action #'(lambda () (option-click
                                                                       (table-view-table
                                                                        (view-named
                                                                         'table-array-item
                                                                         (front-window))))
                                                            (revalidate-view (view-named
                                                                              'table-array-item
                                                                              (front-window)))))
                        (MAKE-INSTANCE 'table-window-menu-item :MENU-ITEM-TITLE "do nothing"
                                       :help-spec "Double-click menu item for table."
                                       :slot-to-check 'double
                                       :menu-item-action #'(lambda () (dbl-click 
                                                                       (table-view-table
                                                                        (view-named
                                                                         'table-array-item
                                                                         (front-window))))
                                                            (revalidate-view (view-named
                                                                              'table-array-item
                                                                              (front-window)))))))
                (MAKE-INSTANCE 'MENU
                  :MENU-TITLE "Variables"
                  :MENU-ITEMS
                  (LIST (MAKE-INSTANCE 'dock-menu-item :MENU-ITEM-TITLE "X Dock" :dock-name "X Dock"
                                       :command-key #-MCG-TEST #\X #+MCG-TEST #\- :affects 'x-var-item
                                       :help-spec "Sends the selected variables to the x dock."
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-dock (front-window) 'x-var-dock))))
                        (MAKE-INSTANCE 'dock-menu-item :MENU-ITEM-TITLE "Y Dock" :dock-name "Y Dock"
                                       :command-key #-MCG-TEST #\Y #+MCG-TEST #\= :affects 'y-var-item
                                       :help-spec "Sends the selected variables to the y dock."
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-dock (front-window) 'y-var-dock))))
                        (MAKE-INSTANCE 'dock-menu-item :MENU-ITEM-TITLE "Cache" :Dock-name "Cache"
                                       :command-key #\Z :affects 'z-var-item
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-dock (front-window) 'gp-dock))))
                        ;;
                        (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                        (MAKE-INSTANCE 'mrr-menu-item :MENU-ITEM-TITLE "X"
                                       :command-key #\[ :affects 'x-var-item
                                       :help-spec "Replaces the x arguments with the most recent result."
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-mrr (front-window) 'x-var-item))))
                        (MAKE-INSTANCE 'mrr-menu-item :MENU-ITEM-TITLE "Y"
                                       :command-key #\] :affects 'y-var-item
                                       :help-spec "Replaces the y arguments with the most recent result."
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-mrr (front-window) 'y-var-item))))
                        (MAKE-INSTANCE 'mrr-menu-item :MENU-ITEM-TITLE "Z"
                                       :command-key #\} :affects 'z-var-item
                                       :help-spec "Replaces the y arguments with the most recent result."
                                       :menu-item-action
                                       #'(lambda () (eval-enqueue `(operate-on-mrr (front-window) 'z-var-item))))
                        ;;
                        (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                        #+OBSOLETE
                        (MAKE-INSTANCE 'one-variable-menu-item :MENU-ITEM-TITLE "Rename Variable"
                                       :help-spec "Renames the selected variable. Enabled when the front window is a CLASP window with one variable selected."
                                       :command-key #\R
                                       :menu-item-action
                                       #'(lambda ()
                                           (eval-enqueue '(rename-selected-variable))))
                        ;;
                        (MAKE-INSTANCE 'openable-selection-menu-item :MENU-ITEM-TITLE "Change Type"
                                       :command-key #\J :help-spec "Changes the type to categorical or whatever."
                                       :menu-item-action
                                       #'(lambda ()
                                           (change-selected-variable-types)))
                        (MAKE-INSTANCE 'openable-selection-menu-item :MENU-ITEM-TITLE "Edit"
                                       :command-key #-MCG-TEST #\E #+MCG-TEST #\Tab
                                       :help-spec "Opens the selected variables and datasets from the data view for editing/viewing."
                                       :menu-item-action 'open-openable-selection)
                        ))
                ))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Transform"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "User Defined"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(make-user-defined-dialog))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Sort"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(sort-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Sort by Index"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(sort-index-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Add a Constant"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(add-constant-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Multiply by Constant"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(mul-constant-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Natural Log"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(one-var-trans-dialog-make 'clasp::natural-log "Natural Log"
                                                   :help '*natural-log-help*
                                                   :categorical-ok NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Log10"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(one-var-trans-dialog-make 'clasp::log10 "Log Base 10"
                                                   :help '*log10-help* :categorical-ok NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Recode Categorical"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(make-instance 'recode-variable-dialog))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Reassign Bins"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(rebin-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Smooth"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(smooth-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Smooth 4235h"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(one-var-trans-dialog-make 'clasp::smooth-variable-4253h "Smooth 4253h"
                                                   :help '*smooth-4253h-help* :categorical-ok NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Discrete Derivative"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(one-var-trans-dialog-make 'clasp::Difference-a-column "Discrete Derivarive"
                                                   :help '*derivative-help* :categorical-ok NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Standardize"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(one-var-trans-dialog-make 'clasp::standardize-variable "Standardize"
                                                   :help '*standardize-help* :categorical-ok NIL))))))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Test"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Confidence Interval Using z Statistic"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(confidence-interval-data-dialog-make
                                      'clasp::confidence-interval-z 
                                      (make-table-maker
                                       *confidence-labels*
                                       :table-type :row-table
                                       :name-format (format nil "~A Z Confidence Interval on ~A" "~5F" "~:(~A~)")
                                       :name-args '(CONFIDENCE X-ARGS))
                                      "Confidence Interval Using Z Statistic"
                                      :help '*confi-z-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Confidence Interval Using t Statistic"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(confidence-interval-data-dialog-make
                                      'clasp::confidence-interval-t 
                                      'confidence-t-formatter
                                      "Confidence Interval Using T Statistic"
                                      :help '*confi-t-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Confidence Interval of a Proportion"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(confidence-prop-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "t-test One Sample"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(t-test-one-sample-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "t-test Two Samples"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(t-test-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "t-test Matched Pairs"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(t-test-matched-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "D test"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(d-test-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :menu-item-title "Analysis of Contrasts"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(make-instance 'contrast-toolbox-dialog))))
                (MAKE-INSTANCE 'clasp-window-menu-item :menu-item-title "Analysis of Variance"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(make-instance 'anova-toolbox-dialog))))
                #+OBVIOUS
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Anova - One Way"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(anova-one-way-dialog-make))))
                #+OBVIOUS
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Anova - Two Way"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(anova-two-way-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Chi-Square Counts"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(chi-square-2x2-counts-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Chi-Square 2x2"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(two-variable-dialog-make
                                                   'clasp::chi-square-2x2 "Chi-Square 2x2"
                                                   'chi-2x2-format
                                                   :help '*chi-2x2-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Chi-Square RxC"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(two-variable-dialog-make
                                                   'clasp::chi-square-rxc "Chi-Square RxC"
                                                   'chi-rxc-format
                                                   :help '*chi-rxc-help*))))
                (MAKE-INSTANCE 'clasp-window-menu-item :menu-item-title "3d Contingency Table Analysis"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(make-instance 'analyze-3d-dialog))))
                (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Linear Regression - brief"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(two-variable-dialog-make 
                                      'clasp::linear-regression-brief
                                      "Linear Regression - brief"
                                      (make-table-maker *brief-lr-labels*
                                                        :table-type :column-table
                                                        :name-format "Linear Regression of ~:(~A~) against ~:(~A~)"
                                                        :name-args '(X-ARGS Y-ARGS))
                                      :help '*brief-lr-help*
                                      :categorical-x NIL :categorical-y NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Linear Regression - verbose"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(two-variable-dialog-make
                                      'clasp::linear-regression-verbose
                                      "Linear Regression - verbose"
                                      'verbose-lr-formatter
                                      :help '*verbose-lr-help*
                                      :categorical-x NIL :categorical-y NIL))))
                (MAKE-INSTANCE 'clasp-window-menu-item
                  :MENU-ITEM-TITLE
                  "Multiple Linear Regression - verbose"
                  :menu-item-action
                  #'(lambda ()
                      (eval-enqueue '(multi-linear-regr-dialog-make))))))
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Sample"
          :MENU-ITEMS
          (LIST (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Uniform Distribution"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(uniform-dist-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Normal Distribution"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(normal-dist-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Binomial Distribution"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(binomial-dist-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Poisson Distribution"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(poisson-dist-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Gamma Distribution"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(gamma-dist-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Monte Carlo Sampling"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(monte-carlo-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Bootstrap/Randomize"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(data-sampling-dialog-make))))
                (MAKE-INSTANCE 'clasp-window-menu-item :MENU-ITEM-TITLE "Mixed Resampling"
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue `(bootstrap-dialog-make))))))
        ;;
        (setf *clasp-windows-menu*
         (MAKE-INSTANCE 'clasp-windows-menu
          :MENU-TITLE "Windows"
          :help-spec "Window Menu

Use this select from the various windows available in the CLASP interface."
          :MENU-ITEMS
          (setf *clasp-windows-menu-default-items*
             (list
           ;; FIX: Put following somewhere else?
           (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "CLASP Window"
                               :help-spec "Selects the current CLASP window or creates a new one if there isn't one."
                               :command-key #\1
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(select-current-or-new-clasp-window))))
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "CLASP Variable Palette"
                               :help-spec "Selects the CLASP variable dock or creates one if there isn't one."
                               :command-key #\2
                               :menu-item-action
                               #'(lambda ()
                                   (eval-enqueue '(find-clasp-dock :force-access t))))
                (make-instance 'menu-item :menu-item-title "CLASP Notebook"
                               :help-spec "Selects the CLASP Notebook or opens a new one if there isn't one."
                               :command-key #\3
                               :menu-item-action
                               #+DEMO 'sorry-demo-version
                               #-DEMO 'open-clasp-notebook)
                (make-instance 'menu-item :menu-item-title "Graph Palette"
                               :help-spec "a list of generated graphs"
                               :command-key #\4
                               :menu-item-action 'open-graph-caddy)
                (make-instance 'menu-item :menu-item-title "Table Palette"
                               :help-spec "a list of generated tables"
                               :command-key #\5
                               :menu-item-action 'open-table-caddy)
                (setf *rw-menu-item* 
                      (make-instance 'menu-item :menu-item-title "Ribbon Window"
                                     :help-spec "is the ribbon window open ?"
                                     :command-key #\6
                                     :menu-item-checked *use-ribbon-window*
                                     :menu-item-action
                                     #'(lambda ()
                                         (set-menu-item-check-mark *rw-menu-item*
                                                                   (setf *use-ribbon-window* (not *use-ribbon-window*)))
                                         (if *use-ribbon-window*
                                           (open-ribbon)
                                           (ignore-errors (window-close (find-window "gibbon-monkey")))))))
                (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
                ))
          ))
        #+CLASP-MCL-INTERFACE-DEBUG
        (MAKE-INSTANCE 'MENU
          :MENU-TITLE "Debug"
          :help-spec "Debug Menu

This menu should be only be enabled in the development version."
          :MENU-ITEMS
          (list
           (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Install CLASP Menu"
                          :menu-item-action #'(lambda () 
                                                (install-default-menubar)
                                                (install-clasp-menu)))
           (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Install CLASP Menubar"
                          :menu-item-action 'install-clasp-menubar)
           (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
           (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Install Default Menubar"
                          :menu-item-action 'install-default-menubar)))

          ))

(defclass clasp-windows-menu (menu)
  ())

(defmethod menu-update ((item clasp-windows-menu))
  (let ((front (front-window))
        (windows (windows)))
    (apply #'remove-menu-items *clasp-windows-menu* (menu-items *clasp-windows-menu*))
    (apply #'add-menu-items *clasp-windows-menu* 
           (append *clasp-windows-menu-default-items*
                   (loop for w in windows
                         collect (make-instance 'ccl::windows-menu-menu-item
                                   :disabled (eq w front)
                                   :command-key (when (eq w *top-listener*) #\L) 
                                   :menu-item-title (ccl::window-title w)
                                   :window w))))))
      

(defvar *clasp-windows-menu*)
(defvar *clasp-windows-menu-default-items*)

(defun l-menu-item-clasp-apple-menu ()
  "Returns a list of CLASP menu items that are installed on the apple
menu, including the separator. See note in clasp-menubar-make
concerning binding this to a global variable."
  ;;
  (LIST (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "About CLASP"
                       :menu-item-action
                       'display-about-box)
        (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "Help"
                       :COMMAND-KEY #\?
                       :menu-item-action
                       #'(lambda ()
                           (display-help (front-window))))
        ;;
        (MAKE-INSTANCE 'MENU-ITEM :MENU-ITEM-TITLE "-" :DISABLED T)
        ;;
        ))


;;;
;;; Define menu installation functions.
;;;

(defun install-clasp-menu ()
  "Installs (l-menu-item-clasp-apple-menu) and (clasp-menubar-make)
items in a new CLASP menu, for development."
  ;;
  (let* ((menu-old (find-menu "CLASP"))
         ;(clasp-menubar-sans-windows (remove *windows-menu* (clasp-menubar-make)))
         (menu (make-instance 'menu :menu-title "CLASP"
                              :menu-items (append (l-menu-item-clasp-apple-menu)
                                                  (clasp-menubar-make)
                                                  ;clasp-menubar-sans-windows
                                                  ))))
    (when menu-old
      (menu-deinstall menu-old))
    (menu-install menu)))


(defun install-clasp-menubar ()
  "Removes the current menubar and installs (l-menu-item-clasp-apple-menu)
and (clasp-menubar-make) ."
  ;;
  (PROGN
    (LET ((APPLE-MENU *APPLE-MENU*))
      (APPLY #'REMOVE-MENU-ITEMS APPLE-MENU (MENU-ITEMS APPLE-MENU))
      (APPLY #'ADD-MENU-ITEMS APPLE-MENU (l-menu-item-clasp-apple-menu)))
    (SET-MENUBAR (clasp-menubar-make))))


(defun install-default-menubar ()
  "Installs *DEFAULT-MENUBAR* ."
  ;;
  ;; FIX: Doesn't save default apple menu items. (See CCL::ABOUT-CCL ?)
  ;;
  (SET-MENUBAR *DEFAULT-MENUBAR*))

#+MCG-TEST
(install-clasp-menu)                    ;for now install developer-style menu
#-MCG-TEST
(install-clasp-menubar)

#|
(install-clasp-menu)
(install-clasp-menubar)
(install-default-menubar)
|#