#|
======================
Tables================
======================

the table class for mac CLASP

in progress

20-Jul-94 ms started
22-Jul-94 ms beta finish (?)
7-Sep-94  ms semi converted to dynamic cell sizing. _table-largest-column
             tracks the biggest column and view/cell sizes are constructed from it.
13-Sep-94 ms FULLY FONT SENSITIVE TABLES !! YAY. font preferences moved to preferences.lisp.

|#

(in-package "CLASP-INTERFACE")

;; a note on tables & table data :
;;  tables should be sequences of sequences. each inner-sequence should
;;  be a row of data items. 

(defvar *hdr-char* "|" "The indicator prefix for table headers (labels).")
(defvar *ttl-char* "_" "The indicator prefix for table titles.")
(defvar *clasp-tables* '())

(defclass statistical-table (clasp::named-object-mixin)
  ((column-names
    :initform nil
    :initarg :column-names
    :type LIST
    :accessor table-column-names
    :documentation "The names of the columns of the table.")
   (row-names
    :initform nil
    :initarg :row-names
    :type LIST
    :accessor table-row-names
    :documentation "The names of the rows of the table.")
   (table-data
    :initform nil
    :initarg :table-data
    :type LIST
    :accessor table-data
    :documentation "The meat 'n potatoes of the table.")
   (largest-column
    :initform nil
    :type FIXNUM
    :accessor table-largest-column
    :documentation "The largest column of the table, in characters.")
   )
  (:documentation "The class of tables."))

(defmethod initialize-instance :after ((statistical-table statistical-table) &key (primary-table nil))
  "makes sure that the column & row names are qualified with a header
character *header-char*. only strings prefaced by the header character 
are printed bold. Any preprocessing necessary for subclasses is done first."
  (let ((col-names (table-column-names statistical-table))
        (row-names (table-row-names statistical-table)))
    (setf (table-column-names statistical-table)
          (mapcar #'(lambda (string) (concatenate 'string *hdr-char*
                                                  string))
                   col-names))
    (setf (table-row-names statistical-table)
          (mapcar #'(lambda (string) (concatenate 'string *hdr-char*
                                                  string))
                   row-names)))
  (setq *clasp-tables* (append *clasp-tables* (list statistical-table)))
  (setf (table-largest-column statistical-table) (largest-column (list 
                                                                  (table-row-names statistical-table)
                                                                  (table-column-names statistical-table)
                                                                  (table-data statistical-table))))
  (if (and primary-table *AUTO-OPEN-PRIMARY-TABLES*)
    (open-clasp-object statistical-table)))

;;
;; specialized table classes
;;

(defclass scheffe-table (statistical-table)
  ((significance :accessor significance :initarg :significance)
   (scheffe :accessor scheffe :initarg :scheffe)
   (control :initform "Sceffe Statistics")
   (option  :initform "Scheffe Significance"))
  (:documentation "A scheffe table; can switch between scheffe and significance.")) 

;; 2d contingency-tables
(defclass contingency-table (statistical-table)
  ((actual-data :accessor actual-data)
   (y-variable :initarg :y-variable)
   (row-percents :accessor row-percents)
   (column-percents :accessor column-percents)
   (control :initform "Row proportions")
   (option  :initform "Column proportions")
   (double  :initform "Observed frequencies"))
  (:documentation "A subclass of statistical-table."))

(defun calc-row-percents (data)
  (let ((buff data))
    (mapcar #'(lambda (list)
                (let ((maxi (length list))
                      lbuff)
                  (dotimes (i (1- maxi))
                    (setf lbuff (append lbuff (list (/ (elt list i) (elt list (1- maxi)))))))
                  (setf lbuff (append lbuff (list 1.0)))
                  lbuff))
            buff)))

(defun calc-col-percents (data)
  (let (buff result)
    (dotimes (i (1- (length data)))
      (setf buff '())
      (dotimes (ii (length (elt data i)))
        (setf buff (append buff (list (/ (elt (elt data i) ii)
                                         (elt (car (last data)) ii))))))
      (setf result (append result (list buff))))
    (append result (list (make-list (length (car data)) :initial-element 1.0)))))

(defmethod initialize-instance :after ((ct contingency-table) &key)
  (setf (actual-data ct) (table-data ct))
  (setf (row-percents ct) (calc-row-percents (table-data ct)))
  (setf (column-percents ct) (calc-col-percents (table-data ct))))

;; group means tables
(defclass group-means (statistical-table)
  ((confidence-intervals
    :initform nil
    :initarg :confidence-intervals
    :accessor confidence-intervals))
  (:documentation "A subclass of statistical-table."))

;; lag table
(defclass lag-table (statistical-table)
  ()
  (:documentation "A subclass of a statistical-table."))

;; censored-data-table
(defclass censored-data-table (statistical-table)
  ()
  (:documentation "A table with mlr independant variables which appear to be colinear."))

;;;
;;; general methods on tables, etc.
;;;

(defun largest-column (table)
  (cond
   ((null table) 0)
   ((atom (car table))
    (max (+ 2 (length (format nil (if (and (not (integerp (car table))) (numberp (car table))) "~,4F" (if (stringp (car table)) "~A" "~S"))
                              (car table))))
         (largest-column (cdr table))))
   (t (max (largest-column (car table)) (largest-column (cdr table))))))

(defmethod f-can-open-clasp-object ((object statistical-table))
  t)

(defmethod open-clasp-object ((table statistical-table))
  (let ((win (find-if #'(lambda (window)
                          (ignore-errors (equal table (table-view-table (view-named 'table-array-item window)))))
                      (windows))))
    (if win
      (window-select win)
      (let* ((maxcollen (table-largest-column table))
             (cols (if (table-column-names table)
                     (+ (length (table-column-names table)) (if (table-row-names table) 1 0))
                     (+ (length (car (table-data table))) (if (table-row-names table) 1 0))))
             (rows (if (table-row-names table)
                     (+ (length (table-row-names table)) (if (table-column-names table) 1 0))
                     (+ (length (table-data table)) (if (table-column-names table) 1 0))))
             ;; screen x width and y width. note font-sensitive x spacing.
             (sx (min *screen-width* (+ 16 (* (string-width (make-string maxcollen :initial-element #\9) *table-header-font*)
                                              cols))))
             (sy (min *screen-height* (+ 30 (* (table-char-height) (1+ rows))))))
      (make-instance 'table-view-window :window-title (CLASP::name table)
                     :table table
                     :view-size
                     (make-point sx sy)
                     :view-position (best-spot sx sy))))))

;;;
;;; viewing stuff for tables
;;;

(defclass table-view (array-dialog-item)
  ((statistical-table
    :initform nil
    :initarg :table
    :type STATISTICAL-TABLE
    :accessor table-view-table
    :documentation "the statistical table being viewed.")
   (view-array
    :initform nil
    ;; init'd by intialize instance
    :type LIST
    :accessor view-array-table-view
    :documentation "the actual viewing stuff."))
    (:documentation "an array-dialog-item subclass for showing a
table's contents.")
    (:DEFAULT-INITARGS
      :view-font *table-header-font*
      :table-hscrollp t
      :table-vscrollp t
      :table-print-function
      #'(lambda (print-item stream)
          (declare (ignore stream))
          (let ((font-spec (if (ignore-errors (search *hdr-char* print-item))
                             *table-header-font*
                             *table-entry-font*)))
            (multiple-value-bind (ff ms) (font-codes font-spec)
              (with-font-codes ff ms
                (with-pstrs ((p-str (remove (aref *hdr-char* 0) print-item)))
                  (#_DrawString p-str))))))))

(defun table-char-height ()
  (+ (car (last *table-entry-font*)) 2))

(defun table-char-width ()
  (car (last *table-entry-font*)))

(defmethod initialize-instance :after ((table-view table-view) &key)
  "sets the cell size to font-sensitive width, then sets up the array of numbers for the array subview"
  (set-cell-size table-view 
                 (string-width (make-string (table-largest-column (table-view-table table-view)) :initial-element #\9) *table-header-font*)
                 (table-char-height))
  (set-table-array table-view
                   (array-from-table (table-view-table table-view))))

(defmethod revalidate-view ((table-view table-view))
  (set-cell-size table-view 
                 (string-width (make-string (table-largest-column (table-view-table table-view)) :initial-element #\9) *table-header-font*)
                 (table-char-height))
  (set-table-array table-view
                   (array-from-table (table-view-table table-view))))

;;
;;


(defclass table-view-window (window)
  ()
  (:documentation "A window subclass used for viewing statistical tables."))

(defmethod window-size-parts :after ((table-view-window table-view-window))
  "Sets the array-dialog-item to fill the window."
  (let ((table-array-item (view-named 'table-array-item table-view-window)))
    (when table-array-item
      (set-view-position table-array-item 0 0)
      (set-view-size table-array-item (view-size table-view-window)))))

(defmethod initialize-instance :after ((table-view-window table-view-window)
                                           &key table)
  "Adds the actual table view to the window."
  (add-subviews table-view-window
                (make-instance 'table-view :view-nick-name 'table-array-item
                               :table table))
  (window-size-parts table-view-window))

(defmethod view-activate-event-handler :after ((window table-view-window))
  (ribbon-display (window-title window)))

;;;
;;; the following functions are for preparing output.
;;;

(defun si (list maxlen)
  "StringIze is a support function for prepare table which arguably
belongs built in."
  (mapcar #'(lambda (item)
                (format nil (if (integerp item)
                              (format nil "~~~DD " (- maxlen 1))
                              (if (numberp item)
                                (format nil "~~~D,4F " (- maxlen 1))
                                (format nil "~~~D@A  " (- maxlen 1))))
                        (if (stringp item) (string-trim " " item) item)))
            list))

(defun prepare-table (table column-names row-names columns rows max-field-len)
  "pads out a table-sequence so it is square & fits r X c.
and makes numbers to strings, if applicable."
  (if (= rows 0)
    nil
    (if column-names
      (append (list (si (if row-names (cons " " column-names) column-names) max-field-len));; shift col names if row names
              (prepare-table table '() row-names columns (1- rows) max-field-len))         ;; recurse
      (append (list 
               (si (append (if (car row-names) (list (car row-names)))         ;; paste row name if any
                           (if (= (length (car table)) (1- columns))           ;; need to pad ?
                             (car table)                                       ;; nope
                             (append (car table)                               ;; paste " "'s
                                     (make-list (- columns (length (car table)) (if row-names 1 0))
                                                :initial-element " ")))) 
                   max-field-len))
              (prepare-table (cdr table) '() (cdr row-names) columns (1- rows) max-field-len)))))

;;;
;;; this gets ugly. we want to rotate the table (while it is still a sequence) because
;;;  of the way LISP understands dimensions is different from the row X column way
;;;

(defun rotate-list-table (list &optional (result '()))
  "makes a list-tables rows into columns & vice versa."
  (if (null (remove nil list))
    (reverse result)
    (rotate-list-table (mapcar #'cdr list)
                       (append (list (mapcar #'car list)) result))))

(defmethod array-from-table ((table statistical-table))
  "Returns an array filled with the contents of the table passed in."
  (with-slots (column-names row-names table-data) table
    (let ((columns (+ (length column-names) (if row-names 1 0) (if column-names 0 1)))
          (rows (+ (length row-names) (if column-names 1 0) (if row-names 0 1))))
      (make-array (list columns rows)
                  :initial-contents (rotate-list-table
                                     (prepare-table table-data
                                                           column-names
                                                           row-names
                                                           columns
                                                           rows
                                                           (table-largest-column table)))))))

;;;
;;;  Holy cats ! this is the unified table code. We have :
;;;  
;;;  Unified-results-table : a class which has multiple tables in its belly.
;;;  Unified-result-window : a window that knows how to view them all.
;;;
;;;  17-Aug-94  ms  Created
;;;
;;;  Due to limitations ingrained in the original table class, this class is something
;;;  of a bastard complete with redundant code. It is my intention to rework this code
;;;  when there's less of a pinch to be friendlier instead of forcing many tables into
;;;  the "old framework" and use multiple views. 


(defclass composite-statistical-table (clasp::named-object-mixin)
  ((tables
    :initform nil
    :initarg :tables
    :type list
    :accessor ctable-tables 
    :documentation "A list of the tables to be included.")
   (table-names
    :initform nil
    :initarg :table-names
    :type list
    :accessor ctable-table-names 
    :documentation "A pairwise matched list of names to the ctable-tables.
These names are intended to be shorthand for the verbose names carried by the actual
tables.")
   (largest-column
    :initform nil
    :type FIXNUM
    :accessor ctable-largest-column
    :documentation "The largest column of the table, in characters."))
  (:documentation "A little like an overlaid graph. The idea is that you can look at
a bunch of tables simultaneously, in one window or contiguously in a log file."))

(defmethod initialize-instance :after ((cst composite-statistical-table) &key (primary-table nil))
  (setq *clasp-tables* (append *clasp-tables* (list cst)))
  (setf (ctable-largest-column cst) (apply #'max 0 (mapcar #'table-largest-column (ctable-tables cst))))
  (if (and primary-table *AUTO-OPEN-PRIMARY-TABLES*)
    (open-clasp-object cst)))

(defun pad-out-table (table rows columns maxlen)
  (if (= rows 0)
    '()
    (append (list (si (if (= (length (car table)) columns)           ;; need to pad ?
                        (car table)                                       ;; nope
                        (append (car table)                               ;; paste " "'s
                                (make-list (- columns (length (car table)))
                                           :initial-element " ")))
                      maxlen
                      ))
            (pad-out-table (cdr table) (1- rows) columns maxlen))))

(defmethod unify-table-data ((cst composite-statistical-table) total-rows total-columns)
  (let (result)
    (loop for ct in (ctable-tables cst)
          for ctn in (ctable-table-names cst) do
          (with-slots (column-names row-names table-data) ct
                      (let* ((columns (+ (length column-names) (if row-names 1 0) (if column-names 0 1)))
                             (rows (+ (length row-names) (if column-names 1 0) (if row-names 0 1))))
                        (setq result 
                              (append (list (list (format nil "~C~A" *ttl-char* ctn)))
                                      (prepare-table table-data column-names row-names
                                                     columns rows (table-largest-column ct))
                                      (list (list ""))
                                      result)))))
    (pad-out-table result total-rows total-columns (ctable-largest-column cst))))
                              

(defmethod f-can-open-clasp-object ((cst composite-statistical-table))
  t)

(defmethod open-clasp-object ((cst composite-statistical-table))
  "cheats. makes the thing into a big regular table and hands off to the normal mechanism."
  (let* ((win (find-if #'(lambda (window)
                           (ignore-errors (equal cst (ctw-table window))))
                       (windows)))
         (rowmax (apply #'+ (* 2 (length (ctable-tables cst)))
                        (mapcar #'(lambda (item) 
                                    (if (table-row-names item)
                                      (+ (length (table-data item))
                                         (if (table-column-names item)
                                           1 0))))
                                (ctable-tables cst))))
         (colmax (apply #'max 1 (mapcar #'(lambda (item)
                                            (+ (length (car (table-data item)))
                                               (if (table-column-names item)
                                                 1 0)))
                                        (ctable-tables cst))))
         (unified-table (unify-table-data cst rowmax colmax))

         (sx (min *screen-width* (+ 16 (* (string-width (make-string (ctable-largest-column cst) :initial-element #\9) *table-header-font*)
                                          colmax))))
         (sy (min *screen-height* (+ 16  (* (table-char-height) rowmax)))))
    (if win
      (window-select win)
      (make-instance 'composite-table-window :table cst
                     :window-title (CLASP::name cst)
                     :data (make-array (list colmax rowmax)
                                       :initial-contents (rotate-list-table
                                                          unified-table))
                     :cell-size-h (ctable-largest-column cst)
                     :view-size
                     (make-point sx sy)
                     :view-position (best-spot sx sy)))))

(defclass composite-table-window (window)
  ((composite-table
    :initform nil
    :initarg :table
    :type composite-statistical-table
    :accessor ctw-table
    :documentation "The table that the window is viewing."))
  (:documentation "A temporary rig to view composite tables.")
  (:default-initargs :view-size #@(400 200)))

(defmethod initialize-instance :after ((ctw composite-table-window) &key data cell-size-h)
  (add-subviews ctw
                (make-instance 'array-dialog-item
                        :view-nick-name 'table-item
                        :cell-size (make-point 
                                    (string-width (make-string cell-size-h :initial-element #\9) *table-header-font*)
                                    (table-char-height))
                        :view-size (view-size ctw)
                        :table-array data
                        :view-font *table-header-font*
                        :table-hscrollp t
                        :table-vscrollp t
                        :table-print-function
                        #'(lambda (print-item stream)
                            (declare (ignore stream))
                            (let ((font-spec (if (ignore-errors (search *hdr-char* print-item))
                                               *table-header-font*
                                               (if (ignore-errors (search *ttl-char* print-item))
                                                 *table-title-font*
                                                 *table-entry-font*))))
                              (multiple-value-bind (ff ms) (font-codes font-spec)
                                (with-font-codes ff ms
                                  (with-pstrs ((p-str (remove (aref *ttl-char* 0)
                                                              (remove (aref *hdr-char* 0) print-item))))
                                    (#_DrawString p-str)))))))))

(defmethod window-size-parts :after ((ctw composite-table-window))
  "Sets the array-dialog-item to fill the window."
  (let ((table-array-item (view-named 'table-item ctw)))
    (when table-array-item
      (set-view-position table-array-item 0 0)
      (set-view-size table-array-item (view-size ctw)))))

(defmethod view-activate-event-handler :after ((window composite-table-window))
  (ribbon-display (window-title window)))

;;;
;;; toggle specifications for windows ::
;;;   control-click and option-click specify toggle state.
;;;

(defmethod view-click-event-handler :after ((view table-view) (pt-where fixnum))
  (if (control-key-p)
    (progn (control-click (table-view-table view)) (revalidate-view view))
    (if (option-key-p)
      (progn (option-click (table-view-table view)) (revalidate-view view))
      (if (double-click-p)
        (progn (dbl-click (table-view-table view)) (revalidate-view view))))))

(defmethod option-click ((window t))
  '())
(defmethod control-click ((window t))
  '())
(defmethod dbl-click ((window t))
  '())

;;; special methods for the contingency table clicks
(defmethod option-click ((table contingency-table))
  (setf (table-data table) (column-percents table)))
(defmethod control-click ((table contingency-table))
  (setf (table-data table) (row-percents table)))
(defmethod dbl-click ((table contingency-table))
  (setf (table-data table) (actual-data table)))

;;; special methods for the scheffe table clicks
(defmethod option-click ((table scheffe-table))
  (setf (table-data table) (significance table)))
(defmethod control-click ((table scheffe-table))
  (setf (table-data table) (scheffe table)))