;;;
;;; notebook type
;;;

#|

Notebook class. Based on the caddy class and adds a few operations
including an interface to the Write function for writing MacWrite files.

16-Aug-94  ms   Created
|#

(in-package "CLASP-INTERFACE")

(defvar *clasp-recorded-items* '())

(defclass clasp-notebook (clasp-caddy)
  ()
  (:DEFAULT-INITARGS
    :caddy-for '*clasp-recorded-items*
    :window-title "CLASP Notebook"))

(defmethod initialize-instance :after ((notebook clasp-notebook) &key)
  (add-subviews notebook
                (make-instance 'button-dialog-item 
                  :dialog-item-text "Log"
                  :border-p nil
                  :dialog-item-action #'(lambda (item) (eval-enqueue `(log-list ',*clasp-recorded-items*)))
                  :view-nick-name 'log-button))
  (setf (table-print-function (view-named 'sequence-view notebook))
        #'(lambda (command-or-result stream)
            "Formats command-or-result to stream and draws a heavy
line if command-or-result is typep 'clasp-command, to visually
separate command-result pairs."
            (declare (ignore stream))
            ;;
            (let ((font-spec   ;; FIX: Use different fonts or draw a line.
                   ;; choose the correct font spec based on item type
                   (cond 
                    ((typep command-or-result 'clasp-command)
                     *font-spec-history-command*)
                    ((typep command-or-result 'simple-string)
                     *font-spec-history-label*)
                    ((typep command-or-result 'comment)
                     *font-spec-history-comment*)
                    ((typep command-or-result 'clasp-result)
                     *font-spec-history-result*)
                    (t             *font-spec-history-result*)))
                  ;; Print control var (~A, ~S, or *float-format-string) based on type
                  (string (format nil (if (or (stringp command-or-result) (typep command-or-result 'clasp-result)
                                              (typep command-or-result 'comment))  ;; ascii output
                                        "~A"
                                        (if (or (typep command-or-result 'ratio)  ;; do float formatting ?
                                                (typep command-or-result 'float))
                                          *float-format-string*
                                          "~S"))
                                  ;; this hairy sequence of if's returns the actual print item depending on type.
                                  (if (typep command-or-result 'clasp-command)    ;; pretty-print commands
                                    (let ((describe (sexp-describe-clasp-command command-or-result))
                                          (command (get-descript-tag command-or-result 'COMMAND)))
                                      (if command  ;; prepare the command for printing
                                        (mapcar #'(lambda (item) (if (equal (ignore-errors (car item))
                                                                            'COMMAND)
                                                                   (cadr item)
                                                                   item))
                                                describe)
                                        (sexp-clasp-command command-or-result))) ;; for untagged commands
                                    (if (typep command-or-result 'ratio)         ;; ratios are ugly
                                      (coerce command-or-result 'float)
                                      (if (typep command-or-result 'comment)     ;; comments must be extracted
                                        (comment-contents command-or-result)
                                        (if (typep command-or-result 'clasp-result)  ;; print the text of the result
                                          (clasp-result-text command-or-result)
                                          command-or-result)))))))                   ;; what is this ?? (floats)
              (multiple-value-bind (ff ms) (font-codes font-spec)
                (with-font-codes ff ms
                  (with-pstrs ((p-str string))
                    (#_DrawString p-str)))))))
  (window-size-parts notebook))

(defmethod window-size-parts :after ((notebook clasp-notebook))
  (let* ((it (view-named 'log-button notebook))
         (grow-icon-dim nil)
         (int-grow-icon-top
          (multiple-value-bind (pt-top-left pt-bottom-right)
                               (ccl::grow-icon-corners notebook)
            (setf grow-icon-dim
                  (- (point-v pt-bottom-right) (point-v pt-top-left)))
            (point-v pt-top-left))))
    (if it
      (set-view-position it (round (/ (- (point-h (view-size notebook)) grow-icon-dim (point-h (view-size it))) 2))
                         (1+ int-grow-icon-top)))))

(defun open-clasp-notebook ()
  (make-instance 'clasp-notebook))
