;;;-*- Mode: Lisp; Package: CLASP-INTERFACE -*-

;; This code was all scarfed from oodles-of-utils.

(in-package :CLASP-INTERFACE)

(eval-when (compile load eval)

(defmacro hset (pointer accessor thing)
  `(ccl::rset ,pointer ,accessor ,thing :storage :handle))
  
(defmacro pset (pointer accessor thing)
  `(ccl::rset ,pointer ,accessor ,thing :storage :pointer))

;; for Not in ROM traps requiring a fn defn
(defmacro deftrap-NotInROM (symbol result-type (&rest typed-arglist) &body body)
  (declare (ignore result-type))
  (let* ((trap-sym       (ensure-trap-sym symbol))
         (fn-sym         (gen-NIR-fn-sym trap-sym))
         (macro-sym      (gen-NIR-macro-sym trap-sym))
         (arglist        (mapcar #'first typed-arglist))
         (type-spec-list (mapcar #'second typed-arglist)))
    
    `(eval-when (:compile-toplevel :load-toplevel :execute)
       (setf (get ',trap-sym :NotInROM-fn-sym)    ',fn-sym)
       (setf (get ',trap-sym :NotInROM-macro-sym) ',macro-sym)
       
       (defmacro ,macro-sym ,arglist
         (NIR-arglist-check ',fn-sym ',arglist ',type-spec-list)
         `(,',fn-sym ,,@arglist))
       
       (defun ,fn-sym ,arglist ,@body))))


;; for Not in ROM traps simply requiring a renaming
(defmacro deftrap-alt-name (alt-trap-symbol asm-trap-symbol)
  (let ((alt-trap-sym (ensure-trap-sym alt-trap-symbol))
        (asm-trap-sym (ensure-trap-sym asm-trap-symbol)))
    `(eval-when (:compile-toplevel :load-toplevel :execute)
       (setf (get ',alt-trap-sym :NotInROM-macro-sym) ',asm-trap-sym))))


;;analogous to require-trap
(defmacro require-trap-NotInROM (trap-symbol &rest arglist)
  (if (trap-prefixp trap-symbol)
    `(require-trap ,trap-symbol ,@arglist)
    `(,trap-symbol ,@arglist)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;takes a potential trap symbol, makes sure it in the right package and begins with an _
(defun ensure-trap-sym (symbol)
  (if (trap-prefixp symbol)
    (if (eq (symbol-package symbol) ccl::*traps-package*)
      symbol
      (intern (symbol-name symbol) :traps))
    (error "trap symbol ~s does not start with an _" symbol)))

;;generates a symbol for the Not in ROM fn
(defun gen-NIR-fn-sym (symbol)
  (intern (concatenate 'string "NIR" (symbol-name symbol)) :traps))

;;generates a symbol for the macro to do compile time arg checking for the Not in ROM fn
(defun gen-NIR-macro-sym (symbol)
  (intern (concatenate 'string "TCHECK-NIR" (symbol-name symbol)) :traps))

;;checks if a symbol starts with an _
(defun trap-prefixp (symbol)
  (char= #\_ (char (symbol-name symbol) 0)))

;;compile time type checking
(defun NIR-arglist-check (fn-symbol arglist type-spec-list)
  (flet ((arg-check (arg type-spec)
           (when (constantp arg)
             (unless (funcall (ccl::mactype-ct-type-check (ccl::find-arg-mactype type-spec))
                              (typecase arg (symbol (symbol-value arg)) (t arg)))
               (error "~s value ~s is not of expected type: ~s" fn-symbol arg type-spec)))))
    (declare (dynamic-extent #'arg-check))
    (mapc #'arg-check arglist type-spec-list)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;stream for reading symbols adding an underscore prefix

(defclass 1char-prefix-stream (input-stream)
  ((prefix-char   :accessor prefix-char
                  :initform nil)
   (suffix-stream :accessor suffix-stream)))

(defmethod stream-tyi ((s 1char-prefix-stream))
  (if (and (prefix-char s) (not (stream-eofp (suffix-stream s))))
    (prog1 (prefix-char s) (setf (prefix-char s) nil))
    (stream-tyi (suffix-stream s))))

(defmethod stream-untyi ((s 1char-prefix-stream) char)
  (setf (prefix-char s) char))

(defvar *prefix-stream* (make-instance '1char-prefix-stream))

;;reads in the next symbol from the stream into the traps package, prepending an _
(defun read-trap-sym (stream)
  (let ((prefix-stream (or *prefix-stream* (make-instance '1char-prefix-stream)))
        (*prefix-stream* nil))
    (setf (suffix-stream prefix-stream) stream)
    (unread-char #\_ prefix-stream)
    (let ((*package* ccl::*traps-package*))
      (read prefix-stream t nil t))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;#~ dispatching macro char

;;This is my chance to map a symbol to something else, like a Not In ROM macro or fn.
;; It would be nice to auto-load any necessary defns at this point too.
;; But that's not currently implemented.
(defun NotInROM-lookup (symbol)
  (or (get symbol :NotInROM-macro-sym) (get symbol :NotInROM-fn-sym) symbol))


(defun NotInROM-dispatch (s macro-char int)
  (declare (ignore macro-char))
  
  (when (and int (not *read-suppress*))
    (error "Reader dispatch macro character #\~ doesn't take an argument."))

  (let* ((symbol (NotInROM-lookup (read-trap-sym s))))
    (unless *read-suppress*
      (when (eq symbol 'traps::_)
        (error "illegal form following #~~ : ~s" (read s t nil t)))

      ;;if it turns out to be a regular trap symbol we must load it
      (when (trap-prefixp symbol) (ccl::load-trap symbol)))
    
    symbol))

(set-dispatch-macro-character #\# #\~ #'NotInROM-dispatch)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(deftrap-NotInROM _FSOpen :OSErr ((fileName (:string 255)) (vRefNum :signed-integer) (refNum (:pointer :signed-integer)))
  (rlet ((pb :ParamBlockRec 
             :ioNamePtr fileName
             :ioVRefNum vRefNum
             :ioVersNum 0
             :ioPermssn #$fsCurPerm
             :ioMisc (%null-ptr)))
    (prog1
      (#_PBOpen pb)
      (%put-word refNum (pref pb :ParamBlockRec.ioRefNum)))))

(deftrap-NotInROM _Create :OSErr ((fileName (:string 255)) (vRefNum :signed-integer) (creator :ostype) (fileType :ostype))
  (rlet ((fi :FInfo
             :fdType      fileType 
             :fdCreator   creator
             :fdFlags     0
             :fdLocation  #@(0 0)
             :fdFldr      0)
         (pb :ParamBlockRec
             :ioNamePtr   fileName
             :ioVRefNum   vRefNum
             :ioFVersNum  0
             :ioFDirIndex 0))
    (when (zerop (#_PBCreate pb))
      (when (zerop (#_PBGetFInfo pb))
        (setf (pref pb :ParamBlockRec.ioFlFndrInfo) fi)
        (#_PBSetFInfo pb)))
    (pref pb :ParamBlockRec.ioResult)))

(deftrap-NotInROM _FSRead :OSErr ((refNum :signed-integer) (count (:pointer :signed-long)) (buffPtr :pointer))
  (rlet ((pb :ParamBlockRec
             :ioVRefNum   0
             :ioRefNum    refNum
             :ioBuffer    buffPtr
             :ioReqCount  (%get-signed-long count)
             :ioPosMode   #$fsAtMark
             :ioPosOffset 0))
    (prog1
      (#_PBRead pb)
      (%put-long count (pref pb :ParamBlockRec.ioActCount)))))

(deftrap-NotInROM _GetFPos :OSErr ((refNum :signed-integer) (filePos (:pointer :signed-long)))
  (rlet ((pb :ParamBlockRec
             :ioRefNum refNum))
    (prog1
      (#_PBGetFPos pb)
      (%put-long filePos (pref pb :ParamBlockRec.ioPosOffset)))))

(deftrap-NotInROM _SetFPos :OSErr ((refNum :signed-integer) (posMode :signed-integer) (posOff :signed-long))
  (rlet ((pb :ParamBlockRec
             :ioRefNum    refNum
             :ioPosMode   posMode
             :ioPosOffset posOff))
    (#_PBSetFPos pb)))

(deftrap-NotInROM _GetEOF :OSErr ((refNum :signed-integer) (logEOF (:pointer :signed-long)))
  (rlet ((pb :ParamBlockRec
             :ioRefNum refNum))
    (prog1
      (#_PBGetEOF pb)
      (%put-long logEOF (%ptr-to-int (pref pb :ParamBlockRec.ioMisc))))))

(deftrap-NotInROM _SetEOF :OSErr ((refNum :signed-integer) (logEOF :signed-long))
  (rlet ((pb :ParamBlockRec
             :ioRefNum refNum
             :ioMisc (%int-to-ptr logEOF)))
    (#_PBSetEOF pb)))

(deftrap-NotInROM _FSClose :OSErr ((refNum :signed-integer))
  (rlet ((pb :ParamBlockRec
             :ioRefNum refNum))
    (#_PBFlushFile pb)
    (#_PBClose pb)))

)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmacro with-res-file-refNum ((refNum
                                 &key
                                 (close-when-done-p nil)
                                 (write-changes-p nil))
                                &body body)
  (let ((new-refNum (gensym))
        (old-refNum (gensym)))
    `(let ((,new-refNum)
           (,old-refNum (require-trap #_CurResFile)))
       (declare (dynamic-extent ,old-refNum ,new-refNum) 
                (fixnum ,old-refNum ,new-refNum))
       (unwind-protect
         (progn
           ;must protect evaluation of refNum in case it alters
           ;the current res file (like a call to open-res-file)
           (setf ,new-refNum ,refNum) 
           (require-trap #_UseResFile ,new-refNum)
           ,@body)
         (when ,new-refNum
           (when ,write-changes-p (require-trap #_UpdateResFile ,new-refNum))
           (when ,close-when-done-p (require-trap #_CloseResFile ,new-refNum)))
         (require-trap #_UseResFile ,old-refNum)))))

(defmacro with-res-file ((pathname
                          &key
                          (if-does-not-exist :error)
                          (if-no-rsrc-fork :error)
                          (if-not-open :close-when-done)
                          (if-open :leave-open)
                          (write-changes-p nil))
                         &body body)
  (let ((close-flag (gensym)))
    `(let ((,close-flag (ecase (if (opened-res-file-p ,pathname) ,if-open ,if-not-open)
                          (:close-when-done t)
                          (:leave-open nil))))
       (declare (dynamic-extent ,close-flag))
       (with-res-file-refNum ((open-res-file ,pathname
                                             :if-does-not-exist ,if-does-not-exist
                                             :if-no-rsrc-fork ,if-no-rsrc-fork)
                              :close-when-done-p ,close-flag
                              :write-changes-p ,write-changes-p)
         ,@body))))

(defmacro without-res-load (&body body)
  `(unwind-protect
     (progn
       (require-trap #_SetResLoad nil)
       ,@body)
     (require-trap #_SetResLoad t)))

(defmacro with-purgeable-resource ((rsrc-handle &key (changed-p nil)) &body body)
  (let ((hState (gensym))
        (rsrc_h (gensym)))
    `(let ((,rsrc_h ,rsrc-handle)
           (,hState nil))
       (declare (dynamic-extent ,hState ,rsrc_h)
                (fixnum ,hState))
       (unwind-protect
         (progn
           (without-interrupts
            (require-trap #_LoadResource ,rsrc_h)
            (require-trap #_HNoPurge ,rsrc_h)
            (setf ,hState (require-trap #_HGetState ,rsrc_h)))
           ,@body)
         (when ,changed-p (require-trap #_ChangedResource ,rsrc_h))
         (require-trap #_HSetState ,rsrc_h ,hState)))))

(defun my-get-resource (rsrc-type rsrc-id-or-name &key (errorp t))
  (without-interrupts
   (let ((rsrc_h (etypecase rsrc-id-or-name
                   (fixnum (#_GetResource rsrc-type rsrc-id-or-name))
                   (string (with-pstrs ((name_p rsrc-id-or-name))
                             (#_GetNamedResource rsrc-type name_p))))))
     (when (and errorp (%null-ptr-p rsrc_h))
       (error "Failed to load resource ~s of type ~s (ResError = ~a)."
              rsrc-id-or-name rsrc-type (#_ResError)))
     rsrc_h)))

(defun my-release-resource (rsrc-handle)
  (if (resource-handlep rsrc-handle)
    (#_ReleaseResource rsrc-handle)
    (error "~s is not a resource handle" rsrc-handle)))

(defun get-resource-id (rsrc-type rsrc-name &key (errorp t))
  (with-returned-pstrs ((name_p rsrc-name))
    (rlet ((id_p :integer)
           (type_p :OSType))
      (with-macptrs ((rsrc_h (without-res-load (#_GetNamedResource rsrc-type name_p))))
        (without-interrupts
         (#_GetResInfo rsrc_h id_p type_p name_p)
         (when (and errorp (not (zerop (#_ResError))))
           (error "resource ~s of type ~s not found." rsrc-name rsrc-type))))
      (%get-signed-word id_p))))

(defun get-resource-name (rsrc-type rsrc-id &key (errorp t))
  (with-returned-pstrs ((name_p ""))
    (rlet ((id_p :integer)
           (type_p :OSType))
      (with-macptrs ((rsrc_h (without-res-load (#_GetResource rsrc-type rsrc-id))))
        (without-interrupts
         (#_GetResInfo rsrc_h id_p type_p name_p)
         (when (and errorp (not (zerop (#_ResError))))
           (error "resource ~s of type ~s not found." rsrc-id rsrc-type))))
      (%get-string name_p))))

(defun resource-handlep (handle)
  (logbitp 5 (#_HGetState handle)))

(defun resource-purgeablep (rsrc-handle)
  (logtest #$resPurgeable (#_GetResAttrs rsrc-handle)))
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; resource files

(defmacro on-trap-nz-error (trap-form &rest body)
  (let* ((result (gensym))
         (trap-call (if (eq 'require-trap (first trap-form))
                      (rest trap-form)
                      trap-form))
         (args (rest trap-call)))
      `(let ((,result ,trap-call))
         (declare (dynamic-extent ,result)
                  (fixnum ,result))
         (unless (zerop ,result)
           ,@body
           (error (MacOS-nz-error-string ',trap-form ',args ,result) ,@args))
         0)))

(defmacro trap-nz-echeck (trap-call)
  `(on-trap-nz-error ,trap-call))

(defun MacOS-nz-error-string (trap-form args result)
  (format nil
          "mac error code = ~s~%> from trap call: ~s~%>  with arg vals: ~{~%>    ~s = ~~s~}"
          result
          trap-form
          args))

(defun opened-res-file-p (pathname)
;;Determines if the resource fork of the specified resource file is open
  (with-pstrs ((fileName (mac-namestring pathname)))
    (rlet ((pb :HParamBlockRec
               :ioNamePtr fileName
               :ioVRefNum 0
               :ioFDirIndex 0
               :ioDirID 0))
      (trap-nz-echeck (#_PBHGetFInfo pb))
      (logbitp 2 (pref pb :HParamBlockRec.ioFlAttrib)))))

(defun open-res-file (pathname &key (if-does-not-exist :error) (if-no-rsrc-fork :error))
;;Opens the resource fork of the the specified file and returns the refNum
  (with-pstrs ((fn (mac-namestring pathname)))
    (unless (probe-file pathname)
      (when if-does-not-exist
        (ecase if-does-not-exist
          (:create (#_CreateResFile fn) (open-res-file pathname))
          (:error (error "~s does not exist" pathname)))))
    (let ((refNum (#_OpenResFile fn)))
      (if (plusp refNum)
        refNum
        (when if-no-rsrc-fork
          (ecase if-no-rsrc-fork
            (:create (#_CreateResFile fn) (open-res-file pathname))
            (:error (error "~s has no resource fork" pathname))))))))

(defun close-res-file (refNum-or-pathname)
  (let ((refNum (etypecase refNum-or-pathname
                  (string (when (opened-res-file-p refNum-or-pathname)
                            (open-res-file refNum-or-pathname)))
                  (pathname (when (opened-res-file-p refNum-or-pathname)
                              (open-res-file refNum-or-pathname)))
                  (fixnum refNum-or-pathname))))
    (when refnum
      (without-interrupts
       (#_CloseResFile refNum)
       (zerop (#_ResError))))))



;;; -----------------------------------------------------------------------------

(defclass rsrc-svm ()
  ((rsrc-type                :initarg :rsrc-type)
   (rsrc-id                  :initarg :rsrc-id)
   (rsrc-name                :initarg :rsrc-name)
   (rsrc-handle              :initarg :rsrc-handle)
   (detach-p                 :initarg :detach-p)
   (dispose-rsrc-on-remove-p :initarg :dispose-rsrc-on-remove-p))
  (:default-initargs    :dispose-rsrc-on-remove-p t :detach-p nil))

(defmethod install-view-in-window :after ((sv rsrc-svm) window)
  (declare (ignore window))
  (rsrc-handle-install sv))

(defmethod remove-view-from-window :after ((sv rsrc-svm))
  (rsrc-handle-remove sv))

(defmethod rsrc-handle-install ((sv rsrc-svm))
  (with-slots (rsrc-type rsrc-id rsrc-name rsrc-handle detach-p) sv
    (cond ((slot-boundp sv 'rsrc-handle))
          ((slot-boundp sv 'rsrc-id)
           (setf rsrc-handle (rsrc-get-fn sv rsrc-type rsrc-id)))
          ((slot-boundp sv 'rsrc-name)
           (setf rsrc-handle (rsrc-get-fn sv rsrc-type rsrc-name))))
    (when (and detach-p (slot-boundp sv 'rsrc-handle) (resource-handlep rsrc-handle))
      (#_DetachResource rsrc-handle))))

(defmethod rsrc-handle-remove ((sv rsrc-svm))
  (when (slot-boundp sv 'rsrc-handle)
    (with-macptrs ((rsrc-handle (slot-value sv 'rsrc-handle)))
      (slot-makunbound sv 'rsrc-handle)
      (when (slot-value sv 'dispose-rsrc-on-remove-p)
        (rsrc-dispose-fn sv rsrc-handle (resource-handlep rsrc-handle))))))

(defmethod set-view-resource ((sv rsrc-svm) &key rsrc-type rsrc-id rsrc-name rsrc-handle)
  (when rsrc-type (setf (slot-value sv 'rsrc-type) rsrc-type))
  (slot-makunbound sv 'rsrc-id)
  (slot-makunbound sv 'rsrc-name)
  (without-interrupts
   (rsrc-handle-remove sv)
   (cond (rsrc-handle (setf (slot-value sv 'rsrc-handle) rsrc-handle))
         (rsrc-id     (setf (slot-value sv 'rsrc-id)     rsrc-id))
         (rsrc-name   (setf (slot-value sv 'rsrc-name)   rsrc-name)))
   (when (wptr sv) (rsrc-handle-install sv))))


(defmethod rsrc-get-fn ((sv rsrc-svm) rsrc-type rsrc-id-or-name)
  (my-get-resource rsrc-type rsrc-id-or-name))

(defmethod rsrc-dispose-fn ((sv rsrc-svm) rsrc-handle rsrc-handlep)
  (if rsrc-handlep
    (#_ReleaseResource rsrc-handle)
    (#_DisposeHandle rsrc-handle)))

;;; ---

(defclass graphic-rsrc-svm (rsrc-svm)
  ((graphic-scaling      :initarg :graphic-scaling)
   (graphic-default-size :initarg :graphic-default-size)
   (erase-on-set-rsrc-p  :initarg :erase-on-set-rsrc-p))
  (:default-initargs
    :graphic-scaling      :adjust-view-size
    :graphic-default-size #@(32 32)
    :erase-on-set-rsrc-p  t
    ))

(defmethod graphic-size ((sv graphic-rsrc-svm) rsrc-handle)
  (declare (ignore rsrc-handle))
  (slot-value sv 'graphic-default-size))

(defmethod draw-graphic ((sv graphic-rsrc-svm) rsrc-handle rect)
  (declare (ignore sv rsrc-handle rect)))

(defmethod draw-bad-handle ((sv graphic-rsrc-svm))
  (multiple-value-bind (topLeft botRight) (graphic-corners sv)
    (rlet ((r :Rect :topLeft topLeft :botRight botRight))
      (#_FillRect r *light-gray-pattern*)
      (#_FrameRect r))))

(defmethod view-draw-contents ((sv graphic-rsrc-svm))
  (if (slot-boundp sv 'rsrc-handle)
    (with-slots (rsrc-handle) sv
      (with-purgeable-resource (rsrc-handle)
        (multiple-value-bind (topLeft botRight) (graphic-corners sv)
          (if (handlep rsrc-handle)
            (ecase (slot-value sv 'graphic-scaling)
              (:scale-to-view (rlet ((r :Rect :topLeft topLeft :botRight botRight))
                                (draw-graphic sv rsrc-handle r)))
              (:adjust-view-size (rlet ((r :Rect
                                           :topLeft topLeft
                                           :botRight (add-points topLeft (graphic-size sv rsrc-handle))))
                                   (draw-graphic sv rsrc-handle r)))
              (:clip-to-view (rlet ((clip-rect :Rect
                                               :topLeft topLeft
                                               :botRight botRight)
                                    (r :Rect :topLeft topLeft :botRight (add-points topLeft (graphic-size sv rsrc-handle))))
                               (with-clip-rect clip-rect
                                 (draw-graphic sv rsrc-handle r)))))
            (draw-bad-handle sv)))))
    (draw-bad-handle sv))
  (call-next-method))

(defmethod focusing-view ((sv simple-view)) (view-container sv))

(defmethod focused-corners ((sv simple-view)) (view-corners sv))

(defmethod view-window-corners ((sv simple-view))
  (let ((offset (subtract-points #@(0 0) (view-origin sv))))
    (values offset (add-points (view-size sv) offset))))

(defmethod erase-corners ((sv simple-view) topLeft botRight)
  (rlet ((r :Rect :topLeft topLeft :botRight botRight))
    (with-macptrs ((rgn (#_NewRgn)))
      (#_RectRgn rgn r)
      (let ((erase-rgn (ccl::window-erase-region (view-window sv))))
        (#_UnionRgn rgn erase-rgn erase-rgn))
      (#_DisposeRgn rgn))))

(defmethod erase-view ((sv simple-view))
  (multiple-value-call #'erase-corners sv (view-window-corners sv)))

(defmethod graphic-margins ((sv graphic-rsrc-svm))
  (declare (ignore sv))
  (values #@(0 0) #@(0 0)))

(defmethod graphic-corners ((sv graphic-rsrc-svm))
  (multiple-value-bind (topLeft botRight) (focused-corners sv)
    (multiple-value-bind (tl-margin br-margin) (graphic-margins sv)
      (values (add-points topLeft tl-margin) (subtract-points botRight br-margin)))))

(defmethod view-default-size ((sv graphic-rsrc-svm))
  (add-points (multiple-value-call #'add-points (graphic-margins sv))
              (slot-value sv 'graphic-default-size)))

(defmethod scale-view-size ((sv graphic-rsrc-svm))
  (when (slot-boundp sv 'rsrc-handle)
    (with-slots (rsrc-handle) sv
      (let ((margin-size (multiple-value-call #'add-points (graphic-margins sv))))
        (set-view-size sv (add-points (graphic-size sv rsrc-handle) margin-size))))))

(defmethod set-view-size :before ((sv graphic-rsrc-svm) h &optional v)
  (declare (ignore h v))
  (erase-view sv))

(defmethod rsrc-handle-install :after ((sv graphic-rsrc-svm))
  (if (eq :adjust-view-size (slot-value sv 'graphic-scaling))
    (scale-view-size sv)
    (invalidate-view sv t)))

(defmethod set-view-resource :after ((sv graphic-rsrc-svm)  &key rsrc-type rsrc-id rsrc-name rsrc-handle)
  (declare (ignore rsrc-type rsrc-id rsrc-name rsrc-handle))
  (invalidate-view sv (slot-value sv 'erase-on-set-rsrc-p)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro rlength (record-type)
  (if (constantp record-type)
    (record-descriptor-length (find-record-descriptor record-type))
    `(record-descriptor-length (find-record-descriptor ,record-type))))
 
(defmacro with-PICT-file ((pathname refNum PICT-size PICT-record
                                    &key
                                    (direction         :input)
                                    (if-does-not-exist (ecase direction (:input :error) (:output :create)))
                                    (if-exists         :error)
                                    (creator           "????"))
                          &body body)
  (let ((fn (gensym)))
    `(flet ((,fn (,refNum ,PICT-size ,PICT-record) ,@body))
       (declare (dynamic-extent #',fn))
       (call-with-PICT-file ,pathname #',fn
                            :direction         ,direction
                            :if-does-not-exist ,if-does-not-exist
                            :if-exists         ,if-exists
                            :creator           ,creator))))

(defun call-with-PICT-file (pathname fn &key
                                     (direction         :input)
                                     (if-does-not-exist (ecase direction (:input :error) (:output :create)))
                                     (if-exists         :error)
                                     (creator           "????"))
  (let ((set-EOF-p nil))
    (if (probe-file pathname)
      (ecase direction
        (:input )
        (:output (ecase if-exists
                   (:error     (error "~s already exists." pathname))
                   (:overwrite (setf set-EOF-p t)))))
      (ecase direction
        (:input  (error "~s doesn't exist - can't open for input." pathname))
        (:output (ecase if-does-not-exist
                   (:error  (error "~s does not exist." pathname))
                   (:create (with-pstrs ((filename (mac-namestring pathname)))
                              (let ((ecode (#~Create filename 0 creator "PICT")))
                                (unless (zerop ecode)
                                  (error "(~a) attempting to create ~s." ecode pathname))))
                            (setf set-EOF-p t))))))
    
    (unless (eq (mac-file-type pathname) :PICT)
      (error "~s is not a PICT file." pathname))
    
    (let ((refNum (with-pstrs ((fileName (mac-namestring pathname)))
                    (rlet ((refNum_p :integer))
                      (let ((ecode (#~FSOpen fileName 0 refNum_p)))
                        (unless (zerop ecode) (error "(~a) opening file: ~s." ecode pathname))
                        (%get-word refNum_p))))))
      (unwind-protect
        (rlet ((PICT-record :Picture))
          
          (when set-EOF-p
            (let ((ecode (#~SetEOF refNum 512)))
              (unless (zerop ecode) (error "(~a) calling SetEOF on ~s." ecode pathname))))
          
          (let ((file-size (rlet ((logicalEOF_p :long))
                             (let ((ecode (#~GetEOF refNum logicalEOF_p)))
                               (unless (zerop ecode) (error "(~a) calling GetEOF on ~s." ecode pathname))
                               (%get-long logicalEOF_p)))))
            
            (ecase direction
              (:input
               ;read the PICT header & leave file position just beyond (+ 512 (rlength :Picture))
               (let ((ecode (#~SetFPos refNum #$fsFromStart 512)))
                 (unless (zerop ecode) (error "(~a) calling SetFPos (preparing to read picture record) on ~s." ecode pathname)))
               (rlet ((byteCount_p :long (rlength :Picture)))
                 (let ((ecode (#~FSRead refnum byteCount_p PICT-record)))
                   (unless (and (zerop ecode) (= (rlength :Picture) (%get-long byteCount_p)))
                     (error "(~a) calling PBRead. Read ~a bytes of PICT header data from ~s."
                            ecode (%get-long byteCount_p) pathname)))))
              (:output
               ;set the file position to 512 & invalidate PICT-record passed to fn
               (%setf-macptr PICT-record (%null-ptr))
               (let ((ecode (#~SetFPos refNum #$fsFromStart 512)))
                 (unless (zerop ecode) (error "(~a) calling SetFPos (preparing to write picture record) on ~s." ecode pathname)))))
            
            
            (funcall fn refNum (- file-size 512) PICT-record)))
        (#~FSClose refNum)))))

(defun get-picture-from-file (pathname)
  ;;Returns a PICT handle from the specified PICT file.
  (with-PICT-file (pathname refnum PICT-size PICT-record)
    (let ((PICT-handle (#_NewHandle PICT-size)))
      (if (%null-ptr-p PICT-handle)
        (error "unable to allocate a ~a byte PICT handle for ~s." PICT-size pathname)
        (with-dereferenced-handles ((PICT_p PICT-handle))
          (pset PICT_p :Picture PICT-record)
          (%incf-ptr PICT_p (rlength :Picture))
          (rlet ((byteCount_p :long (- PICT-size (rlength :Picture))))
            (let ((ecode (#~FSRead refnum byteCount_p PICT_p)))
              (unless (and (zerop ecode) (= (- PICT-size (rlength :Picture)) (%get-long byteCount_p)))
                (error "(~a) calling FSRead. Read ~a/~a bytes of PICT data from ~s."
                       ecode (%get-long byteCount_p) PICT-size pathname))))))
      PICT-handle)))

(defun get-PICT-file-info (pathname picture-record-ptr)
;;Returns the size (in bytes) of the PICT in the specified PICT file.
;;picture-record-ptr's picFrame is filled in.
  (with-PICT-file (pathname refnum PICT-size PICT-record)
    (declare (ignore refnum))
    (pset picture-record-ptr :Picture PICT-record)
    PICT-size))

;QuickDraw bottle neck routine for getPicProc that reads PICT
;;data from an open file. see IM V pp. 87-89
(defpascal dpfd-StdGetPic (:ptr dataPtr :word byteCount)
  (declare (special *dpfd-refNum*))
  (rlet ((byteCount_p :long byteCount))
    (#~FSRead *dpfd-refNum* byteCount_p dataPtr)))

(defmacro with-QDProcs ((&key textProc lineProc rectProc rRectProc ovalProc arcProc polyProc
                              rgnProc bitsProc commentProc txMeasProc getPicProc putPicProc
                              opCodeProc newProc1 newProc2 newProc3 newProc4 newProc5 newProc6
                              )
                        &body body)
  ;;GrafPorts & CGrafPorts are handled identically! This works because:
  ;; - the first 13 fields of a CQDProc are a QDProc
  ;; - both GrafPorts & CGrafPorts use the same stdProcs for these 13 fields.     
  (let ((thePort (gensym))
        (old-procs (gensym))
        (new-procs (gensym)))
    `(with-macptrs ((,thePort (ccl::%getport))
                    (,old-procs (pref ,thePort :GrafPort.grafProcs)))
       (rlet ((,new-procs :CQDProcs))
         (if (%null-ptr-p ,old-procs)
           (require-trap #_SetStdCProcs ,new-procs)
           (require-trap #_BlockMove ,old-procs ,new-procs (rlength :CQDProcs)))
         ,@(when textProc    `((pset ,new-procs :CQDProcs.textProc    ,textProc   )))
         ,@(when lineProc    `((pset ,new-procs :CQDProcs.lineProc    ,lineProc   )))
         ,@(when rectProc    `((pset ,new-procs :CQDProcs.rectProc    ,rectProc   )))
         ,@(when rRectProc   `((pset ,new-procs :CQDProcs.rRectProc   ,rRectProc  )))
         ,@(when ovalProc    `((pset ,new-procs :CQDProcs.textProc    ,ovalProc   )))             
         ,@(when arcProc     `((pset ,new-procs :CQDProcs.arcProc     ,arcProc    )))
         ,@(when polyProc    `((pset ,new-procs :CQDProcs.polyProc    ,polyProc   )))
         ,@(when rgnProc     `((pset ,new-procs :CQDProcs.rgnProc     ,rgnProc    )))
         ,@(when bitsProc    `((pset ,new-procs :CQDProcs.bitsProc    ,bitsProc   )))
         ,@(when commentProc `((pset ,new-procs :CQDProcs.commentProc ,commentProc)))
         ,@(when txMeasProc  `((pset ,new-procs :CQDProcs.txMeasProc  ,txMeasProc )))
         ,@(when getPicProc  `((pset ,new-procs :CQDProcs.getPicProc  ,getPicProc )))
         ,@(when putPicProc  `((pset ,new-procs :CQDProcs.putPicProc  ,putPicProc )))
         ,@(when opCodeProc  `((pset ,new-procs :CQDProcs.opCodeProc  ,opCodeProc )))
         ,@(when newProc1    `((pset ,new-procs :CQDProcs.newProc1    ,newProc1   )))
         ,@(when newProc2    `((pset ,new-procs :CQDProcs.newProc2    ,newProc2   )))
         ,@(when newProc3    `((pset ,new-procs :CQDProcs.newProc3    ,newProc3   )))
         ,@(when newProc4    `((pset ,new-procs :CQDProcs.newProc4    ,newProc4   )))
         ,@(when newProc5    `((pset ,new-procs :CQDProcs.newProc5    ,newProc5   )))
         ,@(when newProc6    `((pset ,new-procs :CQDProcs.newProc6    ,newProc6   )))
         (unwind-protect
           (progn
             (pset ,thePort :CGrafPort.grafProcs ,new-procs)
             ,@body)
           (pset ,thePort :CGrafPort.grafProcs ,old-procs))))))

(defun draw-picture-from-file (pathname rect &optional (scale-to-rect-p t))
  ;;Draws the specfied PICT file either scaledd to rect or at the topLeft
  ;;corner of rect. The PICT size (in bytes) is returned.
  (let ((PICT_h (#_NewHandle (rlength :Picture))))
    (when (%null-ptr-p PICT_h)
      (error "unable to allocate a ~a temporary picture record handle (~a bytes) for ~s." (rlength :Picture) pathname))
    (unwind-protect
      (with-PICT-file (pathname refnum PICT-size PICT-record)
        (with-dereferenced-handles ((PICT_p PICT_h))
          (pset PICT_p :Picture PICT-record))
        (rlet ((r :Rect
                  :topLeft (pref rect :Rect.topLeft)
                  :botRight (if scale-to-rect-p
                              (pref rect :Rect.botRight)
                              (add-points (pref rect :Rect.topLeft)
                                          (subtract-points (href PICT_h :Picture.picFrame.botRight)
                                                           (href PICT_h :Picture.picFrame.topLeft))))))
          (let ((*dpfd-refNum* refNum))
            (declare (special *dpfd-refNum*))
            (without-interrupts
             (with-QDProcs (:getPicProc dpfd-StdGetPic)
               (#_DrawPicture PICT_h r)))))
        PICT-size)
      (#_DisposeHandle PICT_h))))

#|

(defparameter *test-w* (make-instance 'window :color-p t))

(with-focused-view *test-w*
  (rlet ((r :Rect :topLeft 0 :botRight (view-size *test-w*)))
    (draw-picture-from-file "oou:examples;MrT.PICT" r nil)))

(invalidate-view *test-w* t)

(defparameter *p-handle* (get-picture-from-file "oou:examples;MrT.PICT"))

(rlet ((r :Rect :topLeft #@(100 100) :botRight #@(300 300)))
  (with-focused-view *test-w* (#_DrawPicture *p-handle* r)))

(invalidate-view *test-w* t)

(write-picture-to-file *p-handle* "HD:temp")

(with-focused-view *test-w*
  (rlet ((r :Rect :topLeft 0 :botRight (view-size *test-w*)))
    (draw-picture-from-file "HD:temp" r nil)))

(#_DisposeHandle *p-handle*)

(rlet ((p :Picture))
  (print (get-PICT-file-info "oou:examples;MrT.PICT" p))
  (terpri)
  (print-record (pref p :Picture.picFrame) :Rect)
  (print (pref p :Picture.picSize)))

|#

;;; ---------------------------------------------------------------------------------------------


(defclass PICT-svm (graphic-rsrc-svm)
  ((rsrc-id         :initarg :PICT-id)
   (rsrc-name       :initarg :PICT-name)
   (rsrc-handle     :initarg :PICT-handle)
   (PICT-file       :initarg :PICT-file)
   (PICT-storage    :initarg :PICT-storage)
   (graphic-scaling :initarg :PICT-scaling))
  (:default-initargs
    :rsrc-type "PICT"
    :PICT-storage :memory
    :graphic-default-size #@(100 100)))

(defmethod rsrc-handle-install ((sv PICT-svm))
  (call-next-method)
  (when (and (not (slot-boundp sv 'rsrc-handle)) (slot-boundp sv 'PICT-file))
    (setf (slot-value sv 'rsrc-handle)
          (ecase (slot-value sv 'PICT-storage)
            (:memory (get-picture-from-file (slot-value sv 'PICT-file)))
            (:disk   (let ((PICT_h (#_NewHandle (rlength :Picture))))
                       (when (%null-ptr-p PICT_h)
                         (error "unable to allocate a ~a picture record handle (~a bytes) for ~s"
                                (rlength :Picture)
                                (slot-value sv 'PICT-file)))
                       (with-dereferenced-handles ((PICT_p PICT_h))
                         (get-PICT-file-info (slot-value sv 'PICT-file) PICT_p))
                       PICT_h))))))

(defmethod rsrc-get-fn ((sv PICT-svm) rsrc-type rsrc-id-or-name)
  (declare (ignore rsrc-type))
  (etypecase rsrc-id-or-name
    (fixnum (#_GetPicture rsrc-id-or-name))
    (string (#_GetPicture (get-resource-id "PICT" rsrc-id-or-name :errorp nil)))))

(defmethod graphic-size ((sv PICT-svm) PICT-handle)
  (declare (ignore sv))
  (subtract-points
   (href PICT-handle :Picture.picFrame.botRight)
   (href PICT-handle :Picture.picFrame.topLeft )))

(defmethod draw-graphic ((sv PICT-svm) PICT-handle rect)
  (ecase (slot-value sv 'PICT-storage)
    (:memory (#_DrawPicture PICT-handle rect))
    (:disk   (draw-picture-from-file (slot-value sv 'PICT-file) rect))))

(defmethod view-default-size ((sv PICT-svm))
  (add-points (multiple-value-call #'add-points (graphic-margins sv)) #@(100 100)))

(defmethod set-view-PICT ((sv PICT-svm) &key PICT-id PICT-name PICT-handle PICT-file PICT-storage)
  (when PICT-file (setf (slot-value sv 'PICT-file) PICT-file))
  (when PICT-storage (setf (slot-value sv 'PICT-storage) PICT-storage))
  (set-view-resource sv :rsrc-id PICT-id :rsrc-name PICT-name :rsrc-handle PICT-handle))

;;; ---


(defclass PICT-di (PICT-svm dialog-item simple-view) ())

;(defclass PICT-button-di (disable-dim button-dim PICT-di) ())

(pushnew :PICT-DIALOG-ITEM *features*)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#|

(open-res-file "EKSL:Applications:LISP:MCL 3.0:clasp-resources")
;(open-res-file "ccl:oodles-of-utils;examples;examples.rsrc")


;keep the PICT in RAM
(setf *test-w*
      (make-instance 'dialog
        :window-type :document
        :color-p t
        :view-position :centered
        :view-size #@(300 300)
        :window-title "PICT demo"
        :view-subviews (list (make-instance 'PICT-di ;'PICT-button-di
                               :view-position #@(10 10)
                               :PICT-name "Clasp Hand"
                               ;:dialog-item-action #'(lambda (item) (declare (ignore item)) (ed-beep))
                               :view-nick-name :I-pity-the-fool
                               ))))

;spool the PICT off disk (from a PICT file)
(setf *test-w*
      (make-instance 'dialog
        :window-type :document
        :color-p t
        :view-position :centered
        :view-size #@(300 300)
        :window-title "PICT demo"
        :view-subviews (list (make-instance 'PICT-di ;'PICT-button-di
                               :view-position #@(10 10)
                               :PICT-file "oou:examples;MrT.PICT"
                               :PICT-storage :disk
                               ;:dialog-item-action #'(lambda (item) (declare (ignore item)) (ed-beep))
                               :view-nick-name :I-pity-the-fool
                               ))))

(close-res-file "EKSL:Applications:LISP:MCL 3.0:clasp-resources")

|#

;;; --- 
;; EOF
