;;;
;;;  ribbon-window.lisp (extracted from interface-support-fns on 2-8-95 by Westy)
;;;

#| ribbon window |#
;; this is a "ribbon" window that is used to display random things that are sometimes too long
;; for regular views. currently, it is used for displaying long variable names, etc.

(in-package :clasp-interface)

;; Use this to figure out where to put the ribbon since it is missing a title-bar.
(defvar *title-bar-size* 18)

(defclass ribbon-window (windoid)
  ((my-clip-region :initform (ccl:new-region))
   (what-to-display :initform nil :accessor what-i-display)
   (warehouse :initform nil :accessor what-i-was :initarg :what-i-will-be))
  (:default-initargs
    :window-title "gibbon-monkey"
    :view-position (make-point 0 (- *screen-height* 20))
    :view-font '("Helvetica" 10)
    :view-size (make-point *screen-width* 17)
    :window-type :shadow-edge-box
    :what-i-will-be (make-point 0 (- *MENUBAR-BOTTOM* *title-bar-size*))))

(defun ribbon-clear ()
  (let ((window (find-window "gibbon-monkey" 'ribbon-window)))
    (ccl:erase-region window (ccl:clip-region window (slot-value window 'my-clip-region)))))

(defun ribbon-display (message)
  (let ((window (find-window "gibbon-monkey" 'ribbon-window)))
    (if window 
      (progn
        (setf (what-i-display window) message)
        (ribbon-clear)
        (move-to window 5 12)
        (format window "~A"
                (remove-stupid-garbage message))))))

(defun close-ribbon ()
  (let ((window (find-window "gibbon-monkey" 'ribbon-window)))
    (if window
      (window-close window))))

(defun open-ribbon ()
  (if (and *use-ribbon-window* (not (find-window "gibbon-monkey" 'ribbon-window)))
    (make-instance 'ribbon-window)))

(defmethod view-click-event-handler :after ((window ribbon-window) (where fixnum))
  (let ((next (what-i-was window)))
    (setf (what-i-was window) (view-position window))
    (set-view-position window next)))

;;; look here to see how I molest windows that think they're bad in CLASP

(defmethod set-view-position :after ((window window) h &optional v)
  (let ((ribbon (find-window "gibbon-monkey" 'ribbon-window)))
    (when (and (not (eql ribbon window)) 
               ribbon
	       ;; if the ribbon window is at the top
               (< (point-v (view-position ribbon)) 40))
      (let ((vert (or v (point-v h)))
            (horiz (point-h h))
            (min-level (+  40 (point-v (view-size (find-window "gibbon-monkey" 'ribbon-window))))))
        (when (< vert min-level)
          (set-view-position window horiz min-level))))))


(defmethod initialize-instance :after ((window window) &key)
  (if (and (not (typep window 'ribbon-window)) (find-window "gibbon-monkey" 'ribbon-window))
    (let* ((pos (view-position window))
           (vert (point-v pos))
           (horiz (point-h pos))
           (min-level (+  40 (point-v (view-size (find-window "gibbon-monkey" 'ribbon-window))))))
      (when (and (< vert min-level)
               (< (point-v (view-position (find-window "gibbon-monkey" 'ribbon-window))) 40))
        (set-view-position window (make-point horiz min-level)))))
  (when (not (point<= (view-size window) (make-point *screen-width* *screen-height*)))
    (set-view-size window (make-point (- *screen-width* 30) (- *screen-height* 80)))))

(defmethod view-draw-contents ((window ribbon-window))
  (with-slots (what-to-display) window
    (if what-to-display
      (progn
        (ribbon-clear)
        (move-to window 5 12)
        (format window "~A" (remove #\newline what-to-display :test #'char=))))))

;;; just an example of how we might use the ribbon window
(defmethod view-click-event-handler :after ((sdi array-dialog-item) (pt-where fixnum))
  (ignore-errors ;; oh how I love ignore-errors
   (let* ((cell (first (selected-cells sdi)))
         (x (point-h cell))
         (y (point-v cell))
         (item (aref (table-array sdi) x y)))
    (clasp-interface::ribbon-display (format nil "~A" (if (stringp item)
                                                        (string-trim " _|" item)
                                                        item))))))

;;; ---
;;; EOF