#|

patch to add standardize variable to the MacCLASP release 1.3.3

09-03-94  ms Created.

FIX:: place in clasp/shortcuts.lisp

|#

(in-package "CLASP")

;;;
;;; standardize variable
;;;

;;; THESE 2 SHOULD GO IN SHORTCUTS.LISP

(defun standardize-variable (variable)
  (let* ((data (variable-value variable))
         (m (mean data))
         (sd (standard-deviation data)))
    (add-variable-to-dataset
     (variable-dataset variable)
     (mapcar #'(lambda (number) (/ (- number m) sd))
             data)
     (format nil "Standardized ~a" (name variable)))))

(defun mul-by-constant (variable constant)
  (add-variable-to-dataset
   (variable-dataset variable)
   (mapcar #'(lambda (datum) (* datum constant))
           (variable-value variable))
   (format nil "* ~F ~a" constant (name variable))))

;;;
;;; correlation table
;;;

;;; MAYBE THIS SHOULD GO IN STATISTICS.LISP ??

(defun make-correlation-table (&rest number-lists)
  (let (r-list ir-list)
    (dolist (ox number-lists)
      (setf ir-list '())
      (dolist (ix number-lists)
        (push (r-score ox ix) ir-list))
      (push ir-list r-list))
    (values r-list)))

;;;
;;; mean smoothing functions
;;;

;;; THESE SHOULD DEFINATELY GO IN STATISTICS.LISP

(defun smooth-mean-2 (data)
  "With a window of size two, the median and mean smooth functions are the
same."
  (smooth-median-2 data))

(defun smooth-mean-3 (data)
  "Smooths `data' by replacing each element with the mean of it and its two
neighbors.  The ends are handled by duplicating the end elements.  This function
is not destructive; it returns a list the same length as `data,' which should be
a sequence of numbers."
  (flet ((mean-3 (a b c)
           (/ (+ a b c) 3)))
    (let ((n (data-length data)))
      (with-temp-vector (temp (+ n 2))
        (setf (aref temp 0) (elt data 0))
        (replace temp data :start1 1)
        (setf (aref temp (1+ n)) (elt data (1- n)))
        (map 'list
             #'mean-3
             (make-array n :displaced-to temp :displaced-index-offset 0)
             (make-array n :displaced-to temp :displaced-index-offset 1)
             (make-array n :displaced-to temp :displaced-index-offset 2))))))

(defun smooth-mean-4 (data)
  "Smooths `data' by replacing each element with the mean of it, its left
neighbor, and its two right neighbors.  The ends are handled by duplicating the
end elements.  This function is not destructive; it returns a list the same
length as `data,' which should be a sequence of numbers."
  (flet ((mean-4 (a b c d)
	   (/ (+ a b c d) 4)))
    (let ((n (data-length data)))
      (with-temp-vector (temp (+ n 3))
	(setf (aref temp 0) (elt data 0))
	(replace temp data :start1 1)
	(let ((last (elt data (- n 1))))
	  (setf (aref temp (+ n 1)) last)
	  (setf (aref temp (+ n 2)) last))
	(map 'list
	     #'mean-4
	     (make-array n :displaced-to temp :displaced-index-offset 0)
	     (make-array n :displaced-to temp :displaced-index-offset 1)
	     (make-array n :displaced-to temp :displaced-index-offset 2)
	     (make-array n :displaced-to temp :displaced-index-offset 3))))))

(defun smooth-mean-5 (data)
  "Smooths `data' by replacing each element with the median of it, its two left
neighbors and its two right neighbors.  The ends are handled by duplicating the
end elements.  This function is not destructive; it returns a list the same
length as `data,' which should be a sequence of numbers."
  ;; median-5 can definitely be improved.
  (flet ((mean-5 (a b c d e)
           (/ (+ a b c d e) 5)))
    (let ((n (data-length data)))
      (with-temp-vector (temp (+ n 4))
	(let ((first (elt data 0)))
	  (setf (aref temp 0) first)
	  (setf (aref temp 1) first))
	(replace temp data :start1 2)
	(let ((last (elt data (- n 1))))
	  (setf (aref temp (+ n 2)) last)
	  (setf (aref temp (+ n 3)) last))
	(map 'list
	     #'mean-5
	     (make-array n :displaced-to temp :displaced-index-offset 0)
	     (make-array n :displaced-to temp :displaced-index-offset 1)
	     (make-array n :displaced-to temp :displaced-index-offset 2)
	     (make-array n :displaced-to temp :displaced-index-offset 3)
	     (make-array n :displaced-to temp :displaced-index-offset 4))))))

;;
;; Partial correlations, compliments of Lisa Ballesteros
;;

;;; Calculates the partial correlation, given a from and a to variable and a list of 
;;; other variable(s) to hold constant.
;; STATISTICS.LISP ??

(defun partials-from-parents (from to parents-list)
  (labels ((radical (value)
             (sqrt (- 1 (clasp::square value))))
           (parents-partial (from to parents-list)
             (cond ((null parents-list)
                    (correlation from to))
                   ((null (cdr parents-list))
                    (let ((parent (car parents-list)))
                      (/ (- (correlation from to) (* (correlation from parent) (correlation to parent)))
                         (* (radical (correlation from parent)) (radical (correlation to parent))))))
                   (t (/ (- (parents-partial from to (cdr parents-list))
                            (* (parents-partial from (car parents-list) (cdr parents-list))	
                               (parents-partial to (car parents-list) (cdr parents-list))))
                         (* (radical (parents-partial from (car parents-list) (cdr parents-list)))
                            (radical (parents-partial to (car parents-list) (cdr parents-list)))))))))
    (parents-partial from to (reverse parents-list))))

;;
;; general-bootstrapping
;;
;; compute-bootstrap is a more generalized boot function which can handle multivariate 
;; bootstraps. generalization to multivariate expressions comes at a price, however.
;; the keyword arguments are pretty much required -- sampling-function arguments at least.
;; the way these arguments work is that they are mapped lists of functions and the arguments.
;; so, you might say :sampling-functions '(sample-with-replacement sample-with-replacement)
;;                   :sampling-function-arguments '((( <data1> )) (( <data2>)))
;; and the function would map each sample-with.. to the corresponding arg list (list of single data list)
;;


(defun compute-bootstrap (boot-function &rest boot-function-arguments &key 
		                           (sampling-functions #'clasp::sample-with-replacement-from-sequence)
		                           sampling-function-arguments
		                           (number-of-iterations 500))
  (remf boot-function-arguments :sampling-functions)
  (remf boot-function-arguments :sampling-function-arguments)
  (remf boot-function-arguments :number-of-iterations)
  (let (temp)
    (loop for i from 1 to number-of-iterations
          collect (apply boot-function
                         (car (setq temp (mapcar #'(lambda (function arguments)
                                                     (apply function (car arguments) (cdr arguments)))
                                                 sampling-functions
                                                 sampling-function-arguments)))
                         (append (cdr temp)
                                 boot-function-arguments)))))

#+IGNORE
(defun generate-normal-distribution (mean sd samples)
  (let ((dist (make-instance 'normal-distribution :mean mean :standard-deviation sd)))
    (sample-to-list dist samples)))
#+IGNORE
(defun generate-uniform-distribution (min max samples)
  (let ((dist (make-instance 'uniform-distribution :minimum min :maximum max)))
    (sample-to-list dist samples)))

#+IGNORE
(defun generate-binomial-distribution (p n samples)
  (let ((dist (make-instance 'binomial-distribution :p p :n n)))
    (sample-to-list dist samples)))

#+IGNORE
(defun generate-gamma-distribution (order samples)
  (let ((dist (make-instance 'gamma-distribution :ia order)))
    (sample-to-list dist samples)))

#+IGNORE
(defun generate-poisson-distribution (mean samples)
  (let ((dist (make-instance 'poisson-distribution :mean mean)))
    (sample-to-list dist samples)))

;; two variables to bootstrap
#+TEST
(compute-bootstrap #'clasp::correlation 
                   :sampling-functions (list #'clasp::sample-without-replacement-from-sequence
                                             #'clasp::sample-without-replacement-from-sequence)
                   :sampling-function-arguments '(((1 2 3 4 5)) ((10 20 30 40 50)))
                   :number-of-iterations 5)
;; one variable to bootstrap
#+TEST
(compute-bootstrap #'clasp::mean
                   :sampling-functions (list 'clasp::sample-without-replacement-from-sequence)
                   :sampling-function-arguments '(((1 2 3 4 5 6)))
                   :number-of-iterations 5)
