#|
**********************************
* interface code for the custom  *
* test dialogs in Mac CLASP      *
**********************************


|#

(in-package "CLASP-INTERFACE")

;;;
;;; annova-one-way-var-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::anova-one-way-variables .


================================================================
Status =========================================================
================================================================
In-progress.

FIX: confidence-intervals !?
Note: 'scheffe-tests-pop-up-menu should probably be a checkbox, whose
 CHECK-BOX-CHECKED-P would be checked to get the proper value for the
 clasp function. I used a pop-up for demonstrative purposes because
 it's more general and a bit more complicated.


================================================================
Change history =================================================
================================================================
6-Jun-94 mc	Created.
30-Jun-94 ms    Made scheffe-tests-pop-up-menu into scheffe-tests-checkbox

|#


;;;
;;; Define anova-one-way-dialog .
;;;

(defclass anova-one-way-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::anova-one-way-variables function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Anova One Way Variables"
    :VIEW-SIZE #@(300 92)
    :view-position (best-spot 300 92)
    :help '*anova-one-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(S)"
                         :view-nick-name 'x-var-item
                         :view-position #@(8 9)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(S)"
                         :view-nick-name 'y-var-item :allow-categorical-p NIL
                         :view-position #@(8 30)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (MAKE-DIALOG-ITEM 'CHECK-BOX-DIALOG-ITEM #@(9 72) #@(140 20)
                            "Scheffe tests" NIL 
                            :VIEW-NICK-NAME 'scheffe-enable-cbdi
                            :check-box-checked-p t)
          (MAKE-instance 'disabler-checkbox-item :view-position #@(9 53) :view-size #@(153 20)
                            :dialog-item-text "Confidence intervals" :watches-over '(confidence-value)
                            :VIEW-NICK-NAME 'confi-enable-cbdi
                            :check-box-checked-p t)
          (make-instance 'number-entry-item :view-position #@(170 55) :view-size #@(40 16)
                         :allow-negatives NIL :allow-floats t
                         :view-nick-name 'confidence-value :dialog-item-text "0.95")
          (make-instance 'compute-button-di :view-position #@(226 69)))))

(defmethod f-can-compute-statistics-dialog ((anova-one-way-dialog anova-one-way-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   anova-one-way-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   anova-one-way-dialog)))
         (f-confi-p (check-box-checked-p (view-named 
                                          'confi-enable-cbdi
                                          anova-one-way-dialog)))
         (confi-v (number-item-value (view-named 'confidence-value anova-one-way-dialog)))
         (f-can-compute (and l-var-x l-var-y
                             (or (not f-confi-p) 
                                 (and (>= confi-v 0) (<= confi-v 1)))))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one X variable")
                                     ((null l-var-y)
                                      "Need at least one Y variable")
                                     (t "Confidence must be in the range 0..1"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((anova-one-way-dialog anova-one-way-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   anova-one-way-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   anova-one-way-dialog)))

         (f-scheffe-tests-p (check-box-checked-p (view-named 
                                                   'scheffe-enable-cbdi
                                                   anova-one-way-dialog)))
         (f-confi-p (check-box-checked-p (view-named 
                                          'confi-enable-cbdi
                                          anova-one-way-dialog)))
         (confi-v (number-item-value (view-named 'confidence-value anova-one-way-dialog)))
         )
    (if (> (length l-var-y) 1)
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-anova-one-way-variables x y f-scheffe-tests-p (or (not f-confi-p) confi-v)))
      (loop for x in l-var-x
            do (compute-anova-one-way-variables x (car l-var-y) f-scheffe-tests-p (or (not f-confi-p) confi-v))))))

;;;
;;;
;;;

(defmethod compute-anova-one-way-variables ((iv clasp::variable)
                                             (dv clasp::variable)
                                             (f-scheffe-tests-p t) (f-confi-p t))
  "Calls clasp::anova-one-way-variables on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::anova-one-way-variables
                          (clasp::simplify-and-sort ,iv)
                          (clasp::simplify-and-sort-by ,dv ,iv)
                          ,f-scheffe-tests-p ,f-confi-p)
                        :format-function
                        'anova-one-way-formatter
                        :description
                        (list (list 'command 'clasp::anove-one-way-variables)
                              (list 'x-args iv)
                              (list 'y-args dv)
                              (list 'scheffe-tests? f-scheffe-tests-p)
                              (list 'confidence-intervals f-confi-p)))))
    (run-clasp-command clasp-command
                       :str-message "Computing anova one way variables"
                       :f-update-dataset-view t)))


(defun anova-one-way-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
anova-one-way-variables function."
  ;;
  (make-instance 'anova-one-way-dialog))

;;;
;;; annova-one-way-var-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::anova-two-way-variables .


================================================================
Status =========================================================
================================================================
In-progress.

================================================================
Change history =================================================
================================================================
12-Jun-94 ms	Created.
20-Jul-94 ms    Beta finish.

|#


;;;
;;; Define anova-two-way-dialog .
;;;

(defclass anova-two-way-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::anova-two-way-variables function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Anova Two Way Variables"
    :VIEW-SIZE #@(300 105)
    :view-position (best-spot 300 105)
    :HELP '*anova-two-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)  "
                         :view-nick-name 'y-var-item :allow-categorical-p NIL
                         :view-position #@(8 49)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X1(s)"
                         :view-nick-name 'x-var-item
                         :view-position #@(8 5)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X2(s)"
                         :view-nick-name 'z-var-item
                         :view-position #@(8 27)
                         :view-size #@(200 20))
          (make-instance 'compute-button-di :view-position #@(120 80)))))


(defmethod f-can-compute-statistics-dialog ((anova-two-way-dialog anova-two-way-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((l-var-dv (l-variables-var-pum (view-named 'y-var-item
                                                   anova-two-way-dialog)))
         (l-var-iv1 (l-variables-var-pum (view-named 'x-var-item
                                                     anova-two-way-dialog)))
         (l-var-iv2 (l-variables-var-pum (view-named 'z-var-item
                                                     anova-two-way-dialog)))
         (f-can-compute (and l-var-dv
                             l-var-iv2
                             l-var-iv1))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-dv)
                                      "Need at least one dependent variable")
                                     ((or (null l-var-iv1) (null l-var-iv2))
                                      "Need at least one independent variable per dependent")
                                     (t "Variables must be of the same length"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((anova-two-way-dialog anova-two-way-dialog))
  (let* ((l-var-dv (l-variables-var-pum (view-named 'y-var-item
                                                   anova-two-way-dialog)))
         (l-var-iv1 (l-variables-var-pum (view-named 'x-var-item
                                                   anova-two-way-dialog)))
         (l-var-iv2 (l-variables-var-pum (view-named 'z-var-item
                                                   anova-two-way-dialog)))
         )
    (if (> (length l-var-dv) 1)
      (loop for a in l-var-dv
            for b in l-var-iv1
            for c in l-var-iv2
            do (compute-anova-one-way-variables a b c))
      (loop for dv in l-var-dv do
            (loop for iv1 in l-var-iv1
                  for iv2 in l-var-iv2
                  do (compute-anova-two-way-variables dv iv1 iv2))))))

;;;
;;;

(defmethod compute-anova-two-way-variables ((dv clasp::variable)
                                                 (iv1 clasp::variable)
                                                 (iv2 clasp::variable))
  "Calls clasp::anova-two-way-variables on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::anova-two-way-variables-unequal-cell-sizes
				(clasp::simplify ,dv)
				(clasp::simplify ,iv1)
				(clasp::simplify ,iv2))
                        #+OLD
                        `(clasp::anova-two-way-variables
                          (clasp::simplify-and-sort ,dv)
                          (clasp::simplify-and-sort-by ,dv ,iv1)
                          (clasp::simplify-and-sort-by ,dv ,iv2))
                        :format-function
                        'anova-two-way-formatter
                        :description
                        (list #+OLD
                              (list 'command 'clasp::anova-two-way-variables)
                              (list 'command 'clasp::anova-two-way-variables)
                              (list 'y-args dv)
                              (list 'x1-args iv1)
                              (list 'x2-args iv2)))))
    (run-clasp-command clasp-command
                       :str-message "Computing anova two way variables"
                       :f-update-dataset-view t)))


(defun anova-two-way-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
anova-two-way-variables function."
  ;;
  (make-instance 'anova-two-way-dialog))

;;;
;;; chi-squre-2x2-counts.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::chi-square-2x2-counts-counts .


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
12-Jul-94 ms	Created.
20-Jul-94 ms    Beta finish.

|#


;;;
;;; Define chi-square-2x2-counts-dialog .
;;;

(defclass chi-square-2x2-counts-Dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::chi-square-2x2-counts function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Chi-Square 2x2 Counts"
    :VIEW-SIZE #@(250 115)
    :view-position (best-spot 250 115)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :HELP '*chi-counts-help*
    :VIEW-SUBVIEWS
    (LIST (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'a-number-item
                         :allow-floats NIL
                         :view-position #@(73 15)
                         :view-size #@(50 16))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'b-number-item
                         :allow-floats NIL
                         :view-position #@(127 15)
                         :view-size #@(50 16))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'c-number-item
                         :allow-floats NIL
                         :view-position #@(73 35)
                         :view-size #@(50 16))
          (make-instance 'number-entry-item :dialog-item-text "0"
                         :view-nick-name 'd-number-item
                         :allow-floats NIL
                         :view-position #@(127 35)
                         :view-size #@(50 16))
          (make-instance 'check-box-dialog-item :dialog-item-text "Continuity correction"
                         :view-nick-name 'yates-cbdi
                         :view-position #@(50 60)
                         :check-box-checked-p t)
          (make-instance 'compute-button-di :view-position #@(95 88)))))

(defmethod f-can-compute-statistics-dialog ((chi-square-2x2-counts-dialog chi-square-2x2-counts-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((f-can-compute t)
         (str-why-cant-compute (cond (t "CLASP hates you."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((chi-square-2x2-counts-dialog chi-square-2x2-counts-dialog))
  (let* ((a (number-item-value (view-named 'a-number-item chi-square-2x2-counts-dialog)))
         (b (number-item-value (view-named 'b-number-item chi-square-2x2-counts-dialog)))
         (c (number-item-value (view-named 'c-number-item chi-square-2x2-counts-dialog)))
         (d (number-item-value (view-named 'd-number-item chi-square-2x2-counts-dialog)))
         (yates (check-box-checked-p (view-named 'yates-cbdi chi-square-2x2-counts-dialog)))
         )
    (compute-chi-square-2x2-count a b c d yates)))


;;;
;;;
;;;

(defmethod compute-chi-square-2x2-count (a b c d (yates t)
                                           )
  "Calls clasp::anova-one-way-variables on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::chi-square-2x2-counts
                          ,a ,b ,c ,d ,yates
                          )
                        :format-function
                        (make-line-mapper "Chi square of ~A is ~,4F significance ~,4F"
                                                                     '(ARGS 0 1))
                        :description
                        (list (list 'command 'clasp::chi-square-2x2-counts)
                              (list 'args a b c d)
                              (list 'yates? yates))
                        )))
    (run-clasp-command clasp-command
                       :str-message "Computing Chi-Square 2x2 Counts"
                       :f-update-dataset-view t)))


(defun chi-square-2x2-counts-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
anova-one-way-variables function."
  ;;
  (make-instance 'chi-square-2x2-counts-dialog))

#|
================================================================
conifdence interval using z or t   =============================
================================================================
dialog box for confidence-interval-data (z and t stats)

6-Jul-94 ms	Created.
20-Jul-94 ms    Beta finish.

|#
;;;
;;; Define confidence-interval-data-dialog .
;;;

(defclass confidence-interval-data-dialog (statistic-dialog-mixin dialog)
  ((statistical-function :initarg :statistical-function)
   (statistical-function-name :initarg :statistical-function-name)
   (format-function :initarg :format-function))
  (:documentation "A dialog for getting data for the
Median function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Confidence Interval"
    :VIEW-SIZE #@(350 60)
    :view-position (best-spot 350 60)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(9 8)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'static-text-dialog-item :dialog-item-text "Confidence"
                         :view-position #@(9 33))
          (make-instance 'number-entry-item :dialog-item-text "0.95"
                         :allow-negatives nil
                         :view-nick-name 'confidence-number-item
                         :view-position #@(95 35)
                         :view-size #@(70 15))
          (make-instance 'compute-button-di :view-position #@(250 35)))))

(defmethod f-can-compute-statistics-dialog ((confidence-interval-data-dialog confidence-interval-data-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   confidence-interval-data-dialog)))
         (conf (number-item-value (view-named 'confidence-number-item
                                              confidence-interval-data-dialog)))
         (f-can-compute (and l-var-x (and (> conf 0) (< conf 1))))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need at least one data item")
                                     (t "confidence must be in range 0..1"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((confidence-interval-data-dialog confidence-interval-data-dialog))
  (with-slots (statistical-function statistical-function-name format-function)  confidence-interval-data-dialog
    (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                     confidence-interval-data-dialog)))
           (confidence (number-item-value (view-named 'confidence-number-item
                                                      confidence-interval-data-dialog))))
      (loop for x in l-var-x
            do (compute-confidence-statistic x confidence statistical-function
                                             format-function
                                             statistical-function-name)))))


(defmethod compute-confidence-statistic ((iv clasp::variable) confidence function ffunction name)
  "Calls function on the arguments, saving
results in the history."
  (let ((clasp-command (make-clasp-command
                        `(,function
                          (clasp::simplify ,iv)
                          ,confidence)
                        :format-function ffunction
                        :description
                        (list (list 'command function)
                              (list 'x-args iv)
                              (list 'confidence confidence))
                        )))
    (run-clasp-command clasp-command
                       :str-message (format nil "Computing ~a" name)
                       :f-update-dataset-view t)))


(defun confidence-interval-data-dialog-make (function &optional (ffunction NIL) (name (string-capitalize (string function)))
                                                      &key (help "No help for this statistic."))
  "Returns a new dialog instance for editing the function."
  ;;
    (make-instance 'confidence-interval-data-dialog 
      :window-title name
      :statistical-function function 
      :format-function ffunction
      :help help
      :statistical-function-name name))

;;;
;;; confi-prop-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::confidence-interval-proportion .


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
12-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#

(let ((l '()))
  (dotimes (x 1000)
    (push 5 l))
  (clasp::mean l))
  

;;;
;;; Define confidence-prop-dialog .
;;;

(defclass confidence-prop-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
confidence interval of a proportion function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Confidence Interval Proportion"
    :VIEW-SIZE #@(250 130)
    :view-position (best-spot 250 130)
    :help '*confi-prop-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Trials"
                         :view-position #@(15 15))
          (make-instance 'number-entry-item :dialog-item-text "10"
                         :view-nick-name 'total-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 15)
                         :view-size #@(50 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Successes"
                         :view-position #@(15 42))
          (make-instance 'number-entry-item :dialog-item-text "5"
                         :view-nick-name 'success-number-item
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-position #@(125 42)
                         :view-size #@(50 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Confidence"
                         :view-position #@(15 70))
          (make-instance 'number-entry-item :dialog-item-text "0.90"
                         :view-nick-name 'confidence-number-item
                         :allow-floats t
                         :allow-negatives NIL
                         :view-position #@(125 70)
                         :view-size #@(50 16))
          (make-instance 'compute-button-di :view-position #@(95 100)))))

(defmethod f-can-compute-statistics-dialog ((confidence-prop-dialog confidence-prop-dialog))
  "Just check the numbers."
  ;;
  (let* ((successes (number-item-value (view-named 'success-number-item
                                                   confidence-prop-dialog)))
         (trials (number-item-value (view-named 'total-number-item
                                                confidence-prop-dialog)))
         (conf (number-item-value (view-named 'confidence-number-item
                                              confidence-prop-dialog)))
         (f-can-compute (and (> successes 0) (> trials 0) 
                             (and (> conf 0) (< conf 1))
                             (> trials successes)))
         (str-why-cant-compute (cond ((< successes 0)
                                      "invalid # of successes")
                                     ((< trials 1)
                                      "invalid # of trials")
                                     ((> successes trials)
                                      "more successes than trials")
                                     (t "confidence must be in rannge 0..1"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((confidence-prop-dialog confidence-prop-dialog))
  (let* ((n (number-item-value (view-named 'total-number-item confidence-prop-dialog)))
         (x (number-item-value (view-named 'success-number-item confidence-prop-dialog)))
         (c (number-item-value (view-named 'confidence-number-item confidence-prop-dialog)))
         )
    (compute-confi-proportions n x c)))


;;;
;;;
;;;

(defmethod compute-confi-proportions (n x c)
  "Calls clasp::anova-one-way-variables on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::confidence-interval-proportion
                          ,x ,n ,c)
                        :format-function
                        (make-table-maker
                         *confidence-labels*
                         :table-type :row-table
                         :name-format "~5F Confidence interval of a proportion ~D:~D"
                         :name-args '(CONFIDENCE SUCCESSES TRIALS))
                        :description
                        (list (list 'command 'clasp::confidence-interval-proportion)
                              (list 'successes x)
                              (list 'trials n)
                              (list 'confidence c)))))
    (run-clasp-command clasp-command
                       :str-message "Computing Confidence Interval Proportions"
                       :f-update-dataset-view t)))


(defun confidence-prop-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
anova-one-way-variables function."
  ;;
  (make-instance 'confidence-prop-dialog))

#|
================================================================
pop-up-tails-dialog-item========================================
================================================================
Pop up dialog item for :tails parameter found in the t-tests

1-Jul-94 ms	Created.
21-Jul-94 ms    Beta finish.

|#

(defclass tails-pop-up-menu (pop-up-menu)
  ()
  (:documentation "A pop-up-menu subclass with three items,
'Both', 'Positive', and 'Negative'."))

(defmethod initialize-instance :after ((tails-pop-up-menu tails-pop-up-menu)
                                       &key)
  "Inserts the three tails items"
  (add-menu-items tails-pop-up-menu
                  (make-instance 'menu-item :menu-item-title "Both")
                  (make-instance 'menu-item :menu-item-title "Positive")
                  (make-instance 'menu-item :menu-item-title "Negative")))

(defmethod f-menu-value-tails-pum ((tails-pop-up-menu tails-pop-up-menu))
  "returns :both :positive or :negative"
  (let* ((menu-item-selected (selected-item tails-pop-up-menu))
         (str-pop-up-selected (menu-item-title menu-item-selected)))
    (cond ((string-equal str-pop-up-selected "Both")
           :both)
          ((string-equal str-pop-up-selected "Positive")
           :positive)
          ((string-equal str-pop-up-selected "Negative")
           :negative)
          (t "~S is an invalid tail type." str-pop-up-selected))))

#|
================================================================
d-test==========================================================
================================================================
dialog box for the  d-test

5-Jul-94 ms	Created.
22-Jul-94 ms    Beta finish

|#
;;;
;;; Define d-test-dialog .
;;;

(defclass d-test-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
d-test function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "D Test"
    :VIEW-SIZE #@(350 145)
    :help '*d-test-help*
    :view-position (best-spot 350 145)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(8 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'y-var-item :allow-categorical-p NIL
                         :view-position #@(8 40)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'tails-pop-up-menu
                         :view-nick-name 'd-test-tails-pop-up-menu
                         :view-position #@(9 78)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Tails")
          (make-instance 'number-entry-item :dialog-item-text "250"
                         :allow-floats NIL
                         :allow-negatives NIL
                         :view-nick-name 'bootstrap-times-ftdi
                         :view-position #@(230 80)
                         :view-size #@(101 16))
          (make-instance 'static-text-dialog-item :dialog-item-text "Resamples"
                         :view-position #@(143 78))
          (make-instance 'compute-button-di :view-position #@(140 110)))))


(defmethod f-can-compute-statistics-dialog ((d-test-dialog d-test-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   d-test-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   d-test-dialog)))
         (bootstraps (number-item-value (view-named 'bootstrap-times-ftdi
                                                    d-test-dialog)))
         (f-can-compute (and (> bootstraps 0)
                             l-var-x l-var-y))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((or (null l-var-x)
                                          (null l-var-y))
                                      "Need at least one Sample per slot")
                                     (t "Invalid # of resamples"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((d-test-dialog d-test-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   d-test-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   d-test-dialog)))
         (bootstraps (number-item-value (view-named 'bootstrap-times-ftdi
                                                   d-test-dialog)))
         (tails (f-menu-value-tails-pum (view-named 'd-test-tails-pop-up-menu
                                                    d-test-dialog))))
    (if (> (length l-var-y) 1)
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-d-test x y tails bootstraps))  ;; lexical pairing
      (loop for x in l-var-x
            do (compute-d-test x (car l-var-y) tails bootstraps)))))


(defmethod compute-d-test ((iv clasp::variable)
                             (dv clasp::variable)
                             tails
                             boots)
  "Calls d-test on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::d-test
                          (clasp::simplify ,iv)
                          (clasp::simplify ,dv)
                          ,tails
                          :times ,boots)
                        :format-function
                        'd-test-format
                        :description
                        (list (list 'COMMAND 'clasp::d-test)
                              (list 'x-args iv)
                              (list 'y-args dv)
                              (list 'tails tails)
                              (list 'bootstraps boots))
                        )))
    (run-clasp-command clasp-command
                       :str-message "Computing d-test two samples"
                       :f-update-dataset-view t)))


(defun d-test-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
d-test-one-sample function."
  ;;
  (make-instance 'd-test-dialog))

#|
================================================================
t-test-one-sample-dialog========================================
================================================================
dialog box for the one sample version of the t-test

1-Jul-94 ms	Created.

|#
;;;
;;; Define t-test-one-sample-dialog .
;;;

(defclass t-test-one-sample-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
t-test function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "T-Test One Sample"
    :VIEW-SIZE #@(350 120)
    :view-position (best-spot 350 120)
    :help '*t-test1-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Sample"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(8 15)
                         :view-size #@(250 20))         ;FIX: 20 is hardcoded
          (MAKE-DIALOG-ITEM 'STATIC-TEXT-DIALOG-ITEM #@(50 48) #@(100 15)
                            "Sample Mean")
          (make-instance 'number-entry-item :dialog-item-text "0.00"
                         :view-nick-name 'mean-float-entry
                         :view-position #@(150 48)
                         :view-size #@(150 15)
                         )
          (make-instance 'tails-pop-up-menu
                         :view-nick-name 't-test-tails-pop-up-menu
                         :view-position #@(42 78)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Tails")
          (make-instance 'compute-button-di :view-position #@(210 80)))))

(defmethod f-can-compute-statistics-dialog ((t-test-one-sample-dialog t-test-one-sample-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-one-sample-dialog)))
         (f-can-compute l-var-x)
         (str-why-cant-compute (cond (f-can-compute "")
                                     (t "need at least one sample"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((t-test-one-sample-dialog t-test-one-sample-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-one-sample-dialog)))
         (mean (number-item-value (view-named 'mean-float-entry t-test-one-sample-dialog)))
         (tails (f-menu-value-tails-pum (view-named 't-test-tails-pop-up-menu
                                                    t-test-one-sample-dialog))))
    (loop for x in l-var-x
         do (compute-t-test-one-sample x mean tails))))


(defmethod compute-t-test-one-sample ((iv clasp::variable)
                                             mean
                                             tails)
  "Calls t-test on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::t-test-one-sample
                          (clasp::simplify ,iv)
                          ,tails 
                          ,mean)
                        :format-function
                        (make-table-maker *t-test-labels*
                                          :name-format "T Test of ~:(~A~) mean ~5F"
                                          :name-args '(X-ARGS MEAN))
                        :description
                        (list (list 'command 'clasp::t-test-one-sample)
                              (list 'X-ARGS iv)
                              (list 'tails tails)
                              (list 'mean mean)))))
    (run-clasp-command clasp-command
                       :str-message "Computing t-test one sample"
                       :f-update-dataset-view t)))


(defun t-test-one-sample-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
t-test-one-sample function."
  ;;
  (make-instance 't-test-one-sample-dialog))

#|
================================================================
t-test==========================================================
================================================================
dialog box for the one sample version of the t-test

5-Jul-94 ms	Created.
21-Jul-94 ms    Beta finish.

|#
;;;
;;; Define t-test-dialog .
;;;

(defclass t-test-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
t-test function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "T-Test Two Samples"
    :VIEW-SIZE #@(350 120)
    :HELP '*t-test2-help*
    :view-position (best-spot 350 120)
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'y-var-item :allow-categorical-p NIL
                         :view-position #@(9 40)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'tails-pop-up-menu
                         :view-nick-name 't-test-tails-pop-up-menu
                         :view-position #@(42 78)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Tails")
          (make-instance 'compute-button-di :view-position #@(210 80)))))

(defmethod f-can-compute-statistics-dialog ((t-test-dialog t-test-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   t-test-dialog)))
         (f-can-compute (and l-var-x l-var-y))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((or (null l-var-x)
                                          (null l-var-y))
                                      "Need at least one Sample per slot")
                                     (t "Variables must be of same length"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((t-test-dialog t-test-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   t-test-dialog)))
         (tails (f-menu-value-tails-pum (view-named 't-test-tails-pop-up-menu
                                                    t-test-dialog))))
    (if (> (length l-var-y) 1)
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-t-test x y tails))  ;; lexical pairing
      (loop for x in l-var-x
            do (compute-t-test x (car l-var-y) tails)))))


(defmethod compute-t-test ((iv clasp::variable)
                             (dv clasp::variable)
                             tails)
  "Calls t-test on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::t-test
                          (clasp::simplify ,iv)
                          (clasp::simplify ,dv)
                          ,tails)
                        :format-function
                        (make-table-maker *t-test-labels*
                                          :name-format "T Test of ~:(~A~) and ~:(~A~)"
                                          :name-args '(X-ARGS Y-ARGS))
                        :description
                        (list (list 'command 'clasp::t-test)
                              (list 'x-args iv)
                              (list 'y-args dv)
                              (list 'tails tails)))))
    (run-clasp-command clasp-command
                       :str-message "Computing t-test"
                       :f-update-dataset-view t)))


(defun t-test-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
t-test-one-sample function."
  ;;
  (make-instance 't-test-dialog))

#|
================================================================
t-test-matched==================================================
================================================================
dialog box for the one sample version of the t-test-matched function

5-Jul-94 ms	Created.
21-Jul-94 ms    Beta finish.

|#
;;;
;;; Define t-test-matched-dialog .
;;;

(defclass t-test-matched-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for getting data for the
t-test-matched function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "T-Test Matched"
    :VIEW-SIZE #@(350 120)
    :view-position (best-spot 350 120)
    :help '*t-testm-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(9 15)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Y(s)"
                         :view-nick-name 'y-var-item :allow-categorical-p NIL
                         :view-position #@(9 40)
                         :view-size #@(225 20))         ;FIX: 20 is hardcoded
          (make-instance 'tails-pop-up-menu
                         :view-nick-name 't-test-matched-tails-pop-up-menu
                         :view-position #@(42 78)
                         #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "Tails")
          (make-instance 'compute-button-di :view-position #@(210 80)))))

(defmethod f-can-compute-statistics-dialog ((t-test-matched-dialog t-test-matched-dialog))
  "We can compute if there is at least one sample in the x-var-pop-up-menu."
  ;;
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-matched-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   t-test-matched-dialog)))
         (f-can-compute (and l-var-x
                             l-var-y))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((or (null l-var-x)
                                          (null l-var-y))
                                      "Need at least one sample per slot")
                                     (t "Variables must be of same length"))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((t-test-matched-dialog t-test-matched-dialog))
  (let* ((l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   t-test-matched-dialog)))
         (l-var-y (l-variables-var-pum (view-named 'y-var-item
                                                   t-test-matched-dialog)))
         (tails (f-menu-value-tails-pum (view-named 't-test-matched-tails-pop-up-menu
                                                    t-test-matched-dialog))))
    (if (> (length l-var-y) 1)
      (loop for x in l-var-x
            for y in l-var-y
            do (compute-t-test-matched x y tails))  ;; lexical pairing
      (loop for x in l-var-x
            do (compute-t-test-matched x (car l-var-y) tails)))))  ;; mapping


(defmethod compute-t-test-matched ((iv clasp::variable)
                                       (dv clasp::variable)
                                       tails)
  "Calls t-test-matched on the arguments, saving
results in the history."
  ;;
  (let ((clasp-command (make-clasp-command
                        `(clasp::t-test-matched
                          (clasp::simplify ,iv)
                          (clasp::simplify ,dv)
                          ,tails)
                        :format-function
                        (make-table-maker *t-test-labels*
                                          :name-format "T Test Matched of ~:(~A~) and ~:(~A~)"
                                          :name-args '(X-ARGS Y-ARGS))
                        :description
                        (list (list 'command 'clasp::t-test-matched)
                              (list 'x-args iv)
                              (list 'y-args dv)
                              (list 'tails tails)))))
    (run-clasp-command clasp-command
                       :str-message "Computing matched t-test "
                       :f-update-dataset-view t)))


(defun t-test-matched-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
t-test-matched-one-sample function."
  ;;
  (make-instance 't-test-matched-dialog))

;;;
;;; multiple-lr-dialog.lisp
;;;

#|
================================================================
Purpose ========================================================
================================================================
Defines the drag-and-drop statistics function dialog for
clasp::multiple-linear-regression-verbose.


================================================================
Status =========================================================
================================================================
In-progress.


================================================================
Change history =================================================
================================================================
5-Jul-94 ms	Created.
22-Jul-94 ms    Beta Finish.

|#

;;;
;;; Define multi-lr-dialog .
;;;

(defclass multi-lr-dialog (statistic-dialog-mixin dialog)
  ()
  (:documentation "A dialog for inputting data for the
clasp::linear-regression-verbose function.")
  (:default-initargs
    :WINDOW-TYPE :DOCUMENT
    :WINDOW-TITLE "Multiple Linear Regressions - verbose"
    :VIEW-SIZE #@(300 100)
    :view-position (best-spot 300 82)
    :help '*multi-lr-help*
    :VIEW-FONT '("Chicago" 12 :SRCOR :PLAIN)
    :VIEW-SUBVIEWS
    (LIST (make-instance 'static-text-dialog-item :dialog-item-text "Y"
                         :view-position #@(8 40)) 
          (make-instance 'variable-stdi :dialog-item-text "<NONE>"
                         :view-nick-name 'y-var-item
                         :view-position #@(48 40)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'variable-pop-up-menu #-CCL-3 :dialog-item-text #+CCL-3 :menu-title "X(s)"
                         :view-nick-name 'x-var-item :allow-categorical-p NIL
                         :view-position #@(8 15)
                         :view-size #@(200 20))         ;FIX: 20 is hardcoded
          (make-instance 'compute-button-di :view-position #@(120 70)))))


(defmethod f-can-compute-statistics-dialog ((multi-lr-dialog multi-lr-dialog))
  "We can compute when there are one or more pairs of x(s) and y(s),
with the same number of each."
  ;;
  (let* ((l-var-y (variable-vset-stdi (view-named 'y-var-item
                                                   multi-lr-dialog)))
         (l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   multi-lr-dialog)))
         (f-can-compute (and l-var-y
                             l-var-x))
         (str-why-cant-compute (cond (f-can-compute "")
                                     ((null l-var-x)
                                      "Need one dependent variable")
                                     ((null l-var-y)
                                      "Need at least one independent variable")
                                     (t "Some one won't let you do this."))))
    (values f-can-compute
            str-why-cant-compute)))


(defmethod compute-statistics-dialog ((multi-lr-dialog multi-lr-dialog))
  (let* ((l-var-y (variable-vset-stdi (view-named 'y-var-item
                                                   multi-lr-dialog)))
         (l-var-x (l-variables-var-pum (view-named 'x-var-item
                                                   multi-lr-dialog)))
         )
    (compute-multi-lin-regr l-var-y l-var-x)))

;;;
;;;

(defmethod compute-multi-lin-regr ((dv clasp::variable)
                                      (iv list)
                                      )
  "Calls clasp::multiple-linear-regression on the arguments, saving
results in the history."
  ;;
  (let* ((clasp-command (make-clasp-command
                         `(apply
                           #'clasp::multiple-linear-regression-verbose
                           (clasp::simplify ,dv)
                           (mapcar #'clasp::simplify ',iv))
                         :format-function
                         'multi-lr-formatter
                         :description
                         (list (list 'command 'clasp::multiple-linear-regression)
                               (list 'x-args iv)
                               (list 'y-arg dv))
                         )))
    (run-clasp-command clasp-command
                       :str-message "Computing Multiple Linear Regressions"
                       :f-update-dataset-view t)))


(defun multi-linear-regr-dialog-make ()
  "Returns a new dialog instance for editing CLASP's
linear-regression function in the multiple argument tradition."
  ;;
  (make-instance 'multi-lr-dialog))
