(define-application-frame test-frame ()
  ((use-default-top-level :initform t))
  (:command-table (test
		    :inherit-from
		    (user-command-table
		     accept-values-pane)))
  
  (:panes
   (mouse-doc :pointer-documentation
              :text-style 
	      (make-text-style :sans-serif :roman :small)
	      :max-height 12)
   (notebook :interactor
	     :scroll-bars :both
	     :default-text-style (make-text-style :sans-serif :roman :normal)
	     :height +fill+ :width +fill+))
  (:pointer-documentation t)
  (:documentation "TEST")
  (:layouts
   (default
    (vertically ()
      notebook
      mouse-doc)))
  (:top-level (test-top-level)))

(defun test-top-level (frame)
  (loop
   (catch 'up
     (cond ((slot-value frame 'use-default-top-level)
            (format (frame-standard-output frame) "~&Default> ")
            (default-frame-top-level frame))
       (t
        ;; This is the rudiments of a read-eval-print loop using 'command-or-form.
        (let* ((instream (frame-standard-input frame))
               (outstream (frame-standard-output frame))
	       (command-table (frame-command-table frame))
	       (presentation-type
                `((command-or-form :command-table ,command-table)
		  :auto-activate nil)))
          (loop
	   (catch-abort-gestures ("Return to ~A command level"
				  (frame-pretty-name frame))
	     (multiple-value-bind (command-or-form type numeric-arg)
		 (accept presentation-type
			 :stream instream
			 :prompt "Custom> " :prompt-mode :raw)
	       (declare (ignore numeric-arg))
	       (cond ((eq type ':keystroke)
		      (beep outstream))
		     ((null command-or-form))
		     ((eq (presentation-type-name type) 'command)
		      (terpri outstream)
		      (apply (command-name command-or-form)
			     (command-arguments command-or-form))
		      (terpri outstream))
		     (t
		      (let ((values 
			     (multiple-value-list
			      (eval command-or-form))))
			(terpri outstream)
			(dolist (value values)
			  (present value 'expression :stream outstream :single-box :highlighting)
			  (terpri outstream))))))))))))))


(define-test-frame-command (com-quit :name "Quit" :menu "Quit")
    ()
   (frame-exit *application-frame*))

(define-test-frame-command (com-change-top-level :name "Change Top Level" :menu "Change Top Level")
    ()
   
   (setf (slot-value *application-frame* 'use-default-top-level) (not (slot-value *application-frame* 'use-default-top-level)))
   (throw 'up))

(defun test ()
  (run-frame-top-level (make-application-frame 'test-frame :width 500 :height 300)))

