;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/data-manipulation-commands.lisp *-*
;;;; *-* Last-edit: Thursday, February 18, 1993  00:54:05; Edited-By: carlson *-* 
;;;; *-* Machine: Miles (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       Data Manipulation Commands                       *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-18-93 File Created.                         (carlson)
;;;  06-21-95 Added winspect command for Lispworks. (schmill)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :CLASP-INTERFACE)

;;; --*--
;;; ***************************************************************************


(define-command-table clasp-data-manipulation)

(add-menu-item-to-command-table 'clasp "Manipulate" 
                                :menu 'clasp-data-manipulation 
				:documentation "Data manipulation"
                                :errorp nil :after "Test")

(define-clasp-command (com-rename-dataset
		       :command-table clasp-data-manipulation :name t
		       :menu ("Rename Dataset"
			      :documentation
			      "Change the name of a dataset"))
    ((dataset-1 'dataset :prompt "Dataset")
     (string-2 'string :prompt "New name"))
  (rename-dataset dataset-1 string-2)
  (terpri)
  (update-datasets-display))

(define-clasp-command (com-rename-variable
		       :command-table clasp-data-manipulation :name t
		       :menu ("Rename Variable"
			      :documentation
			      "Change the name of a variable"))
    ((column-1 'column :prompt "Variable")
     (string-2 'string :prompt "New name"))
  (rename-variable column-1 string-2)
  (update-datasets-display))

(define-clasp-command (com-delete-result
		       :command-table clasp-data-manipulation :name "Delete Object"
		       :menu ("Delete"
			      :documentation "Delete something"))
    ((result-1 'result :prompt "Object" :mapping-option :map))
  (delete-result result-1))

(defmethod delete-result ((self result))
  (close-it self)
  (clasp::kill self)
  (update-results-display))

(defmethod delete-result ((self dataset))
  (close-it self)
  (delete-dataset self)
  (update-datasets-display))

(defmethod delete-result ((self column))
  (close-it self)
  (delete-variable self)
  (update-datasets-display))

(define-clasp-command
    (com-make-dataset-from-rows
     :command-table clasp-data-manipulation :name t
     :menu
     ("Make Dataset from Rows"
      :documentation
      "Create a dataset from row-major data

 Enter the dataset name, the data for the dataset and names for each
 variable.  The data is a semicolon separated sequence of comma
 separated sequences.  That is if you want to enter data for a table
 called DS1, with variables V1, V2, V3 and V4 which looks like:

 2 4 2 3
 5 7 4 3

 Enter DS1 in response to the Dataset prompt, 2, 4, 2, 3; 5, 7, 4, 3
 in response to the Data prompt, and V1, V2, V3, V3 in response to the
 Variable names prompt."))
    ((name 'string :prompt "Name")
     (data '((sequence (sequence (or number symbol))) :separator #\;)
	   :prompt "Data")
     (variable-names '(sequence string) :prompt "Variable names"))
  (let ((new-dataset (make-dataset-from-rows name data variable-names)))
    (present-with-update new-dataset
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command
    (com-make-dataset-from-columns
     :command-table clasp-data-manipulation :name t
     :menu
     ("Make Dataset from Columns"
      :documentation
      "Create a dataset from column-major data

 Enter the dataset name, the data for the dataset and names for each
 variable.  The data is a semicolon separated sequence of comma
 separated sequences.  That is if you want to enter data for a table
 which looks like:

 2 4 2 3
 5 7 4 3

 Enter DS1 in response to the Dataset prompt, 2, 5; 4, 7; 2, 4; 3, 3
 in response to the Data prompt, and V1, V2, V3, V3 in response to the
 Variable names prompt."))
    ((name 'string :prompt "Name")
     (data '((sequence (sequence (or number symbol))) :separator #\;)
	   :prompt "Data")
     (variable-names '(sequence string) :prompt "Variable names"))
  (let ((new-dataset (make-dataset-from-columns name data variable-names)))
    (present-with-update new-dataset
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-add-variable-to-dataset
		       :command-table clasp-data-manipulation :name t
		       :menu ("Add Variable To Dataset"
			      :documentation
			      "Add a new variable to a dataset"))
    ((dataset-1 'dataset :prompt "Dataset")
     (expression-2 '(sequence (or number symbol)) :prompt "Data")
     (string-3 'string :prompt "Name"))
  (let ((values
	 (multiple-value-list (funcall #'add-variable-to-dataset
				       dataset-1
				       expression-2
				       string-3))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'add-variable-to-dataset (length values) 1)
    (present-with-update (nth 0 values)
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(defun build-where-lambda (variables expression)
  (compile nil 
           `(lambda ,(mapcar #'(lambda (v) (form-symbol (name v))) variables)
              ,@(mapcar #'(lambda (v) (form-symbol (name v))) variables)
              ,expression)))

(define-clasp-command (com-partition-dataset
		       :command-table clasp-data-manipulation :name t
		       :menu ("Partition Dataset"
			      :documentation
			      "Select rows from a dataset"))
    ((dataset-1 'dataset :prompt "Dataset")
     (expression-2 #+clim-1 'expression
		   #+clim-2 '((expression) :auto-activate t) :prompt "Partition clause")
     (variables '(sequence column) :prompt "Includes variables [default All]"
		:default nil :display-default nil)

     ;(pvariables '(sequence column) :default (dataset-variables dataset-1)
     ;            :prompt "Variable Args" :display-default nil)
     )
  
  (when (null variables)
    (setf variables (dataset-variables dataset-1)))

   ;(clasp::spy pvariables variables)
  (let ((values
	 (multiple-value-list (partition-dataset
			       dataset-1
                               #+OLD-VARS
			       expression-2
                               (build-where-lambda (dataset-variables dataset-1) expression-2)
                               (dataset-variables dataset-1)
			       variables
                               expression-2))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      '"partition dataset" (length values) 1)
    (present-with-update (nth 0 values)
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-partition-on
		       :command-table clasp-data-manipulation :name t
		       :menu ("Partition On"
			      :documentation
			      "Partition a dataset on one of its variables"))
    ((dataset 'dataset :prompt "Dataset")
     (variables '(sequence column) :prompt "Partition variables")
     (ivariables '(sequence column) :prompt "Included variables [default All]"
		 :default (dataset-variables dataset) :display-default nil))
  
   (when (null ivariables)
     (setf ivariables (dataset-variables dataset)))
   
  #+LATER
  ;; **** Workaround for bug in CLIM...
  (when (equal ivariables '(nil))
    (setf ivariables (dataset-variables dataset)))
  ;; ****
  (setf ivariables (remove-duplicates (append variables ivariables)))
  (let ((datasets (partition-on
		   dataset
                   variables
		   ivariables)))
    (dolist (dataset datasets)
      (present-with-update dataset :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))))
  
(define-clasp-command (com-merge-datasets
		       :command-table clasp-data-manipulation
		       :name t
		       :menu ("Merge Datasets"
			      :documentation
			      "Combine datasets, stacking like-named columns"))
    ((datasets '(sequence dataset) :prompt "Datasets"))
  (let ((new-dataset (merge-datasets datasets)))
    (present-with-update new-dataset
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))
   
(define-clasp-command (com-open :command-table clasp-data-manipulation :name t
				:menu ("Open"
				       :documentation
				       "Display a result in a window"))
    ((icon 'result :prompt "Object" :mapping-option :map))
  (open-it icon))

(define-clasp-command (com-close :command-table clasp-data-manipulation :name t
				 :menu ("Close"
					:documentation
					"Remove a window displaying a result"))
    ((icon 'result :prompt "Object" :mapping-option :map))
  (close-it icon))

(clim:define-presentation-to-command-translator open-result
    (result com-open clasp-data-manipulation
	    :gesture :select
	    :pointer-documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Open ~a" (name object)))
	    :documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Open ~a" (name object)) )
	    :menu t
	    :tester
	    ((object)
	     (and (not (window-open-p object))
		  (clasp::find-instance-by-id (id object) (class-of object)
						t))))
  (object)
  #-lucid (list (list object))
  #+lucid (list `',(list object)))

(clim:define-presentation-to-command-translator close-result
    (result com-close clasp-data-manipulation
	    :gesture :select
	    :pointer-documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Close ~a" (name object)))
	    :documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Close ~a" (name object)))
	    :menu t
	    :tester
	    ((object)
	     (and (window-open-p object)
		  (clasp::find-instance-by-id (id object) (class-of object)
						t))))
  (object)
  #-lucid (list (list object))
  #+lucid (list `',(list object)))

(define-clasp-command (com-describe :command-table clasp-data-manipulation
				    :name t
				    :menu ("Describe"
					   :documentation
					   "Show the description of a result"))
    ((result 'result :prompt "Result"))
  (present (description result) 'string
	   :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)
  (terpri))

(define-presentation-to-command-translator describe-result
    (result com-describe clasp-data-manipulation
	    :gesture :describe
	    :pointer-documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Describe ~a" (clasp::name object)))
	    :documentation
	    ((object presentation context-type frame stream)
	     (declare (ignore presentation frame context-type))
	     (format stream "Describe ~a" (clasp::name object)))
	    :menu t)
  (object)
  (list object))

;;; ---------------------------------------------------------------------------
;;; this is window inspector stuff for Lispworks

#+LISPWORKS
(define-gesture-name :winspect :pointer-button (:middle :control))

#+LISPWORKS
(define-command 
    (com-winspect :name "winspect " :command-table clasp)
    ((exp 'result))
  (tools:w-inspect exp))
  
#+LISPWORKS
(define-presentation-to-command-translator com-winspect
    (result
     com-winspect
     clasp
     :pointer-documentation
     ((object stream) (format stream "Inspect ~S" object))
     :gesture :winspect
     :echo t)
    (object)
  (list object))

#+LISPWORKS
(define-presentation-to-command-translator result-to-winspect
    (result com-winspect clasp-data-manipulation
            :gesture :winspect
            :pointer-documentation
            ((object presentation context-type frame stream)
             (declare (ignore presentation frame context-type))
             (format stream "Inspect ~a" (clasp::name object)))
            :documentation
            ((object presentation context-type frame stream)
             (declare (ignore presentation frame context-type))
             (format stream "Inspect ~a" (clasp::name object)))
            :menu t)
    (object)
  (list object))

;;; ***************************************************************************
;;; EOF

