;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Package: CLIM-DEMO; Base: 10; Lowercase: Yes -*-

(in-package :clim-user)

(define-application-frame lisp-listener ()
  ((results :initform 
	    (list (make-instance 'result :name "Default"))
	    :accessor results))
  (:command-table (lisp-listener :inherit-from (user-command-table file debug)))
  (:command-definer t)
  ;(:menu-bar nil)
  (:top-level (lisp-listener-top-level))
  (:panes
   (interactor :interactor 
	       #+allegro :excl-recording-p #+allegro t
	       :scroll-bars :both)
   (results :application
	    :default-text-style (clim::make-text-style :san-serif :roman :small)
	    :scroll-bars :both
	    :end-of-line-action :allow
	    :end-of-page-action :allow
	    :display-function 'draw-results-pane)
   (mouse-doc :pointer-documentation
              :text-style (clim:make-text-style :sans-serif :roman :small)
	      :height 12
	      :max-height 12))
  (:layouts (default
		(vertically ()
		  (horizontally ()
		    (1/5 results)
		    (4/5 interactor))
		  mouse-doc))))

;;; ---------------------------------------------------------------------------

(defclass named-object ()
  ((name :initarg :name :accessor name)))

(defclass result (named-object)
  ())

(defmethod draw-results-pane ((application lisp-listener) stream)
  (dolist (result (results application))
    (with-output-as-presentation (stream result 'result)
      (write-string (string (name result)) stream)
      (terpri stream))))

;;; --------------------------------------------------------------------------

(define-command-table file)
(define-command-table debug)

(add-menu-item-to-command-table 'lisp-listener "File" :menu 'file :errorp nil
				:documentation "I/O commands"
				:after :start)

(define-command (com-load-file :command-table file :name t :menu "Load File")
    ((pathnames '(sequence pathname) 
		:provide-default t :prompt "file"))
  (map nil #'load pathnames))

(define-command (com-copy-output-history :command-table file :name t :menu "Save History")
    ((pathname 'pathname :prompt "file"))
  (with-open-file (stream pathname :direction :output)
    (copy-textual-output-history *standard-output* stream)))

(define-command (com-quit :command-table file :name "Quit"
			  :menu ("Quit"
				 :documentation
				 "Leave Lisp Listener"))
    ()
  (frame-exit *application-frame*))

(add-menu-item-to-command-table 'lisp-listener "Debug" :menu 'Debug :errorp nil
				:documentation "other commands"
				:after "File")

(define-command (com-make-dummy-result :command-table debug :name t :menu "Make Dummy Result")
    ()
  (push (make-instance 'result :name (gensym)) (results *application-frame*))
  (redisplay-frame-pane  *application-frame* 'results :force-p t))

(define-command (com-rename-result :command-table debug :name t :menu "Rename Result")
    ((result 'result :prompt "Result")
     (string 'string :prompt "New name"))
  (setf (name result) (intern string))
  (terpri)
  (redisplay-frame-pane  *application-frame* 'results :force-p t))

(define-command (com-refresh-result-pane :command-table debug :name t :menu "Refresh Results")
    ()
  (redisplay-frame-pane  *application-frame* 'results :force-p t))

(define-command (com-use-native-debugger :command-table debug :name t :menu "Use Native Debugger")
    ((boolean 'boolean
	      :prompt "yes or no"
	      :default (not *use-native-debugger*)))
  (setq *use-native-debugger* boolean))

;;; ---------------------------------------------------------------------------

;; override the default behavior in CLIM-2 which is to use the first
;; application frame that it finds
(defmethod frame-standard-output ((the-frame lisp-listener))
  (get-frame-pane the-frame 'interactor))

(defmethod frame-standard-input ((the-frame lisp-listener))
  (get-frame-pane the-frame 'interactor))

;;; --------------------------------------------------------------------------

(defun listener ()
  (let ((frame (make-application-frame 'lisp-listener  :frame-manager (find-frame-manager :port (find-port)))))
    (run-frame-top-level frame)))

;;; ---------------------------------------------------------------------------
;;; All the rest of this code is straight out of the listener demo.

(defmethod frame-maintain-presentation-histories ((frame lisp-listener)) t)

(defmacro condition-restart-loop ((conditions description . args) &body body)
  #---ignore (declare (ignore conditions))
  (let ((tag (clim-utils:gensymbol 'restart)))
    `(tagbody ,tag
       (restart-case
	   (progn ,@body)
	 (nil ()
	   #|| :test (lambda (condition)
		       (some #'(lambda (x) (typep condition x)) ',conditions)) ||#
	   :report (lambda (stream)
		     (format stream ,description ,@args))))
       (go ,tag))))

(defvar *listener-depth* -1)

#+allegro
(progn
  ;; The stack-group object corresponding to the stack we are debugging.
  (defvar *tpl-current-stack-group* nil)
  ;; The newest (possibly invisible) frame at the current break level.
  (defvar *top-top-frame-pointer* nil)
  ;; The newest (possibly invisible) frame at the current break level.
  (defvar *top-frame-pointer* nil)
  ;; Current stack frame pointer.
  (defvar *current-frame-pointer* nil))

(defvar *lisp-listener-frame*)
(defvar *lisp-listener-io*)

(defvar *use-native-debugger* nil)

(defvar *prompt-arrow-1* 
	(make-pattern #2A((0 0 0 0 0 0 0 0 0 0 0 0)
			  (0 0 0 0 0 1 0 0 0 0 0 0)
			  (0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 1 1 1 1 1 1 1 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 0 0 0)
			  (0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 0 0 0 0 0 0 0 1 1 1 0)
			  (0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 1 1 1 1 1 1 1 1 0 0 0)
			  (0 1 1 1 1 1 1 1 0 0 0 0)
			  (0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 0 0 0 0 1 0 0 0 0 0 0))
		      (list +background-ink+ +foreground-ink+)))

(defvar *prompt-arrow-2* 
	(make-pattern #2A((0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0)
			  (0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 0)
			  (0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0))
		      (list +background-ink+ +foreground-ink+)))

(defun lisp-listener-top-level (frame)
  "Run a simple Lisp listener using the window provided."
  (enable-frame frame)
  (let* ((*lisp-listener-frame* frame)
	 (window (frame-standard-input frame))
	 (command-table (frame-command-table frame))
	 (presentation-type `((command-or-form :command-table ,command-table)
			      :auto-activate #+(or Genera Cloe-Runtime) t
					     #-(or Genera Cloe-Runtime) nil)))
    (with-input-focus (window)
      (let* ((*lisp-listener-io* window)
	     (*standard-input* *lisp-listener-io*)
	     (*standard-output* *lisp-listener-io*)
	     #+(or Minima allegro) (*error-output* *lisp-listener-io*)
	     (*query-io* *lisp-listener-io*)
	     #+Minima (*debug-io* *lisp-listener-io*)
	     (*package* *package*)
	     (*listener-depth* (1+ *listener-depth*))
	     (*** nil) (** nil) (* nil)
	     (/// nil) (// nil) (/ nil)
	     (+++ nil) (++ nil) (+ nil)
	     (- nil)
	     #+allegro (*tpl-current-stack-group* sys::*current-stack-group*)
	     #+allegro (*top-top-frame-pointer*
			 (debug::newest-frame si::*current-stack-group* :visible-only-p nil))
	     #+allegro (*top-frame-pointer*
			 (or (db::find-interesting-frame *top-top-frame-pointer*)
			     *top-top-frame-pointer*))
	     #+allegro (*current-frame-pointer* *top-frame-pointer*))
	(terpri *lisp-listener-io*)
	(with-command-table-keystrokes (keystrokes command-table)
	  (condition-restart-loop (#+Genera (sys:error sys:abort)
				   #-Genera (error)
				   "Restart CLIM lisp listener")
	    (lisp-listener-command-reader
	      frame *standard-input* command-table presentation-type
	      :keystrokes keystrokes
	      :listener-depth *listener-depth*
	      :prompt (case *listener-depth*
			(0 *prompt-arrow-1*)
			(1 *prompt-arrow-2*)
			(otherwise 
			  (concatenate 'string 
			    (make-string (1+ *listener-depth*) :initial-element #\=)
			    "> "))))))))))

(defun lisp-listener-command-reader (frame stream command-table presentation-type 
				     &key keystrokes listener-depth (prompt "=> "))
  (catch-abort-gestures ("Return to ~A command level ~D"
			 (frame-pretty-name frame) listener-depth)
    ;; Eat any abort gestures that might be hanging around.
    ;; We need to do this because COMMAND-OR-FORM is wierd.
    (let* ((abort-gestures *abort-gestures*)
	   (*abort-gestures* nil))
      ;;--- What test does this need?
      (when (member (stream-read-gesture stream :timeout 0 :peek-p t) abort-gestures)
	(stream-read-gesture stream :timeout 0)))
    (fresh-line stream)
    (if (stringp prompt)
	(write-string prompt stream)
	(multiple-value-bind (x y) (stream-cursor-position stream)
	  (draw-pattern* stream prompt x y)
	  (stream-increment-cursor-position stream (pattern-width prompt) nil)))
    (multiple-value-bind (command-or-form type numeric-arg)
	(block keystroke
	  (handler-bind ((accelerator-gesture
			   #'(lambda (c)
			       ;; The COMMAND-OR-FORM type is peeking for the
			       ;; first character, looking for a ":", so we
			       ;; have to manually discard the accelerator
			       (stream-read-gesture stream :timeout 0)
			       (return-from keystroke
				 (values
				   (accelerator-gesture-event c)
				   :keystroke
				   (accelerator-gesture-numeric-argument c)))))
			 (simple-parse-error
			   #'(lambda (c)
			       (let ((args (clim-internals::parse-error-format-arguments c)))
				 (when (and (= (length args) 1)
					    (equal (first args) ""))
				   ;; Hmm, user must have just hit <Return>
				   (return-from lisp-listener-command-reader)))
			       ;; Otherwise decline to handle this
			       nil)))
	    (let ((*accelerator-gestures* keystrokes))
	      (accept presentation-type
		      :stream stream
		      :prompt nil :prompt-mode :raw
		      :additional-activation-gestures '(#+Genera #\End)))))
      (when (eq type :keystroke)
	(let ((command (lookup-keystroke-command-item command-or-form command-table 
						      :numeric-argument numeric-arg)))
	  (unless (clim-internals::keyboard-event-p command)
	    (when (partial-command-p command)
	      (setq command (funcall *partial-command-parser*
				     command command-table stream nil
				     :for-accelerator t)))
	    (setq command-or-form command
		  type 'command))))
      (cond ((eq type ':keystroke)
	     (beep))
	    ((eq (presentation-type-name type) 'command)
	     (terpri)
	     (let ((*debugger-hook* 
		     (unless *use-native-debugger*
		       (and (zerop listener-depth) #'listener-debugger-hook))))
	       (apply (command-name command-or-form)
		      (command-arguments command-or-form)))
	     (terpri))
	    (t
	     (terpri)
	     (let ((values 
		     (multiple-value-list
		       (let ((*debugger-hook* 
			       (unless *use-native-debugger*
				 (and (zerop listener-depth) #'listener-debugger-hook))))
			 (eval command-or-form)))))
	       (fresh-line)
	       (dolist (value values)
		 (present value 'expression :single-box :highlighting)
		 (terpri))
	       (setq - command-or-form)
	       (shiftf +++ ++ + -)
	       (when values
		 ;; Don't change this stuff if no returned values
		 (shiftf /// // / values)
		 (shiftf *** ** * (first values)))))))))

(defvar *debugger-condition* nil)
(defvar *debugger-restarts* nil)

(defun listener-debugger-hook (condition hook)
  (declare (ignore hook))
  (let* ((*application-frame* *lisp-listener-frame*)
	 #+Minima (*debug-io* (frame-query-io *application-frame*))
	 (*error-output* (frame-standard-output *application-frame*))
	 (*debugger-condition* condition)
	 (*debugger-restarts* (compute-restarts)))
    (describe-error *error-output*)
    (with-output-recording-options (*lisp-listener-io* :draw t :record t)
      (lisp-listener-top-level *application-frame*))))

(defvar *enter-debugger* '#:enter-debugger)
(defun enter-debugger (stream)
  #-Genera stream
  #+Genera
  (clim-internals::with-debug-io-selected (stream)
    (cl:break "Debugger break for ~A" (frame-pretty-name (pane-frame stream)))))

(define-presentation-type restart-name ())

(define-presentation-method presentation-typep (object (type restart-name))
  (or (eql object *enter-debugger*)
      (typep object 'restart)))

(define-presentation-method present (object (type restart-name) stream (view textual-view)
				     &key)
  (if (eql object *enter-debugger*)
      (princ "Enter the debugger" stream)
      (prin1 (restart-name object) stream)))

(define-lisp-listener-command (com-invoke-restart :name t)
    ((restart 'restart-name :gesture :select))
  (if (eql restart *enter-debugger*)
      (enter-debugger *standard-input*)
    (let (values)
      #+Allegro
      (case (restart-name (find-restart restart))
	(excl::return-value
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "Enter value to return")))))
	(excl::try-a-different-function
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "enter expression which will evaluate to the function to call")))))
	(excl::try-a-different-function-setf
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "enter expression which will evaluate to the function to call"))))))
      (apply #'invoke-restart restart values))))

(define-lisp-listener-command (com-describe-error :name t) ()
  (describe-error *error-output*))

(defun describe-error (stream)
  (when *debugger-condition*
    (with-output-as-presentation (stream *debugger-condition* 'form
				  :single-box t)
      (format stream "~2&Error: ~A" *debugger-condition*))
    (let ((restarts *debugger-restarts*))
      (when restarts
	(let ((actions '(invoke-restart)))
	  (dolist (restart (reverse restarts))
	    (let ((action (member (restart-name restart)
				  '(abort continue muffle-warning store-value use-value))))
	      (when action
		(pushnew (first action) actions))))
	  (format stream "~&Use~?to resume~:[~; or abort~] execution:"
	    "~#[~; ~S~; ~S or ~S~:;~@{~#[~; or~] ~S~^,~}~] "
	    actions (member 'abort actions)))
	(fresh-line stream)
	(formatting-table (stream :x-spacing '(2 :character))
	  (dolist (restart restarts)
	    (with-output-as-presentation (stream restart 'restart-name
					  :single-box t :allow-sensitive-inferiors nil)
	      (formatting-row (stream)
		(formatting-cell (stream) stream)
		(formatting-cell (stream)
		  (when (restart-name restart)
		    (format stream "~S:" (restart-name restart))))
		(formatting-cell (stream)
		  (format stream "~A" restart)))))
	  #+Genera
	  (with-output-as-presentation (stream *enter-debugger* 'restart-name
					:single-box t :allow-sensitive-inferiors nil)
	    (formatting-row (stream)
	      (formatting-cell (stream) stream)
	      (formatting-cell (stream) stream)
	      (formatting-cell (stream)
		(write-string "Enter the debugger" stream)))))))
    (let ((process (clim-sys:current-process)))
      (when process
	(format stream "~&In process ~A." process))))
  (force-output stream))


;;; Lisp-y stuff

(defun quotify-object-if-necessary (object)
  (if (or (consp object)
	  (and (symbolp object)
	       (not (keywordp object))
	       (not (eq object nil))
	       (not (eq object t))))
      (list 'quote object)
    object))

(define-presentation-translator describe-lisp-object
    (expression form lisp-listener
     :documentation
       ((object stream)
	(let ((*print-length* 3)
	      (*print-level* 3)
	      (*print-pretty* nil))
	  (present `(describe ,(quotify-object-if-necessary object)) 'expression
		   :stream stream :view +pointer-documentation-view+)))
     :gesture :describe)
    (object)
  `(describe ,(quotify-object-if-necessary object)))

(define-presentation-translator expression-identity
    (expression nil lisp-listener
     :tester
       ((object context-type)
	(if (and (eq (presentation-type-name context-type) 'sequence)
		 (or (listp object)
		     (vectorp object)))
	    (clim-utils:with-stack-list
	      (type 'sequence (reasonable-presentation-type (elt object 0)))
	      (clim-internals::presentation-subtypep-1 type context-type))
	    (clim-internals::presentation-subtypep-1
	      (reasonable-presentation-type object) context-type)))
     :tester-definitive t
     :documentation ((object stream)
		     (let ((*print-length* 3)
			   (*print-level* 3)
			   (*print-pretty* nil))
		       (present object 'expression 
				:stream stream :view +pointer-documentation-view+)))
     :gesture :select)
    (object)
  object)

(defun reasonable-presentation-type (object)
  (let* ((class (class-of object))
	 (class-name (class-name class)))
    (when (presentation-type-specifier-p class-name)
      ;; Don't compute precedence list if we don't need it
      (return-from reasonable-presentation-type class-name))
    (dolist (class (class-precedence-list class))
      (when (presentation-type-specifier-p (class-name class))
	(return-from reasonable-presentation-type (class-name class))))
    nil))

(define-gesture-name :describe-presentation :pointer-button (:middle :super))

(define-presentation-translator describe-presentation
    (t form lisp-listener
     :documentation
       ((object presentation stream)
	(declare (ignore object))
	(let ((*print-length* 3)
	      (*print-level* 3)
	      (*print-pretty* nil))
	  (present `(describe ,(quotify-object-if-necessary presentation)) 'expression
		   :stream stream :view +pointer-documentation-view+)))
     :gesture :describe-presentation)
    (object presentation)
  (declare (ignore object))
  `(describe ,(quotify-object-if-necessary presentation)))

;;; ***************************************************************************
;;; EOF



