;;; -*- Syntax: Common-lisp; Package: DWIM -*-
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :dwim)

;; From TV.LISP
;; 1-20-94 Rewrote the code called when `master' is non-nil for
;; CLIM 2.0.  (Westy)

(defmethod start-frame (frame &key
			      (wait-until-done t)
			      master
			      (backing-store *enable-backing-store*))
  #+CLIM-0.9
  (progn
      ;; If these are X windows, enable backing-store.
      #+xlib
      (let* ((pane (clim:frame-pane frame))
	     (port (clim:port pane)))
	(when (typep* port 'on-x::x-port)
	  (setf (xlib:window-backing-store (w::sheet-mirror! pane))
	    backing-store)))    
      (cond (master
	     ;; Change the event queue and reinitialize.
	     ;; How should this be undone if this frame is recycled?
	     (setf (slot-value frame 'ws::queue) (ws::frame-queue master))
	     (ci::initialize-stream-queues frame)
	     (clim:enable-frame frame)
	     (clim:panes-need-redisplay frame)
	     (clim:redisplay-frame-panes frame))
	    (T (clim:start-frame frame wait-until-done))))
    
    #+CLIM-1.0
    (labels ((set-backing-store (window value)
	       #+xlib
	       (setf (xlib:window-backing-store (slot-value window 'clim::window))
		 value)
	       (dolist (child (slot-value window 'clim::children))
		 (set-backing-store child value))))
      (cond (master
	     (let ((b (clim:stream-input-buffer
		       (clim:frame-top-level-window master)))
		   (top-level-window (clim:frame-top-level-window frame)))
	       (labels ((set-input-buffer (window buffer)
			  (setf (clim:stream-input-buffer window) buffer)
			  (dolist (w (clim:window-children window))
			    (set-input-buffer w buffer))))
		 (set-input-buffer top-level-window b)
		 (set-backing-store (clim:frame-top-level-window frame)
				    backing-store)
		 (clim:window-expose top-level-window)
		 (clim:redisplay-frame-panes frame :force-p t)
		 ;; return the window just created
		 (values top-level-window frame))))
	    ((not wait-until-done)
	     (process-run-function
	      "Frame Top Level"
	      'start-frame frame
	      :wait-until-done t
	      :master nil
	      :backing-store backing-store)
	     frame)
	    (T
	     (push (list frame (current-process)) *activated-frames*)
	     (setq *deactivated-frames* (delete frame *deactivated-frames*))
	     #+xlib
	     (let ((xwin (slot-value (clim:frame-top-level-window frame) 'clim::window)))
	       (xlib:set-wm-class xwin "clim" "clim"))
	     (set-backing-store (clim:frame-top-level-window frame)
				backing-store)
	     (unwind-protect
		 (let ((clim::*frame-layout-changing-p* t)) ; forces redisplay
		   (clim:run-frame-top-level frame))
	       (deactivate-frame frame)))))
    #+(or CLIM-2.0 CLIM-2)
    (cond (master  ;; 1-20-94 Rewrote this section. (Westy)
	   (let ((b (clim:sheet-event-queue 
		     (clim:frame-top-level-sheet master)))
		 (top-level-window (clim:frame-top-level-sheet frame)))
	     (clim:map-over-sheets #'(lambda (sheet)
				       (setf (clim:sheet-event-queue sheet) b))
				   top-level-window)
	     (clim:enable-frame frame)
	     (clim:redisplay-frame-panes frame :force-p t)
	     (values frame)))
	  ((not wait-until-done)
	   (process-run-function
	    "Frame Top Level"
	    'start-frame frame
	    :wait-until-done t
	    :master nil
	    :backing-store backing-store)
	   frame)
	  (T
	   (push (list frame (current-process)) *activated-frames*)
	   (unwind-protect
	       (progn
		 (clim:run-frame-top-level frame))
	     (deactivate-frame frame))))
    #-CLIM
    (cond (master (error "MASTER operation not supported."))
	  ((not wait-until-done)
	   (scl:send frame :mouse-select))
	  (T
	   (setf (scl:symbol-value-in-instance frame 'tv:process) nil)
	   (let* ((superior (scl:send frame :superior))
		  (shadow (if (typep* superior 'tv:basic-screen)
			      (first (scl:send superior :exposed-inferiors))))
		  (dw:*program-frame* frame)
		  (dw:*program* (scl:send frame :program)))
	     (tv:with-window-shadowed-for-selection (shadow frame :reselect t)
	       (tv:window-call (frame :deactivate)
			       (unwind-protect
				   (dw::run-program-top-level dw:*program*)
				 (deactivate-frame frame))))))))


;; From EXTENSIONS.LISP
;; 1-20-94 `clim-sys:make-process' was being called with incorrect args.
;; Fixed it. (Westy)

(defun process-run-function (name-or-keywords function &rest args)
  (let* ((new-args (copy-list args)) ; in case of stack-allocation
	 (predicate
	  (if args #'(lambda () (apply function new-args)) function)))
    #FEATURE-CASE
    ((:ALLEGRO
      (funcall #'mp:process-run-function name-or-keywords predicate))
     (:GENERA
      (funcall #'scl:process-run-function name-or-keywords predicate))
     (:LUCID
      (flet ((lucid-procees-run-function-hack (NAME-OR-KEYWORDS
					       &rest FNCT-LIST)
	       (let ((FNCT-NAME (first FNCT-LIST))
		     (FNCT-ARGS (copy-list (cdr FNCT-LIST))))
		 (if (consp NAME-OR-KEYWORDS)
		     (apply #'lcl::make-process
			    :function FNCT-NAME
			    :args FNCT-ARGS
			    NAME-OR-KEYWORDS)
		   (lcl::make-process
		    :name NAME-OR-KEYWORDS
		    :function FNCT-NAME
		    :args FNCT-ARGS)))))
	(apply #'lucid-procees-run-function-hack
	       name-or-keywords function args)))
     (:MCL
      ;; No multiprocessing.  Fake it.
      (funcall predicate))
     (:CLIM-1.0
      (clim-utils::make-process predicate :name name-or-keywords))
     (:CLIM-2
      ; 1-20-94 Added :NAME here. (Westy)
      (CLIM-SYS:make-process predicate :name name-or-keywords)))))


(defparameter *use-color-for-postscript* t)

(defun color-stream-p (stream)
  "Is STREAM capable of rendering color?"
  #FEATURE-CASE
  (((not :clim)
    (and (color:color-exists-p)
	 ;; error, for example if stream = cold-load-stream
	 (ignore-errors (color:color-stream-p stream))))
   (:clim-2
    (and (clim:extended-output-stream-p stream)
	 (if (and (type-specifier-p 'postscript-clim::postscript-stream)
		  (typep stream 'postscript-clim::postscript-stream))
	     *use-color-for-postscript*
	   (or #+ignore			; disallowed
	       (clim:palette-color-p 
		(clim:medium-palette (clim:sheet-medium stream)))
	       #+ignore			; currently broken & undocumented
	       (clim:color-stream-p stream)
	       #-ignore			; "official" answer.
	       (clim:palette-color-p
		(let ((frame (clim:pane-frame stream)))
		  (if frame
		      (clim:frame-palette frame)
		    (clim:port-default-palette
		     (clim:port (clim:sheet-medium stream))))))))))
   (:clim-0.9
    (if (typep stream 'ci::encapsulating-stream-mixin)
	(color-stream-p (slot-value stream 'ci::stream))
      (let ((port (clim:port stream)))
	(cond #+Genera
	      ((typep port 'on-genera::genera-port)
	       (dolist (inf (tv:sheet-inferiors (on-genera::genera-screen port)))
		 (ignore-errors
		  (return (color:color-stream-p inf)))))
	      #+XLIB
	      ((typep port 'on-x::x-port)
	       (> (slot-value port 'on-x::depth) 1))
	      (t nil)))))
   ((and :clim-1.0 :clx)
    (if (typep stream 'clim::clx-window)
	(slot-value stream 'clim::color-p)
      t))
   ((and :clim-1.0 (not mcl))
    (if (typep stream 'clim::sheet-window-stream)
	(slot-value stream 'clim::color-p)
      t))
   ((and :clim-1.0 mcl)
    t)))
