;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-
#|
To start up the frame:
   - Evaluate (CLASP-INTERFACE::clasp).

|#


(in-package :CLASP-INTERFACE)

;;;;----------------------------------------------------------------
;;;; Frame definition
;;;;----------------------------------------------------------------

(define-gesture-name :multiple-select :button :left :shifts (:shift))
(define-gesture-name :disjoint-select :button :left :shifts (:shift :control))
(define-gesture-name :deselect        :button :left :shifts (:control))

;;;;----------------------------------------------------------------

(defvar *clasp-frame* nil)
(defvar *clasp-frame-main-window* nil)

(clim::define-application-frame clasp-frame ()
    ((inferiors :initform nil
		:accessor inferiors))
  (:command-table (clasp
		    :inherit-from
		    (user-command-table
		     clasp-file 
                     clasp-description clasp-transform clasp-tests clasp-data-manipulation 
                     clasp-graphing
		     clasp-sample
		     #+ignore tests
		     user-command-table accept-values-pane icon-command-table :graph)))
  
  (:command-definer t)
  (:panes ((display :application
                    :incremental-redisplay t
                    :scroll-bars :both
                    :end-of-line-action :allow)
	   (mouse-doc :pointer-documentation)
	   (menu :command-menu)
	   (interactor :interactor)
	   (datasets :application
                      :default-text-style (clim::make-text-style :sans-serif :roman :normal)
                      :incremental-redisplay t
                      :scroll-bars :both
                      :end-of-line-action :allow
                      :end-of-page-action :allow
                      :display-function 'draw-datasets-pane) 
           ))
  (:layout ((standard
	     (:column 1
		      (:row :rest  
			    (:column 1/5 
				     (menu :compute)
				     (datasets :rest))
			    (display :rest))
		      
		      (interactor 1/5)
		      (mouse-doc :compute)))
	    (simple
	     (:column 1
	       (display :rest)
	       (menu :compute)
	       (interactor 1/5)
	       (mouse-doc :compute))))))

;; This fixes the scroll bar not workig bug in MCL.
#+MCL
(defun set-scroll-max (stream)
  (ccl:set-scroll-bar-max (slot-value stream 'ccl::v-scroller) 
                          (round (* 2.5 (window-inside-height stream))))
  (ccl:set-scroll-bar-max (slot-value stream 'ccl::h-scroller) 
                          (round (* 2.5 (window-inside-width stream))))
  )

(defun stream-of (pane-name)
  (get-frame-pane clim::*application-frame* pane-name))

(defmethod display-or-inferior-pane-p ((frame clasp-frame) window)
  (or (eq window (get-frame-pane frame 'display))
      ;; Check for killed windows at some point.
      (member window (inferiors frame) :test #'eq)))

(defmacro with-inferior-stream ((stream frame &rest options) &body body)
  `(let ((,stream (make-inferior-window ,frame . ,options)))
     . ,body))

;;; ----------------------------------------------------------------------------

(defmethod make-inferior-window ((frame clasp-frame) &rest options)
  (let ((pane (apply #'open-window-stream :parent (clasp-root)
		     :width 450 :height 400
		     :input-buffer (stream-input-buffer 
				    (get-frame-pane frame 'interactor))
               options)))
    (push pane (inferiors frame))	; remember it
    (window-expose pane)		; see it
    (values pane)
    ))

;;; ----------------------------------------------------------------------------

;; Need to augment this so that different result types 
;; use different initargs for the window. Also add autosizing?
(defmethod open-it ((result result) &key refresh label)
  (let ((window (display-window result)))
    (multiple-value-bind (visible error)
	(and window
	     (ignore-errors (window-visibility window)))
      (declare (ignore visible))
      (cond ((or error (null window))
             (with-inferior-stream (stream *application-frame* :label (or label (clasp::name-string result)))
               (setf (display-window result) stream)
               (display result stream)))
            (t
             ;(unless visible ; always expose
               (window-expose window);)
             (when refresh (display result window)))))))

(defmethod place ((result result) &optional (stream *standard-output*))
  (fresh-line stream)
  (write-string "        " stream)
  (multiple-value-bind (x y)
                       (stream-cursor-position* stream)
    (place-in-window result stream x y))
  (terpri stream))

;;; ----------------------------------------------------------------------------

(setf clasp::*dataset-class* 'dataset)

(defmethod print-object ((dataset dataset) stream)
  (print-unreadable-object (dataset stream)
    (princ (clasp::name dataset) stream)))

;(define-presentation-type dataset ())

(defmethod display ((dataset dataset) stream)
  (fresh-line stream)
  (formatting-table (stream)
    (loop for variable in (slot-value dataset 'clasp::variables) do
      ;; Column-oriented.
	(formatting-column (stream)
	   (with-output-as-presentation (:object variable
					 :type 'column
					 :stream stream)
	     (formatting-cell (stream :align-x :center)
			      (princ (clasp::name variable) stream)))
	   ;; Replace with `variable-values'.
	   (loop for value in (clasp::variable-value variable) do
		 (formatting-cell (stream)
				  (princ value stream)))))))

;;; ----------------------------------------------------------------------------

;; We call variables columns because there is a 
;; symbol `variable' in the COMMON-LISP package (at least in allegro).
(setf clasp::*variable-class* 'column)

(defmethod print-object ((column column) stream)
  (print-unreadable-object (column stream)
    (princ (clasp::name column) stream)))

;(define-presentation-type column ())

(defmethod display ((variable column) stream)
  (fresh-line stream)
  (formatting-table (stream)
    (formatting-column (stream)
      (with-output-as-presentation (:object variable
					    :type 'column
					    :stream stream)
	     (formatting-cell (stream :align-x :center)
               (princ (clasp::name variable) stream)))
      (loop for value in (clasp::variable-value variable) do
            (formatting-cell (stream)
              (princ value stream))))))

;;; ----------------------------------------------------------------------------

;(setf graph-user::*dataset-class* 'graph-data)

(defun make-graph-data-icon (&rest options)
  (apply #'make-instance 'graph-data-icon options))

(defmethod open-it ((the-data-icon graph-data-icon) &key)
  (with-slots (graph) the-data-icon
      (unless graph
	(setf graph (make-graph :x-label (x-label the-data-icon) 
				:y-label (y-label the-data-icon)
				:title (clasp::name-string the-data-icon))))
    (add-dataset graph (graph-data the-data-icon))
    (open-it graph :label (clasp::name-string the-data-icon))))

;;; ----------------------------------------------------------------------------

(setf graph-user::*graph-class* 'graph)

(defmethod make-graph (&rest options)
  (apply #'make-instance graph-user::*graph-class* options))

(defmethod open-it ((graph graph) &key refresh label)
  (let ((window (display-window graph)))
    (multiple-value-bind (visible error)
	(and window
	     (ignore-errors (window-visibility window)))
      (declare (ignore visible))
      (cond ((or error (null window))
	     ;; I had to hack `graph::view-graphs' to return a window here.
	     (setf (display-window graph) (graph::view-graphs (list graph) 
							:master *application-frame* 
							:title (or label (clasp::name-string graph)))))
	     (t
	      (window-expose window)
	      (when refresh (window-refresh (display-window graph))))))))


#+obsolete
(defmethod display ((the-graph graph) stream)
  (graph-user::display-graph 
   the-graph
   :width 440 :height 390
   :scroll-if-necessary t
   :stream stream))

(defvar *colors* '(:gold :green :magenta :cyan :red))

(defmethod add-dataset ((graph graph) (graph-data graph:graph-data))
  ;; Pick another color if the color has already been used in this graph.
  (unless (member graph-data (graph::datasets graph) :test #'eq)
    (let* ((used-colors (mapcar #'color (graph::datasets graph)))
	   (next-color (first (set-difference *colors* used-colors :test #'eq))))
      (when (member (color graph-data) used-colors)
	(setf (color graph-data) next-color))))
  (graph::add-dataset graph graph-data))
    
(defmethod execute-on-target ((graph graph) (the-data-icon graph-data-icon))
  (let* ((new-title (concatenate 'string (graph::title graph) " and " (clasp::name-string the-data-icon)))
	 (new-graph (make-graph :x-label (x-label the-data-icon) 
				:y-label (y-label the-data-icon)
				:title new-title))
	(old-datasets (graph::datasets graph)))
    (dolist (dataset old-datasets)
      (graph::add-dataset new-graph dataset))
    (add-dataset new-graph (graph-data the-data-icon))
    (open-it new-graph :refresh t :label new-title)))

;;; ----------------------------------------------------------------------------

(defun redisplayable-format (stream string &rest args)
  (updating-output (stream
		    :unique-id string
		    :cache-value args
		    :cache-test #'equal
		    :copy-cache-value t)
     (apply #'format stream string args)))

(define-presentation-method present 
    (window-point (type window-point) stream (view clim:textual-view) &key)
  (redisplayable-format stream "~A (~D.~D)" 
			(window window-point)
			(point-x window-point)
			(point-y window-point)))

(defun make-window-point (window x y)
  (make-instance 'window-point :window window :x x :y y))

(define-presentation-translator screen-location 
    (blank-area 
     window-point
     clasp
     :documentation "Click to pick a location"
     :tester ((window)
	      (display-or-inferior-pane-p *application-frame* window)))
  
  (window x y)
  (values (make-window-point window x y)))

;;; ----------------------------------------------------------------------------

(defmethod draw-datasets-pane ((application clasp-frame) stream)
  (dolist (dataset (clasp::find-instances 'dataset))
    (updating-output (stream :unique-id (clasp::name dataset)
                             :cache-value (clasp::dataset-variables dataset))
      (clim:with-text-face (:bold)
	(present dataset 'dataset :stream stream))
      (terpri stream)
      (dolist (variable (clasp::dataset-variables dataset))
        (write-char #\space stream)
	(write-char #\space stream)
	(updating-output (stream :unique-id (clasp::name variable))
	   (present variable 'column :stream stream))
        (terpri stream))))
  #+MCL
  (set-scroll-max stream))
  
;;; ----------------------------------------------------------------------------

(defvar *command-counter* 0)

(defmethod execute-frame-command :before ((frame clasp-frame) command)
  (fresh-line)
  (format *standard-output* "~s> " (incf *command-counter*))
  (present command 'command)
  (terpri)
  (format *standard-output* "~s< " *command-counter*)
  )

(defmethod execute-frame-command :after ((frame clasp-frame) command)
  (declare (ignore command))
  #+MCL
  (set-scroll-max (get-frame-pane frame 'display)))


;;; ----------------------------------------------------------------------------
;;; Starting up the frame

(defun clasp ()
  (run-frame-top-level
    (make-application-frame
      'clasp-frame
      :parent (clasp-root))))

(defvar *clasp-root* nil)

(defun clasp-root (&optional reinit)
  (or (and (not reinit) *clasp-root*)
      (setf *clasp-root* (open-root))))
  
(defun open-root ()
  ;;thanks to Oliver Christ <oli@adler.ims.uni-stuttgart.de> for the code below
  #+Lucid (open-root-window
	   :clx
	   :host (lcl:environment-variable "DISPLAY"))
  #+Allegro-v4.1 (open-root-window
		  :clx
		  :host (system:getenv "DISPLAY"))
  #+MCL          (open-root-window :mcl)
  #+Genera       (open-root-window :sheet)
  ;;Please contact keunen@nrb.be if you modify this source code.
  #-(or Lucid Allegro-v4.1 MCL Genera)
  (warning "Unknown CLIM/LISP combination.  Please modify the
`open-root' function to your needs."))


;;;;----------------------------------------------------------------
;;;; For easy access

(defmethod run-frame-top-level :before
		((the-frame clasp-frame))
  (setf *clasp-frame* the-frame)
  (setf *clasp-frame-main-window*
	(get-frame-pane *clasp-frame* 'display)))

;;; ----------------------------------------------------------------------------
  
#|

(defclass graph-icon (result)
	  ((graph :initarg :graph :reader graph))
  (:default-initargs
    :bitmap graph-pattern
    :command-form #'print-args
    :command-documentation "Graph"
    :target-form #'print-args))

(defun make-graph-icon (&key graph x-label y-label title)
  (make-instance 'graph-icon
    :graph (or graph
               (make-graph
                 :title (or title (format nil "Graph Number ~D" (incf graph-user::graph-number)))
                 :x-label (or x-label "Fortnights")
                 :y-label (or y-label "Furlongs")
                 :present-self-p nil))))

(defmethod print-object ((the-graph-icon graph-icon) stream)
  (print-unreadable-object (the-graph-icon stream)
    (princ (clasp::name the-graph-icon) stream)))

;(define-presentation-type graph-icon ())

(defmethod display ((the-graph-icon graph-icon) stream)
  (graph-user::display-graph 
   (graph the-graph-icon)
   :width 440 :height 390
   :scroll-if-necessary t
   :stream stream))

(defmethod execute-on-target (icon-1 icon-2)
  nil)

(defmethod execute-on-target ((the-graph-icon graph-icon) (the-data-icon graph-data-icon))
  (add-dataset (graph the-graph-icon) (graph-data the-data-icon))
  (open-it the-graph-icon :refresh t))

|#
;;; ----------------------------------------------------------------------------




