;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :CLASP-INTERFACE)

;; ----------------------------------------------------------------------------
;;; The FILE menu

(define-command-table clasp-file)

(add-menu-item-to-command-table 'clasp "File" :menu 'clasp-file :errorp nil
				:documentation "I/O and customization commands"
				:after :start)

#+ignore
(define-command (com-help-me :command-table clasp-file :name "Help"
			     :menu  "Help") ()) 

(define-presentation-type clasp-pathname () :inherit-from 'pathname :options (new-name new-type label))

#+mcl
(define-presentation-method accept ((type clasp-pathname) (stream t) (view t) &key)
  (let ((pathname (cond (new-name
                         (ccl::choose-new-file-dialog 
                          :directory (make-pathname :defaults (ccl::choose-file-default-directory) 
                                                    :name new-name
                                                    :type (or new-type "clasp"))
                          :prompt "Save as..."
                          :button-string (or label "Save")))
                        (t
                         (ccl::choose-file-dialog :button-string (or label "Open"))))))
    (replace-input stream (namestring pathname))
    (values pathname)))

#+(and clim-2 (not mcl))
(define-presentation-method accept ((type clasp-pathname) (stream t) (view t) &key)
  (if (stream-rescanning-p stream)
      (accept 'pathname :stream stream :view view)
    (let ((pathname (select-file (clasp-frame))))
      (presentation-replace-input stream pathname 'pathname view)
      (values pathname 'clasp-pathname))))

(define-command (com-load-dataset
		 :command-table clasp-file :name t
		 :menu ("Load Dataset" 
			:documentation
			"Load a dataset from a CLASP format file"))
    ((pathname #-MCL 'pathname
               #+MCL '((clasp-pathname) :label "Load")
	       :prompt #+clim-1 "Pathname" #+clim-2 nil))
  (let ((ds (load-dataset pathname :defaults nil :report-stream *standard-output*)))
    (setf (save-pathname ds) pathname)
    (setf (modified-p ds) nil)
    (present-with-update ds :stream *standard-output*
                         :view #+clim-1 clim:+dialog-view+
                               #+clim-2 clim:+textual-dialog-view+)))

(define-command (com-import-dataset
		 :command-table clasp-file :name t
		 :menu ("Import Dataset"
			:documentation
			"Load a dataset from an ascii file"))
    ((pathname #-MCL 'pathname
               #+MCL '((clasp-pathname) :label "Import")
               :prompt "Pathname")
     (separator 'character :prompt "Separator character")
     (include-labels-p 'boolean
		       :prompt "Include variable names [Yes or No]"))
  (let ((ds (import-dataset pathname :separator separator
			    :include-labels-p include-labels-p)))
    (setf (save-pathname ds) pathname)
    (setf (modified-p ds) nil)
    (present-with-update ds :stream *standard-output*
                         :view #+clim-1 clim:+dialog-view+
                               #+clim-2 clim:+textual-dialog-view+)))
	       
(define-command (com-save-dataset :command-table clasp-file :name t
				  :menu
				  ("Save Dataset"
				   :documentation
				   "Save a dataset to a CLASP format file"))
    ((dataset 'dataset :prompt "Dataset")
     (pathname #-MCL 'pathname
               #+MCL `((clasp-pathname) :new-name ,(clasp::name-string dataset)
					:new-type "clasp" :label "Save")
               :prompt "Pathname"))
  (save dataset pathname))

(define-command (com-export-dataset :command-table clasp-file :name t
				    :menu ("Export Dataset"
					   :documentation
					   "Save a dataset to an ascii file"))
    ((dataset 'dataset :prompt "Dataset")
     (pathname #-MCL 'pathname
               #+MCL `((clasp-pathname) :new-name ,(clasp::name-string dataset) :new-type "text" :label "Save")
               :prompt "Pathname")
     (separator 'character :prompt "Separator Character")
     (include-labels-p 'boolean
		       :prompt "Include variable names [Yes or No]"))
  (export-dataset dataset pathname :separator separator
		  :include-labels-p include-labels-p))

#+ignore
(define-clasp-command (com-set-default-directory :command-table clasp-file
						 :name t
						 :menu "Set Default Directory")
    ((directory 'string :prompt "Directory")
     (type 'string :prompt "Extension"))
  (setf clasp::*data-file-defaults* (make-pathname :host "CLASP-DATA"
						   :directory directory
						   :type type)))

(define-command (com-kill-everything :command-table clasp-file
				     :name "Clear All"
				     :menu ("Clear All"
					    :documentation
					    "Clear All Data"))
    ()
  #+ignore
  (dolist (result (clasp::find-instances 'result))
	  (when (and (slot-boundp result 'display-window)
		     (not (null (display-window result))))
		(setf (window-visibility (display-window result)) nil)))
  (with-clasp-frame (clasp-frame)
    (dolist (window (inferiors clasp-frame))
       (unless (or (and (window-open-p (datasets clasp-frame))
			(eq window (display-frame (datasets clasp-frame))))
		   (and (window-open-p (results clasp-frame))
			(eq window (display-frame (results clasp-frame)))))
		(close-it window)))
    (clasp::forget-instances 'dataset)
    (clasp::forget-instances 'result t)
    ;; Handle the clasp variables as well.
    (setf clasp::*clasp-active-datasets* nil)
    (setf clasp::*clasp-datasets* nil)
    (clrhash clasp::*clasp-dataset-count*)
    (clrhash clasp::*clasp-variable-count*)
    (setf (slot-value clasp-frame 'history) nil)
    (clim:redisplay-frame-panes clasp-frame :force-p t)
    (update-datasets-display)
    (update-results-display)
    (window-clear *clasp-frame-main-window*)))

(define-command (com-clear-desktop :command-table clasp-file
				   :name "Clear Notebook"
				   :menu ("Clear Notebook"
					  :documentation
					  "Clear the interactor pane"))
    ()
  (with-clasp-frame (clasp-frame)
    (setf (slot-value clasp-frame 'history) nil))
  (window-clear *clasp-frame-main-window*))

(defmethod hardcopy ((the-result result) filename)
  (with-open-file (file-stream filename :direction :output
		   :if-exists :overwrite
		   :if-does-not-exist :create)
    (clim:with-output-to-postscript-stream
	(postscript-stream file-stream :multi-page t)
      (display the-result postscript-stream))))

(define-clasp-command (com-save-notebook :command-table clasp-file :name t
					 :menu
					 ("Save Notebook"
					  :documentation
					  "Save the contents of the Notebook pane to a file"))
    ((pathname #-MCL 'pathname
               #+MCL `((clasp-pathname) :new-name ,(clasp::name-string result) :new-type "notebook" :label "Save")
               :prompt "Filename"))
  (with-open-file (stream pathname :direction :output
		   :if-exists :overwrite
		   :if-does-not-exist :create)
    (dump-history *clasp-frame* :stream stream)))
  
(define-clasp-command (com-print :command-table clasp-file :name t
				 :menu
				 ("Print"
				  :documentation
				  "Print a CLASP object to a postscript file"))
    ((result 'result :prompt "Object")
     (pathname #-MCL 'pathname
               #+MCL `((clasp-pathname) :new-name ,(clasp::name-string result) :new-type "ps" :label "Save")
               :prompt "Filename for Postscript File"))
  (hardcopy result pathname))

  
#+FOR-IDIOTS
(define-clasp-command (com-easy-print :command-table clasp-file :name t
				 :menu
				 ("Print for Idiots"
				  :documentation
				  "Print a CLASP object to a postscript file"))
    ((result 'result :prompt "Object"))
  (hardcopy result "/usr/users/eksl/systems/clasp/clasp/out.ps"))

#+clim-1
(defmethod clim::maybe-set-color ((stream clim::postscript-stream) (ink clim-utils::pattern))
  nil)

#+clim-2
(defmethod postscript-clim::maybe-set-color ((stream postscript-clim::postscript-medium) (ink clim-utils::pattern))
  nil)

(define-command (com-preferences :command-table clasp-file
				 :name "Preferences"
				 :menu ("Preferences"
					:documentation
					"Customize display"))
    ()
  (with-clasp-frame (clasp-frame)
    (let* ((result-subclasses (class-direct-subclasses (find-class 'modifiable-display-option-mixin)))
	   display-options
	   (length 0)
	   (error-handling-preferences (error-handling-preferences clasp-frame))
	   ;;(datasets-and-results-display (frame-current-layout clasp-frame))
           (stream #+clim-1 (frame-top-level-window clasp-frame) #+clim-2 (frame-top-level-sheet clasp-frame)))
      (dolist (class result-subclasses)
	(setf display-options (acons class (display-option 
					    (progn 
					      #+allegro ; some hair to deal with a finicky allegro
					      (unless (class-finalized-p class)
						(clos:finalize-inheritance class))
					      (class-prototype class)))
				     display-options))
	  (setf length (max length (length (format nil "~a" (class-name class))))))
      (setf display-options (sort display-options #'string-lessp :key #'(lambda (x) (string (class-name (car x))))))
      (restart-case
	  (accepting-values
	   (stream :own-window t :label "Preferences")
	   (format stream "Result display options:")
	   (fresh-line stream)
	   (with-accept-help ((:subhelp "How should new results be displayed?"))
	    (dolist (display-option display-options)
	      (setf (cdr display-option)
		    (accept '(member :notebook :window :icon :none) :stream stream
			    :prompt (format nil "~v<~@(~a~)~>" length (class-name (car display-option)))
			    :default (cdr display-option)))
	      (fresh-line stream)))
	   (terpri stream)
           (format stream "Error handling options:")
	   (with-accept-help ((:subhelp "How do you want clasp to handle errors?"))
	    (setf error-handling-preferences
		  (accept (list 'subset
                                :use-native-debugger
                                :gaussian-cdf-signals-zero-standard-deviation-error
				#+(or lucid explorer) :trap-floating-underflow-errors)
			  :stream stream
			  :prompt "Error handling"
			  :default error-handling-preferences)))
	   (terpri stream)
           #+ignore
           (format stream "Datasets and Results list display options")
	   #+ignore
           (with-accept-help ((:subhelp "Where should the datasets and results lists be displayed?"))
	    (setf datasets-and-results-display
		  (accept '(member notebook #-MCL window none) :stream stream
			  :prompt "Display datasets and results" :default datasets-and-results-display)))
	   (terpri stream))
	  (abort () (return-from com-preferences (values))))
      (dolist (display-option display-options)
	(setf (display-option (class-prototype (car display-option))) (cdr display-option)))
      (setf (error-handling-preferences clasp-frame) error-handling-preferences)
      #+(and ignore clim-1)
      (setf clasp::*debug* (member :use-native-debugger error-handling-preferences))
      #+(or clim-1 clim-2)
      (setf *use-native-debugger* (member :use-native-debugger error-handling-preferences))
      (setf clasp::*gaussian-cdf-signals-zero-standard-deviation-error*
            (member :gaussian-cdf-signals-zero-standard-deviation-error error-handling-preferences))
      #+explorer
      (setf ticl::zunderflow (member :trap-underflow-errors error-handling-preferences))
      #+lucid
      (when (member 'user::floating-point-underflow user::supported-floating-point-conditions)
	(if (member :trap-underflow-errors error-handling-preferences)
	    (pushnew 'user::floating-point-underflow (user::enabled-floating-point-traps))
	  (setf (user::enabled-floating-point-traps)
		(remove 'user::floating-point-underflow (user::enabled-floating-point-traps))))))
    (let ((frame-layout "NOTEBOOK"))
      (case (slot-value (datasets clasp-frame) 'display-option)
	    (:notebook
	     (setf frame-layout (concatenate 'string frame-layout "+DATASETS"))
	     (close-it (datasets clasp-frame)))
	    (:window
	     (open-it (datasets clasp-frame)))
	    ((:icon nil)
	     (close-it (datasets clasp-frame))))
      (case (slot-value (results clasp-frame) 'display-option)
	    (:notebook
	     (setf frame-layout (concatenate 'string frame-layout "+RESULTS"))
	     (close-it (results clasp-frame)))
	    (:window
	     (open-it (results clasp-frame)))
	    ((:icon nil)
	     (close-it (results clasp-frame))))
      (setf (frame-current-layout clasp-frame) (intern frame-layout :clasp-interface))))
  (values))

(define-clasp-command (com-redraw-clasp :command-table clasp-file
                                        :name "Redraw Clasp"
                                        :menu "Redraw Clasp")
    ()
   (with-clasp-frame (clasp-frame)
     (redisplay-frame-pane clasp-frame 'notebook :force-p t)
     (update-datasets-display)
     (update-results-display)))

(define-command (com-toggle-listener :command-table clasp-file :name "Toggle Listener"
				:menu ("Toggle Listener"
				       :documentation
				       "Change the nature of the top-level interactor (commands only or commands and lisp"))
    ((value 'boolean :prompt "Use Complex Listener" :default (not *use-listener-top-level*)))
    
  (setf *use-listener-top-level* value)
  (throw 'change-top-level nil))

(define-command (com-quit-clasp :command-table clasp-file :name "Quit"
				:menu ("Quit"
				       :documentation
				       "Leave CLASP"))
    ()
   (with-clasp-frame (clasp-frame)
     (frame-exit clasp-frame)))

#+TEST
(define-command (com-maybe-save :command-table clasp-file :name "Save Everything"
				:menu ("Save Everything"
				       :documentation
				       "Maybe Save Everything"))
    ()
  (let ((self *application-frame*))
  (when (and
         (some #'(lambda (object)
		   (and (save-before-quit-p object)
		        (modified-p object)))
	       (clasp::find-instances (find-class 'result)))
	 (notify-user self "Some datasets have been modified" 
                      ;:exit-boxes '((:exit "Save") (:abort "Don't Save"))
		      ))
    (let (save-objects
	  (length 0)
	  (stream #+clim-1 (frame-top-level-window self) #+clim-2 (frame-top-level-sheet self)))
      (dolist (object (clasp::find-instances (find-class 'result)))
	(when (save-before-quit-p object)
	  (setf save-objects
		(cons (list object (when (modified-p object) :SAVE)
			    (if (> (length (namestring (save-pathname object)))
				   0)
				(save-pathname object)
			      (make-pathname :name (name object) :type "clasp")))
		      save-objects)
		length (max length (length (name object))))))
      (setf save-objects (sort save-objects #'string-lessp
			       :key #'(lambda (x) (name (car x)))))
      (when save-objects
	(restart-case
	    (progn
	      (accepting-values
		  (stream :own-window t :label "Exit" :exit-boxes '((:exit "Doit") (:abort "Cancel")))
		(dolist (object save-objects)
	                  ;(format stream "~v<~a~>: " length (name (car object)))
		  (write-string (name (car object)) stream)
		  (write-char #\: stream)
		  (terpri stream)
		  (write-string "     " stream)
		  (setf (second object)
			(accept '(member-alist (("Save" . :SAVE) ("Kill" . :KILL) ("Unmod" . :UNMOD) ("Nothing" . nil)))
				:stream stream :default (second object)
				:prompt "Action?"
				:query-identifier (list (car object) 'save?)))
		  (terpri stream)
		  (write-string "     " stream)
		  (setf (third object)
			(accept 'pathname :stream stream :default (third object)
				:prompt "File"
				:query-identifier (list (car object) 'pathname)))
		  (fresh-line stream)))
	      (dolist (object save-objects)
		(ecase (second object) 
		  (:SAVE
		   (save (first object) (third object))
		   (setf (save-pathname (first object)) (third object)
			 (modified-p (first object)) nil))
		  (:UNMOD
		   (setf (modified-p (first object)) nil)
		   ;; Still need to change the pathname.
		   (setf (save-pathname (first object)) (third object)))
		  (:KILL
		   (delete-result (first object)))
		  ((nil)))))
	  (abort () (return-from com-maybe-save))))))))

;;;----------------------------------------------------------------------------

(defmethod frame-exit :around ((self clasp-frame) #+lispworks &key)
   (when (and
         (some #'(lambda (object)
		   (and (save-before-quit-p object)
		        (modified-p object)))
	       (clasp::find-instances (find-class 'result)))
	 (notify-user self "Some datasets have been modified" 
                      :exit-boxes '((:exit "Save") (:abort "Don't Save"))))
    (let (save-objects
	  (length 0)
	  (stream #+clim-1 (frame-top-level-window self) #+clim-2 (frame-top-level-sheet self)))
      (dolist (object (clasp::find-instances (find-class 'result)))
	(when (save-before-quit-p object)
	  (setf save-objects
		(cons (list object (when (modified-p object) :SAVE)
			    (if (> (length (namestring (save-pathname object)))
				   0)
				(save-pathname object)
			      (make-pathname :name (name object) :type "clasp")))
		      save-objects)
		length (max length (length (name object))))))
      (setf save-objects (sort save-objects #'string-lessp
			       :key #'(lambda (x) (name (car x)))))
      (when save-objects
	(restart-case
	    (progn
	      (accepting-values
		  (stream :own-window t :label "Exit" :exit-boxes '((:exit "Doit") (:abort "Cancel")))
		(dolist (object save-objects)
	                  ;(format stream "~v<~a~>: " length (name (car object)))
		  (write-string (name (car object)) stream)
		  (write-char #\: stream)
		  (terpri stream)
		  (write-string "     " stream)
		  (setf (second object)
			(accept '(member-alist (("Save" . :SAVE) ("Kill" . :KILL) ("Unmod" . :UNMOD) ("Nothing" . nil)))
				:stream stream :default (second object)
				:prompt "Action?"
				:query-identifier (list (car object) 'save?)))
		  (terpri stream)
		  (write-string "     " stream)
		  (setf (third object)
			(accept 'pathname :stream stream :default (third object)
				:prompt "File"
				:query-identifier (list (car object) 'pathname)))
		  (fresh-line stream)))
	      (dolist (object save-objects)
		(ecase (second object) 
		  (:SAVE
		   (save (first object) (third object))
		   (setf (save-pathname (first object)) (third object)
			 (modified-p (first object)) nil))
		  (:UNMOD
		   (setf (modified-p (first object)) nil)
		   ;; Still need to change the pathname.
		   (setf (save-pathname (first object)) (third object)))
		  (:KILL
		   (delete-result (first object)))
		  ((nil)))))
	  (abort () (return-from frame-exit))))))
  (dolist (window (inferiors self))
    (close-it window))
  (call-next-method))


;;;----------------------------------------------------------------------------

#+documentation
(define-clasp-command (com-print-menus :command-table clasp :name t)
    ()
  (do-command-table-inheritance
      (command-table (find-command-table 'clasp))
    (let ((name (format nil "~A" (command-table-name command-table))))
      (when (search "CLASP-" name)
	(with-open-file (file-stream (concatenate 'string name "-menu.ps")
			 :direction :output :if-exists :overwrite
			 :if-does-not-exist :create)
	  (with-output-to-postscript-stream (stream file-stream)
	    (display-command-table-menu command-table stream
					:max-width 333 :max-height 385)))))))

#+documentation
(define-clasp-command (com-print-interactor :command-table clasp :name t)
    ()
  (with-clasp-frame (clasp-frame)
    (with-open-file (file-stream "interactor-pane.ps"
		     :direction :output :if-exists :overwrite
		     :if-does-not-exist :create)
      (with-output-to-postscript-stream (stream file-stream)
	(let ((*standard-output* stream))
	  (redisplay-frame-pane clasp-frame 'interactor :force-p t))))))

;;; ----------------------------------------------------------------------------
