;;; -*- Mode: LISP; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-
;;;
;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

(in-package :CLASP-INTERFACE)

;;; ***************************************************************************

(define-command-table icon-command-table
    :inherit-from (user-command-table))

(defmethod initialize-instance :after ((self icon) &rest ignore)
  (declare (ignore ignore))
  (setf (width self)  (pattern-width (bitmap self))
	(height self) (pattern-height (bitmap self))))

;;; present as text
(define-presentation-method present
    (icon (type icon) stream (view textual-view) &key)
#|
Date: Thu, 17 Nov 94 20:57:55 PST
From: Colin Meldrum <colin@franz.com>
To: westy@cs.umass.edu
Class: bh
Bh-Id: spr11805
Bh: append spr11805 expire
Subject: Re: [spr11805] Problems with command-or-form in ACL?
Cc: bugs@franz.com, clim@bbn.com


  Date: Thu, 17 Nov 1994 15:03:27 -0500
  From: "David L. Westbrook" <westy@lindy.cs.umass.edu>

  Allegro CL 4.2 [SPARC; R1] (10/19/94 12:34)

  The following code takes the lisp-listener frame from the demo and
  extends it with a hierarchy of menus on a menu bar and an additional
  pane that presents some arbitrary objects. I have encountered two
  problems:

  [rest deleted]
  
The problem with clicking on argyments to be inserted into the command
line occurs because of a bug in ACL with *print-readably*.

The following hack is a present method which cause an error (in
particular not a print-not-readable error) if *print-readably* is t

(define-presentation-method present (object (type named-object) stream
					    (view textual-view) &key)
  (if *print-readably*
      (error "foo")
    (write-string (name object) stream)))

Alternatively you could define both present and accept methods which
would give you better intergration with input-editor (see the flight
planner demo for an example)
|#

  (if *print-readably*
      (error "foo")
    (with-text-face (#+clim-2 stream :bold #+clim-1 stream)
      (format stream "~A" (name icon))
      )))

#+clim-2
(export '(clim::draw-icon*) :clim)

#+clim-2
(defun draw-icon* (stream pattern x y &rest args &key clipping-region transformation)
  (declare (ignore clipping-region transformation))
  (apply #'draw-pattern* stream pattern x y args))

;;; draws the icon bitmap in its window at its coords
(defmethod draw ((self icon) &key (stream (icon-window self))
				  (x (x self))
				  (y (y self))
				  (with-label t))
  (when stream
    (draw-icon* stream (bitmap self) x y)
    #+clim-1 (stream-increment-cursor-position* stream (width self) 0) 
    #+clim-2 (stream-increment-cursor-position stream (width self) 0) 
    (when with-label
      (draw-label self :stream stream :x x :y y)))
  )

;;; erases the icon output record and clears the slots
(defmethod erase ((self icon) STREAM)
  (declare (ignore STREAM))
    (erase-output-record (presentation self) (icon-window self))
    (setf (icon-window self) nil)
    (setf (presentation self) nil))

(defmethod draw-label ((self icon)
		       &key (stream (icon-window self))
			    (x (x self))
			    (y (y self))
			    (style '(:sans-serif :bold :small))
			    )
  (let* ((textx (+ x (width self) 2))
	 (texty (+ y (truncate (height self) 2)))
	 (rec (with-new-output-record (stream)
		(draw-text* stream (name self)
				 textx texty
				 :align-x :left
				 :align-y :center
				 :text-style style
				 :ink
                                 #+clim-1 +foreground+
                                 #+clim-2 +foreground-ink+))))
    (with-bounding-rectangle* (left top right bottom) 
      rec
      (declare (ignore top bottom))
      #+clim-1 (stream-increment-cursor-position* stream (- right left) 0)
      #+clim-2 (stream-increment-cursor-position stream (- right left) 0))))

;;; present in a dialog view, rather than text
(define-presentation-method present
    (icon (type icon) stream (view textual-dialog-view) &key)
  (when (not (icon-window icon))
    (multiple-value-bind (x y)
	#+clim-1 (stream-cursor-position* stream)
	#+clim-2 (stream-cursor-position stream)
      (setf (icon-window icon) stream)
      (setf (x icon) x)
      (setf (y icon) y)))
  (setf (presentation icon)
    (with-output-as-presentation
	#+clim-1 (:stream stream :object icon :type (type-of icon) :single-box t)
	#+clim-2 (stream icon (type-of icon) :single-box t)
      (draw icon :stream stream))))

(defconstant *open-result-text-style* (make-text-style nil :italic nil))
(defconstant *closed-result-text-style* (make-text-style nil nil nil))

(define-presentation-method present
    (icon (type icon) stream (view textual-datasets-view) &key)
  (when (not (icon-window icon))
    (multiple-value-bind (x y)
	#+clim-1 (stream-cursor-position* stream)
	#+clim-2 (stream-cursor-position stream)
      (setf (icon-window icon) stream)
      (setf (x icon) x)
      (setf (y icon) y)))
  (setf (presentation icon)
    (with-output-as-presentation
	#+clim-1 (:stream stream :object icon :type (type-of icon) :single-box t)
	#+clim-2 (stream icon (type-of icon) :single-box t)
        (with-text-style
         (#+clim-2 stream
          ;; YUCK! This assumes that this object includes a window mixin. We should
          ;; have a method specialized on the window mixin.
          (merge-text-styles (if (window-open-p icon)
                                 *open-result-text-style*
                               *closed-result-text-style*)
                             (medium-merged-text-style stream))
          #+clim-1 stream)
	 (write-string (name icon) stream)))))

(define-presentation-method present
    (icon (type icon) stream (view textual-results-view) &key)
  (when (not (icon-window icon))
    (multiple-value-bind (x y)
	#+clim-1 (stream-cursor-position* stream)
	#+clim-2 (stream-cursor-position stream)
      (setf (icon-window icon) stream)
      (setf (x icon) x)
      (setf (y icon) y)))
  (setf (presentation icon)
    (with-output-as-presentation
	#+clim-1 (:stream stream :object icon :type (type-of icon) :single-box t)
	#+clim-2 (stream icon (type-of icon) :single-box t)
        (with-text-style
         (#+clim-2 stream
          (merge-text-styles (if (window-open-p icon)
                                 *open-result-text-style*
                               *closed-result-text-style*)
                             (medium-merged-text-style stream))
          #+clim-1 stream)
         (write-string (name icon) stream)))))

;;; presentation highlighting method for icons
;;; bolds the icon boundary
(define-presentation-method highlight-presentation ((type icon) record stream state)
  state
  (multiple-value-bind (xoff yoff)
      (convert-from-relative-to-absolute-coordinates
       stream (output-record-parent record))
    (with-bounding-rectangle* (left top right bottom) record
	(draw-rectangle* stream
			       (+ left xoff)
			       (+ top yoff)
			       (+ right xoff)
			       (+ bottom yoff)
			       :ink +flipping-ink+
			       :filled nil
			       :line-thickness 3)
	)))

;;; ***************************************************************************
;;; EOF

