;;;; -*- Mode: Common-Lisp; Package: CLASP-INTERFACE; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/inferior-frame.lisp *-*
;;;; *-* Last-Edit: Wednesday, June 21, 1995  13:44:51; Edited-By: schmill *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                         INFERIOR FRAME METHODS                         *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: David L. Westbrook
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-04-95 File Created.  (westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(defmethod display-or-inferior-pane-p ((frame clasp-frame) window)
  (or (eq window (get-frame-pane frame 'notebook))
      ;; Check for killed windows at some point.
      (member window (inferiors frame) :test #'eq)))

(defmacro with-inferior-stream ((stream frame &rest options) &body body)
  `(let ((,stream (frame-standard-output (make-inferior-window ,frame . ,options))))
     . ,body))

(defmacro with-inferior-window ((inferior-frame inferior-stream frame &rest options) &body body)
  `(let* ((,inferior-frame (make-inferior-window ,frame . ,options))
          (,inferior-stream (frame-standard-output ,inferior-frame)))
     . ,body))

(define-application-frame inferior-frame () ()
  #+clim-1
  (:panes ((display :application
		    :incremental-redisplay t)))
  #+clim-2
  (:panes (display :application
                   :end-of-line-action :allow
                   :display-time t
		   :incremental-redisplay t))
  #+ignore
  (scrolling
   ()
   (make-pane 'application-pane
	      :display-time t
	      :incremental-redisplay t))
  #+clim-2
  (:pointer-documentation t)
  #+clim-2
  (:layouts (default (vertically
		      ()
		      display))))

(defmethod make-inferior-window ((frame clasp-frame) &rest options)
  (let ((new-frame (apply #'make-application-frame 'inferior-frame
			  #+clim-1 :parent #+clim-1 (clasp-root)
                          #+clim-2 :frame-manager #+clim-2 (frame-manager frame)
                          options)))
    (dwim:start-frame new-frame :master frame)
    (push new-frame (inferiors frame))
    new-frame))


(defmethod close-it ((self inferior-frame) &key &allow-other-keys)
  (frame-exit self))

(defmethod close-it ((self graph-data-icon) &key &allow-other-keys)
  (close-it (graph self))
  (setf (window-open-p self) nil))

(defmethod close-it ((self graph::graph-viewer) &key &allow-other-keys)
  (frame-exit self))

(defmethod open-it :around ((self window-mixin)
                            &rest args &key &allow-other-keys)
  (cond
   ;; THIS IS GROSS! How about doing it the right way?
   ((or (eq (class-of self) (find-class 'graph-data-icon))
	(eq (class-of self) (find-class 'graph)))
    (call-next-method))
   ((display-frame self)
    (unless (window-open-p self)
      (dwim:start-frame (display-frame self) :master (clasp-frame))
      (setf (window-open-p self) t)))
   (t
    (setf (display-frame self) (apply #'make-inferior-window (clasp-frame) args))
    (call-next-method))))

(defmethod close-it ((self window-mixin) &key &allow-other-keys)
  (when (display-frame self)
    (close-it (display-frame self))
    (setf (window-open-p self) nil)))

;;; ---------------------------------------------------------------------------

(defmacro with-auto-sizing ((stream
			     &key (max-width most-positive-fixnum)
				  (max-height most-positive-fixnum))
			    &body body)
  (let ((output-record (gensym "OUTPUT-RECORD"))
	#+allegro (stream-symbol (gensym "STREAM"))) ; see below
    `(let ((,output-record
	    (with-new-output-record (,stream)
	      ,@body)))
       (with-bounding-rectangle* (left top right bottom)
	 ,output-record
	 (window-set-inside-size
	  ,stream (min (+ (- right left) 5) ,max-width)
	  (min (+ (- bottom top) 5) ,max-height))
         #+clim-1 (window-set-viewport-position* ,stream 0 0)
	 ;; There seems to be a bug in the setf* methood for window-viewport-position, so I've hacked it -AC 1/21/93
	 #+(and clim-2 (not allegro)) (setf (window-viewport-position ,stream) (values 0 0))
	 #+(and clim-2 allegro) (let ((,stream-symbol ,stream))
				  (funcall #'clim-utils::|SETF* CLIM:WINDOW-VIEWPORT-POSITION| 0 0 ,stream-symbol))))))

;;; ----------------------------------------------------------------------------

#+OLD
(defmethod make-inferior-window ((frame clasp-frame) &rest options)
  (let ((pane (apply #'open-window-stream
                     :parent (clasp-root)
		     :width 800 :height 400
		     :input-buffer (stream-input-buffer 
				    (get-frame-pane frame 'notebook))
               options)))
    (push pane (inferiors frame))	; remember it
    ;(window-expose pane)		; see it
    (values pane)
    ))

;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************