;;;; -*- Mode: Common-Lisp; Package: CLASP-INTERFACE; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/dev/interface/listener.lisp *-*
;;;; *-* Last-Edit: Friday, February 9, 1996  14:41:25; Edited-By: Westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                        CLASP Lisp Interactor                           *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-04-95 Header added.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :clasp-interface)

#|

Your problem report has been assigned a tracking id of spr14873.
Please use this id in the subject line of any mail related to
it so that we may better track communication on your inquiry.
Also, please address mail to bugs@franz.com as well as me,
so your questions can be answered if I am unavailable.

  Date: Tue, 2 Jul 1996 20:19:54 -0400 (EDT)
  From: "David L. Westbrook" <westy@lindy.cs.umass.edu>

  There seems to be a problem accepting a sequence of things as part of
  a command defined under the lisp listener demo. It beeps and will not
  complete.

  Repeat by defining all this code:

The problem arises because in the last example CLIM needs to reparse
the input (why this occurs only in the last example isn't really
important). When you click on items to enter arguments CLIM puts a
special marker in the input-editor buffer so that when it reparses the 
input it gets hold of the actual object. 

Unfortunately this is all done at the stream layer in the input editor
code. This is OK as long as you're clicking on presentations that
represent complete arguments but if you have an argument that consists 
of multiple presentations eg a sequence of items then this breaks
badly. This is because by the time it gets down to the accept-method
for a sequence of items it doesn't know about these special markers in 
the input stream. It would be a major modularity violation to let the
the accept method code know about this and I having spent considerable 
time looking at this I don't see any way to fix the problem without
making major changes to CLIM.

The only thing I can suggest is a workaround where you define textual
based accept and present methods for all your datatypes. This way when 
you click on an item as an argument it inserts into the input buffer
a string of characters which when accept'ed will get you back the
original object. I managed to work around the error in your example
by adding the following definitions:

|#

(defvar *results*
    (make-array 10 :adjustable t :fill-pointer 0))

(define-presentation-method accept
    ((type result) stream (view textual-view) &key)
  (let ((token (with-delimiter-gestures (#\:)
		 (read-token stream)))
	(separator (read-char stream))
	(index (parse-integer (read-token stream))))
    (declare (ignore token separator))
    (aref *results* index)))

(define-presentation-method present
    (x (type result) stream (view textual-view) &key)
  (let ((index (or (position x *results*)
		   (prog1 (fill-pointer *results*)
		     (vector-push-extend x *results*)))))
    (format stream "~a:~d" type index)))

#|

The text based representation for the objects is arbitary but it
does ensure uniqueness. You may be able to come up with a better
representation by taking into account the particular nature of the
objects in your application domain.

Hope this is helpful.

|#


;;; ----------------------------------------------------------------------------------
;;; EOF