;;;; -*- Mode: Lisp; Package: CLASP-INTERFACE; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/dev/interface/listener.lisp *-*
;;;; *-* Last-Edit: Friday, February 9, 1996  14:41:25; Edited-By: Westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                        CLASP Lisp Interactor                           *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Copyright (c) 1990 - 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-04-95 Header added.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :clasp-interface)

#|

(defun lisp-listener-command-reader (frame stream command-table presentation-type 
				     &key keystrokes listener-depth (prompt "=> "))
  (catch-abort-gestures ("Return to ~A command level ~D"
			 (frame-pretty-name frame) listener-depth)
    
    ;;Date: Thu, 17 Nov 94 20:57:55 PST
    ;;From: Colin Meldrum <colin@franz.com>
    ;;Bh-Id: spr11805
    ;;Subject: Re: [spr11805] Problems with command-or-form in ACL?
    
    ;;  Date: Thu, 17 Nov 1994 15:03:27 -0500
    ;;  From: "David L. Westbrook" <westy@lindy.cs.umass.edu>
    
    ;;  Allegro CL 4.2 [SPARC; R1] (10/19/94 12:34)
    
    ;;  The following code takes the lisp-listener frame from the demo and
    ;;  extends it with a hierarchy of menus on a menu bar and an additional
    ;;  pane that presents some arbitrary objects. I have encountered two
    ;;  problems:
    
    ;;  [rest deleted]
    
    ;; The problem with trying to activate commands with no arguments from a
    ;; menu arises because the lisp-listener top level doesn't use
    ;; read-frame-command (which has a default around method which makes this
    ;; work). You can work around the problem by redefining your top level as
    ;; follows (additional code in caps)
    
    (FLET ((MAYBE-ECHO-COMMAND (COMMAND)
	   (LET ((PTYPE `(COMMAND :COMMAND-TABLE ,(FRAME-COMMAND-TABLE FRAME))))
	     (PRESENT COMMAND PTYPE :STREAM STREAM :ALLOW-SENSITIVE-INFERIORS NIL)
	     (PUSH-HISTORY-ELEMENT (PRESENTATION-TYPE-HISTORY PTYPE)
				   (CLIM-INTERNALS::MAKE-PRESENTATION-HISTORY-ELEMENT :OBJECT COMMAND :TYPE PTYPE)))))
	  ;; Eat any abort gestures that might be hanging around.
	  ;; We need to do this because COMMAND-OR-FORM is wierd.
	  (let* ((abort-gestures *abort-gestures*)
		 (*abort-gestures* nil))
	    ;;--- What test does this need?
	    (when (member (stream-read-gesture stream :timeout 0 :peek-p t) abort-gestures)
	      (stream-read-gesture stream :timeout 0)))
    (fresh-line stream)
    (if (stringp prompt)
	(write-string prompt stream)
      (multiple-value-bind (x y) (stream-cursor-position stream)
	(draw-pattern* stream prompt x y)
	(stream-increment-cursor-position stream (pattern-width prompt) nil)))
    (multiple-value-bind (command-or-form type numeric-arg)
	(block keystroke
	  (handler-bind ((accelerator-gesture
			  #'(lambda (c)
			      ;; The COMMAND-OR-FORM type is peeking for the
			      ;; first character, looking for a ":", so we
			      ;; have to manually discard the accelerator
			      (stream-read-gesture stream :timeout 0)
			      (return-from keystroke
				(values
				 (accelerator-gesture-event c)
				 :keystroke
				 (accelerator-gesture-numeric-argument c)))))
			 (simple-parse-error
			  #'(lambda (c)
			      (let ((args (clim-internals::parse-error-format-arguments c)))
				(when (and (= (length args) 1)
					   (equal (first args) ""))
				  ;; Hmm, user must have just hit <Return>
				  (return-from lisp-listener-command-reader)))
			      ;; Otherwise decline to handle this
			      nil))
			 (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT
			  #'(LAMBDA (C)
				    (LET ((COMMAND (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT-COMMAND C))) 
					 (WHEN (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT-ECHO-P C)
					       (MAYBE-ECHO-COMMAND COMMAND))
					 (RETURN-FROM KEYSTROKE COMMAND))))
			 
			 )
	    (let ((*accelerator-gestures* keystrokes))
	      (accept presentation-type
		      :stream stream
		      :prompt nil :prompt-mode :raw
		      :additional-activation-gestures '(#+Genera #\End)))))
      (when (eq type :keystroke)
	(let ((command (lookup-keystroke-command-item command-or-form command-table 
						      :numeric-argument numeric-arg)))
	  (unless (clim-internals::keyboard-event-p command)
	    (when (partial-command-p command)
	      (setq command (funcall *partial-command-parser*
				     command command-table stream nil
				     :for-accelerator t)))
	    (setq command-or-form command
		  type 'command))))
      (cond ((eq type ':keystroke)
	     (beep))
	    ((eq (presentation-type-name type) 'command)
	     (terpri)
	     (let ((*debugger-hook* 
		     (unless *use-native-debugger*
		       (and (zerop listener-depth) #'listener-debugger-hook))))
	       (apply (command-name command-or-form)
		      (command-arguments command-or-form)))
	     (terpri))
	    (t
	     (terpri)
	     (let ((values 
		     (multiple-value-list
		       (let ((*debugger-hook* 
			       (unless *use-native-debugger*
				 (and (zerop listener-depth) #'listener-debugger-hook))))
			 (eval command-or-form)))))
	       (fresh-line)
	       (dolist (value values)
		 (present value 'expression :single-box :highlighting)
		 (terpri))
	       (setq - command-or-form)
	       (shiftf +++ ++ + -)
	       (when values
		 ;; Don't change this stuff if no returned values
		 (shiftf /// // / values)
		 (shiftf *** ** * (first values))))))))))

|#

#|
   Date: Fri, 18 Mar 1994 16:36:46 -0500
   From: Adam Carlson <carlson%lindy@cs.umass.edu>

   Clim 1.1
     Lucid 4.1 - Sparc
     Lispworks 3.1 - Sparc & Alpha
   Clim 2.0 beta
     Lucid 4.1 - Sparc
     Lispworks 3.2b1 - Alpha

   1) Command name completion doesn't work. (Clim 1 & Clim 2)

   If there are two commands, (com-foo :name "Foo") and (com-foo-bar :name "Foo
   Bar"), then in order to use the first command, you must type return after
   typing :foo, but _before_ hitting space.  This is especially annoying in
   harlequin where tab isn't a completion character, so you have to type space to
   do command name completion and then hit backspace so that the longer command
   name isn't assumed.  This causes problems if the command takes arguments,

Completion gestures, like other "magic" input editing gestures, can
usually be set up by the user.  These are the additional gestures I
set up for Harlequin CLIM 2.0-beta:

Thus, #\Control-Tab acts as a completion gesture for me in the
circumstance you describe.

|#

#+CLIM-2
(clim:define-gesture-name :help :keyboard (:h :control) :unique nil)
#+CLIM-2
(clim:define-gesture-name :complete :keyboard (:tab :control) :unique nil)
#+CLIM-2
(clim:define-gesture-name :preview-command :keyboard (:tab :meta) :unique nil)
#+CLIM-2
(clim:define-gesture-name :ie-clear-input :keyboard (:backspace :control) :unique nil)

(defmethod frame-maintain-presentation-histories ((frame clasp-frame)) t)

(defmacro condition-restart-loop ((conditions description . args) &body body)
  #---ignore (declare (ignore conditions))
  (let ((tag (gensym (string 'restart))))
    `(tagbody ,tag
       (restart-case
	   (progn ,@body)
	 (nil ()
	   #|| :test (lambda (condition)
		       (some #'(lambda (x) (typep condition x)) ',conditions)) ||#
	   :report (lambda (stream)
		     (format stream ,description ,@args))))
       (go ,tag))))

(defvar *listener-depth* -1)

#+allegro
(progn
  ;; The stack-group object corresponding to the stack we are debugging.
  (defvar *tpl-current-stack-group* nil)
  ;; The newest (possibly invisible) frame at the current break level.
  (defvar *top-top-frame-pointer* nil)
  ;; The newest (possibly invisible) frame at the current break level.
  (defvar *top-frame-pointer* nil)
  ;; Current stack frame pointer.
  (defvar *current-frame-pointer* nil))

(defvar *clasp-io*)

(defvar *use-native-debugger* nil)

(defvar *prompt-arrow-1* 
	(make-pattern #2A((0 0 0 0 0 0 0 0 0 0 0 0)
			  (0 0 0 0 0 1 0 0 0 0 0 0)
			  (0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 1 1 1 1 1 1 1 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 0 0 0)
			  (0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 0 0 0 0 0 0 0 1 1 1 0)
			  (0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 1 1 1 1 1 1 1 1 0 0 0)
			  (0 1 1 1 1 1 1 1 0 0 0 0)
			  (0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 0 0 0 0 1 0 0 0 0 0 0))
		      (list +background-ink+ +foreground-ink+)))

(defvar *prompt-arrow-2* 
	(make-pattern #2A((0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0)
			  (0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 0)
			  (0 0 0 0 0 0 0 0 0 0 0 1 1 1 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0)
			  (0 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 1 0 0 0 0 0)
			  (0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 0))
		      (list +background-ink+ +foreground-ink+)))

(defvar *clasp-user-package* (find-package 'clasp-user))

(defun clasp-top-level-initializations (frame)
  #+ignore
  (unless (and (slot-boundp frame 'datasets-window)
	       (not (null (datasets-window frame))))
    (setf (datasets-window frame)
	  (make-inferior-window frame #+clim-1 :label #+clim-2 :name "Datasets")))
  #+ignore
  (unless (and (slot-boundp frame 'results-window)
	       (not (null (results-window frame))))
    (setf (results-window frame)
	  (make-inferior-window frame #+clim-1 :label #+clim-2 :name "Results")))
  #+(and ignore clim-1)
  (setf clasp::*debug* (member :use-native-debugger (error-handling-preferences frame)))
  #+(or clim-1 clim-2)
  (setf *use-native-debugger* (member :use-native-debugger (error-handling-preferences frame)))
  #+explorer
  (setf ticl::zunderflow (member :trap-underflow-errors (error-handling-preferences frame)))
  #+lucid
  (when (member 'user::floating-point-underflow user::supported-floating-point-conditions)
    (if (member :trap-underflow-errors (error-handling-preferences frame))
	(pushnew 'user::floating-point-underflow (user::enabled-floating-point-traps))
	(setf (user::enabled-floating-point-traps)
	      (remove 'user::floating-point-underflow (user::enabled-floating-point-traps)))))
  (when (window-open-p (datasets frame))
    (open-it (datasets frame)))
  (when (window-open-p (results frame))
    (open-it (results frame)))
  (dolist (result-window (clasp::find-instances 'result))
    (when (window-open-p result-window)
      (setf (window-open-p result-window) nil)
      (open-it result-window))))

(defvar *use-listener-top-level* t)

(defun clasp-top-level (frame)
  "Run clasp using the window provided."
  (loop
   (catch 'change-top-level
     (if *use-listener-top-level*
       (progn #+clim-2 (enable-frame frame)
         (let* ((*clasp-frame* frame)
	        (*pointer-documentation-output*
                 (frame-pointer-documentation-output frame))
	        (window (frame-standard-input frame))
	        (command-table (frame-command-table frame))
	        (presentation-type
                 #+clim-1 `(command-or-form :command-table ,command-table)
                 #+clim-2 `((command-or-form :command-table ,command-table)
	                    ;; :auto-activate #+(or Genera Cloe-Runtime) t
	                    ;; #-(or Genera Cloe-Runtime) nil
			    )))
           
           (clasp-top-level-initializations frame)
           (with-input-focus (window)
	     (let* ((*clasp-io* window)
		    (*standard-input* *clasp-io*)
		    (*standard-output* *clasp-io*)
		    #+(or Minima allegro) (*error-output* *clasp-io*)
		    (*query-io* *clasp-io*)
		    #+Minima (*debug-io* *clasp-io*)
		    (*package* *clasp-user-package*)
		    (*listener-depth* (1+ *listener-depth*))
		    (*** nil) (** nil) (* nil)
		    (/// nil) (// nil) (/ nil)
		    (+++ nil) (++ nil) (+ nil)
		    (- nil)
		    #+allegro (*tpl-current-stack-group* sys::*current-stack-group*)
		    #+allegro
		    (*top-top-frame-pointer*
		     #+(and allegro-version>= (not (version>= 4 3)))
         		     (debug::newest-frame si::*current-stack-group* :visible-only-p nil)
		     #+(and allegro-version>= (version>= 4 3))
			    (excl::int-newest-frame si::*current-stack-group* :visible-only-p nil))
                    #+allegro
		    (*top-frame-pointer*
			       (or (db::find-interesting-frame *top-top-frame-pointer*)
				   *top-top-frame-pointer*))
		    #+allegro (*current-frame-pointer* *top-frame-pointer*))
	       (terpri *clasp-io*)
	       (with-command-table-keystrokes (keystrokes command-table)
	         (condition-restart-loop (#+Genera (sys:error sys:abort)
						   #-Genera (error)
						   "Restart CLIM lisp listener")
				         (clasp-command-reader
				          frame *standard-input* command-table presentation-type
				          :keystrokes keystrokes
				          :listener-depth *listener-depth*
				          :prompt (case *listener-depth*
						    (0 *prompt-arrow-1*)
						    (1 *prompt-arrow-2*)
						    (otherwise 
						     (concatenate 'string 
							          (make-string (1+ *listener-depth*) :initial-element #\=)
							          "> "))))))))))
       
       (default-frame-top-level frame)))))

(defun clasp-command-reader (frame stream command-table presentation-type 
			     &key keystrokes listener-depth (prompt "=> "))
  (#-allegro progn 
   #+allegro FLET 
   #+allegro ((MAYBE-ECHO-COMMAND (COMMAND)
	       (LET ((PTYPE `(COMMAND :COMMAND-TABLE ,(FRAME-COMMAND-TABLE FRAME))))
		    (PRESENT COMMAND PTYPE :STREAM STREAM :ALLOW-SENSITIVE-INFERIORS NIL)
		    (PUSH-HISTORY-ELEMENT (PRESENTATION-TYPE-HISTORY PTYPE)
					  (CLIM-INTERNALS::MAKE-PRESENTATION-HISTORY-ELEMENT :OBJECT COMMAND :TYPE PTYPE)))))
  (climi::catch-abort-gestures ("Return to ~A command level ~D"
			      (frame-pretty-name frame) listener-depth)
    ;; Eat any abort gestures that might be hanging around.
    ;; We need to do this because COMMAND-OR-FORM is wierd.
    (let* ((abort-gestures #+clim-1 clim::*abort-characters*
                           #+clim-2 *abort-gestures*)
	   (#+clim-1 clim::*abort-characters*
            #+clim-2 *abort-gestures* nil))
      ;;--- What test does this need?
      (when (member (stream-read-gesture stream :timeout 0 :peek-p t) abort-gestures)
	(stream-read-gesture stream :timeout 0)))
    (fresh-line stream)
    (if (stringp prompt)
	(write-string prompt stream)
	(multiple-value-bind (x y)
            #+clim-1 (stream-cursor-position* stream)
	    #+clim-2 (stream-cursor-position stream)
	  #+clim-1 (draw-icon* stream prompt x y)
          #+clim-1 (stream-increment-cursor-position* stream (clim::pattern-width prompt) nil)
          #+clim-2 (draw-pattern* stream prompt x y)
	  #+clim-2 (stream-increment-cursor-position stream (pattern-width prompt) nil)))
    (multiple-value-bind (command-or-form type numeric-arg)
	(block keystroke
	  (handler-bind ((clim::accelerator-gesture
			   #'(lambda (c)
			       ;; The COMMAND-OR-FORM type is peeking for the
			       ;; first character, looking for a ":", so we
			       ;; have to manually discard the accelerator
			       (stream-read-gesture stream :timeout 0)
			       (return-from keystroke
				 (values
				   (clim::accelerator-gesture-event c)
				   :keystroke
				   (clim::accelerator-gesture-numeric-argument c)))))
			  #+allegro
			  (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT
			   #'(LAMBDA (C)
			       (LET ((COMMAND (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT-COMMAND C))) 
				    (WHEN (CLIM-INTERNALS::SYNCHRONOUS-COMMAND-EVENT-ECHO-P C)
					  (MAYBE-ECHO-COMMAND COMMAND))
				    (RETURN-FROM KEYSTROKE COMMAND))))
                         ;; Tried this - Westy
                         #+DID-NOT-WORK
			 (input-not-of-required-type
			   #'(lambda (c)
			       (let ((args (clim-internals::parse-error-format-arguments c)))
				 (clasp::spy args)
				 (when (and (= (length args) 1)
					    (equal (first args) ""))
				   ;; Hmm, user must have just hit <Return>
				   (return-from clasp-command-reader)))
			       ;; Otherwise decline to handle this
			       nil))
                         #+clim-2
			 (simple-parse-error
			   #'(lambda (c)
			       (let ((args (clim-internals::parse-error-format-arguments c)))
				 (clasp::spy args)
				 (when (and (= (length args) 1)
					    (equal (first args) ""))
				   ;; Hmm, user must have just hit <Return>
				   (return-from clasp-command-reader)))
			       ;; Otherwise decline to handle this
			       nil)))
	    (let (#+clim-1 (clim::*accelerator-characters* keystrokes)
                  #+clim-2 (*accelerator-gestures* keystrokes))
	      (accept presentation-type
		      :stream stream
		      :prompt nil :prompt-mode :raw
		      #+clim-1 :additional-activation-characters
                      #+clim-2 :additional-activation-gestures #-genera nil #+Genera '(#\End)))))
      (when (eq type :keystroke)
	(let ((command (lookup-keystroke-command-item command-or-form command-table 
						      :numeric-argument numeric-arg)))
	  (unless #+clim-1 (characterp command)
                  #+clim-2 (clim-internals::keyboard-event-p command)
	    (when (partial-command-p command)
	      (setq command (funcall *partial-command-parser*
				     command command-table stream nil
				     :for-accelerator t)))
	    (setq command-or-form command
		  type 'command))))
      (cond ((eq type ':keystroke)
	     (beep))
	    ((null command-or-form))
	    ((eq (presentation-type-name type) 'command)
	     (terpri)
	     (let ((*debugger-hook* 
		     (unless *use-native-debugger*
		       (and (zerop listener-depth) #'clasp-debugger-hook))))
	       (apply (command-name command-or-form)
		      (command-arguments command-or-form)))
	     (terpri)
             (update-history frame command-or-form *last-command-result*)
	     (update-datasets-display)
	     (update-results-display))
	    (t
	     (terpri)
	     (let ((values 
		     (multiple-value-list
		       (let ((*debugger-hook* 
			       (unless *use-native-debugger*
				 (and (zerop listener-depth) #'clasp-debugger-hook))))
			 (eval command-or-form)))))
	       (fresh-line)
	       (dolist (value values)
		 (present value 'expression :single-box :highlighting)
		 (terpri))
               (update-history frame command-or-form values)
               (update-datasets-display)
               (update-results-display)
	       (setq - command-or-form)
	       (shiftf +++ ++ + -)
	       (when values
		 ;; Don't change this stuff if no returned values
		 (shiftf /// // / values)
		 (shiftf *** ** * (first values))))))))))

(defvar *debugger-condition* nil)
(defvar *debugger-restarts* nil)

#+clim-1
(defun clasp-debugger-hook (condition hook)
  (declare (ignore hook))
  (let ((*debug-io* (frame-query-io *application-frame*))
	(*error-output* (frame-query-io *application-frame*))
        (*debugger-condition* condition)
	(*debugger-restarts* (compute-restarts)))
    (describe-error *error-output*)
    (clasp-top-level *application-frame*)))

#+clim-2
(defun clasp-debugger-hook (condition hook)
  (declare (ignore hook))
  (let* ((*application-frame* *clasp-frame*)
	 #+Minima (*debug-io* (frame-query-io *application-frame*))
	 (*error-output* (frame-standard-output *application-frame*))
	 (*debugger-condition* condition)
	 (*debugger-restarts* (compute-restarts)))
    (describe-error *error-output*)
    (with-output-recording-options (*clasp-io* :draw t :record t)
      (clasp-top-level *application-frame*))))

(defvar *enter-debugger* '#:enter-debugger)
(defun enter-debugger (stream)
  #-Genera (declare (ignore stream))
  #+Genera
  (clim-internals::with-debug-io-selected (stream)
    (cl:break "Debugger break for ~A" (frame-pretty-name (pane-frame stream)))))

(define-presentation-type restart-name ())

(define-presentation-method presentation-typep (object (type restart-name))
  (or (eql object *enter-debugger*)
      (typep object 'restart)))

(define-presentation-method present (object (type restart-name) stream (view textual-view)
				     &key)
  (if (eql object *enter-debugger*)
      (princ "Enter the debugger" stream)
      (prin1 (restart-name object) stream)))

(define-clasp-command (com-invoke-restart :name t)
    ((restart 'restart-name :gesture :select))
  (if (eql restart *enter-debugger*)
      (enter-debugger *standard-input*)
    (let (values)
      #+Allegro
      (case (restart-name (find-restart restart))
	(excl::return-value
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "Enter value to return")))))
	(excl::try-a-different-function
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "enter expression which will evaluate to the function to call")))))
	(excl::try-a-different-function-setf
	 (setq values
	   (list (eval
		  (accept 'form :stream *standard-input*
			  :prompt "enter expression which will evaluate to the function to call"))))))
      (apply #'invoke-restart restart values))))

(define-clasp-command (com-describe-error :name t) ()
  (describe-error *error-output*))

(defun describe-error (stream)
  (when *debugger-condition*
    (clim::with-output-as-presentation
     #+clim-1 (:object *debugger-condition* :type 'form :stream stream :single-box t)
     #+clim-2 (stream *debugger-condition* 'form :single-box t)
      (format stream "~2&Error: ~A" *debugger-condition*))
    (let ((restarts *debugger-restarts*))
      (when restarts
	(let ((actions '(invoke-restart)))
	  (dolist (restart (reverse restarts))
	    (let ((action (member (restart-name restart)
				  '(abort continue muffle-warning store-value use-value))))
	      (when action
		(pushnew (first action) actions))))
	  (format stream "~&Use~?to resume~:[~; or abort~] execution:"
	    "~#[~; ~S~; ~S or ~S~:;~@{~#[~; or~] ~S~^,~}~] "
	    actions (member 'abort actions)))
	(fresh-line stream)
	(formatting-table (stream
                           #+clim-1 :inter-column-spacing
                           #+clim-2 :x-spacing            '(2 :character))
	  (dolist (restart restarts)
	    (clim::with-output-as-presentation
             #+clim-1 (:object restart :type 'restart-name :stream stream
			       :single-box t :allow-sensitive-inferiors nil)
             #+clim-2 (stream restart 'restart-name
                              :single-box t :allow-sensitive-inferiors nil)
	      (formatting-row (stream)
		(formatting-cell (stream) stream)
		(formatting-cell (stream)
		  (when (restart-name restart)
		    (format stream "~S:" (restart-name restart))))
		(formatting-cell (stream)
		  (format stream "~A" restart)))))
	  #+Genera
	  (clim::with-output-as-presentation (stream *enter-debugger* 'restart-name
					:single-box t :allow-sensitive-inferiors nil)
	    (formatting-row (stream)
	      (formatting-cell (stream) stream)
	      (formatting-cell (stream) stream)
	      (formatting-cell (stream)
		(write-string "Enter the debugger" stream)))))))
    #+clim-2
    (let ((process (clim-sys:current-process)))
      (when process
	(format stream "~&In process ~A." process))))
  (force-output stream))

(define-clasp-command (com-use-native-debugger :name t)
    ((boolean 'boolean
	      :prompt "yes or no"
	      :default (not *use-native-debugger*)))
  (setq *use-native-debugger* boolean)
  (with-clasp-frame
   (clasp-frame)
   (if boolean
       (pushnew :use-native-debugger (error-handling-preferences clasp-frame))
     (setf (error-handling-preferences clasp-frame)
           (remove :use-native-debugger (error-handling-preferences clasp-frame))))))


;;; Lisp-y stuff

(defun quotify-object-if-necessary (object)
  (if (or (consp object)
	  (and (symbolp object)
	       (not (keywordp object))
	       (not (eq object nil))
	       (not (eq object t))))
      (list 'quote object)
    object))

(define-presentation-translator describe-lisp-object
    (expression form clasp ;replaced lisp-listener
     :documentation
       ((object stream)
	(let ((*print-length* 3)
	      (*print-level* 3)
	      (*print-pretty* nil))
	  (present `(describe ,(quotify-object-if-necessary object)) 'expression
		   :stream stream :view +pointer-documentation-view+)))
     :gesture :describe)
    (object)
  `(describe ,(quotify-object-if-necessary object)))

(define-presentation-translator expression-identity
    (expression nil clasp ;replaced lisp-listener
    :tester
    ((object context-type)
     (if (and (eq (presentation-type-name context-type) 'sequence)
	      (or (listp object)
		  (vectorp object)))
	 (dwim:with-stack-list
	  (type 'sequence (reasonable-presentation-type (elt object 0)))
          #+clim-1
          (presentation-subtypep type context-type)
	  #+clim-2
          (clim-internals::presentation-subtypep-1 type context-type))
       #+clim-1
       (presentation-subtypep
        (reasonable-presentation-type object) context-type)
       #+clim-2
       (clim-internals::presentation-subtypep-1
	(reasonable-presentation-type object) context-type)))
    :tester-definitive t
    :documentation ((object stream)
		    (let ((*print-length* 3)
			  (*print-level* 3)
			  (*print-pretty* nil))
		      (present object 'expression 
			       :stream stream :view +pointer-documentation-view+)))
    :gesture :select)
   (object)
   object)

(defun reasonable-presentation-type (object)
  (let* ((class (class-of object))
	 (class-name (class-name class)))
    (when (presentation-type-specifier-p class-name)
      ;; Don't compute precedence list if we don't need it
      (return-from reasonable-presentation-type class-name))
    (dolist (class (class-precedence-list class))
      (when (presentation-type-specifier-p (class-name class))
	(return-from reasonable-presentation-type (class-name class))))
    nil))

;;; ----------------------------------------------------------------------------------
