;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       ANOVA/Contrast CLIM UI                           *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;

;;; Copyright (c) 1990 - 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;  06-20-95 File Created. (Schmill)
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :clasp-interface)

;;; ---------------------------------------------------------------------------

(proclaim '(inline cross-string))
(defun cross-string (&rest things)
  (format nil "~{~A~^ X ~}" (mapcar #'name things)))

;;; ---------------------------------------------------------------------------
;;;
(define-clasp-command (com-anova-one-way :command-table clasp-tests
						   :name "One Way Anova"
						   :menu "One Way Anova")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (confidence-intervals '(null-or-type #-lucid (real 0 1) #+lucid real) :prompt "Confidence intervals" :default nil))
   (let* ((iv-values (simplify-and-sort column-1))
          (values
	   (multiple-value-list (funcall
				 #'clasp::anova-one-way
				 iv-values
				 (simplify-and-sort-by column-2 column-1)
				 t
				 confidence-intervals)))
	  (result (funcall 'make-anova-one-way-result-from-list (subseq values 0 5)
			   :iv column-1 :dv column-2 :groups (sixth values))))
    (setf (description result)
      (format nil "anova of ~a on ~a"
	      (description column-1) (description column-2)))
    (present-with-update result
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ---------------------------------------------------------------------------
;;; one way repeated measures

(define-clasp-command (com-anova-one-way-repeated :command-table clasp-tests
                                                  :name "ANOVA One Way Repeated Measures"
                                                  :menu "ANOVA One Way Repeated Measures")
    ((column-f 'numbers :prompt "Fixed Factor" :mapping-option :map)
     (column-r 'numbers :prompt "Random Factor" :mapping-option :map)
     (column-v 'numbers :prompt "X" :mapping-option :map)
     #+HOLD (scheffe-p 'boolean :prompt "Scheffe Tests ?"))
   (let* ((values (multiple-value-list
                   (clasp::anova-one-way-repeated-measures (variable-value column-f)
						           (variable-value column-r)
						           (variable-value column-v)
                                                           #+HOLD scheffe-p)))
          (scheffe-result (make-scheffe-table-from-list (fourth values) :group-means (second values)))
          (result (make-generic-table-from-lists
                   (list (name column-f) (name column-r) (cross-string column-f column-r) "Total")
                   '("SS" "DF" "MS" "F" "P")
                   (first values)
                   "Source" :name "anova-one-way-repeated-table"
                   :other-stuff (list (list "Scheffe Tests" +display-condensed+ scheffe-result)))))
     (present-with-update result
			  :stream *standard-output*
                          :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ---------------------------------------------------------------------------
;;; two-way mixed design

(define-clasp-command (com-anova-two-way-mixed :command-table clasp-tests
					       :name "ANOVA Two Way Mixed (NRM)"
					       :menu "ANOVA Two Way Mixed (NRM)")
    ((column-f 'numbers :prompt "Fixed Factor" :mapping-option :map)
     (column-r 'numbers :prompt "Random Factor" :mapping-option :map)
     (column-v 'numbers :prompt "Effect" :mapping-option :map))
   (let* ((values (multiple-value-list
                   (clasp::anova-two-way-mixed (variable-value column-f)
						           (variable-value column-r)
						           (variable-value column-v))))
          (result (make-generic-table-from-lists
                   (list (name column-f) (name column-r) (cross-string column-f column-r) "Within" "Total")
                   '("SS" "DF" "MS" "F" "P")
                   (first values)
                   "Source" :name "anova-two-way-mixed-table")))
     (present-with-update result
			  :stream *standard-output*
                          :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ---------------------------------------------------------------------------
;;; two-way repeated measures

(define-clasp-command (com-anova-two-way-repeated :command-table clasp-tests
					          :name "ANOVA Two Way Repeated Measures"
					          :menu "ANOVA Two Way Repeated Measures")
    ((Y  'numbers :prompt "Y"  :mapping-option :map)
     (X1 'numbers :prompt "Fixed1" :mapping-option :map)
     (X2 'numbers :prompt "Fixed2" :mapping-option :map)
     (R3 'numbers :prompt "Random" :mapping-option :map))
   (let* ((values (multiple-value-list
                   (clasp::anova-two-way-repeated-measures (variable-value x1) (variable-value x2)
                                                           (variable-value R3) (variable-value Y))))
          (result (make-generic-table-from-lists
                   (list (name x1) (name r3) (cross-string x1 r3) (name x2)
                         (cross-string x2 r3) (cross-string x1 x2)
                         (cross-string x1 x2 r3) "TOTAL")
                   '("SS" "DF" "MS" "F" "P")
                   (first values)
                   "Source" :name "anova-two-way-repeated-table")))
     (present-with-update result
			  :stream *standard-output*
                          :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))


;;; ---------------------------------------------------------------------------
;;; three way factorial anova

(define-clasp-command (com-anova-three-way :command-table clasp-tests
					   :name "ANOVA Three Way"
					   :menu "ANOVA Three Way")
    ((Y  'numbers :prompt "Y"  :mapping-option :map)
     (X1 'numbers :prompt "X1" :mapping-option :map)
     (X2 'numbers :prompt "X2" :mapping-option :map)
     (X3 'numbers :prompt "X3" :mapping-option :map))
   (let* ((values (multiple-value-list
                   (clasp::anova-three-way (variable-value X1) (variable-value X2) (variable-value X3)
                                           (variable-value Y))))
          (result (make-generic-table-from-lists
                   (list (name x1) (name X2) (name X3)
                         (cross-string x1 x2) (cross-string x1 x3) (cross-string x2 x3)
                         (cross-string x1 x2 x3) "WITHIN GROUPS")
		   (list "SS" "DF" "MS" "F" "P")
		   (first values)
		   "Source"
                   :name "three-way-anova-table")))
     (present-with-update result
			  :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ---------------------------------------------------------------------------
;;; contingency-table analysis

(defparameter *annoying-length* 15)

(define-clasp-command (com-three-way-analysis :command-table clasp-tests
					      :name "Three Way Analysis"
					      :menu "Three Way Analysis")
    ((F1 'numbers :prompt "Factor A" :mapping-option :map)
     (F2 'numbers :prompt "Factor B" :mapping-option :map)
     (F3 'numbers :prompt "Factor C" :mapping-option :map))
   (let* ((values (multiple-value-list (clasp::analyze-3way-relationship
                                        (variable-value f1) (variable-value f2) (variable-value f3))))
          (gps (first values))
          (vl (prin1 (length (prin1 gps))))
          (names (prin1 (if (> (apply #'max (mapcar #'(lambda (v) (length (name v))) (list F1 F2 F3))) *annoying-length*)
                     '("A" "B" "C")
                     (mapcar #'name (list F1 F2 F3)))))
          (result (make-generic-table-from-lists
		   (append (when (>= vl 1) '("Complete Ind."))
                           (when (>= vl 2) names)
                           (when (>= vl 3)
			     (list (format nil "~A-~A|~A" (second names) (third names) (first names))
				   (format nil "~A-~A|~A" (first names) (third names) (second names))
				   (format nil "~A-~A|~A" (first names) (second names) (third names)))))
		   '("G" "P")
		   (append (when (>= vl 1) (list (first gps)))
			   (when (>= vl 2) (second gps))
			   (when (>= vl 3) (third gps))))))
     (present-with-update result
                          :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))
