;;;; -*- Mode: Lisp; Syntax: Common-Lisp; Base: 10; Package:CLASP-INTERFACE -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/numeric-result-classes.lisp *-*
;;;; *-* Last-Edit: Tuesday, November 15, 1994  19:05:02; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       Result Classes for Numerics                      *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************


;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-25-93 File Created.  (Carlson)
;;;  11-15-94 Added support for `mode-results' that come from
;;;           `mode-for-continuous-data' which does not return occurences.
;;;           (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :clasp-interface)

;;; --*--
;;; ***************************************************************************

(defclass numeric-result (result)
	  ((value :initarg :value :accessor value)))

(defmethod show-in-results-display ((self numeric-result))
  nil)

(defmethod display ((self numeric-result) stream)
  (present (value self) 'nice-real :stream stream
                                   :view #+clim-1 clim:+dialog-view+
                                         #+clim-2 clim:+textual-dialog-view+))

(defmethod initialize-instance :after ((self numeric-result) &key)
  (unless (and (slot-boundp self 'name)
	       (> (length (name self)) 0))
    (setf (name self) (format nil "~a" (value self)))))

;;;---------------------------------------------------------------------------

(defclass mode-result (numeric-result)
	  ((number-of-occurences :initform nil
                                 :initarg :number-of-occurences
				 :accessor number-of-occurences)))

(defmethod display ((self mode-result) stream)
  (flet ((format-item (name)
	   (let ((value (slot-value self name)))
	     (formatting-cell
	      (stream :align-x :right)
	      (present value 'nice-real :stream stream))))
	 (format-label (label)
	   (formatting-cell
	    (stream :align-x :right)
	    (with-text-style
		(#+clim-2 stream *label-text-style* #+clim-1 stream)
	      (format stream label)))))
  (formatting-table
   (stream #+clim-1 :inter-column-spacing #+clim-2 :x-spacing '(2 :character))
   (formatting-row
    (stream)
    (format-label "Mode") (format-item 'value))
   (when (slot-value self 'number-of-occurences)
     (formatting-row
      (stream)
      (format-label "Number of occurences")
      (format-item 'number-of-occurences))))))

;;;---------------------------------------------------------------------------

(defclass statistical-summary-result (numeric-result)
	  ((column-length       :initarg :column-length)
	   (min                 :initarg :min)
	   (max                 :initarg :max)
	   (range               :initarg :range)
	   (median              :initarg :median)
	   (mode                :initarg :mode)
	   (mean                :initarg :mean)
	   (variance            :initarg :variance)
	   (standard-deviation  :initarg :standard-deviation)
	   (interquartile-range :initarg :interquartile-range)
           (skewness            :initarg :skewness)))

(defun make-statistical-summary-result (&rest options)
  (apply #'make-instance 'statistical-summary-result options))

(defun make-statistical-summary-result-from-list
    (list &rest initargs &key &allow-other-keys)
  (destructuring-bind 
      (column-length min max range median mode mean variance
		     standard-deviation interquartile-range skewness)
      list
    (apply #'make-statistical-summary-result
     :column-length column-length :min min :max max :range range
     :median median :mode mode :mean mean :variance variance
     :standard-deviation standard-deviation
     :interquartile-range interquartile-range
     :skewness skewness
     initargs)))


(defmethod display ((self statistical-summary-result) stream)
    (flet ((format-labeled-item (name &optional label)
	   (let ((value (slot-value self name)))
	     (formatting-row
	      (stream)
	      (formatting-cell
	       (stream :align-x :right)
	       (with-text-style
		   (#+clim-2 stream *label-text-style* #+clim-1 stream)
		 (format stream "~@(~a~)" (or label name))))
	      (formatting-cell
	       (stream :align-x :right)
	       (present value 'nice-real :stream stream))))))
      (formatting-table
       (stream #+clim-1 :inter-column-spacing #+clim-2 :x-spacing '(2 :character))
	(format-labeled-item 'column-length "length")
	(format-labeled-item 'min)
	(format-labeled-item 'max)
	(format-labeled-item 'range)
	(format-labeled-item 'median)
	(format-labeled-item 'mode)
	(format-labeled-item 'mean)
	(format-labeled-item 'variance)
	(format-labeled-item 'standard-deviation "standard deviation")
	(format-labeled-item 'interquartile-range "interquartile range")
        (format-labeled-item 'skewness "skewness"))))

;;; ***************************************************************************
;;; EOF