;;;; -*- Mode: Common-Lisp; Package: CLASP-INTERFACE; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/result-methods.lisp *-*
;;;; *-* Last-Edit: Tuesday, April 4, 1995  14:13:23; Edited-By: westy *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                             RESULT METHODS                             *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: David L. Westbrook
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-04-95 File Created.  (westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :CLASP-INTERFACE)

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *


(defconstant *result-types*
    '(result numeric-result column number-sequence dataset
      linear-regression-verbose-result linear-regression-brief-result
      anova-one-way-table anova-one-way-result anova-two-way-table
      scheffe-table scheffe-table-entry confidence-interval-result
      significance t-test-result chi-square-result d-test-result
      graph graph-data-icon))

(defconstant *undisplayed-result-types*
    '(dataset column variable numeric-result scheffe-table scheffe-table-entry
      significance anova-one-way-table graph))

(defconstant *label-text-style*
    (make-text-style :sans-serif :bold :small))

(defclass datasets-list (modifiable-display-option-mixin)
  ((display-option :allocation :class :initform :notebook :initarg :display-option)))

(defmethod show-in-results-display ((self datasets-list)) nil)

(defclass results-list (modifiable-display-option-mixin)
  ((display-option :allocation :class :initform :notebook :initarg :display-option)))

(defmethod show-in-results-display ((self results-list)) nil)

(defmethod open-it ((result result)
		    &key (auto-size t)
		    &allow-other-keys)
  (let ((stream (display-stream result))
	(max-width (if (typep result 'dataset)
		       500
		     most-positive-fixnum))
	(max-height (if (or (typep result 'dataset)
			    (typep result 'column))
			500
		      most-positive-fixnum)))
    (if auto-size
	(with-auto-sizing (stream :max-width max-width
				  :max-height max-height)
			  (with-output-as-presentation #+clim-1 (:stream stream :object result :type (type-of result))
						       #+clim-2 (stream result (type-of result))
						       (display result stream)))
      (with-output-as-presentation #+clim-1 (:stream stream :object result :type (type-of result))
				   #+clim-2 (stream result (type-of result))
				   (display result stream)))))

(defmethod toggle-open-close ((self result) &rest args &key &allow-other-keys)
  (if (window-open-p self)
      (apply #'close-it self args)
    (apply #'open-it self args)))

;;; ----------------------------------------------------------------------------

(setf clasp::*dataset-class* 'dataset)

(defmethod print-object ((dataset dataset) stream)
  (print-unreadable-object (dataset stream)
    (princ (clasp::name dataset) stream)))

;(define-presentation-type dataset ())

(defmethod display ((dataset dataset) stream)
  (fresh-line stream)
  (formatting-table (stream)
    (loop for variable in (slot-value dataset 'clasp::variables) do
      ;; Column-oriented.
	(formatting-column (stream)
	  (with-output-as-presentation #+clim-1 (:stream stream :object variable :type 'column)
				       #+clim-2 (stream variable 'column)
	     (formatting-cell (stream :align-x :center)
			      (with-text-style
				  (#+clim-2 stream *label-text-style* #+clim-1 stream)
				(princ (name variable) stream))))
	   ;; Replace with `variable-values'.
	   (loop with values = (clasp::variable-value variable)
                 for cnt from 0 to (1- (length values))
                 for value = (elt values cnt) do
		 (with-output-as-presentation
		     #+clim-1
		     (:stream stream
                      :object (if (typep value 'number) value `',value)
		      :type (if (typep value 'number) 'number 'symbol))
		     #+clim-2
		     (stream
                      (if (typep value 'number) value `',value)
		      (if (typep value 'number) 'number 'symbol))
		   (formatting-cell (stream :align-x :right)
				    (princ value stream))))))))

;;; ----------------------------------------------------------------------------

;; We call variables columns because there is a 
;; symbol `variable' in the COMMON-LISP package (at least in allegro).
(setf clasp::*variable-class* 'column)
(setf clasp::*row-number-variable-class* 'row-number-column)

(defmethod print-object ((column column) stream)
  (print-unreadable-object (column stream)
    (princ (clasp::name column) stream)
    (write-char #\space stream)
    (princ (clasp::name (clasp::variable-dataset column)) stream)))

;(define-presentation-type column ())

(defmethod display ((variable column) stream)
  (fresh-line stream)
  (formatting-table (stream)
    (formatting-column (stream)
      (with-output-as-presentation #+clim-1 (:stream stream :object variable :type 'column)
				   #+clim-2 (stream variable 'column)
        (formatting-cell (stream :align-x :center)
			 (with-text-style
			     (#+clim-2 stream *label-text-style* #+clim-1 stream)
			  (princ (name variable) stream))))
      (loop with values = (clasp::variable-value variable)
	    for cnt from 0 to (1- (length values))
	    for value = (elt values cnt) do
            (formatting-cell (stream :align-x :right)
              (princ value stream))))))

;;; ----------------------------------------------------------------------------

;(setf graph::*dataset-class* 'graph-data)

(defun make-graph-data-icon (&rest options)
  (apply #'make-instance 'graph-data-icon options))

(defun make-graph-for-dataset (the-data-icon)
  (with-slots (graph) the-data-icon
    (unless graph
      (setf graph (make-graph :x-label (x-label the-data-icon) 
			      :y-label (y-label the-data-icon)
			      :title (description the-data-icon))))
    (when (slot-boundp the-data-icon 'graph-data)
      (add-dataset graph (graph-data the-data-icon)))
    (values graph)))

(defmethod open-it ((the-data-icon graph-data-icon)
		    &key refresh label (auto-size t)
		    &allow-other-keys)
  (declare (ignore refresh label auto-size))
  (let ((graph (make-graph-for-dataset the-data-icon)))
    (open-it graph :label (name the-data-icon))
    (setf (display-frame the-data-icon)
	  (display-frame graph))))

(defmethod display ((the-data-icon graph-data-icon) stream)
  (let ((graph (make-graph-for-dataset the-data-icon)))
    (display graph stream)))

(defparameter *graph-hardcopy-transformation*
   (make-translation-transformation -50 0))

(defmethod hardcopy ((the-data-icon graph-data-icon) filename)
  (with-open-file (file-stream filename :direction :output
		   :if-exists :overwrite
		   :if-does-not-exist :create)
    (clim:with-output-to-postscript-stream (postscript-stream file-stream :multi-page nil)
      (let ((dwim::*use-color-for-postscript* nil))
	(with-slots (graph) the-data-icon
	  ;; If we ever turn off auto open we will have to change this.
	  (when graph
	    (with-drawing-options (postscript-stream :transformation *graph-hardcopy-transformation*)
             (graph::display-graph graph :stream postscript-stream))))))))

;;; ----------------------------------------------------------------------------

(setf graph::*graph-class* 'graph) ;; was graph-user:: -AC

(defun make-graph (&rest options)
  (apply #'make-instance graph::*graph-class* options)) ;; was graph-user:: -AC

(defmethod open-it ((graph graph)
		    &key refresh label (auto-size t)
		    &allow-other-keys)
  (declare (ignore auto-size))
  (let ((stream (display-stream graph)))
    (multiple-value-bind (visible error)
	(and stream
	     (ignore-errors (window-visibility stream)))
      (declare (ignore visible))
      (when (> (length (graph::title graph)) 30)
	(setf (graph::title graph)
	  (format nil "~a..." (subseq (graph::title graph) 0 26))))
      (when (> (length (graph::y-label graph)) 15)
	(setf (graph::y-label graph)
	  (format nil "~a..." (subseq (graph::y-label graph) 0 11))))
      (cond ((or error (null stream))
	     ;; I had to hack `graph::view-graphs' to return a frame here.
	     (with-clasp-frame (clasp-frame)
		(setf (display-frame graph) (graph::view-graphs
					     (list graph) 
					     :master clasp-frame
					     :title (or label (clasp::name graph))))
		(push (display-frame graph) (inferiors clasp-frame))))
	    (t
	     (window-expose window)
	     (when refresh (window-refresh (display-stream graph))))))))


(defmethod display ((the-graph graph) stream)
  (when (> (length (graph::title the-graph)) 30)
    (setf (graph::title the-graph)
      (format nil "~a..." (subseq (graph::title the-graph) 0 26))))
  (when (> (length (graph::y-label the-graph)) 15)
    (setf (graph::y-label the-graph)
      (format nil "~a..." (subseq (graph::y-label the-graph) 0 11))))
  (graph::display-graph ;; was graph-user:: -AC
   the-graph
   ;:width 440 :height 390
   :scroll-if-necessary t
   :stream stream))

(defvar *colors* '(:gold :green :magenta :cyan :red))

(defmethod add-dataset ((graph graph) (graph-data graph:graph-data))
  ;; Pick another color if the color has already been used in this graph.
  (unless (member graph-data (graph::datasets graph) :test #'eq)
    (let* ((used-colors (mapcar #'color (graph::datasets graph)))
	   (next-color (first (set-difference *colors* used-colors :test #'eq))))
      (when (member (color graph-data) used-colors)
	(setf (color graph-data) next-color))))
  (graph::add-dataset graph graph-data))

#+OLD
(defmethod execute-on-target ((graph graph) (the-data-icon graph-data-icon))
  (let* ((new-title (concatenate 'string (graph::title graph) " and " (clasp::name the-data-icon)))
	 (new-graph (make-graph :x-label (x-label the-data-icon) 
				:y-label (y-label the-data-icon)
				:title new-title))
	(old-datasets (graph::datasets graph)))
    (dolist (dataset old-datasets)
      (graph::add-dataset new-graph dataset))
    ;; the-data-icon objects have a graph-data slot (if there is only
    ;; one scigraph::dataset attached to the icon) and a graph slot (which
    ;; is used for things link scatterplot w/ regression line.)  This checks
    ;; if the graph slot is being used and uses its datasets, otherwise, it
    ;; just uses the graph-data slot.
    (if (graph the-data-icon)
	(dolist (dataset (graph::datasets (graph the-data-icon)))
	  (add-dataset new-graph dataset))
	(add-dataset new-graph (graph-data the-data-icon)))
    (open-it new-graph :refresh t :label new-title)))

;;; ----------------------------------------------------------------------------

(defun redisplayable-format (stream string &rest args)
  (updating-output (stream
		    :unique-id string
		    :cache-value args
		    :cache-test #'equal
		    :copy-cache-value t)
     (apply #'format stream string args)))

(define-presentation-method present 
    (window-point (type window-point) stream (view clim:textual-view) &key)
  (redisplayable-format stream "~A (~D.~D)" 
			(window window-point)
			(point-x window-point)
			(point-y window-point)))

(defun make-window-point (window x y)
  (make-instance 'window-point :window window :x x :y y))

(define-presentation-translator screen-location 
    (blank-area 
     window-point
     clasp
     :documentation "Click to pick a location"
     :tester ((window)
	      (with-clasp-frame (clasp-frame)
		(display-or-inferior-pane-p clasp-frame window))))
  
  (window x y)
  (values (make-window-point window x y)))

;;; ----------------------------------------------------------------------------

(defmethod display ((self datasets-list) stream)
  (window-clear stream)
  (dolist (dataset (clasp::find-instances 'dataset))
	  (with-text-face (stream :bold)
			  (present dataset 'dataset :stream stream :view +textual-results-view+))
	  (terpri stream)
	  (dolist (variable (clasp::dataset-variables dataset))
		  (write-char #\space stream)
		  (write-char #\space stream)
		  (present variable 'column :stream stream :view +textual-results-view+)
		  (terpri stream))
	  (terpri stream))
  #+MCL
  (set-scroll-max stream))

(defmethod open-it ((self datasets-list) &key &allow-other-keys)
  (display self (display-stream self)))

(defmethod display ((self results-list) stream)
  (window-clear stream)
  (dolist (result (clasp::find-instances 'result))
	  (when (show-in-results-display result)
		(present result (type-of result) :stream stream :view +textual-results-view+)
		(terpri stream)))
  #+MCL
  (set-scroll-max stream))

(defmethod open-it ((self results-list) &key &allow-other-keys)
  (display self (display-stream self)))


;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************