;;;; -*- Mode:Common-Lisp; Package:CLASP-INTERFACE; Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/clasp/clasp/development/interface/stat-commands.lisp *-*
;;;; *-* Last-Edit: Friday, April 21, 1995  16:51:16; Edited-By: westy *-*
;;;; *-* Machine: Lindy (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                        Data Description Commands                       *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************

;;; Copyright (c) 1990 - 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.

;;; For more information write to clasp-support@cs.umass.edu

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-18-93 File Created.  (Westy)
;;;  11-15-94 Made com-mode use `mode-for-continuous-data'.  (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :CLASP-INTERFACE)

;;; --*--
;;; ***************************************************************************

(define-command-table clasp-description)

(add-menu-item-to-command-table 'clasp "Describe" 
                                :menu 'clasp-description 
				:documentation "Descriptive Statistics"
                                :errorp nil 
                                :after "graph")

;;; ---------------------------------------------------------------------------
;;; I ordered these the same way they are in the statistical-fns file.

(define-clasp-command (com-data-length :command-table clasp-description
				       :name t :menu "Data Length")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'data-length (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'data-length (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Length ~a" (name column-1))
			   :description (format nil "~@(data length ~a~)"
						(description column-1)))
			 :stream *standard-output*
                         :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-mean :command-table clasp-description :name t
                                 :menu "Mean")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values
	 (multiple-value-list (funcall #'mean (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'mean (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Mean ~a" (name column-1))
			   :description (format nil "~@(mean ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))
 
(define-clasp-command (com-variance :command-table clasp-description :name t
                                     :menu "Variance")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'variance (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'variance (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Variance ~a" (name column-1))
			   :description (format nil "~@(variance ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))
 
(define-clasp-command (com-standard-deviation :command-table clasp-description
					:name t :menu "Standard Deviation")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'standard-deviation (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'standard-deviation (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Std. Dev. ~a" (name column-1))
			   :description (format nil
						"~@(standard deviation ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-minimum :command-table clasp-description
			     :name t :menu "Minimum")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'minimum (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'minimum (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Min ~a" (name column-1))
			   :description (format nil "~@(minimum ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-maximum :command-table clasp-description
			     :name t :menu "Maximum")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'maximum (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'maximum (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Max ~a" (name column-1))
			   :description (format nil "~@(maximum ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-range :command-table clasp-description
			   :name t :menu "Range")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'range (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'range (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Range ~a" (name column-1))
			   :description (format nil "~@(range ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-quantile :command-table clasp-description :name t
			      :menu "Quantile")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (real-2 #-lucid '(real 0 1) #+lucid 'real :prompt "Percentile (real from 0 to 1)"))
  (let ((values (multiple-value-list
		    (funcall #'quantile (simplify column-1) real-2))))
    (assert (= (length values) 3) nil
      "`~a' returned ~s values, we were expecting ~s"
      'quantile (length values) 3)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Quantile(~1,2f) ~a"
					  real-2 (name column-1))
			   :description (format nil "~@(quantile(~1,2f) ~a~)"
						real-2 (description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-median :command-table clasp-description 
			    :name t :menu "Median")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values	(multiple-value-list
		    (funcall #'median (simplify column-1)))))
    (assert (= (length values) 3) nil
      "`~a' returned ~s values, we were expecting ~s"
      'median (length values) 3)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Median ~a" (name column-1))
			   :description (format nil "~@(median ~a~)" 
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-trimmed-mean :command-table clasp-description
				  :name t :menu "Trimmed Mean")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (real-2 #-lucid '(real 0 1/2) #+lucid 'real :prompt "Trimming factor (real from 0 to .5)"))
  (let ((values (multiple-value-list
		    (funcall #'trimmed-mean (simplify column-1) real-2))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'trimmed-mean (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Trimmed mean (~1,2f) ~a"
					  real-2 (name column-1))
			   :description (format nil
						"~@(trimmed mean (~1,2f) ~a~)"
						real-2 (description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(defun call-for-n-values (n function-symbol &rest args)
  (let ((values (multiple-value-list (apply function-symbol args))))
    (assert (= (length values) n) nil
            "`~a' returned ~s values, we were expecting ~s"
            function-symbol (length values) n)
    (values-list values)))

(define-clasp-command (com-mode :command-table clasp-description :name t
			  :menu "Mode")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
   (let* ((numbers (simplify column-1))
          (continuous? (clasp::data-continuous-p numbers)))
     (multiple-value-bind (value number-of-occurences)
         (call-for-n-values
          (if continuous? 1 2)
          (if continuous? 'mode-for-continuous-data 'mode)
	  numbers)

       (present-with-update (make-instance 'mode-result
			                   :value value
			                   :name (format nil "Mode ~a" (name column-1))
			                   :number-of-occurences (unless continuous? number-of-occurences)
			                   :description (format nil "~@(mode~:[ of the discrete variable~; of the continuous variable~] ~a~)"
                                                                continuous?
						                (description column-1)))
			    :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+))))

(define-clasp-command (com-interquartile-range
		       :command-table clasp-description
		       :name t :menu "Interquartile Range")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'interquartile-range (simplify column-1)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'interquartile-range (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "Interquartile range ~a"
					  (name column-1))
			   :description (format nil
						"~@(interquartile range ~a~)"
						(description column-1)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-statistical-summary
		       :command-table clasp-description
		       :name t :menu "Statistical Summary")
    ((column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'statistical-summary (simplify column-1)))))
    (assert (= (length values) 11) nil
      "`~a' returned ~s values, we were expecting ~s"
      'statistical-summary (length values) 11)
    (present-with-update
     (make-statistical-summary-result-from-list
      values
      :name (format nil "Stat. Sum. ~a" (name column-1))
      :description (format nil "~@(statistical summary of ~a~)"
				  (description column-1)))
     :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-covariance :command-table clasp-description
				      :name t :menu "Covariance")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values (multiple-value-list
		    (funcall #'covariance
			     (simplify column-1) (simplify column-2)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'covariance (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :value (nth 0 values)
			   :name (format nil "covariance ~a/~a"
					  (name column-1) (name column-2))
			   :description (format nil
						"~@(covariance of ~a and ~a~)"
						(description column-1)
						(description column-2)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-correlation :command-table clasp-description
				 :name t :menu "Correlation")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map))
  (let ((values
	 (multiple-value-list (funcall
			       #'correlation
			       (simplify column-1)
			       (simplify column-2)))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'correlation (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :name (format nil "Correlation of ~a and ~a" 
					 (name column-1) (name column-2))
			   :value (nth 0 values)
			   :description (format nil
						"~@(correlation of ~a and ~a~)"
						(description column-1)
						(description column-2)))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-partial-correlation :command-table clasp-description
				               :name t :menu  "Partial Correlation")
    ((column-2 'numbers :prompt "Y" :mapping-option :map)
     (column-1 'numbers :prompt "X" :mapping-option :map)
     (parents '(sequence numbers) :prompt "IV's"))
  (let ((values
	 (multiple-value-list (funcall
			       #'clasp::partials-from-parents
			       (simplify column-1)
			       (simplify column-2)
                               parents))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'correlation (length values) 1)
    (present-with-update (make-instance 'numeric-result
			   :name (format nil "Partial correlation of ~a and ~a holding ~{~a~^, ~} constant." 
					 (name column-1) (name column-2) (mapcar #'name parents))
			   :value (nth 0 values)
			   :description (format nil
						"~@(partial correlation of ~a and ~a holding ~{~a~^, ~} constant.~)"
						(description column-1)
						(description column-2)
                                                (mapcar #'description parents)
                                                ))
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))


(define-clasp-command (com-cross-correlation :command-table clasp-description
				       :name t :menu "Cross Correlation")
    ((column-1 'numbers :prompt "Y" :mapping-option :map)
     (column-2 'numbers :prompt "X" :mapping-option :map)
     (min-lag 'integer :prompt "Min lag" :default 0)
     (max-lag 'integer :prompt "Max lag"))
  (when (< max-lag min-lag)
    (let (temp)
      (setf temp column-1
	    column-1 column-2
	    column-2 temp)
      (setf temp min-lag
	    min-lag max-lag
	    max-lag temp)))
  (let* ((values
	 (multiple-value-list (funcall
			       #'cross-correlation
			       (simplify column-1)
			       (simplify column-2)
			       max-lag min-lag)))
	 (number-seq
	  (make-instance 'number-sequence
	    :value (nth 0 values)
	    :name (format nil "Cross Corr. [~a ~a] ~a/~a"
			    min-lag max-lag (name column-1) (name column-2))
	    :description
	    (format nil
		    "cross correlation of ~a and ~a with lags from ~a to ~a"
		    (description column-1)
		    (description column-2)
		    min-lag max-lag))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'cross-correlation (length values) 1)
    (present-with-update number-seq
			 :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

(define-clasp-command (com-autocorrelation :command-table clasp-description
				     :name t :menu "Autocorrelation")
    ((column-1 'numbers :prompt "X" :mapping-option :map)
     (min-lag 'integer :prompt "Min lag" :default 0)
     (max-lag 'integer :prompt "Max lag"))
  (when (< max-lag min-lag)
    (let ((temp max-lag))
      (setf max-lag min-lag)
      (setf min-lag temp)))
  (let* ((values
	  (multiple-value-list (funcall
				#'autocorrelation
				(simplify column-1)
				max-lag min-lag)))
	 (number-seq
	  (make-instance 'number-sequence
	   :value (nth 0 values)
	   :name (format nil "Autocorr. [~a ~a] ~a" min-lag max-lag
			  (name column-1))
	   :description
	   (format nil "autocorrelation of ~a with lags from ~a to ~a"
		   (description column-1)
		   min-lag max-lag))))
    (assert (= (length values) 1) nil
      "`~a' returned ~s values, we were expecting ~s"
      'autocorrelation (length values) 1)
    (present-with-update number-seq
	     :stream *standard-output* :view #+clim-1 clim:+dialog-view+ #+clim-2 +textual-dialog-view+)))

;;; ----------------------------------------------------------------------------

#+FBD-OBSOLETE
(define-clasp-command (com-fbd-once :command-table clasp-description
			            :name "FBD" :menu  "FBD")
    ((dv 'numbers :prompt "Y")
     (ivs '(sequence numbers) :prompt "X's"))

   (compute-fbd dv ivs t))

#+FBD-OBSOLETE
(define-clasp-command (com-fbd-bs :command-table clasp-description
			          :name "Booted FBD" :menu "Booted FBD")
    ((dv 'numbers :prompt "Y")
     (ivs '(sequence numbers) :prompt "X's")
     (count `(integer 0 *))
     (shuffle 'boolean)
     (resample 'boolean)
     (verbose 'boolean))

   (compute-bs-fbd dv ivs shuffle resample count verbose))

#+FBD-OBSOLETE
(defun compute-bs-fbd (dv ivs sh rs nrs vb)
  (let ((mx (path-analysis::bootstrap-model (mapcar #'simplify (append ivs (list dv)))
                                            (mapcar #'clasp::name  (append ivs (list dv)))
                                            :num-times nrs :verbosep vb
                                            :num-samples (and rs (clasp::data-length (simplify dv)))
                                            :shuffle sh)))
    (path-analysis::show-links mx (mapcar #'clasp::name  (append ivs (list dv))) :num-times nrs)
    mx))

#+FBD-OBSOLETE
(defun compute-fbd (dep indep verbose)
  (path-analysis::fbd
   (path-analysis::make-model-data
    (mapcar #'simplify
            (append indep (list dep)))
    :names-alist
    (path-analysis::variable-names-alist
     :names (mapcar #'clasp::name (append indep (list dep)))))
   :verbosep verbose))

;;; ***************************************************************************
;;; EOF