;;;; -*- Mode: Common-Lisp; Package: EKSL-UTILITIES; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/anaphoric.lisp *-*
;;;; *-* Last-edit: Saturday, May 20, 1995  16:35:38; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                            ANAPHORIC MACROS                            *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************

;;; Copyright (c) 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.
;;;
;;; Written by: David L. Westbrook
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  05-12-95 File Created.  (Westy)
;;;  05-20-95 Made `it' and `self' appear in the correct package, exported all, 
;;;           and documented most.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :EKSL-UTILITIES)

(export '(aif 
	   awhen 
	   awhile 
	   aand 
	   acond 
	   alambda 
	   ablock 
	   aif2 
	   awhen2 
	   awhile2 
	   acond2))

;;; --*--
;;; ***************************************************************************

(defmacro aif (test-form then-form &optional else-form)
  "Anaphoric IF:  Binds the symbol `it' to the value of the `test.'"
  (let ((it (form-symbol "IT")))
    `(let ((,it ,test-form))
       (if ,it ,then-form ,else-form))))

(defmacro awhen (test-form &body body)
  "Anaphoric WHEN:  Binds the symbol `it' to the value of the `test.'"
  `(aif ,test-form
        (progn ,@body)))

(defmacro awhile (expr &body body)
  "Anaphoric WHILE loop: Executes `expr' and `body' as long as `expr' is true.
Binds the symbol `it' to the value of the `test.'"
  (let ((it (form-symbol "IT")))
    `(do ((,it ,expr ,expr))
	 ((not ,it))
       ,@body)))

(defmacro aand (&rest args)
  "Anaphoric AND:  Binds the symbol `it' to the value of the preceding `arg.'"  
  (cond ((null args) t)
        ((null (cdr args)) (car args))
        (t `(aif ,(car args) (aand ,@(cdr args))))))

(defmacro acond (&rest clauses)
  "Anaphoric COND:  Binds the symbol `it' to the value of the preceding `clause.'"
  (if (null clauses)
      nil
      (let ((cl1 (car clauses))
	    (it  (form-symbol "IT"))
            (sym (gensym)))
        `(let ((,sym ,(car cl1)))
           (if ,sym
               (let ((,it ,sym))
                 ,it ;; prevent compiler warnings about unused IT
                 ,@(cdr cl1))
               (acond ,@(cdr clauses)))))))

(defmacro alambda (parms &body body)
  "Anaphoric LAMBDA: Binds the symbol `self' to the function, so that it can
call itself recursively."
  (let ((self (form-symbol "SELF")))
    `(labels ((,self ,parms ,@body))
       #',self)))

(defmacro ablock (tag &rest args)
  `(block ,tag
     ,(funcall (alambda (args)
                 (case (length args)
                   (0 nil)
                   (1 (car args))
                   (t `(let ((it ,(car args)))
                         ,(self (cdr args))))))
               args)))

(defmacro aif2 (test &optional then else)
  "Needs to be documented.  See Paul Graham's book."
  (let ((win (gensym))
	(it  (form-symbol "IT")))
    `(multiple-value-bind (,it ,win) ,test
       (if (or ,it ,win) ,then ,else))))

(defmacro awhen2 (test &body body)
  "Needs to be documented.  See Paul Graham's book."
  `(aif2 ,test
         (progn ,@body)))

(defmacro awhile2 (test &body body)
  "Needs to be documented.  See Paul Graham's book."
  (let ((flag (gensym)))
    `(let ((,flag t))
       (while ,flag
         (aif2 ,test
               (progn ,@body)
               (setq ,flag nil))))))

(defmacro acond2 (&rest clauses)
  "Needs to be documented.  See Paul Graham's book."
  (if (null clauses)
      nil
      (let ((cl1 (car clauses))
            (val (gensym))
            (win (gensym))
	    (it  (form-symbol "IT")))
        `(multiple-value-bind (,val ,win) ,(car cl1)
           (if (or ,val ,win)
               (let ((,it ,val)) ,@(cdr cl1))
               (acond2 ,@(cdr clauses)))))))



;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************