;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/parameters.lisp *-*
;;;; *-* Last-edit: Sunday, May 7, 1995  20:47:33; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                             Mess Parameters                            *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  05-06-95 File Created.  (anderson)
;;;  07-01-95 Stole choose-*-values from Phoenix.  `choose-variable-values'
;;;           should be moved someplace better.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(define-parameter
          undefine-parameter
	  choose-variable-values
	  choose-parameter-values))

;;; --*--
;;; ***************************************************************************


(defvar *parameter-names* nil
  "A list of parameters defined by `define-parameter'.")

(defmacro define-parameter
	  (parameter default-value
	   &optional (short-documentation nil)
	   &key (type t type-specified?) allowed-values cvv-menu-type
	   extra-documentation export?
	   &aux documentation)
  "Define a system parameter so that it's included in menus of parameters.
`short-documentation' should be a concise, one or two line, description of the
parameter.  `extra-documentation' can be used to provide volumes of information
to the users.  `type' should be a standard Common Lisp type.  `allowed-values'
should be a list of values the parameter can accept and is evaluated at compile
time.  `export?' indicates whether the symbol should be available to other
packages.  `cvv-menu-type' specifies the item type that a choose-variable-values
menu might use for this parameter \(`allowed-values' will also be used for
certain cvv item types\)."

  '(when (and type (not (typep default-value type)))
    (error "Default value of `~S' for the `~A' parameter is not of type `~A'."
           default-value parameter type))

  (unless short-documentation
    (setf short-documentation "This Park parameter was not documentated by its creator."))
  ;; `allowed-values' is evaluated at compile time
  (setf allowed-values (eval allowed-values)
        documentation (format nil "~a~@[~%~a~]~@[~%~a~]~@[~%~a~]"
                              short-documentation
                              (when extra-documentation
                                extra-documentation)
                              (when allowed-values
                                (format nil "Allowed values are ~{~s~^, ~}. Default is ~s" allowed-values default-value))
                              (when type-specified?
                                (format nil "Type of value is ~a." type))))
  `(progn
     (defvar ,parameter ,default-value ,documentation)
     (proclaim '(type ,type ,parameter))
     (eval-when (load eval)
       (when ,export?
	 (export  ',parameter))
       (pushnew ',parameter *parameter-names*)
       (setf (get ',parameter 'type               ) ',type)
       (setf (get ',parameter 'default-value      ) ,default-value)
       (setf (get ',parameter 'short-documentation) ',short-documentation)
       (setf (get ',parameter 'cvv-menu-type      ) ',cvv-menu-type)
       ; `allowed-values' must now be evaluated:
       (setf (get ',parameter 'allowed-values     ) ',allowed-values))
     ',parameter))

(defmacro undefine-parameter (parameter &rest ignore)
  "Removes the parameter from the menus."
  (declare (ignore ignore))
  `(progn
     (setf *parameter-names* (remove ',parameter *parameter-names* :test #'eq))
     (makunbound ',parameter)))

;;; ============================================================================

(defun choose-variable-values (item-list &rest options
				         &key abortable border-color &allow-other-keys)
  #-Explorer
  (nyi item-list options border-color abortable)
  #+Explorer
  (apply (if abortable
             #'utils:abortable-choose-variable-values
             #'w:choose-variable-values)
         item-list
         :foreground-color *menu-foreground*
         :background-color *menu-background*
         :label-color      *menu-label-foreground*
         :label-background *menu-label-background*
         :label (let ((label (getf options :label "Choose Variable Values")))
                  (and label
                       (etypecase label
                         (cons (append `(:font *menu-label-font*) label))
                         (string `(:font *menu-label-font* :string ,label)))))
         :allow-other-keys t
         options))

(defun choose-parameter-values ()
  "Pops up a menu that allows one to set the value of all the defined parameters."
  (let ((item-list
          (mapcar #'(lambda (parameter)
                      (let ((actual-type (get parameter 'type))
                            (cvv-menu-type (get parameter 'cvv-menu-type))
                            (allowed-values (get parameter 'allowed-values)))
                        (list* parameter
                               (string-capitalize (string-trim '(#\*) (symbol-name parameter)) :spaces t)
                               :documentation (documentation parameter 'variable)
                               (cond ((eq cvv-menu-type :choose)
                                      `(,cvv-menu-type ,allowed-values))
                                     ((consp cvv-menu-type)
                                      `(,@cvv-menu-type))
                                     (cvv-menu-type
                                      `(,cvv-menu-type))
                                     (actual-type
                                      `(:typep ,actual-type))))))
                  (sort (copy-list *parameter-names*) 'string-lessp))))
    (choose-variable-values
      item-list
      :label "Edit Parameter Values")))

;;; ***************************************************************************
;;; EOF
