;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                              LISP Speech                               *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Matt Schmill

;;; Department of Computer Science
;;; University of Massachusetts
;;; Amherst MA 01003-4610

;;; Copyright (c) 1995 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee, provided that the above
;;; copyright notice of EKSL, this paragraph and the one following appear
;;; in all copies and in supporting documentation.
;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity of
;;; such damages.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  4-16-96 Created (schmill)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(say defsound initialize-speech sound speak*))

(defparameter *delay?*
  #FEATURE-CASE
  ((ALPHA t)
   (MCL NIL)
   (SPARC t)))

;; speech stuff

#+UNIX
(defparameter +best-shell+
    (cond ((probe-file "/usr/local/bin/bash") "/usr/local/bin/bash")
	  ((probe-file "/usr/local/bin/tcsh") "/usr/local/bin/tcsh")
	  ((probe-file "/usr/local/bin/csh") "/usr/local/bin/csh")
	  (t "/bin/sh")))

(defun initialize-speech ()
  #FEATURE-CASE
  ((ALPHA #FEATURE-CASE
	  ((ALLEGRO (excl:run-shell-command "Aaxp &"))
	   (LISPWORKS (system::call-system "Aaxp &" +best-shell+))))))	   

(defun speak* (speak-string &key (wait t))
  (let ((c-string #FEATURE-CASE
		  ((ALPHA (format nil "aplay ~A" speak-string))
		   (SUN   (format nil "cat   ~A > /dev/audio")))))
    #+UNIX
    (apply #FEATURE-CASE
	   ((ALLEGRO #'excl:run-shell-command)
	    (LISPWORKS #'system::call-system))
	   #FEATURE-CASE
	   ((ALLEGRO (list c-string :wait wait))
	    (LISPWORKS (list c-string +best-shell+))))))

(defun system-errorp (code)
  #FEATURE-CASE
  ((UNIX (> code 0))))

(defmacro trying-to-ensure-speech (&body speak-body)
  (let ((r1 (gensym "RESULT")))
    `(let ((,r1 ,@speak-body))
       (if (system-errorp ,r1)
	   (progn (initialize-speech)
		  ,@speak-body)
	 ,r1))))

(defun speak (speak-string)
  (trying-to-ensure-speech (speak* speak-string)))

;;; ================================================================

(defvar +sound-db+ (make-hash-table :test #'equal))

(defclass sound ()
  ((string :initform NIL :initarg :speech :accessor sound-string)
   (tag    :initform NIL :initarg :tag    :accessor sound-tag)
   (delay  :initform NIL :initarg :delay  :accessor sound-delay)))

(defun defsound (tag delay string)
  (setf (gethash tag +sound-db+) (make-instance 'sound :speech string :tag tag :delay delay)))

(defun find-sound (tag)
  (gethash tag +sound-db+))

(defun sound (sound-tag)
  (speak* (sound-string (find-sound sound-tag)))
  (when *delay?*
    (sleep (sound-delay (find-sound sound-tag)))))

(defun say (&rest utterances)
  (loop for u in utterances do
	(sound u)))

;;;
#+TEST
(progn
  (defsound "unit" 0.7 "~schmill/stuff/unit")
  (defsound "one"  0.2 "~schmill/stuff/1")
  (defsound "now"  0.2 "~schmill/stuff/now")
  (defsound "operational" 1.2 "~schmill/stuff/operational"))