;;;; -*- Mode: Common-Lisp; Package: EKSL-UTILITIES; Base: 10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/undefmethod.lisp *-*
;;;; *-* Last-Edit: Saturday, May 6, 1995  12:32:29; Edited-By: anderson *-*
;;;; *-* Machine: Earhart (Alpha) *-*
;;;; *-* Software: Harlequin Common Lisp 3.2 *-*
;;;;
;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                              UNDEFMETHOD                               *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  05-06-95 File Created.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

(in-package :EKSL-UTILITIES)

(proclaim '(optimize (speed 3) (safety 1)))

(export '(undefmethod))

;;; ---------------------------------------------------------------------------
;;; Macro to help in dynamically removing a method definition.  It's easy to
;;; redefine a method, but it's hard to make it go away entirely.  This uses 
;;; exactly the same syntax as `defmethod,' but removes it.
;;; ---------------------------------------------------------------------------

(defmacro undefmethod (function-specifier &body body)
  "Syntax is identical to `defmethod,' but removes the method.  Clearly, this
macro has no place in delivered code, but it can be a tremendous help during
development."
  (let ((qualifiers nil))
    (do () ((listp (car body)))
      (push (pop body) qualifiers))
    (setf qualifiers (nreverse qualifiers))
    (spy qualifiers)
    (let* ((arglist      (pop body))
	   (specializers (subseq arglist 0 (position-if #'(lambda (elt) (member elt lambda-list-keywords))
                                                        arglist)))
	   (classes      (mapcar #'(lambda (x)
			             (if (symbolp x) 't (find-class (second x))))
			         specializers)))
      `(let ((m (find-method #',function-specifier ',qualifiers ',classes)))
        (if (y-or-n-p "delete ~s ? " m) (remove-method #',function-specifier m))))))

#+test
(undefmethod foo ((x mess::event)))

#+test
(undefmethod foo :before ((x mess::event)))

#+test
(undefmethod foo :before :sideways ((x mess::event)))



;;; ***************************************************************************
;;; *                              End of File                                *
;;; ***************************************************************************