;;;; -*- Mode:Common-Lisp; Package:eksl-utilities; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/collecting.lisp *-*
;;;; *-* Last-edit: Sunday, May 7, 1995  20:10:24; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                   Macros to Collect Stuff into Lists                   *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Eksl
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  05-07-95 File created and copied code from Phoenix Utils.  (anderson)
;;;  07-08-95 Fixed a bug in `collect-list.'  Return-var was being set to the
;;;           end-var symbol, instead of its value.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)


(export '(with-collection 
	   collect 
	   collect-list 
	   do-collecting 
	   do*-collecting 
	   do-collecting-results))

;;; --*--
;;; ***************************************************************************
;;; Some of these are undoubtedly obsolete, but for now, they
;;; help in porting Phoenix

(defmacro with-collection (&body body)
  "With-collection returns the list of all values that were given to
collect during the evaluation of body. e.g.
  (with-collection (collect 'a) (collect 'b)) ==> '(a b)"
  (with-variables (return-var end-var)
    `(macrolet ((collect (argument)
		  `(if ,',end-var
		       (rplacd ,',end-var (setq ,',end-var (cons ,argument nil)))
		       (setf ,',end-var (cons ,argument nil)
			     ,',return-var ,',end-var)))
		(collect-list (argument)
		  `(progn
		     (if ,',end-var
			 (rplacd ,',end-var (copy-list ,argument))
			 (setf ,',end-var (copy-list ,argument)
			       ,',return-var ,',end-var))
		     (setf ,',end-var (last ,',end-var)))))
       (let* ((,return-var nil)
	      (,end-var nil))
	 ,@body
	 ,return-var))))

#+test
(defun test-with-collection ()
  (print (with-collection (collect 1)))
  (print (with-collection (collect-list '(a b c))))
  (print (with-collection
          (dolist (x '(1 nil (a b c)))
            (if (listp x)
	        (collect-list x)
	        (collect x))))))

(defmacro collect (argument)
  "Adds `argument' to the list of objects being accumulated by the body of a
`with-collection.' It is an error to use this macro outside the lexical scope of
`with-collection.'"
  (declare (ignore argument))
  (error "COLLECT must be used within the lexical scope of WITH-COLLECTION."))

(defmacro collect-list (argument)
  "Adds all the elements of `argument,' which must be a list, to the list of
objects being accumulated by the body of a `with-collection.' It is an error to
use this macro outside the lexical scope of `with-collection.'"
  (declare (ignore argument))
  (error "COLLECT-LIST must be used within the lexical scope of WITH-COLLECTION."))

;;; (with-collection 'a (collect 'x) 'b (collect-list '(c d e)))

(defmacro do-collecting (&body body)
  `(with-collection
     (do ,@body)))

(defmacro do*-collecting (&body body)
  `(with-collection
     (do* ,@body)))


(defmacro do-collecting-results ((iteration-variable list) &body body)
  "Iterates over elements of LIST, binding ITERATION-VARIABLE to each, and
applying BODY.  If BODY returns a non-nil value, that value is collected.
Returns a list of collected values, in order."
  (with-variables (save-var)
    `(with-collection
       (dolist (,iteration-variable ,list)
         (let ((,save-var (progn ,@body)))
           (if ,save-var
               (collect ,save-var)))))))

;;; ***************************************************************************
;;; EOF
