;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: Titanic: /usr/users/eksl/anderson/mess/dev/utils/dump.lisp *-*
;;;; *-* Last-edit: Thursday, April 14, 1994  14:07:11; Edited-By: anderson *-* 
;;;; *-* Machine: Count (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                 Functions to Read/Write CLOS Instances                 *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  04-14-94 File Created.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(make slot-names dump-object))

;;; --*--
;;; ***************************************************************************
;;; Warning: This code uses make-instance and doesn't do anything to explicitly
;;; make a slot unbound.  If a slot was unbound during writing, no
;;; initialization is written out.  If there is an init-form or :after method or
;;; anything else that causes a slot to be given a value when the slot was
;;; unbound during writing, the slot will not be unbound.

(defparameter *allocatable-types?* t
  "If true, assumes that a function `allocate' has been defined that, when given
the name of a type, returns an instance of that type.  See the macro
`make-allocatable.'  If false, `make-instance' is used instead.")

(defun make (type &rest slots-and-values)
  "Makes an instance of `type' and fills its slots from `slots-and-values.'
Returns the object."
  (let ((obj (if *allocatable-types?*
		 (allocate type)
		 (make-instance type))))
    (do ((name      (car  slots-and-values) (car  remaining))
	 (value     (cadr slots-and-values) (cadr remaining))
	 (remaining (cddr slots-and-values) (cddr remaining)))
	((null name))
      (setf (slot-value obj name) value))
    obj))

(defun slot-names (object)
  "Returns the names of all slots of `object.'"
  #+Explorer
  (mapcar #'slot-description-name (class-slots (class-of object)))
  #+(or Lucid Lispworks)
  (mapcar #'clos:slot-definition-name (clos:class-slots (class-of object)))
  #-(or Explorer Lucid Lispworks)
  (error "Don't know how to get slot-names in this Lisp."))

(defun dump-object (obj)
  "Returns a form that evaluates to an object of the same type and the same slot
values as `obj.'"
  `(make ',(type-of obj)
	 ,@(mapcan #'(lambda (sn)
		       (if (slot-boundp obj sn)
			   (let ((val (slot-value obj sn)))
			     (if (symbolp val)
				 `(',sn ',(slot-value obj sn))
				 `(',sn ,(slot-value obj sn))))))
		   (slot-names obj))))

;;; ============================================================================
;;; Test

#+test
(progn (defclass test-class-A () ((a :initform 1)))
       (defclass test-class-B (test-class-A) ((b :initform 'b)))
       (let* ((inst (make-instance 'test-class-B))
	      (form (dump-object inst))
	      (obj  (eval form)))
	 (print inst)
	 (print form)
	 (print obj)
	 (describe obj)))

;;; ============================================================================
;;; Suggestion:  To implement *print-readably* for some class:

#+comment
(defmethod print-object ((self <class>) stream)
  (if *print-readably*
      (format stream "#.~s " (dump-object self))
      <...>))

;;; ***************************************************************************
;;; EOF
