;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/points-and-angles.lisp *-*
;;;; *-* Last-edit: Wednesday, June 21, 1995  17:51:58; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                             Primitives                                 *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Mike Greenberg
;;;             Department of Computer and Information Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  10-24-89 Header added.  (Westy)
;;;  03-16-90 Altered and added documentation.  (Westy)
;;;  04-25-90 Added some error checking to `polyline-length'. (Silvey)
;;;  06-22-90 Added the functions `normalize-angle' and 
;;;           `relative-angle->absolute-angle.'  (Anderson)
;;;  06-27-90 Added the function `angle-difference.'  (Anderson)
;;;  07-06-90 Removed all the search-path definitions, since they are not 
;;;           primitive.  They're now in the route-search code, in the 
;;;           a-star-search file.  (Anderson)
;;;  08-24-90 Fixed a bug in `copy-point'. If a place to copy it was specified
;;;           it was returning the y value not the entire point.  (Westy)
;;;  12-21-90 Added declarations to `time-stamp' and `time-only-stamp' to
;;;           prevent the compiler from optimizing the `format's within them
;;;           incorrectly.  (Westy)
;;;  03-25-91 Moved `make-initialized-array' to FS;MACROS.  (Westy)
;;;  02-18-92 Added `with-temp-points' and `with-temporary-points-enabled' and
;;;           other related implementational items.  (Westy)
;;;  04-12-94 Copied this file from Phoenix into MESS.  Changed the package to
;;;           eksl-utilities.  Removed all but the points and angles functions.
;;;           Exported the symbols.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(proclaim '(optimize (speed 3) (safety 1)))

(export '(2fpi
	   fpi
	   degrees->radians
	   radians->degrees
	   normalize-angle 
	   relative-angle->absolute-angle 
	   angle-difference 
	   point-x 
	   point-y 
	   point 
	   pointp 
           really-real
	   real-point-p
	   rounded-point-p 
	   rounded-point 
           real-point
	   point= 
	   copy-point 
	   point-apply* 
	   point-apply 
	   point-at-resolution 
	   point-at-resolution-with-offsets 
	   cell-center-point))

;;; --*--
;;; ***************************************************************************

(defconstant fpi (coerce pi 'single-float)
  "The constant pi, in single-float format.  Using this constant avoid
run-time double-float contagion.")

(defconstant 2fpi (coerce (* 2.0d0 pi) 'single-float)
  "The constant 2*pi, in single-float format.  Using this constant avoid
run-time double-float contagion.")

(defun degrees->radians (degrees)
  "Convert degrees to radians."
  (* degrees (/ fpi 180.)))

(defun radians->degrees (radians)
  "Convert radians to degrees.  Does not round the result."
  (* radians (/ 180. fpi)))

(proclaim '(inline normalize-angle))
(defun normalize-angle (angle full-circle)
  "Returns an angle equivalent to `angle,' but in the range
[0,`full-circle'\).  If `angle' is in degrees, `full-circle' should be 360; if
it is in radians, `angle' should be 2 pi.  \(Consider using the constant
`2fpi.'\).  Works for both positive and negative angles."
  ;; A possibly more efficient alg can be found in the original Phoenix file.
  #-Lispworks
  (declare (inline mod))
  (mod angle full-circle))

(defun relative-angle->absolute-angle (angle standard-angle full-circle)
  "Convert `angle,' which is an angle relative to `standard-angle' into an
absolute angle.  Relative angles are measured as clockwise rotation from some
standard angle, where a relative-angle of zero means the two vectors point the
same way.  A typical use for relative angles in Phoenix is angle to the wind;
0 is straight downwind, and 180 is straight upwind.  Absolute angles are
clockwise rotation from due East.  The value is normalized and returned in the
same units as the inputs, which is determined by `full-circle'; see the
function `normalize-angle.'"
  (declare (inline normalize-angle))
  (normalize-angle (+ standard-angle angle) full-circle))

(defun angle-difference (angle1 angle2 &optional (units :degrees))
  "Returns the difference between `angle1' and `angle2,' dealing properly with
wrapping around the circle.  For example, whether `angle1' = 330 and `angle2'
= 30 or vice versa, the result is always 60.  The angles are both measured in
`units' which must be either :radians or :degrees."
  (flet ((compute-difference (a1 a2 half-circle full-circle)
	   (let ((diff (abs (- a1 a2))))
	     (if (> diff half-circle)
		 (- full-circle diff)
		 diff))))
    (declare (inline compute-difference))
    (ecase units
      (:degrees (compute-difference angle1 angle2 180 360))
      (:radians (compute-difference angle1 angle2 fpi 2fpi)))))

;;;----------------------------------------------------------------------------
;;; Point definition

(proclaim '(inline point
		   point-x
		   point-y
		   set-point-x
		   set-point-y
		   point=
		   copy-point
		   point-at-resolution
		   point-at-resolution-with-offsets
		   cell-center-point ))

(defun point-x (point)
  (car point))
  
(defun point-y (point)
  (cdr point))

(defun set-point-x (point value)
  (setf (car point) value))

(defun set-point-y (point value)
  (setf (cdr point) value))

(defsetf point-x set-point-x)
(defsetf point-y set-point-y)

(defun point (&optional (x nil x-specified) y (setpoint nil setpoint-specified))
  "Make a new point using `x' and `y'. If `setpoint' is specified it is
destructively modified and returned."
  (if setpoint-specified
      (progn
	(setf (point-x setpoint) x
	      (point-y setpoint) y)
	setpoint)
      (if x-specified
	  (cons x y)
	  (cons 0 0))))

(defun pointp (point)
  (and (consp point)
       (numberp (point-x point))
       (numberp (point-y point))))

(defvar *weird-reals* nil
  "Lispworks seems to generate these things like #| #1E++0 is single-float
plus-infinity |#.  The function real-point-p tries to help track those down
and pushes them onto this list.")

(deftype really-real ()
  `(or integer
       (single-float ,most-negative-single-float ,most-positive-single-float)
       (double-float ,most-negative-double-float ,most-positive-double-float)
       (short-float  ,most-negative-short-float  ,most-positive-short-float)
       (long-float   ,most-negative-long-float   ,most-positive-long-float)))

(defun real-point-p (point)
  (flet ((check-real (x)
            (if (and (typep x 'real)
                     (not (typep x 'really-real)))
		(push x *weird-reals*))
            (typep x 'really-real)))
    (and (consp point)
	 (check-real (point-x point))
	 (check-real (point-y point)))))

(defun rounded-point-p (point)
  (and (consp point)
       (integerp (point-x point))
       (integerp (point-y point))))

(deftype point ()
  `(satisfies pointp))

(deftype real-point ()
  `(satisfies real-point-p))

(deftype rounded-point ()
  `(satisfies rounded-point-p))

(proclaim '(inline point=))		
(defun point= (p1 p2)
  (and (pointp p1) (pointp p2)
       (and (= (point-x p1) (point-x p2))
	    (= (point-y p1) (point-y p2)))))

(defun copy-point (point &optional (to nil to-specified))
  (if to-specified
    (progn 
      (setf (point-x to) (point-x point)
	    (point-y to) (point-y point))
      to)
    (point (point-x point) (point-y point))))

(defmacro point-apply* (fn point)
  "Descructivly apply `fn' successively to x and y components of `point'."
  (once-only (point)
    `(progn
       (setf (point-x ,point) (funcall ,fn (point-x ,point))
	     (point-y ,point) (funcall ,fn (point-y ,point)))
       ,point)))

(defmacro point-apply (fn point)
  "Apply `fn' successively to x and y components of `point' and return a new
point."
  (once-only (point)
    `(point (funcall ,fn (point-x ,point))
	    (funcall ,fn (point-y ,point)))))

(defun point-at-resolution (point resolution &optional (new-point (point)))
  "If `point' is a point on the map, and there is a grid over the map of
`resolution,' return the upper-left point of the cell that `point' is in.  The
point is destructively copied into `new-point' and returned."
  (check-type point rounded-point)
  (point (trunc2 (point-x point) resolution)
	 (trunc2 (point-y point) resolution)
	 new-point))

#+ignore
(defun point-at-resolution* (point resolution)
  (setf (point-x point) (trunc2 (point-x point) resolution)
	(point-y point) (trunc2 (point-y point) resolution))
  point)

(defun point-at-resolution-with-offsets (p1 p2 resolution)
  "Return a point in the same cell as `p1' with the same offsets as `p2'."
  (let* ((size (exp2 resolution)))
    (point (+ (trunc2 (point-x p1) resolution)
	      (mod (point-x p2) size))
	   (+ (trunc2 (point-y p1) resolution)
	      (mod (point-y p2) size)))))


(defun cell-center-point (cell-point resolution &optional (new-point (point)))
  "If `point' is a point on the map, and there is a grid over the map of
`resolution,' return the center point of the cell that `point' is in.  The
point is destructively copied into `new-point' and returned."
  (declare (inline point-at-resolution))
  (point-at-resolution cell-point resolution new-point)
  (let ((size (exp2 (1- resolution))))
    (incf (point-x new-point) size)
    (incf (point-y new-point) size)
    new-point))

#+test
(defun test-cell-center-point (&optional (res *elevation-cell-resolution*))
  (dotimes (n most-positive-fixnum)
    (multiple-value-bind (point window) (spum)
      (if (send window :point-on-screen-p point)
        (progn
          (highlight-point-with-label (point-at-resolution point res)
                                      :label (format nil "UL ~d" n) :color w:yellow)
          (highlight-point-with-label (cell-center-point point res)
                                      :label (format nil "C ~d" n)
                                      :color w:red))
        (return)))))

;;; ***************************************************************************
;;; EOF






