;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: Titanic: /usr/users/eksl/anderson/mess/eksl-utilities/print-table.lisp *-*
;;;; *-* Last-edit: Wednesday, March 1, 1995  20:44:12; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                   A Utility Function to Print Tables                   *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  02-15-95 File Created.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(print-table))

;;; --*--
;;; ***************************************************************************

(defun print-table (stream headings &rest columns)
  "Formats a table, kinda like LaTeX."
  ;; intensive consing
  (setf headings (mapcar #'princ-to-string headings))
  (when (some #'null columns)
    (error "The ~dth column has no data" (position nil columns)))
  (setf columns (mapcar #'(lambda (l)
			    (mapcar #'princ-to-string l))
			columns))
  (let* ((elt-widths  (mapcar #'(lambda (l) (reduce #'max l :key #'length))
			      columns))
	 (col-widths  (mapcar #'(lambda (elt head)
				  (+ 2 (max elt (length head))))
			      elt-widths
			      headings)))
    (fresh-line stream)
    (mapc #'(lambda (cw head)
	      ;; center heading in a column of width col-width
	      (format stream "~v:@<~a~>|" cw head))
	  col-widths
	  headings)
    (terpri stream)
    (apply #'mapc
	   #'(lambda (&rest row)
	       (mapc #'(lambda (cw ew elt)
			 ;; right-justify elt in elt-width, centered in column
                         ;; The extra format string is necessary in Lispworks,
                         ;; because they don't implement ~v correctly.
			 (format stream (format nil "~~~d:@<~~~d@a~~>|" cw ew) elt))
		     col-widths
		     elt-widths
		     row)
	       (terpri stream))
	   columns)))

#+lispworks
(defun foo ()
  (values (format nil "|~20:@<|~10@a|~>|" 'foo)
          '(format nil "|~20:@<~v@a~>|" 10 'foo)
          '(format nil "|~v:@<~v@a~>|" 20 10 'foo)))

#+test
(defun test-print-table ()
  (print-table *standard-output* '(left right) nil nil))	       

#+test
(defun test-print-table ()
  (print-table *standard-output*
	       '(left-column middle right)
	       '(1 2 3 123)
	       '(12 23 34 1234)
	       '(123 234 345 12345)))

;;; ***************************************************************************
;;; EOF
