;;;; -*- Mode:Common-Lisp; Package:cl-user; Fonts:(medfnt); Base:10 -*-
;;;; *-* File: /usr/users/eksl/systems/utils/dev/spy.lisp *-*
;;;; *-* Last-edit: Tuesday, July 18, 1995  08:39:51; Edited-By: anderson *-* 
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                       Debugging Utility Functions                      *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Department of Computer and Information Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; 
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  06-27-90 File Created.  (Anderson)
;;;  10-16-90 Altered `spy' to send its output *trace-output*. The difference
;;;           between *standard-output* and *trace-output* is that the former
;;;           is bound in all processes, while the later is not. The effect of
;;;           setting *trace-output* to point to a particular window works
;;;           globally and therefore is usefull when debugging processes that
;;;           might not have a nice window associated with them.  The problem
;;;           with this is that you have to know where your *trace-output* has
;;;           been diverted to or you could be in for some surprises.  (Westy)
;;;  03-25-91 Moved debugging-tag code out of Phoenix.  (Westy)
;;;  01-13-92 Added an optional list of options to `spy' (made it similar to 
;;;           `timeit' except that these are really optional).  (Westy)
;;;  03-18-94 Copied this file into the eksl-utilities package of the new 
;;;           Phoenix implementation and removed everything except SPY.
;;;           (anderson)
;;;  05-05-95 Added `spyx' as a convenience. (anderson)
;;;  05-20-95 Improved parsing of keyword options.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :cl-user)

(export '(spy spyx))

;;;----------------------------------------------------------------------------
;;; PACKAGE HACK:
;;;
;;; WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
;;; 
;;; This macro is put in the COMMON-LISP package so that it will be universally
;;; available without having to type a package prefix in front of it.  That's
;;; because it is a debugging function, and we want to make it quick to
;;; type.  It shouldn't exist in a finished product.
;;; 
;;; Yes, we know this is an abuse of the COMMON-LISP package, but it's the easiest way
;;; to make it universally accessible.

#+Explorer
(push si:inhibit-fdefine-warnings si:inhibit-fdefine-warnings)

(#+allegro excl::without-package-locks
 #+lispworks let #+lispworks ((lw:*packages-for-warn-on-redefinition* nil))           
 #-(or allegro lispworks) progn
 
(import '(spy spyx) :common-lisp)
(export '(spy spyx) :common-lisp)

;; Also do LISP package for those older LISPs.
#-MCL ; MCL has no lisp package
(import '(spy spyx) :lisp)
#-MCL ; MCL has no lisp package
(export '(spy spyx) :lisp)

;;; The following is really awful, since clim-lisp is supposed to be pristine.
(when (find-package :clim-lisp)
  (import '(spy spyx) :clim-lisp)
  (export '(spy spyx) :clim-lisp))

;;; --*--
;;; ***************************************************************************

;;; Printing rationals as floats doesn't work in Lispworks; it has no effect. (Westy)

(defmacro spy (&rest forms)
  "A debugging tool: wrapping this around a form causes both the form and its
values to be printed on *trace-output*.  Like `progn,' this returns the value\(s\)
of the last form, so that it can be safely wrapped about functional code as
well.  In other words, \(spy \(foo\)\) returns the same values as \(foo\).
The first form can optionally be a list of options, eg., keyword value pairs\).
Options supported:
:STREAM <stream>      - directs output to `stream'
:NO-NEWLINE <boolean> - if true, suppresses insertion of newlines between forms
:NO-EXPAND <boolean>  - if true, reprints the form with values replacing subforms
"
  (let ((no-newline nil)
	(no-expand  t)
	(stream #-Lispworks '*trace-output* #+Lispworks '*standard-output*))
    (when (and (consp (first forms)) (keywordp (first (first forms))))
      (let ((options (pop forms)))
	(do () ((null options))
	  (case (car options)
	    (:stream     (setf stream     (cadr options)))
	    (:no-newline (setf no-newline (cadr options)))
	    (:no-expand  (setf no-expand  (cadr options)))
	    (t           (cerror "ignore it and its argument"
				 "not a known option:  ~s" (car options))))
	  (setf options (cddr options)))))
    (labels ((spy-2 (form)
	       ;; produces a form equivalent to the input, except that all non-car atoms
	       ;; are evaluated, so that when it prints, the user can see the "expanded"
	       ;; version of the form.  Certain special forms are handled specially.
	       (if (atom form)
                   #+CRAP
		   `(write-nicely ,form)
                   #-CRAP
                   form
		   (let ((car (car form)))
		     (case car
		       ((setq setf)
			`(list ',car ,@(let ((place? t))
					 (mapcar #'(lambda (subform)
						     (prog1 (if place?
								`',subform
								(spy-2 subform))
							    (setf place? (not place?))))
						 (cdr form)))))
                       ((and or prog1 progn)
                        `(list ',car ,@(mapcar #'spy-2 (cdr form))))
		       (t (if (or (macro-function car)
				  (special-form-p car))
			      `',form
			      `(list ',car ,@(mapcar #'spy-2 (cdr form))))))))))
      (let ((formo (gensym "ORIGINAL-FORM"))
	    (formx (gensym "EXPANDED-FORM"))
	    (values (gensym "VALUES")))
	(flet ((spy-1 (form)
		 `(let ((,formo   ',form)
			(,formx  ,(unless no-expand (spy-2 form)))
			(,values  (multiple-value-list ,form)))
		    ,(when no-expand `(declare (ignore ,formx)))
		    ,(unless no-newline
		       `(fresh-line ,stream))
		    ,(if no-expand
			 `(format ,stream "~s =>~{ ~s~}"
                                  ,formo
                                  #-CRAP ,values
                                  #+CRAP (mapcar #'write-incely ,values))
			 `(format ,stream "~s =>~%~s =>~{ ~s~}" ,formo ,formx
                                  #-CRAP ,values
                                  #+CRAP (mapcar #'write-nicely ,values)))
		    ,(if no-newline
			 `(write-char #\space ,stream)
			 `(terpri ,stream))
		    (values-list ,values))))
	  `(progn
	     (let (#+Lispworks (*print-rationals-as-floats?* t))
	       #+Lispworks (declare (special *print-rationals-as-floats?*))
	       ,@(mapcar #'spy-1 forms))
	     ,@(when no-newline
		 `((terpri ,stream)))))))))

(defmacro spyx (&rest forms)
  "A version of `spy' which expands the forms.  See the documentation for `spy,'
particularly the :expand option.  This macro just supplies that option."
  `(spy (:no-expand nil) . ,forms))

#+test
(defun test-spy ()
  (let ((c 1/5)) (spy c))
  (spy (/ 1 2))
  (spy (:no-expand t) (floor 4 3))
  (let ((a 1) (b 2)) (spy (/ a b)))
  (let ((a 1) (b 2) c) (spy (/ (setf c (+ a b)) (+ a b))))
  (let ((a 1) (b 2) c) (spy (and a b (or (prog1 a b) (progn c)))))
  (let ((a 1) (b 2)) (spy (multiple-value-bind (c d) (floor a b) (values c d))))
  )

#+Explorer
(pop si:inhibit-fdefine-warnings)

) ; END OF PACKAGE HACK

;;; ***************************************************************************
;;; EOF
