;;;; -*- Mode:Common-Lisp; Package:eksl-utilities; Fonts:(medfnt courierfb hl12i tr12 medfnb cptfonti hl12b); Base:10 -*-
;;;; *-* File: Titanic: /usr/users/eksl/anderson/pyre/utils/testing-functions.lisp *-*
;;;; *-* Last-edit: Tuesday, March 22, 1994  14:46:59; Edited-By: anderson *-* 
;;;; *-* Machine: Chuck (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*
;;;; *-* Lisp: TI Common Lisp System 6.49  *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *                                                                        *
;;;; *                     Macros to Define Test Functions                    *
;;;; *                                                                        *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Scott D. Anderson
;;;             Experimental Knowledge Systems Laboratory
;;;             Paul R. Cohen, Principal Investigator
;;;             David L. Westbrook, Systems Manager
;;;             David M. Hart, Laboratory Manager
;;;             Department of Computer Science
;;;             University of Massachusetts
;;;             Amherst, Massachusetts 01003.
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  03-22-94 File Created.  (anderson)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(*test-functions* deftest call-test-functions))

;;; --*--
;;; ***************************************************************************

(defvar *test-functions* nil
  "A list of the names of functions produced by the \\macroind{deftest} macro.")

(defmacro deftest (name &body forms)
  "Defines a no-arg function called \\ar{name} with body \\ar{forms} that is
called to test something.  Typically, the test function uses \\fn{spy} to print
a call to the function to be tested and print its result; the test function
simply supplies reasonable arguments.  The function \\fn{call-test-functions}
iterates through all these test functions, calling them and reporting the
results.  If the \\kwdind{:test} is not on the \\gvar{*features*} list, this
macro produces nothing."
  (when (find :test *features*)
    `(progn (defun ,name () ,@forms)
	    (pushnew ',name *test-functions*))))

(defun call-test-functions (&optional no-query)
  (dolist (f *test-functions*)
    (restart-case
      (if no-query
	  (progn (format t "~2&~s~%" f)
		 (funcall (symbol-function f)))
	  (when (y-or-n-p "call ~s?" f)
	    (funcall (symbol-function f))))
      (next-test-function ()
			  :report (lambda (s)
				    (format s "Go on to the next test function"))
			  nil))))
	

;;; ***************************************************************************
;;; EOF
