;;;; -*- Mode:Common-Lisp; Package:EKSL-UTILITIES; Base:10; Syntax:common-lisp; Fonts:(medfnt) -*-
;;;; *-* File: Titanic: /usr/users/eksl/systems/utils/dev/utilities.lisp *-*
;;;; *-* Last-edit: Thursday, June 29, 1995  10:05:22; Edited-By: anderson *-* 
;;;; *-* Edited-By: anderson *-*
;;;; *-* Machine: Christa (Explorer II, Microcode 489) *-*
;;;; *-* Software: TI Common Lisp System 6.49 *-*

;;;; **************************************************************************
;;;; **************************************************************************
;;;; *
;;;; *                  Standard EKSL Utilities
;;;; *
;;;; **************************************************************************
;;;; **************************************************************************
;;;
;;; Written by: Kevin Gallagher, Philip Johnson, Daniel Corkill, Kelly Murray,
;;;             David Westbrook, Marty Humphrey, Mike Greenberg, Scott Anderson

;;; Copyright (c) 1994 University of Massachusetts
;;; Department of Computer Science
;;; Experimental Knowledge Systems Laboratory
;;; Professor Paul Cohen, Director.
;;; All rights reserved.

;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted without fee for non-commercial uses
;;; only (not for resale), provided that the above copyright notice of EKSL,
;;; this paragraph and the one following appear in all copies and in
;;; supporting documentation.

;;; EKSL makes no representation about the suitability of this software for any
;;; purposes.  It is provided "AS IS", without express or implied warranties
;;; including (but not limited to) all implied warranties of merchantability
;;; and fitness for a particular purpose, and notwithstanding any other
;;; provision contained herein.  In no event shall EKSL be liable for any
;;; special, indirect or consequential damages whatsoever resulting from loss
;;; of use, data or profits, whether in an action of contract, negligence or
;;; other tortuous action, arising out of or in connection with the use or
;;; performance of this software, even if EKSL is advised of the possiblity
;;; of such damages.

;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;;
;;;  03-01-90 File Created.  (Westy)
;;;  05-05-95 Renamed to EKSL-UTILITIES.  (Westy)
;;;  05-05-95 Added `export' and `provide-utilities' forms.  (anderson)
;;;  05-05-95 Added assert*, a variant of CL assert.  (anderson)
;;;  05-05-95 Since eksl-utilities doesn't use the TICL package, I removed the
;;;           #-Explorer that preceded the definition of once-only.  (anderson)
;;;  05-05-95 Similarly for `push-end.'  (anderson)
;;;  05-05-95 Removed `form-gbb-symbol.'  (anderson)
;;;  05-05-95 Added `doplist,' `doalist,' `doarray,' `fancy-dolist,' `dosequence,' 
;;;           `dolist3,' `group,' and `gensym0.'  (anderson)
;;;  05-08-95 Added `deletef' and `removef' from Phoenix.  Corrected the setfing
;;;           in those and other macros.  (anderson)
;;;  05-09-95 Added `defsubst' and `named-lambda.'  (anderson)
;;;  05-26-95 Moved all of the macros to the file "macros".  (Westy)
;;;  05-30-95 Added `float->integer' and exported `sort-using-list-order.'
;;;           (anderson)
;;;  06-28-95 Added `car-safe' and `cdr-safe' since they're used in Phoenix.  (anderson)
;;;  06-18-96 Added `compact-array'. (Westy)
;;;
;;; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;;; --*--

(in-package :eksl-utilities)

(export '(mapappend
	  find-all
	  find-all-if
	  find-all-if-not
	  compact-array
	  nmerge-list
          circular-list
	  make-initialized-array
	  no-op
	  fixnump
	  object->string
	  float->integer
	  sort-using-list-order
	  lambda-list->args
	  car-safe
	  cdr-safe))
;;; --*--
;;; ***************************************************************************

(provide-utilities
  (defun mapappend (fun &rest args)
    "Applies `fun' to successive elements of list, appends the resulting lists together."
    (if (some #'null args) 
	()
	(append (apply fun (mapcar #'car args))
		(apply #'mapappend  fun (mapcar #'cdr args))))))

;;;----------------------------------------------------------------------------

(setf (symbol-function 'find-all-if) #'remove-if-not)

(setf (symbol-function 'find-all-if-not) #'remove-if)

;; No sense paying extra for a function that does not do anything over
;; what is already there.
(proclaim '(inline find-all))

(defun find-all (item sequence &rest keyword-args
                 &key (test #'eql) &allow-other-keys)
  "Find all those elements of sequence that match item,
  according to the keywords.  Doesn't alter sequence."
  #-Lucid
  (apply #'remove item sequence
	 :test (complement test) keyword-args)
  ;; The :test-not args are deprecated, but Lucid is behind the times.
  #+Lucid
  (apply #'remove item sequence
         :test-not test keyword-args))

;;; ============================================================================

(defun nmerge-list (list1 list2 compare-fn &key (key #'identity))
  "Destructive merge of `list2' into `list1'.  Both lists are may be changed.
This works best if the lists are NOT cdr-coded."
  (when (not list1) (return-from nmerge-list list2))
  (when (not list2) (return-from nmerge-list list1))
  (let (new-list new-list-end)
    (cond ((funcall compare-fn
                    (funcall key (first list1))
                    (funcall key (first list2)))
           (setf new-list list1
                 list1 (cdr list1)))
          (t
           (setf new-list list2
                 list2 (cdr list2))))
    (setf new-list-end new-list)
    (do ()
        (nil)
      (when (not list1)
        (setf (cdr new-list-end) list2)
        (return))
      (when (not list2)
        (setf (cdr new-list-end) list1)
        (return))
      (cond ((funcall compare-fn
                      (funcall key (first list1))
                      (funcall key (first list2)))
             (setf (cdr new-list-end) list1
                   new-list-end list1
                   list1 (cdr list1)))
            (t
             (setf (cdr new-list-end) list2
                   new-list-end list2
                   list2 (cdr list2)))))
    new-list))

;;;----------------------------------------------------------------------------

(defun delete-if! (predicate sequence)
  "Just like `delete-if' except that it returns two values: a list of the
deleted items, and the new sequence."
  (let* ((deleted-items  nil)
         (new-sequence   (delete-if #'(lambda (item)
                                        (if (funcall predicate item)
                                            (push item deleted-items)))
                                    sequence)))
      (values (nreverse deleted-items) new-sequence)))

;;;----------------------------------------------------------------------------

;;; Replacement for (delete-if-not #'identity array) in Allegro which
;;; seems to return a new array without a fill pointer.
;; This could be slightly more efficient, but currently it is only called 
;; once when the firemap is built.
(defun compact-array (array)
  "Destructively compacts an array with fill-pointer. Works on type T
arrays and considers elements with Nil in them to be empty."
  (declare (array array))
  (let* ((next-index-to-fill (aif (position nil array :test #'eq) it -1))
	 (scan-index next-index-to-fill)
	 (length (length array)))
    (declare (fixnum next-index-to-fill scan-index length))
    (when (not (minusp next-index-to-fill))
      ;; Map over the whole array keeping 'next-index-to-fill' pointing at
      ;; next empty element while the 'scan-index' finds full elements to move.
      (loop named scan do
	    ;; look for the next full element
	    ;; or hit the end of the array
	    (loop do (when (= (incf scan-index) length) (return-from scan))
		     (when (aref array scan-index) (return)))
	    ;; if did not hit the end we copy the value to the empty spot
	    (setf (aref array next-index-to-fill) (aref array scan-index))
	    (setf (aref array scan-index) nil)
	    ;; find next empty element to fill
	    (loop until (or (= (incf next-index-to-fill) length)
			    (null (aref array next-index-to-fill)))))
    (setf (fill-pointer array) next-index-to-fill)))
  (values array))

#+test
(defun test-compact-array ()
  (flet ((doit (array)
	   (spy (:no-newline t) "BEFORE" array)
	   (compact-array array)
	   (spy (:no-newline t) "AFTER" array)))
    
    (doit (make-array 0 :fill-pointer 0))
    (doit (make-array 1 :fill-pointer 1  :initial-contents '(1)))
    (doit (make-array 2 :fill-pointer 2  :initial-contents '(nil 1)))
    (doit (make-array 2 :fill-pointer 2  :initial-contents '(1 nil)))
    (doit (make-array 3 :fill-pointer 3  :initial-contents '(nil 1 nil)))
    (doit (make-array 4 :fill-pointer 4  :initial-contents '(nil nil nil nil)))
    (doit (make-array 4 :fill-pointer 4  :initial-contents '(1 2 3 4)))
    (doit (make-array 12 :initial-contents '(1 2 nil 4 5 6 7 8 nil nil nil 12) :fill-pointer 12))
    (doit (make-array 13 :initial-contents '(1 2 nil 4 nil 6 nil 8 nil nil nil 12 nil) :fill-pointer 13))
    (doit (make-array 13 :initial-contents '(nil nil nil nil nil 6 nil 8 nil nil nil 12 nil) :fill-pointer 13))
    ))

#+test
(defun test-compact-array-speed ()
  (flet ((doit (array)
	   (let ((a2 (copy-seq array)))
	     (spy (length (time (compact-array array))))
	     (spy (length (time (setf a2 (delete-if-not #'identity a2))))))))
    
    (doit (make-array 100000 :fill-pointer t 
		      :initial-contents (loop repeat 100000 collect (if (zerop (random 2)) nil t))))))


;;; ============================================================================

(provide-utilities
 (defun circular-list (&rest elements)
   "Builds a circular list with `elements'."
   (when elements
     (setq elements (copy-list elements))
     (rplacd (last elements) elements)
     elements)))

;;; ============================================================================

(defun gensym0 (x)
  "Ignores its argument and returns a gensym.  Useful for mapping of a bunch of
arbitrary forms, since `gensym' requires that its argument be a string or a number."
  (declare (ignore x))
  (gensym "G"))

#+test
(defun test-gensym0 ()
  (print (mapcar #'gensym0 '(1 "A" b (+ 2 3)))))


;;; ============================================================================

(defun group (list n)
  "Return the elements of `list' grouped in sublists of length `n,' which should
be a positive integer.  For example, the list '(a b c d) grouped by 2 yields
'((a b) (c d)).  Definition of `group' taken from On Lisp, by Paul Graham.  The
definition is not hugely efficient, so this function is appropriate for parsing
macro args, but no for inner loops."
  (check-type list list)
  (check-type n (integer 1 #.most-positive-fixnum))
  (labels ((next (list n acc)
	     (let ((rest (nthcdr n list)))
	       (if (consp rest)
		   (next rest n (cons (subseq list 0 n) acc))
		   (nreverse (cons list acc))))))
    (next list n nil)))

#+test
(defun test-group ()
  (print (group '(a b c d) 3))
  (print (group '(a b c d) 2))
  (print (group '(a b c d) 1))
  (print (group '(a b c d) 0)))

;;; ---------------------------------------------------------------------------
;;;
;;; Kevin's Generic Slot Functions :::
;;;
;;; ---------------------------------------------------------------------------

#+PICCL
(defun nostruct (type)
  (error "~A is not a defined structure" type))
#+PICCL
(defmacro getstruct (type &Optional (ifnot nil noerrp))
  (if noerrp
      `(or (sys::symbol-get ,type 'sys::%structure-definition)
	   ,ifnot)
      `(or (sys::symbol-get ,type 'sys::%structure-definition)
	   (nostruct ,type))))


(provide-utilities
  (defun GET-DEFSTRUCT-DESCRIPTION (name)
    
    "GET-DESFSTRUCT-DESCRIPTION name

If `name' has been defined as a structure then return its
description.  Otherwise signal an error."
    
    #+LISPM   (si::get-defstruct-description name)
    #+DEC     (or (get name 'defstruct-description)
		  (error "~s is not the name of a defstruct." name))
    #+PICCL (getstruct name)
    #+MCL (or (ccl::structure-class-p name)
	      (error "~s is not the name of a defstruct." name))
    #-(or LISPM DEC PICCL MCL)
    (nyi name)))

;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun DEFSTRUCT-P (symbol)
    
    "DEFSTRUCT-P symbol

This predicate returns a non-nil value if symbol is
the name of a defstruct type; nil otherwise."
    
    (when (symbolp symbol)
      
      #+LISPM   (si::getdecl symbol 'si::defstruct-description)
      #+DEC     (get symbol 'defstruct-description)
      #+MCL     (ccl::structure-class-p symbol)
      #+PICCL (if (getstruct symbol NIL) T NIL) ;; Probably should just return value of getstruct.
      )
    
    #-(or LISPM DEC PICCL MCL) (nyi)))

;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun STRUCTURE-SLOT-NAMES (type)
    
    "STRUCTURE-SLOT-NAMES type

This function returns a list of structure slot names
given the type (symbol) of the structure."
    
    #-(or LISPM DEC PICCL MCL)
    (nyi type)
    
    #+(or LISPM DEC)
    (let* ((description (get-defstruct-description type)))
      (assert (not (null description)) ()
	      "~S is not a structure instance." type)
      (let* ((slot-data
	       #+DEC    (defstruct-description-slot-data description)
	       #+LISPM  (si::defstruct-description-slot-alist description)))
	(mapcar #'first slot-data)))
    #+MCL
    (mapcar #'first (rest (aref (get-defstruct-description type) 1)))
    #+PICCL
    (let ((dsd (get-defstruct-description type)))
      (mapcar #'piccl::ssd-%name (piccl::sd-slots dsd)))
    ))

;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun STRUCTURE-SLOT-P (type slot)
    
    "STRUCTURE-SLOT-P type slot

   This function returns true if `Slot' is the name of a slot in the
   defstruct type named by `Type.'"
    
    (and (structure-slot-index-1 type slot) t)))

;;; ---------------------------------------------------------------------------
;;;
;;; Stucture Slot Index Computation Function :::
;;;
;;; Common to both access and setting functions.
;;;
;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun STRUCTURE-SLOT-INDEX-1 (type slot)
    
    ;; Returns the index of the slot or nil if the slot doesn't
    ;; exist.  Used by STRUCTURE-SLOT-P and STRUCTURE-SLOT-INDEX.
    
    #+(or DEC LISPM)
    (let* ((description (get-defstruct-description type)))
      
      #+DEC
      (let* ((slot-data (defstruct-description-slot-data description))
	     (index (second (assoc slot slot-data
				   ;; We use string= to avoid package
				   ;; requirements.
				   :TEST #'string=))))
	index)
      
      #+LISPM
      (let* ((slot-data (si::defstruct-description-slot-alist
			  description))
	     (index (second (assoc slot slot-data
				   ;; We use string= to avoid package
				   ;; requirements.
				   :TEST #'string=))))
	(unless (null index)
	  (case (second description)
	    (:NAMED-ARRAY (+ 1 index))
	    (otherwise index)))))
    #+MCL
    (let ((pos (position (symbol-name slot) (rest (aref (GET-DEFSTRUCT-DESCRIPTION type) 1))
			 :key #'first
			 :TEST #'string=)))
      (when pos (1+ pos)))
    #+PICCL
    (let ((dsd (GET-DEFSTRUCT-DESCRIPTION type)))
      (find (symbol-name slot) (sys::sd-slots dsd)
	    :KEY #'sys::ssd-%name
	    :TEST #'string=))
    #-(or DEC LISPM MCL PICCL)
    (nyi type slot)))

;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun STRUCTURE-SLOT-INDEX (type slot)
    
    ;; Returns the index of the slot or signals an error
    ;; if the slot doesn't exist.
    
    (or (structure-slot-index-1 type slot)
	(error "~S is not a slot in ~S." slot type))))

;;;---------------------------------------------------------------------------
;;;
;;; Structure Slot Access Functions :::
;;;
;;; ---------------------------------------------------------------------------

(provide-utilities
  (defun GET-STRUCTURE-SLOT-ACCESSOR-FUNCTION (type slot)
    
    "GET-STRUCTURE-SLOT-ACCESSOR-FUNCTION type slot

Returns the accessor function for ``slot'' in structures of type ``type''."
    
    #-(or DEC LISPM)
    (nyi type slot)
    #+(or DEC LISPM)
    (let* ((description (get-defstruct-description type)))
      
      #+DEC
      (let* ((slot-data (defstruct-description-slot-data description))
	     (defstruct-name (system::defstruct-description-name slot-data))
	     ;; The following means of obtaining the package of the defstruct
	     ;; functions should work in all (?) cases ::
	     (defstruct-package
	       (symbol-package
		 (system::defstruct-description-constructor slot-data))))
	;; The following requires the umass-extended-lisp patch to VAXLisp which
	;; saves the :CONC-NAME on the symbols PList:
	(intern (concatenate 'string
			     (get type :conc-name)
			     (symbol-name slot))
		defstruct-package))
      
      #+LISPM
      (let* ((slot-data (si::defstruct-description-slot-alist
			  description))
	     (accessor (seventh (assoc slot slot-data
				       ;; We use string= to avoid package
				       ;; requirements.
				       :TEST #'string=))))
	accessor))))

;;; ---------------------------------------------------------------------------

(provide-utilities get-structure-slot
		   (defun GET-STRUCTURE-SLOT (object slot)
		     
		     "GET-STRUCTURE-SLOT object slot

Returns the value of a slot in a structure
given the name of the slot.

This function may be used as a place form for SETF."
		     #+DEC
		     (let ((type (type-of object)))
		       (%sp-structref type object (structure-slot-index type slot)))
		     #+Explorer
		     (let ((type (type-of object)))
		       (aref object (structure-slot-index type slot)))
		     #+MCL
		     (let ((type (type-of object)))
		       (ccl::struct-ref object (structure-slot-index type slot)))
		     #+PICCL
		     (let* ((dsd (getstruct
				   ;; Fast but lower
				   (sys::%stref object 0)
				   ;; Slow, but better error message!
				   ;; (type-of object)
				   ))
			    (ssd (find slot (sys::sd-slots dsd)
				       :KEY #'sys::ssd-%name
				       :TEST #'string=)))
		       (sys::%stref object (sys::ssd-index ssd)))
		     #-(or DEC Explorer PICCL MCL)
		     (nyi object slot)
		     )
		   
;;; ---------------------------------------------------------------------------
;;;
;;; Structure Slot Setting Functions :::
;;;
;;; ---------------------------------------------------------------------------
		   
		   (defun SET-STRUCTURE-SLOT (object slot value)
		     
		     ;; SET-STRUCTURE-SLOT object slot value
		     ;;
		     ;; Sets the value of a slot in a structure
		     ;; given the name of the slot.
		     
		     #+DEC
		     (let ((type (type-of object)))
		       (%sp-structset type object (structure-slot-index type slot) value))
		     #+Explorer
		     (let ((type (type-of object)))
		       (si::set-aref object (structure-slot-index type slot) value))
		     #+MCL
		     (let ((type (type-of object)))
		       (ccl::struct-set object (structure-slot-index type slot) value))
		     #+PICCL
		     (let* ((dsd (getstruct
				   ;; Fast but lower
				   (sys::%stref object 0)
				   ;; Slow, but better error message!
				   ;; (type-of object)
				   ))
			    (ssd (find (symbol-name slot) (sys::sd-slots dsd)
				       :KEY #'sys::ssd-%name
				       :TEST #'string=)))
		       (sys::%stset object (sys::ssd-index ssd) value))
		     #-(or DEC Explorer MCL PICCL)
		     (nyi object slot value))
		   
		   (defsetf get-structure-slot set-structure-slot)
		   )				; end of provide-utilities

;;; ---------------------------------------------------------------------------

;; The VaxLisp version of defstruct-conc-name is in BOOKKEEP.LISP ::

#-DEC
(provide-utilities
  (defun DEFSTRUCT-CONC-NAME (STRUCTURE)
    
    "DEFSTRUCT-CONC-NAME structure

Return a string which is the prefix for building defstruct slot
accessors for this structure.  STRUCTURE must be a symbol which
names a structure type."
    
    structure
    #+LISPM
    (let* ((description (get-defstruct-description structure)))
      (assert (not (null description)) (structure)
	      "~s is not the name of a structure." structure)
      (if (si:defstruct-description-conc-name description)
	  (string (si:defstruct-description-conc-name description))
	  ""))
    
    #+PICCL
    (let ((dsd (getstruct structure)))
      (sys::sd-conc-name dsd))
    
    #-(or LISPM PICCL)
    (error "Can't do generic defstruct operations on ~a yet."
	   (machine-type))))

;;;----------------------------------------------------------------------------

;; Not a macro, but it could be.
(provide-utilities
  (defun make-initialized-array (&rest inits)
    "Creates an array initialized from using `inits' which is a list of the form
 \(\(<index-1> <init-form-1>\)
  \(<index-2> <init-form-2>\)
  ...
  \(<index-n> <init-form-n>)\)
Each element indexed will be filled with the result of evaluating the
corresponding initialization form. The array returned will be just large
enough to hold the element with the maximum index."
    
    (let ((size 0)
	  array)
      (doplist (k v inits)
	(setf size (max size k)))
      (setf array (make-array (1+ size)))
      (doplist (k v inits)
	(setf (aref array k) v))
      array)))

;;; ===========================================================================

(defun no-op (&rest args)
  "Ignores its arguments are returns nil.  Useful when you need to pass a
function of arbitrary arity and you want it to do nothing."
  (declare (ignore args)))

;;; ===========================================================================

#-MCL ; already has this
(defun fixnump (arg)
  "Same as (typep arg 'fixnum).  A lot of Explorer code was written using this,
and it's easier to implement it than to change them all."
  (typep arg 'fixnum))

;;; ============================================================================

(defun object->string (object)
  "Uses the Common Lisp `string' function to convert `object' into a string, but
also converts objects of the form 'x to \"x\"."
  (if (and (listp object)
	   (eq (car object) 'quote))
      (string (cadr object))
      (string object)))

;;; ============================================================================

(defun float->integer (number)
  "You can't coerce a float to an integer, because they want to you decide
whether you want to use round, floor, ceiling, and so forth.  Sometimes, though,
you know or believe that the float is the floating point representation of an
integer, so all of the above would produce the same result.  This function is
for that case.  It coerces `number' to an integer, even if it's a float.  The
fractional part of the float must be zero or an error is signalled."
  (etypecase number
    (integer number)
    (float   (multiple-value-bind (int frac) (floor number)
	       (if (zerop frac)
		   int
		   (error "Doesn't represent a whole number: ~s" number))))))

#+test
(defun test-float->integer ()
  (spy (float->integer 3))
  (spy (float->integer 3.0))
  (spy (float->integer 3.1))
  (spy (float->integer 1/3)))

;;;----------------------------------------------------------------------------

;; I wouldn't put this in any time critical places if you are dealing with
;; large lists.
(defun sort-using-list-order (x list &key (test #'eq) key (list-key #'identity))
  "Sorts the list `x' using the other list `list' as a index."
  (flet ((item-lessp-function (list  test item-key list-key)
           (if item-key
             #'(lambda (item-1 item-2)
                 (setf item-2 (funcall item-key item-2)
                       item-1 (funcall item-key item-1))
                 (let ((item-2-present (member item-2 list :test test :key list-key))
                       (item-1-present (member item-1 list :test test :key list-key)))
                   (if (and item-1-present item-2-present)
                     (not (member item-1 item-2-present :test test :key list-key))
                     ;; otherwise
                     (if item-1-present t nil))))
             #'(lambda (item-1 item-2)
                 (let ((item-2-present (member item-2 list :test test :key list-key))
                       (item-1-present (member item-1 list :test test :key list-key)))
                   (if (and item-1-present item-2-present)
                     (not (member item-1 item-2-present :test test :key list-key))
                     ;; otherwise
                     (if item-1-present t nil)))))))
    (let ((lessp-function (item-lessp-function list test key list-key)))
      (stable-sort x lessp-function))))

;;; ============================================================================

(defun lambda-list->args (lambda-list)
  "Takes a formal parameter list, possibly containing things like &optional or (key
default), and returns a list of the argument variables.  This is useful when processing
arguments in a macro that will define a function, say if you want to declare them all
ignoreable or something.  Returns a flat list of symbols."
  ;; mode must go :required -> &optional -> &rest -> &key -> &allow-other-keys -> &aux
  ;; but modes may be skipped.  &rest is weird because there can be only one &rest arg,
  ;; so we have an intermediate mode &rest1, which means that one &rest arg has been
  ;; read, so error if there's another.
  (let ((mode :required)
	(args nil))
    (flet ((okay? (arg)
	     (and (symbolp arg)
		  (not (keywordp arg))
		  (not (member arg lambda-list-keywords)))))
      (dolist (elt lambda-list)
	(ecase mode
	  (:required
	   (cond ((member elt '(&optional &rest &key &aux))
		  (setf mode elt))
		 ((okay? elt)
		  (push elt args))
		 (t
		  (error "Bad required arg in lambda-list:  ~s" elt))))
	  (&optional
	   (cond ((member elt '(&rest &key &aux))
		  (setf mode elt))
		 ((okay? elt)
		  (push elt args))
		 ;; Default value, but no supplied-p var
		 ((and (listp elt)
		       (null (cddr elt))
		       (okay? (car elt)))
		  (push (car elt) args))
		 ;; Default value and supplied-p var
		 ((and (listp elt)
		       (not (null (cddr elt)))
		       (okay? (car elt))
		       (okay? (caddr elt)))
		  (push (car elt) args)
		  (push (caddr elt) args))
		 (t
		  (error "Bad &optional arg in lambda-list:  ~s" elt))))
	  (&rest
	   (cond ((member elt '(&key &aux))
		  (error "No &rest arg in lambda-list:  ~s" lambda-list))
		 ((okay? elt)
		  (push elt args)
		  ;; Special mode
		  (setf mode '&rest1))
		 (t
		  (error "Bad &rest arg in lambda-list:  ~s" elt))))
	  (&rest1
	   (cond ((member elt '(&key &aux))
		  (setf mode elt))
		 (t
		  (error "Bad &rest arg in lambda-list:  ~s" elt))))
	  (&key
	   (cond ((member elt '(&allow-other-keys &aux))
		  (setf mode elt))
		 ((okay? elt)
		  (push elt args))
		 ;; Default value, but no weird stuff
		 ((and (listp elt)
		       (okay? (car elt)))
		  (push (car elt) args))
		 ;; Weird stuff; that is, arg name supplied
		 ((and (listp elt)
		       (listp (car elt))
		       (okay? (cadar elt)))
		  (push (cadar elt) args))
		 (t
		  (error "Bad &key arg in lambda-list:  ~s" elt))))
	  (&allow-other-keys
	   (if (eq elt '&aux)
	       (setf mode elt)
	       (error "Bad stuff after &allow-other-keys: ~s" lambda-list)))
	  (&aux
	   (if (okay? elt)
	       (push elt args)
	       (error "Bad &aux arg:  ~s" elt)))))
      (nreverse args))))

#+test
(defun test-lambda-list->args ()
  (spy (lambda-list->args '(a b c)))
  (spy (lambda-list->args '(a &optional c)))
  (spy (lambda-list->args '(a &optional (c 1))))
  (spy (lambda-list->args '(a &optional (c 1 c?))))
  (spy (lambda-list->args '(a &optional (c 1 c?) &rest d)))
  (spy (lambda-list->args '(a &optional &rest d)))
  (spy (lambda-list->args '(a &rest d)))
  (spy (lambda-list->args '(a &rest d &key e)))
  (spy (lambda-list->args '(a &key e (f 2))))
  (spy (lambda-list->args '(a &key e ((:f g 2)))))
  (spy (lambda-list->args '(a &key e &allow-other-keys)))
  (spy (lambda-list->args '(a &key e &allow-other-keys &aux g)))
  (spy (lambda-list->args '(a &aux g h)))
  )

;;; ============================================================================

(proclaim '(inline car-safe))
(defun car-safe (x)
  "Returns the car of `x' if it's a cons, NIL otherwise."
  (and (consp x) (car x)))

(proclaim '(inline car-safe))
(defun cdr-safe (x)
  "Returns the cdr of `x' if it's a cons, NIL otherwise."
  (and (consp x) (cdr x)))

;;; ---------------------------------------------------------------------------
;;;				  End of File
;;; ---------------------------------------------------------------------------
